@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -66,12 +66,13 @@ class Dict(List):
             id=":'list_%s' % name" class="grid" width=":field.width"
             var2="keys=field.keys(obj);
                   subFields=field.getSubFields(zobj, layoutType);
+                  swidths=field.getWidths(subFields);
                   zobj=oobj|zobj">
       <!-- Header -->
       <tr valign="bottom">
-       <th width=":field.widths[0]"></th>
+       <th width=":swidths[0]"></th>
        <th for="subName, sub in subFields" if="sub"
-           width=":field.widths[loop.subName.nb + 1]">::_(sub.labelId)</th>
+           width=":swidths[loop.subName.nb + 1]">::sub.getListHeader(_ctx_)</th>
       </tr>
       <!-- Rows of data -->
       <x for="rowId, text in keys">:field.pxRow</x>
@@ -106,12 +107,9 @@ class Dict(List):
         # that will determine content and style for the separator row.
         self.keys = keys
 
-    def computeWidths(self, widths):
-        '''Set given p_widths or compute default ones if not given'''
-        if not widths:
-            self.widths = [''] * (len(self.fields) + 1)
-        else:
-            self.widths = widths
+    def getWidths(self, subFields):
+        '''Get the widths to appply to these p_subFields'''
+        return self.widths or [''] * (len(subFields) + 1)
 
     def getStorableValue(self, obj, value, complete=False):
         '''Gets p_value in a form that can be stored in the database'''
@@ -160,7 +158,7 @@ class Dict(List):
     def subValidate(self, obj, value, errors):
         '''Validates inner fields'''
         for key, row in value.iteritems():
-            for name, subField in self.fields:
+            for name, subField in self.getSubFields(obj):
                 message = subField.validate(obj, getattr(row, name, None))
                 if message:
                     setattr(errors, '%s*%s' % (subField.name, key), message)
