@@ -17,7 +17,7 @@
 # ------------------------------------------------------------------------------
 from DateTime import DateTime
 from cStringIO import StringIO
-import time, os.path, mimetypes, shutil, base64
+import os, os.path, time, mimetypes, shutil, base64
 
 from appy.px import Px
 from appy import Object
@@ -48,7 +48,25 @@ def getShownSize(size):
     size = size / 1024.0 # This is the size, in Kb
     if size < 1024: return '%s Kb' % sutils.formatNumber(size, precision=1)
     size = size / 1024.0 # This is the size, in Mb
-    return '%s Mb' % sutils.formatNumber(size, precision=1)
+    if size < 1024: return '%s Mb' % sutils.formatNumber(size, precision=1)
+    size = size / 1024.0 # This is the size, in Gb
+    return '%s Gb' % sutils.formatNumber(size, precision=1)
+
+def getFolderSize(folder, nice=False, withCounts=False):
+    '''Returns the size of this p_folder (all content, recursively)'''
+    r = 0
+    counts = 0 # Count the number of files
+    for root, dirs, files in os.walk(folder):
+        for name in files:
+            r += os.stat(os.path.join(root, name)).st_size
+            counts += 1
+    # Nicely format the size if requested
+    if nice:
+        r = getShownSize(r)
+    # Return the counts of files as well if requested
+    if withCounts:
+        r = (r, counts)
+    return r
 
 # ------------------------------------------------------------------------------
 class FileInfo:
@@ -83,8 +101,8 @@ class FileInfo:
              case, p_fsPath is the absolute path to the file, including the
              filename. If you manipulate such a FileInfo instance, please avoid
              using methods that are used by Appy to manipulate
-             database-controlled files (like methods getFilePath, removeFile,
-             writeFile or copyFile).'''
+             database-controlled files (like methods removeFile, writeFile or
+             copyFile).'''
         self.fsPath = fsPath
         self.fsName = None # The name of the file in fsPath
         self.uploadName = uploadName # The name of the uploaded file
@@ -106,12 +124,15 @@ class FileInfo:
         self.mimeType = guessMimeType(path)
         self.modified = DateTime(info.st_mtime)
 
-    def getFilePath(self, obj):
+    def getFilePath(self, obj, checkTemp=True):
         '''Returns the absolute file name of the file on disk that corresponds
            to this FileInfo instance.'''
+        # For a not-in-db file, its full path is already in p_self.fsPath
+        if not self.fsName: return self.fsPath
+        # Build the full path to this db-controlled file
         dbFolder, folder = obj.o.getFsFolder()
         r = osPathJoin(dbFolder, folder, self.fsName)
-        if not os.path.exists(r):
+        if not os.path.exists(r) and checkTemp:
             # It may already have been deleted by a failed transaction. Try to
             # get a copy we may have made in the OS temp folder.
             folder = os.path.basename(folder)
