@@ -210,13 +210,14 @@ class Pod(Field):
       mapping=None, generateLabel=None, label=None, template=None,
       templateName=None, useTemplateName=None, showTemplate=None,
       freezeTemplate=None, showFrozenOnly=True, maxPerRow=5, context=None,
-      stylesMapping={}, formats=None, getChecked=None, mailing=None,
-      mailingName=None, showMailing=None, mailingInfo=None, view=None,
-      cell=None, edit=None, xml=None, translations=None, downloadName=None,
-      downloadDisposition='attachment', forceOoCall=False,
+      stylesMapping={}, stylesOutlineDeltas=None, formats=None, getChecked=None,
+      mailing=None, mailingName=None, showMailing=None, mailingInfo=None,
+      view=None, cell=None, edit=None, xml=None, translations=None,
+      downloadName=None, downloadDisposition='attachment', forceOoCall=False,
       optimalColumnWidths=False, distributeColumns=None, script=None,
       pdfOptions='ExportNotes=True', tabbedCR=False, fonts=None, confirm=False,
-      raiseOnError=False, action=None, beforeAction=None, multiObjects=False):
+      raiseOnError=False, action=None, beforeAction=None, multiObjects=False,
+      finalizeFunction=None):
         # Param "template" stores the path to the pod template(s). If there is
         # a single template, a string is expected. Else, a list or tuple of
         # strings is expected. Every such path must be relative to your
@@ -306,6 +307,9 @@ class Pod(Field):
         self.context = context
         # A global styles mapping that would apply to the whole template(s)
         self.stylesMapping = stylesMapping
+        # Outline level deltas to apply to ODF styles. Consult the Renderer
+        # constructor for more info.
+        self.stylesOutlineDeltas = stylesOutlineDeltas
         # What are the output formats when generating documents from this pod ?
         self.formats = formats or self.getAllFormats(self.template[0])
         # Parameter "getChecked" can specify the name of a Ref field belonging
@@ -438,6 +442,9 @@ class Pod(Field):
         # is not sufficient to define show='query': you must explicitly set
         # multiObjects=True, too.
         self.multiObjects = multiObjects
+        # One or several "finalize" function(s) can be passed. Consult the
+        # Renderer constructor for more info.
+        self.finalizeFunction = finalizeFunction
         # Call the base constructor
         Field.__init__(self, None, (0,1), None, None, show, page, group,
           layouts, move, False, True, None, False, specificReadPermission,
@@ -547,6 +554,17 @@ class Pod(Field):
             fileName = '%s-%s' % (norm(title)[:40], fileName)
         return format and ('%s.%s' % (fileName, format)) or fileName
 
+    def getVisibleFormats(self, obj, template):
+        '''Returns the list of formats for which this p_template can be
+           generated.'''
+        r = self.showTemplate(obj, template)
+        if not r: return ()
+        if isinstance(r, bool):
+            r = self.getAllFormats(template)
+        elif isinstance(r, basestring):
+            r = (r,)
+        return r
+
     def getVisibleTemplates(self, obj):
         '''Returns, among self.template, the template(s) that can be shown'''
         res = []
@@ -557,12 +575,8 @@ class Pod(Field):
                             freezeFormats=self.getFreezeFormats(obj, template)))
         else:
             for template in self.template:
-                formats = self.showTemplate(obj, template)
+                formats = self.getVisibleFormats(obj, template)
                 if not formats: continue
-                elif isinstance(formats, bool):
-                    formats = self.getAllFormats(template)
-                elif isinstance(formats, basestring):
-                    formats = (formats,)
                 res.append(Object(template=template, formats=formats,
                            freezeFormats=self.getFreezeFormats(obj, template)))
         # Compute the already frozen documents, and update the available formats
@@ -760,11 +774,14 @@ class Pod(Field):
         # Compute the renderer's parameters
         rendererParams = {'template': templatePath, 'context': podContext,
           'result': result, 'stylesMapping': stylesMapping,
+          'stylesOutlineDeltas': self.stylesOutlineDeltas,
           'imageResolver': ztool.getApp(), 'overwriteExisting': True,
           'forceOoCall': self.forceOoCall, 'raiseOnError': self.raiseOnError,
           'optimalColumnWidths': self.optimalColumnWidths,
           'distributeColumns': self.distributeColumns, 'script': script,
-          'pdfOptions': pdfOptions, 'tabbedCR': self.tabbedCR, 'fonts': fonts}
+          'pdfOptions': pdfOptions, 'tabbedCR': self.tabbedCR, 'fonts': fonts,
+          'finalizeFunction': self.finalizeFunction
+        }
         # Add parameters regarding the Python interpreter to use and LO port
         cfg = ztool.getProductConfig(True)
         if cfg.unoEnabledPython:
