@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -291,7 +291,7 @@ class Rich(Multilingual, Field):
             ck.append('%s: %s' % (k, sv))
         return ', '.join(ck)
 
-    def getJs(self, layoutType, r, config):
+    def getJs(self, obj, layoutType, r, config):
         if layoutType not in ('edit', 'view'): return
         # Compute the URL to ckeditor CDN
         ckUrl = Rich.cdnUrl % (config.ckVersion, config.ckDistribution)
