@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -70,8 +70,8 @@ class Mode:
     pxActions = Px('''
      <table>
       <tr><td for="action in actions"
-            var2="field=action; fieldName=field.name;
-                  multi=True">:action.pxRender</td></tr>
+            var2="multi=action.getMulti('queryResult', req['search']);
+                  field=action; fieldName=field.name">:action.pxRender</td></tr>
      </table>''')
 
     @staticmethod
@@ -381,13 +381,20 @@ class Grid(List):
 
 class Calendar(Mode):
     '''Displays search results in a monthly calendar view'''
+
     px = Px('''<x var="layoutType='view';
                        field=tool.getField('calendar')">:field.pxView</x>''')
 
+    # Formats for keys representing dates
+    dayKey = '%Y%m%d'
+
+    # Format for representing hours in the UI
+    hourFormat = '%H:%M'
+
     def __init__(self, *args):
         Mode.__init__(self, *args)
         # For this calendar view to work properly, objects to show inside it
-        # must have the following attributes:
+        # must have the following fields:
         # ----------------------------------------------------------------------
         # date    | an indexed and required Date field with format =
         #         | Date.WITH_HOUR storing the object's start date and hour;
@@ -395,8 +402,13 @@ class Calendar(Mode):
         # endDate | a not necessarily required Date field with format =
         #         | Date.WITH_HOUR storing the object's end date and hour.
         # ----------------------------------------------------------------------
-        # Optionally, if objects define the following attributes, special icons
-        # indicating repeated events will be shown.
+        # Optionnally, if you want to use index "months" (we advice you to do
+        # so), an indexed Computed field with this name, whose attribute p_index
+        # must be "TextIndex", must also be defined (see details in m_init
+        # below).
+        # ----------------------------------------------------------------------
+        # Optionally, too, if objects define the following attributes, special
+        # icons indicating repeated events will be shown.
         # ----------------------------------------------------------------------
         # successor   | Ref field with multiplicity = (0,1), allowing to
         #             | navigate to the next object in the list (for a repeated
@@ -408,6 +420,17 @@ class Calendar(Mode):
         #             | hereabove.
         # ----------------------------------------------------------------------
 
+    def getMonthIndex(self):
+        '''Deduce, from p_self.klass, what is the month index to use'''
+        # Is there, on p_self.klass, a field name "months" ?
+        months = getattr(self.klass, 'months', None)
+        if not months: return 'date'
+        if months.type == 'Computed' and months.indexed == 'TextIndex':
+            r = 'months'
+        else:
+            r = 'date'
+        return r
+
     def init(self, req, dir):
         '''Creates a stub calendar field'''
         Mode.init(self, req, dir)
@@ -432,12 +455,65 @@ class Calendar(Mode):
         #           | day.
         # ----------------------------------------------------------------------
         self.objects = {} # ~{s_YYYYmmdd: [(s_entryType, Object)]}~
-        # Formats for keys representing dates
-        self.dayKey = '%Y%m%d'
-        # Format for representing hours in the UI
-        self.hourFormat = '%H:%M'
         # If filters are defined from a list mode, get it
         self.filters = self.tool.getFilters(self.klass)
+        # The name of the index to use to restrict the search to the currently
+        # shown month. If p_self.klass defines a Computed indexed TextIndex
+        # field named "months", this index will be used. Else, index "date" will
+        # be used. These 2 indexes are described hereafter.
+        # ----------------------------------------------------------------------
+        # "date"   | This index is expected to define the (start) date of the
+        #          | objects to search. So, the search will match any object
+        #          | whose (start) date is included in the range of dates
+        #          | defined by the currently shown month. Recall that this
+        #          | range may be larger than the month itself: because complete
+        #          | weeks are shown, some days from the previous and next
+        #          | months may be present, too.
+        #          |
+        #          | While this index is the easiest to implement, it has the
+        #          | following problem: objects spanning several days (by using
+        #          | fields "date" and "endate") and whose start date is not
+        #          | among the range of visible dates, will not be part of the
+        #          | result. This is why an alternate index named "month' is
+        #          | also proposed (see below).
+        # ----------------------------------------------------------------------
+        # "months" | When using index "months", every object is expected to
+        #          | hold, in a Computed indexed TextIndex field named "months",
+        #          | the space-separated list of months the object crosses
+        #          | (every crossing month must be represented as a string of
+        #          | the form YYYYmm). "Crossing a month" means: there must be a
+        #          | not-empty intersection between the range of visible days
+        #          | for this month, and the object's range of days. For objects
+        #          | defining no end date, this range is reduced to a unique day
+        #          | (defined by field "date").
+        #          |
+        #          | Because this list of crossing months is relatively complex
+        #          | to produce, Appy provides a function that computes it:
+        #          |
+        #          |           appy.shared.dates.getCalendarMonths
+        #          |
+        #          | Here is a complete example of a Appy class using index
+        #          | "months".
+        #          |
+        #          | from appy.shared.dates import getCalendarMonths
+        #          |
+        #          | class Event:
+        #          |     ...
+        #          |     date    = Date(format=Date.Date.WITH_HOUR,
+        #          |                    multiplicity=(1,1), indexed=True, ...)
+        #          |     endDate = Date(format=Date.Date.WITH_HOUR,
+        #          |                    indexed=True, ...)
+        #          |     ...
+        #          |     def computeMonths(self):
+        #          |         '''Compute the months crossed by p_self'''
+        #          |         # p_self.endDate may be None
+        #          |         return ' '.join(getCalendarMonths(self.date,
+        #          |                                           self.endDate))
+        #          |
+        #          |     months  = Computed(method=computeMonths,
+        #          |                        indexed='TextIndex', show=False)
+        # ----------------------------------------------------------------------
+        self.monthIndex = self.getMonthIndex()
 
     def updateAjaxParameters(self, params):
         '''Grid-specific ajax parameters'''
@@ -470,61 +546,86 @@ class Calendar(Mode):
         else:
             r[dateKey].append(entry)
 
-    def addEntries(self, obj):
+    def addEntries(self, obj, gridFirst, gridLast):
         '''Add, in self.objects, entries corresponding to p_obj. If p_obj spans
            a single day, a single entry of the form ("start", p_obj) is added at
            the key corresponding to this day. Else, a series of entries are
            added, each of the form (s_entryType, p_obj), with the same object,
-           for every day in p_obj's timespan.
-
-           For example, for an p_obj starting at "1975/12/11 12:00" and ending
-           at "1975/12/13 14:00" will produce the following entries:
-              key "19751211"  >  value ("start+", obj)
-              key "19751212"  >  value ("pass", obj)
-              key "19751213"  >  value ("end", obj)
-        '''
+           for every day in p_obj's timespan.'''
+        # For example, for an p_obj starting at "1975/12/11 12:00" and ending at
+        # "1975/12/13 14:00", m_addEntries will produce the following entries:
+        #      key "19751211"  >  value ("start+", obj)
+        #      key "19751212"  >  value ("pass", obj)
+        #      key "19751213"  >  value ("end", obj)
+        # ~~~
         # Get p_obj's start and end dates
+        fmt = Calendar.dayKey
         start = obj.date
-        startKey = start.strftime(self.dayKey)
+        startKey = start.strftime(fmt)
         end = obj.endDate
-        endKey = end and end.strftime(self.dayKey) or None
+        endKey = end and end.strftime(fmt) or None
         # Shorthand for self.objects
         r = self.objects
-        if not endKey or (endKey == startKey):
+        if not endKey or endKey == startKey:
             # A single entry must be added for p_obj, at the start date
             self.addEntry(startKey, ("start", obj))
         else:
+            # Insert one event per day, provided the events are in the grid
+            # ~~~
             # Add one entry at the start day
-            self.addEntry(startKey, ("start+", obj))
-            # Add "pass" entries for every day between the start and end days
-            next = start + 1
-            nextKey = next.strftime(self.dayKey)
-            while nextKey != endKey:
+            if start >= gridFirst:
+                self.addEntry(startKey, ("start+", obj))
+                next = start + 1
+            else:
+                # Ignore any day between v_start and p_gridFirst
+                next = gridFirst
+            # Add "pass" entries for every day between the event's start and end
+            # dates.
+            nextKey = next.strftime(fmt)
+            while nextKey != endKey and next <= gridLast:
                 # Add a "pass" event
                 self.addEntry(nextKey, ('pass', obj))
                 # Go the the next day
                 next += 1
-                nextKey = next.strftime(self.dayKey)
+                nextKey = next.strftime(fmt)
             # Add an "end" entry at the end day
-            self.addEntry(endKey, ('end', obj))
+            if end <= gridLast:
+                self.addEntry(endKey, ('end', obj))
 
     def search(self, first, grid):
-        '''Performs the query, limited to the date range defined by p_grid'''
-        # Performs the query, restricted to the visible date range
-        last = DateTime(grid[-1][-1].strftime('%Y/%m/%d 23:59:59'))
-        dateSearch = Search(date=(first, last), sortBy='date', sortOrder='asc')
-        res = self.tool.executeQuery(self.className,
-          search=self.uiSearch.search, maxResults='NO_LIMIT',
-          search2=dateSearch, filters=self.filters)
+        '''Performs the search, restricted to the date range defined by
+           p_grid.'''
+        # # The first date in the p_grid, that may be earlier than the p_first
+        # day of the month.
+        gridFirst = grid[0][0]
+        # The last date in the grid, calibrated
+        gridLast = DateTime(grid[-1][-1].strftime('%Y/%m/%d 23:59:59'))
+        # Get the search parameters being specific to the range of dates
+        params = {'sortBy':'date', 'sortOrder':'asc'}
+        if self.monthIndex == 'date':
+            # Define the search based on every event's (start) date.
+            # ~~~
+            # As first date, prefer the first visible date in the p_grid instead
+            # of the p_first day of the month.
+            params['date'] = gridFirst, gridLast
+        else: # p_self.monthIndex is "months"
+            # Define the search based on index "months"
+            params['months'] = first.strftime('%Y%m')
+        # Create a Search instance for the search-related parameters
+        dateSearch = Search(**params)
+        # Perform the search
+        r = self.tool.executeQuery(self.className, search=self.uiSearch.search,
+                                   maxResults='NO_LIMIT', search2=dateSearch,
+                                   filters=self.filters)
         # Produce, in self.objects, the dict of matched objects
-        for zobj in res.objects:
-            self.addEntries(zobj.appy())
+        for zobj in r.objects:
+            self.addEntries(zobj.appy(), gridFirst, gridLast)
 
     def dumpObjectsAt(self, date):
         '''Returns info about the object(s) that must be shown in the cell
            corresponding to p_date.'''
         # There may be no object dump at this date
-        dateStr = date.strftime(self.dayKey)
+        dateStr = date.strftime(Calendar.dayKey)
         if dateStr not in self.objects: return
         # Objects exist
         r = []
