@@ -327,6 +327,9 @@ class String(Field):
         return String._MODULO_97(obj, value, True)
     BELGIAN_ENTERPRISE_NUMBER = MODULO_97_COMPLEMENT
 
+    # Prefixes for which the Belgian NISS must be patched (see m_BELGIAN_NISS)
+    nissModuloPrefixes = ('0', '1', '2')
+
     @staticmethod
     def BELGIAN_NISS(obj, value):
         '''Returns True if the NISS in p_value is valid'''
@@ -335,13 +338,13 @@ class String(Field):
         niss = sutils.keepDigits(value)
         # NISS must be made of 11 numbers
         if len(niss) != 11: return False
-        # When NRN begins with 0 or 1, it must be prefixed with number "2" for
-        # checking the modulo 97 complement.
-        nissForModulo = niss
-        if niss.startswith('0') or niss.startswith('1'):
-            nissForModulo = '2'+niss
+        # When NRN begins with some prefix, it must be prefixed with number "2"
+        # for checking the modulo 97 complement. It is related to the birth
+        # year, coded on 2 chars: must 20 considered as 1920 or 2020 ?
+        if niss[0] in String.nissModuloPrefixes:
+            niss = '2' + niss
         # Check modulo 97 complement
-        return String.MODULO_97_COMPLEMENT(None, nissForModulo)
+        return String.MODULO_97_COMPLEMENT(None, niss)
 
     @staticmethod
     def IBAN(obj, value):
@@ -820,24 +823,58 @@ class String(Field):
                                        raiseOnError=False)
         return extractor.parse('<p>%s</p>' % value)
 
+    def repairUni(self, value):
+        '''Repair this unilingual, string p_value'''
+        # Return a tuple (s_repairedValue, b_repaired). If the value was not
+        # repaired (because there was no need for repair), it (p_value, False)
+        # is returned.
+        if not value: return value, False
+        # Is p_value parsable ?
+        if StringCleaner.isParsable(value): return value, False
+        # Remove illegal chars
+        new = StringCleaner.clean(value)
+        if not StringCleaner.isParsable(new):
+            # Still unparsable... try adding an ending p
+            anew = new.strip() + '</p>'
+            if not StringCleaner.isParsable(anew):
+                raise Exception(UNPARSABLE % new)
+            else:
+                new = anew
+        return new, True
+
     def repair(self, obj):
         '''Checks whether this XHTML p_value is SAX-parsable. If not, try to
            remove illegal chars. The method returns True if the field content
            has been repaired.'''
         value = self.getValue(obj)
         if not value: return
-        if not StringCleaner.isParsable(value):
-            # Remove illegal chars
-            new = StringCleaner.clean(value)
-            if not StringCleaner.isParsable(new):
-                # Still unparsable... try adding an ending p
-                anew = new.strip() + '</p>'
-                if not StringCleaner.isParsable(anew):
-                    raise Exception(UNPARSABLE % new)
-                else:
-                    new = anew
-            setattr(obj, self.name, new)
-            return True
+        if isinstance(value, dict):
+            # Manage a multilingual value
+            new = None
+            for lang, val in value.iteritems():
+                newVal, repaired = self.repairUni(val)
+                if repaired:
+                    if new is None:
+                        new = {lang: newVal}
+                    else:
+                        new[lang] = newVal
+            if new:
+                # At least one value as been repaired. Complete v_new with the
+                # possibly unchanged values.
+                for lang, val in value.iteritems():
+                    if lang not in new:
+                        new[lang] = val
+                setattr(obj, self.name, new)
+                r = True
+            else:
+                r = False
+        else:
+            # Manage a unilingual value
+            new, repaired = self.repairUni(value)
+            if repaired:
+                setattr(obj, self.name, new)
+            r = repaired
+        return r
 
     def getIndexValue(self, obj, forSearch=False):
         '''Pure text must be extracted from rich content; multilingual content
@@ -1041,7 +1078,7 @@ class String(Field):
             value = value.replace('\r', '')
         # Truncate the result if longer than self.maxChars
         if isString and self.maxChars and (len(value) > self.maxChars):
-            value = value[:self.maxChars]
+            value = obj.getTool().truncateValue(value, width=self.maxChars)
         # Get a multivalued value if required
         if value and self.isMultiValued() and \
            (type(value) not in sutils.sequenceTypes):
