@@ -145,7 +145,7 @@ class AuthenticationContext:
            identifies the context, while s_name is a human-readable name that
            will be shown in the UI.'''
 
-    def _setDefault(self, req, tool):
+    def _setDefault(self, req, tool, setOnRequest=True):
         '''Set a default context when relevant'''
         if self.chooseOnLogin: return
         # If the context was not chosen at login time, but there is a default
@@ -155,7 +155,9 @@ class AuthenticationContext:
             # Update the cookie with this context
             gutils.updateCookie(req, default, onResponse=True)
             # Update authentication context attributes set on the request
-            self._setOnRequest(tool, req, default)
+            if setOnRequest:
+                self._setOnRequest(tool, req, default)
+            return default
 
     # This method does not need to be overridden if there is no default context
     def getDefaultContext(self, tool):
