@@ -206,7 +206,7 @@ class ToolMixin(BaseMixin):
         # Remove <br/> tags
         if '</p>' in res: res = res[3:res.index('</p>')]
         # Add the page title if found
-        if obj and ('page' in self.REQUEST):
+        if obj and 'page' in self.REQUEST:
             pageLabel = '%s_page_%s' % (obj.meta_type, self.REQUEST['page'])
             text = self.translate(pageLabel, blankOnError=True)
             if text:
@@ -1184,7 +1184,7 @@ class ToolMixin(BaseMixin):
             ctx = req.get('__ac_ctx')
             if login: place = 'form'
         # d. Identify the user from a SSO reverse proxy
-        if not login or (login == '_s_s_o_'):
+        if not login or login == '_s_s_o_':
             sso = self.getProductConfig(True).sso
             if sso:
                 login = sso.extractUserLogin(tool, req, warn=not alsoSpecial)
@@ -1261,22 +1261,30 @@ class ToolMixin(BaseMixin):
         if not user: return
         login = user.login # May have been transformed by a source
         # Authenticate the user if required. In the case of SSO, authentication
-        # has already been performed by the reverse proxy.
-        if authenticate and place != 'sso':
-            if user.state == 'inactive' or not user.checkPassword(password) or \
-               (cfg.authContext and \
-                cfg.authContext.isMandatory(tool, authenticate) and \
-                not ctx and place != 'basic'):
-                # Disable the user
-                self._disableUser(req)
-                return
-            # Create an authentication cookie for this user
-            gutils.writeCookie(login, password, ctx, req)
+        # has already been performed by the reverse proxy, but, if
+        # authentication contexts are in use, a default one may need to be
+        # retrieved.
+        authCtx = cfg.authContext
+        if authenticate:
+            if place == 'sso':
+                # Get a potential default context
+                if not ctx and authCtx:
+                    ctx = authCtx._setDefault(req, tool, setOnRequest=False)
+            else:
+                # Disable the user when relevant
+                if user.state == 'inactive' or \
+                   not user.checkPassword(password) or \
+                   (authCtx and authCtx.isMandatory(tool, authenticate) and \
+                    not ctx and place != 'basic'):
+                    self._disableUser(req)
+                    return
+                # Create an authentication cookie for this user
+                gutils.writeCookie(login, password, ctx, req)
         # Cache the user and some precomputed values, for performance
         req.user = user
         req.userLogin = login
-        if ctx and cfg.authContext:
-            cfg.authContext._setOnRequest(tool, req, ctx)
+        if ctx and authCtx:
+            authCtx._setOnRequest(tool, req, ctx)
         else:
             # Initialise attribute "authContext" with the empty context
             req.authContext = None
