@@ -175,7 +175,7 @@ class User(ModelClass):
     _appy_attributes = ['password1', 'password2', 'title', 'name', 'firstName',
       'source', 'login', 'password3', 'password4', 'encrypted', 'email',
       'roles', 'resetPassword', 'groups', 'toTool', 'syncDate', 'lastLoginDate',
-      'changePasswordAtNextLogin']
+      'changePasswordAtNextLogin', 'editablePages']
     # All methods defined below are fake. Real versions are in the wrapper.
     # Passwords are on a specific page.
     def showPassword12(self): pass
@@ -241,6 +241,7 @@ class User(ModelClass):
     # We may force a local user (source=zodb) to change its password at next
     # login (ie, for users created by an admin).
     changePasswordAtNextLogin = gen.Boolean(**hidden)
+    def showEditablePages(self): return # For back ref Page::editablePages
 
 # The Group class --------------------------------------------------------------
 class Group(ModelClass):
@@ -278,7 +279,7 @@ class Translation(ModelClass):
 # The Page class ---------------------------------------------------------------
 class Page(ModelClass):
     _appy_attributes = ['doc', 'title', 'content', 'pages', 'parent',
-                        'expression', 'toTool3']
+                        'expression', 'editors', 'toTool3']
     folder = True
     pa = {'label': 'Page'}
     doc = gen.Pod(template='/gen/templates/Page.odt', formats=('pdf',),
@@ -287,14 +288,28 @@ class Page(ModelClass):
     title = gen.String(show=('edit','xml'), multiplicity=(1,1),
                        indexed=True, **pa)
     content = gen.String(format=gen.String.XHTML, layouts='f')
+
     # If this Python expression returns False, the page can't be viewed
     def showExpression(self): pass
     expression = gen.String(layouts=Field.dLayouts, show=showExpression, **pa)
+
     # Pages can contain other pages
     def showSubPages(self): pass
     pages = gen.Ref(None, multiplicity=(0,None), add=True, link=False,
       composite=True, back=gen.Ref(attribute='parent', show=False, **pa),
       numbered=True, show=showSubPages, **pa)
+
+    # People allowed to edit the page (and sub-pages)
+    def showEditors(self): pass
+    def linkEditor(self, user): pass
+    def unlinkEditor(self, user): pass
+
+    editors = gen.Ref(User, multiplicity=(0,None), add=False, link='popup',
+      afterLink=linkEditor, afterUnlink=unlinkEditor,
+      back=gen.Ref(attribute='editablePages', show=False,
+                   multiplicity=(0,None), label='User'),
+      show=showEditors, showHeaders=True, actionsDisplay='inline',
+      shownInfo=('title', 'login'), **pa)
 Page.pages.klass = Page
 setattr(Page, Page.pages.back.attribute, Page.pages.back)
 
