@@ -290,7 +290,11 @@ def readCookie(request, onResponse=False):
     if not onResponse:
         cookie = request.get('_appy_', None)
     else:
-        cookie = request.RESPONSE.cookies['_appy_']['value']
+        appyCookie = request.RESPONSE.cookies.get('_appy_')
+        if appyCookie:
+            cookie = appyCookie['value']
+        else:
+            cookie = None
     if not cookie: return None, None, None
     cookieValue = base64.decodestring(urllib.unquote(cookie))
     if ':' not in cookieValue: return None, None, None
