@@ -10,6 +10,7 @@ from appy.shared import utils as sutils
 SSO_NO_PASSWORD = 'No password is stored for a SSO user.'
 U_NO_ACL        = 'User "%s" has no corresponding Zope user.'
 U_DELETED       = 'User "%s" deleted.'
+U_NO_ZOPE       = 'ZODB user %s has no Zope user.'''
 
 # ------------------------------------------------------------------------------
 class UserWrapper(AbstractWrapper):
@@ -225,7 +226,11 @@ class UserWrapper(AbstractWrapper):
     def getEncryptedPassword(self):
         '''Gets the encrypted password for this user'''
         if self.source == 'sso': raise Exception(SSO_NO_PASSWORD)
-        return self.getZopeUser()._getPassword()
+        zuser = self.getZopeUser()
+        if zuser is None:
+            self.log(U_NO_ZOPE % self.login, type='error')
+            return
+        return zuser._getPassword()
 
     def showEncryptedPassword(self):
         '''Allow to show encrypted password to admins only, via the xml
