@@ -1151,10 +1151,10 @@ class AbstractWrapper(object):
         self.o.delete(executeMethods=executeMethods, unindex=unindex)
 
     def translate(self, label, mapping=None, language=None, format='html',
-                  blankOnError=False):
+                  field=None, blankOnError=False):
         '''Check documentation of self.o.translate'''
         return self.o.translate(label, mapping, language=language,
-                                format=format, blankOnError=blankOnError)
+                          format=format, field=field, blankOnError=blankOnError)
 
     def do(self, name, comment='', doAction=True, doHistory=True,
            noSecurity=False, data=None):
