@@ -14,11 +14,12 @@
 # You should have received a copy of the GNU General Public License along with
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
-from appy import Object, commercial
+
 from appy.pod import PodError
+from appy.pod.elements import *
+from appy import Object, commercial
 from appy.shared.utils import Traceback
 from appy.shared.errors import CommercialError
-from appy.pod.elements import *
 
 # ------------------------------------------------------------------------------
 EVAL_ERROR = 'Error while evaluating expression "%s". %s'
@@ -105,6 +106,7 @@ class Action:
         '''Evaluates p_expr with p_context. p_expr can contain an error expr,
            in the form "someExpr|errorExpr". If it is the case, if the "normal"
            expr raises an error, the "error" expr is evaluated instead.'''
+        eval = context['_eval_'].run
         if '|' not in expr:
             res = eval(expr, context)
         else:
@@ -180,7 +182,7 @@ class Action:
             fromRes = None
             error = False
             try:
-                fromRes = eval(self.fromExpr, context)
+                fromRes = context['_eval_'].run(self.fromExpr, context)
             except Exception, e:
                 msg = FROM_EVAL_ERROR % (self.fromExpr,self.getExceptionLine(e))
                 self.manageError(result, context, msg, e)
