@@ -508,14 +508,14 @@ class MemoryBuffer(Buffer):
         # Get the meta-condition if found
         if elem and elem.attrs:
             metaCondition = elem.attrs.get('text:condition')
-            if metaCondition and (metaCondition.lower() != 'ooow:true'):
-                metaCondition = metaCondition.split(':', 1)[1]
-                metaWrap = metaCondition[0]
-                if (metaWrap != metaCondition[-1]) or \
-                   (metaWrap not in expr.metaWraps):
-                    raise ParsingError(BAD_META_CONDITION % metaCondition)
-                expr.metaWrap = metaWrap
-                expr.metaCondition = metaCondition.strip('"\'"')
+            if metaCondition:
+                meta = metaCondition.lstrip('ooow:')
+                if meta and meta.lower() != 'true':
+                    metaWrap = meta[0]
+                    if metaWrap != meta[-1] or metaWrap not in expr.metaWraps:
+                        raise ParsingError(BAD_META_CONDITION % meta)
+                    expr.metaWrap = metaWrap
+                    expr.metaCondition = meta.strip('"\'"')
         if tiedHook: tiedHook.tiedExpression = expr
         self.elements[self.getLength()] = expr
         # To be sure that an expr and an elem can't be found at the same index
