@@ -19,10 +19,10 @@ from UserDict import UserDict
 import zipfile, shutil, xml.sax, os, os.path, re, mimetypes, time
 
 import appy.pod
-from appy.pod import PodError
 from appy.pod.lo_pool import LoPool
 from appy.shared.zip import unzip, zip
 from appy.pod.buffers import FileBuffer
+from appy.pod import PodError, Evaluator
 from appy.shared.xml_parser import Escape
 from appy.pod import styles_manager as sm
 from appy.pod.converter import FILE_TYPES
@@ -115,14 +115,14 @@ class Renderer:
 
     def __init__(self, template, context, result, pythonWithUnoPath=None,
       ooServer='localhost', ooPort=2002, stream='auto', stylesMapping={},
-      html=False, forceOoCall=False, finalizeFunction=None,
-      overwriteExisting=False, raiseOnError=False, imageResolver=None,
-      rotateImages=False, stylesTemplate=None, optimalColumnWidths=False,
-      distributeColumns=False, script=None, managePageStyles=None,
-      resolveFields=False, expressionsHolders=defaultExpressionsHolders,
-      metadata=True, pdfOptions='ExportNotes=True', csvOptions=None,
-      deleteTempFolder=True, protection=False, pageStart=1, tabbedCR=False,
-      fonts=None, loPool=None):
+      stylesOutlineDeltas=None, html=False, forceOoCall=False,
+      finalizeFunction=None, overwriteExisting=False, raiseOnError=False,
+      imageResolver=None, rotateImages=False, stylesTemplate=None,
+      optimalColumnWidths=False, distributeColumns=False, script=None,
+      evaluator=None, managePageStyles=None, resolveFields=False,
+      expressionsHolders=defaultExpressionsHolders, metadata=True,
+      pdfOptions='ExportNotes=True', csvOptions=None, deleteTempFolder=True,
+      protection=False, pageStart=1, tabbedCR=False, fonts=None, loPool=None):
         '''Base on a document template (whose path is in p_template), which is
            an ODT or ODS file containing special expressions and statements
            written in Python, this renderer generates an ODT file (whose path is
@@ -172,10 +172,13 @@ class Renderer:
         # "out"      | The input is read from disk and the output is streamed.
         # ----------------------------------------------------------------------
 
-        # If you plan to make "XHTML to OpenDocument" conversions (via the POD
-        # function "xhtml"), you may specify a "styles mapping" in
-        # p_stylesMapping. Moreover, if the input(s) to this function is valid
-        # HTML but not valid XHTML, specify p_html being True.
+        # If you plan to make "XHTML to OpenDocument" conversions, via the POD
+        # function "xhtml", you may specify (a) a "styles mapping" in
+        # p_stylesMapping. You can also (b) alter the outline level of every
+        # style found in your POD template, via dict p_stylesOutlineDeltas,
+        # whose keys are style names and values are deltas to apply to their
+        # outline levels. Finally, (c), if the input(s) to function "xhtml" is
+        # valid HTML but not valid XHTML, specify p_html being True.
 
         # If you specify a function in p_finalizeFunction (or a list/tuple of
         # functions), it will be called by the renderer before re-zipping the
@@ -227,6 +230,18 @@ class Renderer:
         # information, see appy/pod/converter.py, option "-s". Note that when
         # such p_script is specified, p_forceOoCall is forced to True.
 
+        # By default, POD expressions and statement parts are evaluated using
+        # the Python built-in "eval" function. If you consider this to be a
+        # security problem (are POD templates written by external people ? by
+        # advanced users ?), you can use pod in conjunction with the third-party
+        # module named RestrictedPython:
+        #
+        #              https://restrictedpython.readthedocs.io
+        #
+        # In order to do so, pass, in attribute p_evaluator, an instance of
+        # class appy.pod.restricted.Evaluator. For more information and options,
+        # consult appy.pod.restricted.py.
+
         # If this document is a sub-document to be included in a master one, it
         # has sense to set a specific value for parameter p_managePageStyles:
         # ----------------------------------------------------------------------
@@ -347,6 +362,7 @@ class Renderer:
         self.optimalColumnWidths = optimalColumnWidths
         self.distributeColumns = distributeColumns
         self.script = script
+        self.evaluator = evaluator
         self.managePageStyles = managePageStyles
         self.resolveFields = resolveFields
         self.expressionsHolders = expressionsHolders
@@ -375,6 +391,7 @@ class Renderer:
         self.contentXml = info['content.xml']
         self.stylesXml = info['styles.xml']
         # Manage the styles defined into the ODT template
+        self.stylesOutlineDeltas = stylesOutlineDeltas
         self.stylesManager = sm.StylesManager(self)
         # From LibreOffice 3.5, it is not possible anymore to dump errors into
         # the resulting ods as annotations. Indeed, annotations can't reside
@@ -404,7 +421,7 @@ class Renderer:
     # Attributes to clone to a sub-renderer when using m_clone hereafter
     cloneAttributes = ('html', 'raiseOnError', 'imageResolver', 'rotateImages',
       'stylesTemplate', 'optimalColumnWidths', 'distributeColumns',
-      'expressionsHolders', 'protection', 'tabbedCR', 'fonts')
+      'expressionsHolders', 'protection', 'tabbedCR', 'fonts', 'evaluator')
 
     def clone(self, template, context, result, **params):
         '''Creates another Renderer instance, similar to p_self, but for
@@ -452,9 +469,12 @@ class Renderer:
           'NumberedProperties': sm.NumberedProperties,
           'pageBreak': self.insertPageBreak,
           'columnBreak': self.insertColumnBreak,
+          '_eval_': self.evaluator or Evaluator,
           # Variables to use for representing pod-reserved chars
           'PIPE': '|', 'SEMICOLON': ';'})
-        # Developer, forget the following line
+        # Add Evaluator-specific entries
+        r['_eval_'].updateContext(r)
+        # Insert the context as an entry of itself
         if '_ctx_' not in r: r['_ctx_'] = r
         return r
 
@@ -898,7 +918,7 @@ class Renderer:
         pageStyles = None
         mps = self.managePageStyles
         if mps != None:
-            pageStyles = self.stylesManager.pageStyles.init(mps)
+            pageStyles = self.stylesManager.pageStyles.init(mps, self.template)
         # Patch styles.xml and content.xml
         dynamic = self.stylesManager.dynamicStyles
         for name in ('styles', 'content'):
