@@ -55,6 +55,10 @@ HTML_PARA_ODT_TEXT = 'For XHTML element "%s", you must associate a ' \
 HTML_TEXT_ODT_PARA = 'For XHTML element "%s", you must associate an ' \
   'OpenDocument "text" style (that applies to only a chunk of text within a ' \
   'paragraph). "%s" is a paragraph-wide style.'
+IMPLICIT_PS = '%s: no explicit page style is defined in it, but attribute ' \
+  'managePageStyles has been set with an integer value. In that case, you ' \
+  'must explicitly define the default page style on the first page of your ' \
+  'POD template.'
 
 # ------------------------------------------------------------------------------
 class Properties:
@@ -323,6 +327,14 @@ class Style:
         self.fontSizeUnit = None # May be pt, %, ...
         # Were the styles lies within styles and substyles hierarchy
         self.outlineLevel = outlineLevel
+        # A possible delta to apply to the outline level. Do not confuse this
+        # notion of delta, that applies when the outline level needs to be
+        # dumped as attribute to a "text:h" paragraph whose style is p_self,
+        # with the notion of delta being in use for a styles mapping (key "h*"),
+        # allowing to fine-tune the mapping between outline levels implicitly
+        # expressed on XHTML tags ("h1", "h2", etc) and ODF styles's outline
+        # levels.
+        self.outlineDelta = 0
         # Namespace for the ODF "style-name" attribute corresponding to this
         # style
         self.styleNameNs = (family == 'table-cell') and 'table' or 'text'
@@ -354,8 +366,9 @@ class Style:
         if withName:
             res = ' %s:style-name="%s"' % (self.styleNameNs, self.name)
         # Outline level when relevant
-        if self.outlineLevel != None:
-            res += ' text:outline-level="%d"' % self.outlineLevel
+        if self.outlineLevel is not None:
+            level = self.outlineLevel + self.outlineDelta
+            res += ' text:outline-level="%d"' % level
         # Colspan and rowspan when relevant
         if attrs and attrs.has_key('colspan'):
             res += ' table:number-columns-spanned="%s"' % attrs['colspan']
@@ -486,15 +499,18 @@ class PageStyles:
         # the same renderer several times (ie, when re-importing the same
         # sub-pod), we must avoid calling m_init more than once.
         self.initialised = False
+        # The name of the current pod template, for use in error messages
+        self.template = None
 
     def add(self, style):
         '''Adds a newly parsed page p_style'''
         self.styles[style.name] = style
 
-    def init(self, managePageStyles):
+    def init(self, managePageStyles, template):
         '''Post-initialisation'''
         # Do not perform this if initialisation already occurred
         if self.initialised: return
+        self.template = template
         # Take p_managePageStyles into account
         if isinstance(managePageStyles, int):
             self.startNumber = managePageStyles
@@ -543,6 +559,10 @@ class PageStyles:
             self.first = True
             # Replace style uses with their renamed versions
             r = self.CONTENT_USE.sub(self.getNewStyleUse, r)
+            # Raise an exception if no replacement has been made, but a start
+            # number was specified.
+            if self.startNumber is not None and self.first:
+                raise Exception(IMPLICIT_PS % self.template)
         return r.encode('utf-8')
 
 # ------------------------------------------------------------------------------
@@ -1250,6 +1270,7 @@ class StylesManager:
     # Valid value types for some keys within style mappings
     mappingValueTypes = {'h*': int, 'table': TableProperties,
                          'ol': NumberedProperties, 'ul': BulletedProperties}
+
     def __init__(self, renderer):
         self.renderer = renderer
         self.stylesString = renderer.stylesXml
@@ -1270,6 +1291,12 @@ class StylesManager:
         self.stylesGenerator = StylesGenerator(self)
         # Dynamically-generated styles
         self.dynamicStyles = DynamicStyles()
+        # Apply outline deltas, if specified
+        if renderer.stylesOutlineDeltas:
+            styles = self.styles
+            for name, delta in renderer.stylesOutlineDeltas.iteritems():
+                if name in styles:
+                    styles[name].outlineDelta = delta
 
     def checkStylesAdequation(self, htmlStyle, odtStyle):
         '''Checks that p_odtStyle may be used for style p_htmlStyle'''
