@@ -10,7 +10,7 @@ xhtmlInput = '''<table>
       <p>Second line</p>
     </td>
     <td>
-      <p>Text</p>
+      <p class="Signature">Must get style Signature</p>
     </td>
   </tr>
   <tr>
