@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -398,11 +398,14 @@ class HtmlDiff:
             # Return the line without the surrounding tag
             action = 'insert'
             outerTag = htmlTag.match(line)
-            line = outerTag.group(3)
+            if outerTag:
+                line = outerTag.group(3)
+            else:
+                line = '?'
         else:
             action = None
             outerTag = None
-        # Replace found inner inserts with their content.
+        # Replace found inner inserts with their content
         innerDiffs = []
         while True:
             match = innerDiff.search(line)
