@@ -15,8 +15,9 @@
 # You should have received a copy of the GNU General Public License along with
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
-import os, os.path, re, time, sys, subprocess, traceback, unicodedata, shutil, \
-       mimetypes, random, urlparse
+import os, os.path, re, time, sys, tempfile, string
+import subprocess, traceback, unicodedata, shutil, mimetypes, random, urlparse
+
 sequenceTypes = (list, tuple)
 
 # ------------------------------------------------------------------------------
@@ -244,20 +245,11 @@ def getOsTempFolder(sub=False):
     '''Gets the absolute path to the temp folder on this machine. If p_sub is
        True, it creates a sub-folder within this temp folder and returns its
        absolute path instead of the "root" temp folder path.'''
-    tmp = '/tmp'
-    if os.path.isdir(tmp):
-        res = tmp
-    elif os.environ.has_key('TMP'):
-        res = os.environ['TMP']
-    elif os.environ.has_key('TEMP'):
-        res = os.environ['TEMP']
-    else:
-        raise "Sorry, I can't find a temp folder on your machine."
     if sub:
-        subFolder = os.path.join(res, 'Appy%f' % time.time())
-        os.mkdir(subFolder)
-        res = subFolder
-    return res
+        r = tempfile.mkdtemp(prefix='Appy')
+    else:
+        r = tempfile.gettempdir()
+    return r
 
 def getTempFileName(prefix='', extension='', timestamp=True):
     '''Returns the absolute path to a unique file name in the OS temp folder.
@@ -906,4 +898,12 @@ def urlsplit(url):
     # Re-build the URL parts into an URL, but without params
     url = '%s://%s%s' % (protocol, domain, path)
     return url, r
+
+# ------------------------------------------------------------------------------
+def randomName(length=3):
+    '''Returns a randomly name made of p_length chars'''
+    r = []
+    for i in range(length):
+        r.append(random.choice(string.ascii_letters))
+    return ''.join(r)
 # ------------------------------------------------------------------------------
