@@ -836,8 +836,15 @@ class XmlMarshaller:
                 res.write('"')
             if not self.marshallBinaries:
                 # Dump the file location
-                res.write(' location="%s/%s/%s"' % (self.databaseFolder,
-                                          fieldValue.fsPath, fieldValue.fsName))
+                if fieldValue.fsName:
+                    # This is a DB-controlled file
+                    location = '%s/%s/%s' % (self.databaseFolder,
+                                             fieldValue.fsPath,
+                                             Escape.xml(fieldValue.fsName))
+                else:
+                    # A not-in-db file
+                    location = fieldValue.fsPath
+                res.write(' location="%s"' % location)
         res.write('>')
         # Dump the field value
         self.dumpValue(res, fieldValue, fType, isRef=(fieldType=='ref'))
@@ -1293,15 +1300,20 @@ class StringCleaner:
     # Chars (some of the ASCII control characters) provoking SAX parse errors in
     # strings, once being part of a XML data structure that must be parsed with
     # a SAX parser.
-    numbers = (1, 2, 3, 4, 5, 6, 7, 8, 10, 16, 19, '0e')
+    numbers = tuple(range(1,20)) + ('0e',)
     chars = '|'.join(['\\x%s' % str(n).zfill(2) for n in numbers])
     illegal = re.compile(chars)
 
     @classmethod
-    def isParsable(class_, s):
-        '''Returns True if p_s is SAX-parsable'''
+    def isParsable(class_, s, wrap='x'):
+        '''Returns True if p_s is SAX-parsable. If p_s represents a chunk of
+           XHTML code not being wrapped in a single root XML tag, it can be
+           wrapped into p_wrap. Specify p_wrap = None for disabling such
+           wrapping.'''
+        # Wrap p_s if required
+        if wrap: s = '<%s>%s</%s>' % (wrap, s, wrap)
         try:
-            XmlParser().parse('<x>%s</x>' % s)
+            XmlParser().parse(s)
             return True
         except Exception:
             return False
