@@ -20,17 +20,24 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 import sys, argparse
 
+from appy.model.utils import Object as O
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Program:
     '''Abstract base class for all command-line programs'''
 
-    def __init__(self):
-        # Create the argument parser
-        self.parser = argparse.ArgumentParser()
-        # Define arguments
-        self.defineArguments()
-        # Parse arguments
-        self.args = self.parser.parse_args()
+    def __init__(self, **kwargs):
+        # p_kwargs may be passed, if this program is not called via the
+        # command-line, but programmatically.
+        if kwargs:
+            self.args = O(**kwargs)
+        else:
+            # Create the argument parser
+            self.parser = argparse.ArgumentParser()
+            # Define arguments
+            self.defineArguments()
+            # Parse arguments
+            self.args = self.parser.parse_args()
         # Analyse parsed arguments
         self.analyseArguments()
 
