@@ -26,12 +26,17 @@ from appy.server import Server
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 HELP_ACT  = 'Action can be "start" (start the site), "stop" (stop it), ' \
             '"restart" (stop it and directly restart it), "fg" (start it in ' \
-            'the foreground), "clean" (clean temporary objects and pack the ' \
-            'database) or "run" (execute a specific method).'
-HELP_METH = 'When action in "run", specify the name of the method to execute ' \
-            'on your app\'s tool.'
-HELP_WAIT = 'When action is "stop" or "restart", specify a number of seconds ' \
-            'to wait before sending the SIGINT signal to the Appy server.'
+            'the foreground), "clean" (clean temporary objects, pack the ' \
+            'database, detect missing files on the filesystem, remove ' \
+            'phantom files) or "run" (execute a specific method).'
+HELP_METH = 'When action in "run", you must specify the name of the method ' \
+            'to execute on your app\'s tool. When action is "start" or "fg", ' \
+            'you may also specify such a method: it will be executed just ' \
+            'after standard method "onInstall", if defined on the app\'s tool.'
+HELP_WAIT = '(do not use this, this is for Appy internal use only) When ' \
+            'action is "stop" or "restart", "run" will wait WAIT seconds ' \
+            'before sending the SIGINT signal to the Appy server. In this ' \
+            'mode, no output at all is echoed on stdout.'
 ACT_KO    = 'Unknown action "%s".'
 METH_KO   = 'Specify a method name via option "-m".'
 PORT_USED = 'Port %d is already in use.'
@@ -64,10 +69,10 @@ class Run(Program):
         action = self.args.action
         if action not in Run.allowedActions:
             self.exit(ACT_KO % action)
-        # Check that a method name is given when action is "run"
-        method = wait = None
+        # Get other args
+        method = self.args.method
+        wait = None
         if action == 'run':
-            method = self.args.method
             if method is None:
                 self.exit(METH_KO)
         elif action in ('restart', 'stop'):
@@ -76,6 +81,25 @@ class Run(Program):
         self.method = method
         self.wait = wait
 
+    def say(self, something, cr=True):
+        '''Display p_something on stdout by using the standard print function'''
+        # When the Appy server (SV) wants to restart himself (ie, after a
+        # backup or some other job), he does it by forking, via
+        # subprocess.Popen, a subprocess (SUB) that launches this command:
+        #
+        #            <site_path>/bin/site restart -w <seconds>
+        #
+        # In this mode, it is useless for the "run" script (in SUB) to print
+        # anything on stdout: there is nobody to see this. Worse: using the
+        # "print" standard function in SUB with attribute flush=True may block
+        # the communication between SUB and SV and prevent the Appy server from
+        # restarting.
+        if self.wait: return
+        if cr: # End with a *c*arriage *r*eturn
+            print(something)
+        else:
+            print(something, end='', flush=True)
+
     def checkStart(self, exitIfInUse=False):
         '''Checks that the server can be started in "start" mode'''
         # Check if the port is already in use or not
@@ -83,7 +107,7 @@ class Run(Program):
         if config.server.inUse():
             message = PORT_USED % config.server.port
             if exitIfInUse:
-                print(message)
+                self.say(message)
                 sys.exit(1)
             else:
                 raise self.Error(message)
@@ -91,7 +115,7 @@ class Run(Program):
         # stopped. Remove the file and issue a warning.
         pid = config.database.getPidPath()
         if pid.exists():
-            print(PID_EX)
+            self.say(PID_EX)
             pid.unlink()
 
     def checkStop(self, pidFile, restart=False):
@@ -99,7 +123,7 @@ class Run(Program):
            (pid) of the running server.'''
         # Check that the pid file exists
         if not pidFile.exists():
-            print(PID_NF)
+            self.say(PID_NF)
             if not restart:
                 sys.exit(1)
             else:
@@ -112,21 +136,24 @@ class Run(Program):
         self.checkStart(exitIfInUse=exitIfInUse)
         # Spawn a child process and run the server in it, in "bg" mode
         args = [sys.argv[0], 'bg']
+        if self.method:
+            args.append('-m')
+            args.append(self.method)
         # Start the server in the background, in another process
-        pid = subprocess.Popen(args).pid
+        pid = subprocess.Popen(args, stdin=None, stdout=None, stderr=None).pid
         # Create a file storing the process ID, besides the DB file
-        print(S_START % pid)
+        self.say(S_START % pid)
         sys.exit(0)
 
     def waitStopped(self, pidFile, pid):
         '''Wait until p_pidFile has been deleted by the server process'''
         maxWait = self.config.server.maxWait
-        print(WAIT_M % maxWait, end='', flush=True)
+        self.say(WAIT_M % maxWait, cr=False)
         time.sleep(0.3)
         while pidFile.is_file() and maxWait > 0:
             time.sleep(2)
             maxWait -= 2
-            print('.', end='', flush=True)
+            self.say('.', cr=False)
         # Kill brutally if the process is still there
         if pidFile.is_file():
             try:
@@ -161,13 +188,13 @@ class Run(Program):
             try:
                 os.kill(pid, signal.SIGINT)
             except ProcessLookupError:
-                print(S_NRUN)
+                self.say(S_NRUN)
                 wasRunning = False
             if wasRunning:
                 # Wait until we are sure the server is stopped
                 self.waitStopped(pidFile, pid)
         if not restart and wasRunning:
-            print(S_STOP)
+            self.say(S_STOP)
 
     def run(self, site, app, ext=None):
         '''Runs the web server'''
@@ -212,11 +239,11 @@ class Run(Program):
 
         elif action == 'restart':
             self.stop(restart=True)
-            time.sleep(0.5)
+            time.sleep(1)
             try:
                 self.start(exitIfInUse=False)
             except self.Error as e:
                 # The server is probably not stopped yet
-                time.sleep(1.0)
+                time.sleep(1)
                 self.start()
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
