@@ -27,11 +27,11 @@ from persistent.mapping import PersistentMapping
 import ZODB, ZODB.POSException, transaction, transaction.interfaces
 
 from appy.px import Px
-from appy.utils import multicall
 from appy.database.lock import Lock
 from appy.database.lazy import Lazy
 from appy.utils import path as putils
 from appy.database.catalog import Catalog
+from appy.utils import multicall, Function
 
 # Constants  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 DB_CREATED   = 'Database created @%s.'
@@ -44,7 +44,6 @@ DB_PACKED    = 'Done. Went from %s to %s.'
 TMP_STORE_NF = 'Temp store does not exist in %s.'
 O_STORE_NF   = 'Object store does not exist in %s.'
 TOOL_NF      = 'Tool does not exist in %s.'
-METH_NF      = 'Method "%s" does not exist on the tool.'
 TMP_STORE_E  = 'No temp object has been deleted in %s.'
 TMP_OBJS_DEL = '%s temp object(s) removed.'
 O_EXISTS     = 'An object with id "%s" already exists.'
@@ -71,22 +70,27 @@ class Config:
         self.filePath = None
         # The path to the folder containing database-controlled binary files
         self.binariesFolder = None
+        # The path to the folder containing phantom files
+        self.phantomFolder = None
         # Typically, we have this configuration: within a Appy <site>, we have a
-        # folder named "var" containing all database-related stuff: the main
-        # database file named appy.fs and the sub-folders containing
-        # database-controlled binaries. In this case:
+        # folder named "var" containing the main database file named appy.fs,
+        # the sub-folders containing database-controlled binaries, among which
+        # one particular folder named "phantom", containing phantom files =
+        # waste files that weren't properly cleaned during aborted transactions:
+        #
         #   "filePath"       is <site>/var/appy.fs
         #   "binariesFolder" is <site>/var
-
+        #   "phantomFolder"  is <site>/var/phantom
+        #
         # What roles can unlock pages ? By default, only a Manager can do it.
         # You can place here roles expressed as strings or Role instances,
         # global or local.
         self.unlockers = ['Manager']
         # Number of times a transaction is retried after a conflict error has
         # occurred.
-        self.conflictRetries = 3
+        self.conflictRetries = 5
 
-    def set(self, folder, filePath=None):
+    def set(self, folder, filePath=None, phantomFolder=None):
         '''Sets site-specific configuration elements. If filePath is None,
            p_folder will both hold binaries and the database file that will be
            called appy.fs.'''
@@ -95,6 +99,12 @@ class Config:
             self.filePath = self.binariesFolder / 'appy.fs'
         else:
             self.filePath = pathlib.Path(filePath)
+        # Initialise the phantom folder
+        if phantomFolder:
+            phantom = pathlib.Path(phantomFolder)
+        else:
+            phantom = self.binariesFolder / 'phantom'
+        self.phantomFolder = phantom
 
     def getDatabase(self, server, handler, poFiles, method=None):
         '''Create and/or connect to the site's database (as instance of class
@@ -128,9 +138,11 @@ class Config:
             # We are starting the server: initialise the database. This is the
             # "initialization" connection, allowing to perform database
             # initialization or update at server startup.
-            database.init(created, handler, poFiles)
+            database.init(created, handler, poFiles, method=method)
         elif server.mode == 'clean':
-            # Clean the database temp folder and pack it
+            # Clean the database temp folder, pack the database and launch a
+            # database analysis, leading to the potential removal of phantom
+            # files.
             database.clean(handler, logger)
         elif server.mode == 'run':
             # Execute method named p_method on the tool
@@ -236,7 +248,7 @@ class Database:
             transaction.abort()
             self.db.close()
 
-    def init(self, created, handler, poFiles):
+    def init(self, created, handler, poFiles, method=None):
         '''Create the basic data structures and objects in the database'''
         # Get a connection to the database. This is the "initialization"
         # connection, allowing to perform database initialization or update at
@@ -249,9 +261,9 @@ class Database:
         # If the database is being p_created, add the following attributes to
         # the root object. Class "PersistentMapping" is named "PM" for
         # conciseness.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # name       |  type  | description
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # iobjects   |  PM+   | The main persistent dict, storing all persistent
         #            |        | objects. Any object stored here has an attribute
         #            |        | named "iid" storing an identifier being an
@@ -262,7 +274,7 @@ class Database:
         #            |        | (such a key is named an "ikey"), and values are
         #            |        | IOBTrees whose keys are object identifiers and
         #            |        | values are the objects themselves.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # objects    |  PM    | A secondary persistent dict storing objects
         #            |        | that, in addition to their integer ID, have
         #            |        | also a string identifier, in attribute "id".
@@ -287,7 +299,7 @@ class Database:
         #            |        | - has attribute "iid" storing the integer ID;
         #            |        | - is stored both in "iobjects" (based on its
         #            |        |   "iid") and in "objects" (based on is "id").
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # temp       |  PM    | A persistent mapping storing objects being
         #            |        | created via the ui. The process of creating an
         #            |        | object is the following: a temporary object is
@@ -306,19 +318,19 @@ class Database:
         #            |        | Objects created "from code" do not follow the
         #            |        | same route and are directly created in
         #            |        | "(i)objects".
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # lastId     |  int   | An integer storing the last ID granted to an
         #            |        | object being stored in "iobjects".
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # lastTempId |  int   | An integer storing the last ID granted to a
         #            |        | temporary object stored in "temp".
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # catalogs   |  PM    | A persistent mapping of catalogs. For every
         #            |        | "indexable" class in the model, there will be
         #            |        | one entry whose key is the class name and whose
         #            |        | value is a Catalog instance
         #            |        | (see appy/database/catalog.py).
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         if created:
             root.iobjects = PersistentMapping()
             root.objects = PersistentMapping()
@@ -340,11 +352,11 @@ class Database:
         # memory. Getting a recent object loads the sub-node in memory, with
         # objects that could also need to be retrieved by other requests because
         # of their temporal proximity.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # Ensure all base objects are created in the database: the tool, base
         # users, translation files and catalogs. Create any object that would
         # be missing.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         if not hasattr(root, 'objects'):
             raise Database.Error(DB_CORRUPTED % self.db.storage._file_name)
         try:
@@ -356,7 +368,7 @@ class Database:
             # Create or update catalogs
             Catalog.manageAll(root, handler)
             # Let the tool initialise itself and create sub-objects as required
-            tool.init(handler, poFiles)
+            tool.init(handler, poFiles, method=method)
             # Commit changes (forced, even if handler.commit is False)
             transaction.commit()
         except self.Error as e:
@@ -369,7 +381,7 @@ class Database:
             connection.close()
 
     #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    # Methods for searching objects
+    #                     Methods for searching objects
     #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     def search(self, handler, className, ids=False, **kwargs):
@@ -378,7 +390,7 @@ class Database:
            a list of object IDS instead of a list of true objects.'''
         # p_ids being True can be useful for some usages like determining the
         # number of objects without needing to get information about them.
-        # ~~~
+        # ~
         # Ensure there is a catalog for p_className
         catalog = handler.dbConnection.root.catalogs.get(className)
         if not catalog:
@@ -403,7 +415,7 @@ class Database:
     def compute(self, className): pass
 
     #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    # Global database operations
+    #                       Global database operations
     #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     def pack(self, logger=None):
@@ -438,24 +450,33 @@ class Database:
         logger.info(TMP_OBJS_DEL % count)
 
     def clean(self, handler, logger):
-        '''Cleaning the database means (a) removing any temp object from it and
-           (b) packing it.'''
+        '''Clean the database = (a) remove any temp object from it, (b) launch
+           an analysis via the appy.database.analyser and (c) pack it.'''
         # Create a specific connection
         connection = handler.dbConnection = self.db.open()
-        # Get the temp store
+        # (a) Remove temp objects. Get the temp store.
         temp = getattr(connection.root, 'temp', None)
         dbFile = handler.server.config.database.filePath
         dbPath = str(dbFile)
         if temp is None:
+            # This is a structural problem: abort the whole operation
             return self.abort(connection, TMP_STORE_NF % dbPath, logger)
+        commit = False
         count = len(temp)
         if count == 0:
             logger.info(TMP_STORE_E % dbPath)
         else:
             self.cleanTemp(handler, logger, count=count)
+            commit = True
+        # (b) Launch an analysis. importing the Analyser at the start of this
+        #     file produces a circular import.
+        from appy.database.analyser import Analyser
+        Analyser(handler, logger).run()
+        # At this step, close the connection (after committing if necessary)
+        if commit:
             transaction.commit()
         connection.close()
-        # Pack the database
+        # (c) Pack the database
         self.pack(logger)
 
     def run(self, method, handler, logger):
@@ -475,12 +496,12 @@ class Database:
         tool = handler.tool = root.objects.get('tool')
         if tool is None:
             return abort(connection, TOOL_NF % dbPath, logger)
-        # Ensure p_method is defined on the tool
-        if not hasattr(tool, method):
-            return abort(connection, METH_NF % method, logger)
+        # Now that a connection exists, reinit the current user, that may be a
+        # fake one so far.
+        handler.guard.initUser()
         # Execute the method
         try:
-            getattr(tool, method)()
+            Function.scall(tool, method, raiseOnError=True)
         except Exception as err:
             handler.server.logTraceback()
             return abort(connection)
@@ -543,16 +564,16 @@ class Database:
            in use.'''
 
         # If p_o is...
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # not None | p_id and p_store are ignored, and the method checks if p_o
         #          | is present in the database.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #   None   | p_id and p_store must be non empty and the method checks if
         #          | p_id is already in use in p_store.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # If p_raiseError is True, if the object or ID exists, the method raises
         # an error. Else, it returns a boolean value.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
         if id is None:
             id = o.id
@@ -602,9 +623,11 @@ class Database:
         # Find the class corresponding to p_className
         class_ = handler.server.model.classes.get(className)
         if not class_: raise self.Error(CLASS_NF % className)
+        # The guard may be absent (example: a User instance is being created
+        # during the authentication process).
+        guard = getattr(handler, 'guard', None)
         # Security check
-        guard = handler.guard
-        if secure:
+        if guard and secure:
             guard.mayInstantiate(class_, checkInitiator=True, raiseOnError=True)
         # The root database object
         root = handler.dbConnection.root
@@ -618,7 +641,8 @@ class Database:
         if custom: self.exists(id=id, store=root.objects, raiseError=True)
         # Create the object
         id = id or iid
-        o = class_.new(iid, id, guard.userLogin, initialComment, initialState)
+        userLogin = guard.userLogin if guard else 'system'
+        o = class_.new(iid, id, userLogin, initialComment, initialState)
         # Store the newly created object in the database
         store[iid] = o
         if custom:
@@ -673,7 +697,7 @@ class Database:
         # In the remaining of this method, at various places, we will check if
         # p_o has already been deleted or not. Indeed, p_o may just have been a
         # transient object whose only use was to collect data from the UI.
-        # ---
+        # ~
         # Call the custom "onEditEarly" if available. This method is called
         # *before* potentially linking p_o to its initiator.
         if isTemp:
@@ -702,19 +726,19 @@ class Database:
         '''Delete object p_o from the database. When unlinking it from other
            objects, if the concerned Ref fields are historized and p_historize
            is True, this deletion is noted in tied object's histories.'''
-        # ~~~
+        # ~
         # If p_executeMethods is False, the custom "onDelete" method, even if
         # present on p_o, will not be executed.
-        # ~~~
+        # ~
         # If this object is cascade-deleted from another one, p_ignore contains
         # the back ref pointing to the deleting object, that must not be walked
         # again.
-        # ~~~
+        # ~
         # For performance, when available, the p_root database object is passed
-        # ~~~
+        # ~
         # If the deletion is aborted by custom method "onDelete", a translated
         # error message is returned.
-        # ~~~
+        # ~
         isTemp = o.isTemp()
         # Call a custom "onDelete" if it exists
         if not isTemp and executeMethods and hasattr(o, 'onDelete'):
@@ -729,7 +753,7 @@ class Database:
             # Browse only Ref fields
             if field.type != 'Ref': continue
             # Do not browse the field mentioned in p_ignore
-            if ignore and (field.name == ignore.name): continue
+            if ignore and field.name == ignore.name: continue
             # Browse tied objects
             contained = field.composite
             for tied in field.getValue(o, single=False):
@@ -750,7 +774,7 @@ class Database:
                                      comment='%s: %s' % (className, title))
         if not isTemp:
             # Unindex p_o if it was indexed
-            if o.class_.isIndexable(): o.reindex(unindex=True)
+            if o.class_.isIndexable(): o.reindex(unindex=True)  
             # Delete the filesystem folder corresponding to this object
             folder = self.getFolder(o, create=False)
             if folder.exists():
@@ -777,7 +801,7 @@ class Database:
         # If p_logMissing is True and no object is found corresponding to the
         # passed p_id, an error message will be logged. If the caller knows it,
         # the p_className may be passed, producing a more precise error message.
-        # ~~~
+        # ~
         # Convert p_id to an int if it is an integer coded in a string
         if isinstance(id, str) and (id.isdigit() or id.startswith('-')):
             id = int(id)
@@ -795,11 +819,11 @@ class Database:
 
     def getObjects(self, handler, ids, className=None, start=0, size=None):
         '''Returns a list of objects from their p_ids'''
-        # ~~~
+        # ~
         # p_ids can be a list or a IISet. If p_size is not None, it represents a
         # number of IDs: only this number of IDs will be retrieved from p_ids,
         # starting at index p_start.
-        # ~~~
+        # ~
         # IDs for which no object is found are removed from their catalog (if
         # p_className is not None).
         r = [] # The retrieved objects
@@ -858,7 +882,7 @@ class Database:
         # where "path" is the Path instance as described hereabove, and
         # "relative" is a string containing the part of the path that does not
         # contain its start (= the root folder containing binaries).
-        # ~~~
+        # ~
         # Start with getting the root folder storing site binaries
         r = o.config.database.binariesFolder
         # Add the object-specific path, depending on its ID
