@@ -27,6 +27,7 @@ from appy.database.indexes.ref import RefIndex
 from appy.database.indexes.date import DateIndex
 from appy.database.indexes.text import TextIndex
 from appy.database.indexes.rich import RichIndex
+from appy.database.indexes.sort import SortIndex
 from appy.database.indexes.float import FloatIndex
 from appy.database.indexes.boolean import BooleanIndex
 
@@ -100,11 +101,11 @@ class Catalog(PersistentMapping):
         self.all = IITreeSet()
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    # Class methods
+    #                            Class methods
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     @classmethod
-    def populate(class_, root, handler, populate):
+    def populate(class_, root, handler, populate, forceLog=False):
         '''Called by the framework, this method populates new indexes or indexes
            whose type has changed. The list of indexes to populate is given in
            p_populate, as produced by m_manageAll.'''
@@ -132,7 +133,7 @@ class Catalog(PersistentMapping):
             root.catalogs[className].unindexObject(iid)
         # At the time the database is created, there is a single object in it:
         # the tool.
-        if counts.total > 1:
+        if counts.total > 1 or forceLog:
             # Log details about the operation
             names = [] # The names of the populated indexes
             for className, indexes in populate.items():
@@ -148,7 +149,7 @@ class Catalog(PersistentMapping):
     @classmethod
     def manageAll(class_, root, handler):
         '''Called by the framework, this method creates or updates, at system
-           startup, catalogs required by the app's model .'''
+           startup, catalogs required by the app's model.'''
         # Ensure a catalog exists for every indexable class
         model = handler.server.model
         catalogs = root.catalogs
@@ -432,7 +433,7 @@ class Catalog(PersistentMapping):
         o.goto(o.referer, message=o.translate('action_done'))
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    # PX
+    #                                  PX
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     # PX displaying info stored in this catalog's indexes for some object
