@@ -34,6 +34,7 @@ from appy.database.operators import Operator
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 VAL_KO     = 'Index "%s" in catalog "%s": wrong %s "%s".'
 INDEX_ERR  = 'Error while indexing %s::%s.'
+INDEX_REC  = 'Recomputing index %s:%s...'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Index(persistent.Persistent):
@@ -281,6 +282,17 @@ class Index(persistent.Persistent):
             raise self.Error(VAL_KO % (self.name, self.catalog.name,
                                        term, str(value)))
 
+    def recompute(self, o):
+        '''Completely recompute p_self's content'''
+        catalog = self.catalog
+        o.log(INDEX_REC % (catalog.name, self.name))
+        # Empty the index
+        self.init()
+        # Re-populate it
+        handler = o.H()
+        root = handler.dbConnection.root
+        catalog.populate(root, handler, {catalog.name:[self]}, forceLog=True)
+
     #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     # Conversion methods
     #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
