@@ -108,13 +108,17 @@ class Lock:
             del(o.locks[page])
 
     @classmethod
-    def getLockers(class_, o):
-        '''Return the list of logins having locked at least one page on p_o'''
+    def getLockers(class_, o, names=False, sep=None):
+        '''Returns the logins (or names if p_names is True) of the users having
+           locked at least on page on p_o. The p_result is a list, excepted
+           if p_sep is given: in that case, it is a string, joined via p_sep.'''
         r = set()
         if hasattr(o, 'locks'):
             for login, date in o.locks.values():
                 r.add(login)
-        return r
+        if not names: return r
+        r = [o.search1('User', login=login).getTitle() for login in r]
+        return sep.join(r) if sep else r
 
     @classmethod
     def unlockableBy(class_, o, user):
