@@ -1,7 +1,7 @@
 '''Execute commands to a Subversion repository'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -18,65 +18,61 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-import os
-from pathlib import Path
+from appy.deploy.repository import Repository
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-class Subversion:
+class Subversion(Repository):
     '''Represents a distant Subversion server'''
 
     # svn options for non interactive mode
     nonInteractive = '--non-interactive --no-auth-cache'
 
-    def __init__(self, specifier):
-        # A p_specifier is an object with attributes such as "url", "name",
-        # "login", "password"...
-        self.specifier = specifier
+    def __init__(self, url, name=None, login=None, password=None):
+        '''Defines a distant Subversion repo'''
+        super().__init__(url, name)
+        # Optional credentials
+        self.login = login
+        self.password = password
 
-    def build(self, command, path):
-        '''Builds the Subversion p_command based on info from p_self.specifier
-           and the concerned local p_path.'''
-        # Returns a tuple (v_command, v_path), v_command being the complete
-        # Subversion command (p_command and added args), and v_path being the
-        # p_path completed with the local folder name in <site>/lib.
+    def build(self, command, folder):
+        '''Builds the Subversion p_command based on info from p_self and the
+           concerned local p_folder.'''
+        # Returns a tuple (v_command, v_folder), v_command being the complete
+        # Subversion command (p_command and added args), and v_folder being the
+        # p_folder completed with the local folder name in <site>/lib.
         r = ['svn', command]
-        spec = self.specifier
         # Compute the local folder name in <site>/lib
-        if spec.name:
-            local = spec.name
-        else:
-            # The name of the app to checkout is the last part of spec.url
-            local = Path(spec.url).name
-        local = path / local        
+        local = self.getLocalFolder(folder)
         if command == 'checkout':
             # Add the checkout URL and local
-            r.append(spec.url)
+            r.append(self.url)
             # Add the name of the local folder to checkout to, if it is
             # different from the last part of the URL.
-            if spec.name: r.append(spec.name)
+            if self.name: r.append(self.name)
         elif command == 'update':
             # Add the local folder that must be updated
             r.append(str(local))
         # Add parameters for running commands in a non interactive mode
         r.append(Subversion.nonInteractive)
         # Add authentication-related parameters if found
-        if spec.login:
-            r.append('--username %s --password %s' % (spec.login,spec.password))
+        if self.login:
+            r.append('--username %s --password %s' % (self.login,self.password))
         return ' '.join(r), local
 
-    def checkout(self, path):
-        '''Checkout a copy of code retrieved via p_self.specifier, in some local
-           p_path.'''
-        # Remember the current working directory: we will chdir into p_path
-        curdir = os.getcwd()
-        os.chdir(path)
-        # Build the svn command
-        command, local = self.build('checkout', path)
-        # Execute the command
-        print('Executing %s...' % command)
-        os.system(command)
-        # chdir to the initial working directory
-        os.chdir(curdir)
-        # Return the path to the checkouted local copy
-        return local
+    def getDownloadCommand(self, folder):
+        '''Produce the complete command allowing to "svn co" this repo (p_self)
+           in this local p_folder.'''
+        return self.build('checkout', folder)
+
+    def getUpdateCommand(self, folder):
+        '''Produce the complete command allowing to "svn up" this repo (p_self)
+           in this local p_folder.'''
+        return self.build('update', folder)
+
+    def asSpecifier(self):
+        '''Returns p_self's specifier version'''
+        r = '%s,type=svn' % super().asSpecifier()
+        if self.login:
+            r = '%s,login=%s,password=%s' % (r, self.login, self.password)
+        return r
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
