@@ -1,7 +1,7 @@
 '''Management of the model behind a Appy application'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -39,10 +39,13 @@ class Config:
         # detect, add them in the following list. Every role can be a Role
         # instance or a string.
         self.additionalRoles = []
-        # What roles can unlock locked pages ? By default, only a Manager can do
-        # it. You can place here roles expressed as strings or Role instances,
-        # global or local.
-        self.unlockers = ['Manager']
+        # When marshalling File fields via the XML layout, binary content is, by
+        # default, included as a series of fixed size Base64-encoded chunks
+        # wrapped in "part" tags. If the target site has access to this site's
+        # filesystem, an alternative is to marshall the disk location of the
+        # binary file instead of its content. If you want to enable this latter
+        # behaviour, set the following attribute to False.
+        self.marshallBinaries = True
 
     def set(self, appFolder):
         '''Sets site-specific configuration elements'''
@@ -66,7 +69,7 @@ class Model:
     # and standard workflows defined in package appy.model.workflows.standard.
     # These classes and workflows are injected into any app's model.
     baseClasses = ('Page', 'User', 'Group', 'Tool', 'Translation', 'Carousel',
-                   'Document', 'Query')
+                   'Document', 'Query', 'Mover')
     baseWorkflows = ('Anonymous', 'Authenticated', 'Owner', 'TooPermissive')
 
     def __init__(self, config, classes, workflows):
