@@ -53,6 +53,7 @@ INDEX_TEMP  = "A temp object can't be (un)indexed."
 CATALOG_KO  = 'Catalog not found for %s instance.'
 NO_ID       = 'Call to getObject with no ID.'
 NO_STR      = 'Specify the name of the class as a string.'
+F_FREEZ_KO  = 'Field "%s" is not (un)freezable.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Base:
@@ -69,7 +70,7 @@ class Base:
     workflow = workflows.Anonymous
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    # Fields
+    #                             Fields
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     # The following fields are base fields that apply to any class, be it a
@@ -88,8 +89,10 @@ class Base:
                 r.append(val)
         return ' '.join(r)
 
-    searchable = Text(show=False, persist=False, indexed=True,
-                      default=getSearchableText, label='Base', page=None)
+    searchableAttributes = {'show': False, 'persist':False, 'indexed':True,
+      'default': getSearchableText, 'label':'Base', 'page': None}
+
+    searchable = Text(**searchableAttributes)
 
     # Attribute "title" is a mandatory field used at various places by default,
     # for displaying base information about an object.
@@ -98,8 +101,8 @@ class Base:
       # The following lambda allows to produce an index value suitable for
       # sorting. In the call to m_getShownValue, language 'en' is forced in
       # order to bypass the user language and always have a deterministic value.
-      'indexValue': lambda o, v: Normalize.text(o.getShownValue(language='en'),
-                                                keepBlank=False) if v else ''}
+      'indexValue': lambda o, v:
+         Normalize.sortable(o.getShownValue(language='en')) if v else ''}
 
     title = String(**titleAttributes)
 
@@ -123,10 +126,14 @@ class Base:
     # "state" is a field deduced from the object history and identifies the
     # object state according to its workflow. It must be a "select" field,
     # because it will be necessary for displaying the translated state name.
-    state = Select(validator=Selection(lambda o: o.getWorkflow().listStates(o)),
-              svalidator=Selection(lambda tool, c: c.workflow.listStates(tool)),
-              show='result', default=lambda o: o.history[0].state,
-              persist=False, indexed=True, height=5, label='Base')
+    stateAttributes = {
+      'validator': Selection(lambda o: o.getWorkflow().listStates(o)),
+      'svalidator': Selection(lambda tool, c: c.workflow.listStates(tool)),
+      'show': 'result', 'default': lambda o: o.history[0].state,
+      'persist': False, 'indexed': True, 'height': 5, 'label':'Base'
+    }
+
+    state = Select(**stateAttributes)
 
     # Field "allowed" defines an index storing the list of roles and users being
     # allowed to view this object. It will be used within catalog searches with
@@ -153,11 +160,12 @@ class Base:
                      indexed=True, default=getAllowed, label='Base')
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    # History
+    #                             History
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     recordAttributes = {'method': History.view, 'label':'Base', 'layouts':'f',
       'xml': lambda o, value: o.history,
+      'show': lambda o: Show.VX if o.user.hasRole('Manager') else None,
       'page':Page('history', label='Base_page_history',
                   sticky=True, icon='history.svg',
                   show=lambda o: 'view' if o.user.hasRole('Manager') else None)}
@@ -165,7 +173,7 @@ class Base:
     record = Computed(**recordAttributes)
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    # Python interpreter
+    #                        UI Python interpreter
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     interpreter = Python(label='Base', layouts='f',
@@ -174,7 +182,7 @@ class Base:
                 icon='python.svg'))
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    # Properties
+    #                            Properties
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     # The following properties are read-only attributes allowing easy access,
@@ -242,7 +250,7 @@ class Base:
     database = property(lambda o: o.H().server.database, raiseReadOnly)
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    # Base methods
+    #                            Base methods
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     def getHandler(self):
@@ -286,7 +294,16 @@ class Base:
         req = self.req
         if 'pageLayout' in req:
             pl = req.pageLayout
-            return pl if isinstance(pl, Layout) else Layout(pl)
+            if not isinstance(pl, Layout):
+                # A string
+                if '*' in pl:
+                    # A CSS class and alignment are specified
+                    pl, css, align = pl.split('*')
+                else:
+                    css = ''
+                    align = 'left'
+                pl = Layout(pl, css=css, align=align)
+            return pl
         # Retrieve layouts being applicable for p_self
         r = self.class_.getPageLayouts(self)[layout]
         return r if not isinstance(r, str) else Layout(r)
@@ -318,6 +335,12 @@ class Base:
         if not info: return
         return self.getObject(info.split(':', 1)[0])
 
+    def getPublishedObject(self):
+        '''Returns the object being currently published in the UI, if any'''
+        traversal = self.traversal
+        if not traversal: return
+        return traversal.o
+
     def __repr__(self):
         '''Returns the class name and Appy object ID'''
         return '<%s id=%s>' % (self.getClass().name, self.id)
@@ -330,9 +353,9 @@ class Base:
         '''Logs a p_message of some p_type'''
         return self.H().log('app', type, message)
 
-    def say(self, message):
+    def say(self, message, fleeting=None):
         '''Adds p_message to the global message returned in the response'''
-        self.resp.addMessage(message)
+        self.resp.addMessage(message, fleeting=fleeting)
 
     def goto(self, url=None, message=None, fromPopup=False):
         '''Return to some p_url or to the referer page. If you want to come back
@@ -345,7 +368,7 @@ class Base:
         return self.resp.code == 303
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    # Security methods
+    #                           Security methods
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     def raiseUnauthorized(self, msg=None):
@@ -361,7 +384,7 @@ class Base:
         return self.guard.allows(self, permission, raiseError=raiseError)
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    # URL methods
+    #                            URL methods
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     def buildUrl(self, name='', base=None, ram=False, bg=False):
@@ -380,8 +403,15 @@ class Base:
         '''See doc in appy.server.Server::getUrl'''
         return self.H().server.getUrl(self, sub=sub, **params)
 
+    def getGotoUrl(self):
+        '''Get p_self's URL, at the page and layout as defined by request
+           variables "gotoPage" and "gotoLayout".'''
+        req = self.req
+        return self.getUrl(sub=req.gotoLayout, page=req.gotoPage,
+                           nav=req.nav or 'no')
+
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    # Field methods
+    #                            Field methods
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     def getField(self, name, className=None, raiseError=False):
@@ -419,7 +449,7 @@ class Base:
             if type and field.type != type: continue
             # Ignore fields that can't be rendered on p_layout
             if not field.isRenderable(layout): continue
-            # Ignore unshowable fields 
+            # Ignore unshowable fields
             if not field.isShowable(self, layout): continue
             r.append(field)
         return r
@@ -428,9 +458,9 @@ class Base:
         '''Returns p_self's fields, sorted by group, being part of this p_page,
            to be shown on this p_layout.'''
         # If p_collect is...
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  False  | r_ is a list, each element being a field or a group.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  True   | r_ is a tuple (page, l, css, js, phases):
         #         |  - "page"   repeats the current p_page;
         #         |  - "l"      is the list produced when p_collect is False;
@@ -440,10 +470,10 @@ class Base:
         #         |             included because used by the returned fields;
         #         |  - "phases" is an ordered dict of the phases and pages that
         #         |             must currently be shown for p_self.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # If p_fields are given, we use them instead of getting all fields
         # defined on p_self's class.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         r = []
         groups = {} # The already encountered groups (UiGroup instances)
         css = js = phases = None # The elements to collect when relevant
@@ -458,7 +488,8 @@ class Base:
             # Ignore fields whose page is not p_page
             if field.pageName != page: continue
             # Ignore fields that must not be shown
-            if not field.isShowable(self, layout): continue
+            if not field.isRenderable(layout) or \
+               not field.isShowable(self, layout): continue
             if collect:
                 # Collect JS/CSS files
                 if css is None: css = []
@@ -507,13 +538,6 @@ class Base:
         return self.translate('%s_%s' % (self.getWorkflow().name, name),
                               language=language)
 
-    def getShownValue(self, name='title', layout='view', language=None,
-                      value=None):
-        '''Call field.getShownValue on field named p_name'''
-        field = self.getField(name)
-        value = field.getValue(self) if value is None else value
-        return field.getShownValue(self, value, layout, language=language)
-
     def getValue(self, name, type=None, layout='view', language=None,
                  value=None, bypassDefault=False):
         '''Gets the value of field named p_name on p_self'''
@@ -525,7 +549,7 @@ class Base:
         #
         # Use m_getValue if you need some variant of the field value. If p_type
         # is:
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #    None     | The value as stored in the database will be returned
         #             | (or p_value if not None). If p_bypassDefault is True, in
         #             | the absence of a value, the system allowing to retrieve
@@ -534,7 +558,7 @@ class Base:
         #             | useful when specifying one of the 2 following types, in
         #             | order to format an alternate value than the one stored
         #             | on p_self.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # 'formatted' | The "formatted" value, as produced by methods
         #             | <Field>.getFormattedValue, will be returned. For
         #             | example, in the case of a Select field, the formatted
@@ -546,16 +570,20 @@ class Base:
         #             | representations on different layouts. Parameter p_layout
         #             | must be understood as "display the field value as it
         #             | would appear on layout X".
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #   'shown'   | A variant of the "formatted" type, taking care of
         #             | multilingual values, as produced by methods
         #             | <Field>.getShownValue. For a multilingual value, the
         #             | "shown" value will be the part corresponding to the UI
         #             | language (or to p_language if specified).
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  'catalog'  | The field value as stored in the database index, as
         #             | produced by methods <Field>.getCatalogValue.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # [Not implemented yet] If p_name refers to an inner field (has the form
+        # <fieldName>*<subName>), p_value must be passed ans must be one of
+        # the sub-values contained within the outer field. 
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         field = self.getField(name)
         # Get the catalog value
         if type == 'catalog': return field.getCatalogValue(self)
@@ -571,14 +599,28 @@ class Base:
         method = getattr(field, 'get%sValue' % type.capitalize())
         return method(self, r, layout, language=language)
 
+    def getShownValue(self, name='title', layout='view', language=None,
+                      value=None):
+        '''Call field.getShownValue on field named p_name'''
+        field = self.getField(name)
+        value = field.getValue(self) if value is None else value
+        return field.getShownValue(self, value, layout, language=language)
+
     def countRefs(self, name):
         '''Counts the nb of objects linked to this one via Ref field p_name'''
         tied = self.values.get(name)
         return 0 if tied is None else len(tied)
 
-    def getIndexOf(self, name, tied, raiseError=True):
+    def path(self, name):
+        '''Returns the absolute file name of file stored in File field p_named
+           p_name.'''
+        v = getattr(self, name)
+        if v: return v.getFilePath(self)
+
+    def getIndexOf(self, name, tied, raiseError=True, notFoundValue=None):
         '''See docstring in the homonym method on appy/model/fields/ref::Ref'''
-        return self.getField(name).getIndexOf(self, tied, raiseError=raiseError)
+        return self.getField(name).getIndexOf(self, tied, raiseError=raiseError,
+                                              notFoundValue=notFoundValue)
 
     def getPageIndexOf(self, name, tied):
         '''See docstring in the homonym method on appy/model/fields/ref::Ref'''
@@ -602,15 +644,16 @@ class Base:
 
     def sort(self, name, sortKey='title', reverse=False):
         '''Sorts referred elements linked to p_self via Ref field named p_name
-           according to a given p_sortKey which can be:
-           - an attribute set on referred objects ("title", by default);
-           - a method that will be called on every tied object, will receive
-             every such object as unique arg and will return a value that will
-             represent its order among all tied objects. This mimics usage of
-             the "key" param of the standard list.sort method;
-           - None. If None, default sorting will occur, using the method stored
-             in field.insert.
-        '''
+           according to a given p_sortKey.'''
+        # p_sortKey can be:
+        # - an attribute set on referred objects ("title", by default);
+        # - a method that will be called on every tied object, will receive
+        #   every such object as unique arg and will return a value that will
+        #   represent its order among all tied objects. This mimics usage of
+        #   the "key" param of the standard list.sort method;
+        # - None. If None, default sorting will occur, using the method stored
+        #   in field.insert.
+        # ~
         # Get the objects tied to p_self via field p_name
         objects = getattr(self, name, None)
         if not objects: return
@@ -632,8 +675,36 @@ class Base:
         # Perform the sort
         objects.sort(key=keyMethod, reverse=reverse)
 
+    def freezeField(self, name, template=None, format='pdf', secure=False,
+                    freezeOdtOnError=True, value=None):
+        '''Freezes, on p_self, the content of a freezable field named p_name'''
+        # In the case of a Pod field, a given p_template may be given (indeed,
+        # several templates can exist in field.template); the "freeze format"
+        # may be passed in p_format ("pdf" by default). If p_value is not None,
+        # it is frozen as-is, instead of recomputing the field value.
+        field = self.getField(name)
+        if not field.freezable:
+            raise Exception(F_FREEZ_KO % name)
+        # Additional parameters are passed for Pod fields
+        if field.type == 'Pod':
+            return field.freeze(self, template, format, secure=secure,
+                                upload=value, freezeOdtOnError=freezeOdtOnError)
+        else:
+            return field.freeze(self, value=value)
+
+    def unfreezeField(self, name, template=None, format='pdf', secure=False):
+        '''Unfreezes a frozen field'''
+        field = self.getField(name)
+        if not field.freezable:
+            raise Exception(F_FREEZ_KO % name)
+        # Additional parameters are passed for Pod fields
+        if field.type == 'Pod':
+            field.unfreeze(self, template, format, secure=secure)
+        else:
+            field.unfreeze(self)
+
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    # Containment
+    #                              Containment
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     # Attribute "cid" stores identifying information about the object being the
@@ -706,7 +777,7 @@ class Base:
             return container if objectOnly else (container, name)
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    # Catalog-related methods
+    #                          Catalog-related methods
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     def search(self, className=None, **kwargs):
@@ -760,7 +831,7 @@ class Base:
         for iid in self.search(className, ids=True, sortBy=sortBy, \
                                secure=secure, **fields):
             # Stop if the maximum number of objects has been reached
-            if (max is not None) and (i >= max): break
+            if max is not None and i >= max: break
             # Get the object from its ID
             o = self.getObject(iid, logMissing=True)
             # Ignore dead IDs
@@ -777,17 +848,58 @@ class Base:
         if self.isTemp(): raise Exception(INDEX_TEMP)
         return self.database.reindexObject(self.H(), self, **kwargs)
 
-    def getCatalog(self, raiseError=False):
-        '''Returns the catalog storing p_self (if indexed)'''
-        r = self.H().dbConnection.root.catalogs.get(self.class_.name)
+    def getCatalog(self, class_=None, raiseError=False):
+        '''Returns the catalog storing p_self (if indexed) or instances of
+           another p_class_, if passed.'''
+        class_ = class_ or self.class_
+        r = self.H().dbConnection.root.catalogs.get(class_.name)
         if not r and raiseError:
-            raise Exception(CATALOG_KO % self.class_.name)
+            raise Exception(CATALOG_KO % class_.name)
         return r
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    # Sub-PXs
+    #                                Sub-PXs
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
+    # The object preamble is based and reuses the same CSS classes as results's
+    # top zone.
+    pxPreamble = Px('''
+     <div class="sTop"
+          var="class_=o.class_;
+               nav=req.nav if req.nav != 'no' else None;
+               snav=ui.Siblings.get(nav,tool,_ctx_,'iconS') if nav else None">
+
+      <!-- Class name -->
+      <div class="pageTitle">
+       <x>:_(o.class_.name)</x>
+
+       <!-- The total number of objects, if navigation is there -->
+       <x if="snav">
+         <span var="css=class_.getCssFor(tool, 'sep')"
+               class=":'navSep %s' % css if css else 'navSep'">//</span>
+         <span class="btot">:snav.total</span>
+       </x>
+
+       <!-- Class-specific colored border (*t*itle *bot*tom) -->
+       <div var="css=class_.getCssFor(tool, 'tbot')"
+            if="css" class=":'tbot %s' % css"></div>
+      </div>
+
+      <!-- The "back" icon -->
+      <div if="snav"
+           class="pageIcons">::snav.getGotoSource(svg, _, imgCss='iconM')</div>
+
+      <!-- Siblings navigation -->
+      <div if="snav" class="snav" style="margin-left:auto"
+           var2="gotoSource=False">:snav.pxNavigate</div>
+
+      <!-- Pod templates -->
+      <div var="fields=o.getFields('pre', 'main')" style="margin-left:auto">
+       <div class="spod" for="field in fields"
+            var2="fieldName=field.name">:field.pxRender</div>
+      </div>
+     </div>''')
+
     # The header displays, for a given object, its title and breadcrumb
     pxHeader = Px('''
      <div var="breadcrumb=ui.Breadcrumb(o, popup)"
@@ -818,7 +930,7 @@ class Base:
      template=Template.px, hook='content')
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    # Object controls
+    #                           Object controls
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     pxControls = Px('''
@@ -866,7 +978,7 @@ class Base:
       <div if="not isEdit and not popup and guard.mayDelete(o) and \
                not o.Lock.isSet(o, user, page)" var2="label=_('object_delete')">
        <div>
-        <a class="clickable" onclick=":'onDeleteObject(%s)' % q(o.url)">
+        <a class="clickable" onclick=":'onDeleteObject(%s)' % q(o.iid)">
          <img src=":svg('delete')" class=":picto"/>
          <div style=":'display:%s' % config.ui.pageDisplay">::label</div>
         </a>
@@ -881,19 +993,19 @@ class Base:
       </x>
      </x>
 
-     <!-- Fields on layout "buttons". Layout is defined as "view" and not
-          "cell", because "cell" would render a too minimalistic widget. -->
+     <!-- Fields on layout "buttons". Layout is set as "view" and not "buttons"
+          because this latter would render a too minimalistic widget. -->
      <x if="not isEdit"
         var2="fields=o.getFields('buttons', page=req.page or 'main');
               layout='view'">
-      <!-- Call view and not pxRender to avoid having a table -->
-      <div for="field in fields"
-           var2="name=field.name; value=field.getValueIf(o, name, layout)">
-       <div>:field.cell</div></div>
+      <!-- Call PX "buttons", and not "pxRender", to minimize wrapping tags -->
+      <x for="field in fields"
+         var2="name=field.name; value=field.getValueIf(o, name, layout)">
+       <div>:field.buttons</div></x>
      </x>''')
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    # Objet creation method
+    #                         Objet creation method
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     def create(self, _name, secure=False, raiseOnWrongAttribute=True,
@@ -963,13 +1075,13 @@ class Base:
         '''Returns a dict ~{s_fieldName, fieldValue}~ representing p_self's
            dataset ready-to-be-copied to another object.'''
         # Check m_createFrom below to know more about method args
-        # ~~~
+        # ~
         r = {}
         # Browse field values from p_self and store "copyable" ones in "r"
         for field in self.class_.fields.values():
             # Skip non persistent fields, back references & p_exclude(d) fields
-            if not field.persist or (field.name in exclude) or \
-               ((field.type == 'Ref') and field.isBack): continue
+            if not field.persist or field.name in exclude or \
+               (field.type == 'Ref' and field.isBack): continue
             # Use the value in p_fields if given
             if field.name in fields:
                 value = fields[field.name]
@@ -1012,6 +1124,25 @@ class Base:
             history.modified = other.modified
         return r
 
+    def __copy__(self, memo):
+        '''Duplicating an Appy object must be done via m_createFrom'''
+        # This disables duplicating an Appy object via copy.copy
+        return self
+
+    def __deepcopy__(self, memo):
+        '''Duplicating an Appy object must be done via m_createFrom'''
+        # This disables duplicating an Appy object via copy.deepcopy
+        return self
+
+    def updateFrom(self, other, exclude=(), fields={}):
+        '''Updates p_self's field values from this p_other object. Back refs and
+           fields listed on p_exclude will not be copied.'''
+        # Browse all field values to set on p_other
+        for name, value in other.getCopyValues(exclude, fields).items():
+            # Has p_self a field named p_name ?
+            if self.getField(name):
+                self.values[name] = value
+
     def delete(self, historize=False, executeMethods=True):
         '''Deletes p_self (see homonym method on the database object'''
         return self.database.delete(self, historize, executeMethods)
@@ -1021,7 +1152,7 @@ class Base:
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     def translate(self, label, mapping=None, language=None, asText=False,
-                  field=None, blankOnError=False):
+                  field=None, blankOnError=False, noNice=False):
         '''Translates a given p_label with p_mapping'''
         # If p_field is given, p_label does not correspond to a full label name,
         # but to a label type linked to p_field: "label", "descr" or "help".
@@ -1064,7 +1195,7 @@ class Base:
             r = ''
         # If still no result, put a nice name derived from the label instead of
         # a translated message.
-        if not r and not blankOnError:
+        if not r and not blankOnError and not noNice:
             return produceNiceMessage(label.rsplit('_', 1)[-1])
         # Manage carriage returns, depending on the desired format
         return r.replace('<br/>', '\n') if asText else r
@@ -1082,14 +1213,14 @@ class Base:
     def xml(self):
         '''Returns p_self's XML version'''
         self.resp.setContentType('xml')
-        return Marshaller(rootTag=self.class_.name).marshall(self, appy=True)
+        return self # p_self will be automatically marshalled by the traversal
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     # PXs
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     # The object, as shown in a list of referred (tied) objects. In this PX:
-    # v_io refers to the source object ;
+    # v_io refers to the source, *i*nitiator object ;
     # v_o  refers to the tied object.
     pxTied = Px('''
      <tr var="io=initiator.o;
@@ -1102,64 +1233,35 @@ class Base:
               cbId='%s_%s' % (hook, currentNumber);
               mayView=guard.mayView(o);
               id=o.iid"
-         id=":ohook" valign=":ifield.rowAlign" class=":rowCss">
-      <td for="column in columns" width=":column.width" align=":column.align"
-          var2="field=column.field; isCB=field=='_checkboxes'"
-          class=":selector.cbClass if isCB and selector else ''">
-
-       <!-- Special column with the tied element's number -->
-       <x if="objectIndex != None and field == '_number'">:ifield.pxNumber</x>
-
-       <!-- Special column containing the tied element's checkbox -->
-       <input if="isCB and mayView" type="checkbox" name=":hook" id=":cbId"
-              var2="checked=cbChecked|False" checked=":checked"
-              value=":id" onclick="toggleCb(this)"/>
-
-       <!-- Any other column -->
-       <x if="not column.special">:field.pxTied</x>
-      </td>
+         id=":ohook">
+      <x for="col in columns" var2="field=col.field">:col.pxCellRef</x>
 
       <!-- Store data in this tr node allowing to ajax-refresh it -->
-      <script>:ifield.getAjaxDataRow(o, ohook, hook, rowCss=rowCss, \
-               currentNumber=currentNumber, cbChecked=cbId, ajaxSingle=True, \
+      <script>:ifield.getAjaxDataRow(o, ohook, hook, inPickList,
+               currentNumber=currentNumber, cbChecked=cbId, ajaxSingle=True,
                initiator=':%d:%s' % (io.iid, ifield.name))</script>
      </tr>''')
 
     # The object, as shown in a list of search results
     pxResult = Px('''
-     <tr valign=":uiSearch.search.rowAlign" class=":rowCss"
-         var="id=o.iid; ohook=mode.getHookFor(o); mayView=guard.mayView(o);
+     <tr var="id=o.iid; ohook=mode.getHookFor(o); mayView=guard.mayView(o);
               backHook=ohook; cbId='%s_%s' % (mode.checkboxesId, currentNumber)"
          id=":ohook">
-      <td for="column in mode.columns"
-          var2="field=column.field; isCB=field=='_checkboxes'"
-          id=":'field_%s' % field.name|field"
-          width=":column.width" align=":column.align"
-          class=":mode.cbClass if isCB else ''">
-
-       <!-- Special column containing the checkbox -->
-       <input if="isCB" type="checkbox" name=":mode.checkboxesId" id=":cbId"
-              var2="checked=mode.cbChecked|True" value=":id"
-              onclick="toggleCb(this)" checked=":checked"/>
-
-       <!-- Any other column -->
-       <x if="not column.special">:field.pxResult</x>
-      </td>
+      <x for="col in mode.columns" var2="field=col.field">:col.pxCell</x>
 
       <!-- Store data in this tr node allowing to ajax-refresh it -->
-      <script>:mode.getAjaxDataRow(o, ohook, rowCss=rowCss, \
-               currentNumber=currentNumber, cbChecked=cbId)</script>
-     </tr>''')
+      <script>:mode.getAjaxDataRow(o, ohook, currentNumber=currentNumber,
+               cbChecked=cbId)</script></tr>''')
 
     # Base PX for viewing object fields from a given page
     view = Px('''
      <x var="x=guard.mayView(o, raiseError=True);
-             table=o.getPageLayout(layout);
+             pageLayout=o.getPageLayout(layout);
              page=o.getCurrentPage(req, layout);
              page,grouped,css,js,phases=o.getGroupedFields(page, layout)">
       <x>::ui.Includer.getSpecific(tool, css, js)</x>
       <x if="not req.notGlobal">::ui.Globals.getForms(tool)</x>
-      <x var="tagId='pageLayout'; tagName=''; tagCss='';
+      <x var="table=pageLayout;tagId='pageLayout'; tagName=''; tagCss='';
               layoutTarget=o">:table.pxRender</x>
       <x if="not req.notGlobal"
          var="x=o.callOnView()">::ui.Globals.getScripts(tool, q, layout)</x>
@@ -1196,11 +1298,12 @@ class Base:
        <input type="hidden" name="gotoLayout" value=""/>
        <input type="hidden" name="insert" value=":req.insert or ''"/>
        <input type="hidden" name="confirmed" value="False"/>
+       <input type="hidden" name="confirmedData" value=""/>
        <x var="tagId='pageLayout'; tagName=''; tagCss='';
                layoutTarget=o">:table.pxRender</x>
       </form>
       <script if="confirmText">::'askConfirm(%s,%s,%s,null,%d)' % \
-        (q('script'), q('postConfirmedEditForm()'), q(confirmText), \
+        (q('script'), q('postConfirmedEditForm(comment)'), q(confirmText), \
          o.class_.getConfirmPopupWidth(popup))</script>
       <x>::ui.Globals.getScripts(tool, q, layout)</x>
      </x>''',
@@ -1208,20 +1311,21 @@ class Base:
      js='''
       completeAppyForm = function(f) {
         // Complete Appy form p_f via special form element named "_get_"
-        var g = f._get_;
+        let g = f._get_;
         if (!g.value) return;
-        var [source, name, info] = g.value.split(':');
+        let [source, name, info] = g.value.split(':');
         if (source == 'form') {
           // Complete p_f with form elements coming from another form
-          var f2 = getNode(':' + name),
+          let f2 = getNode(':' + name),
               fields = info.split(','),
               fieldName = fieldValue = null;
           if (!f2) return;
-          for (var i=0; i<fields.length; i++) {
+          for (let i=0; i < fields.length; i++) {
             fieldName = fields[i];
             if (fieldName[0] == '*') {
               // Try to retrieve search criteria from the storage if present
-              fieldValue = sessionStorage.getItem(fieldName.substr(1));
+              let spars = getSearchInfo(fieldName.substr(1) + '_customSearch');
+              fieldValue = ('criteria' in spars)? spars['criteria']: null;
               if (!fieldValue) continue;
               fieldName = 'criteria';
             }
@@ -1232,8 +1336,19 @@ class Base:
           }
         }
       }
+
+      retrieveContentEditable = function(f) {
+        // Copy, in textareas, content stored in content-editable divs
+        let divs = f.querySelectorAll('[contenteditable=true]');
+        for (const d of divs) {
+          if (d.innerHTML && d.innerHTML !== '<div></div>' && d.nextSibling) {
+            d.nextSibling.value = d.innerHTML;
+          }
+        }
+      }
+
       submitAppyForm = function(action, gotoPage, gotoLayout) {
-        var f = document.getElementById('appyForm');
+        let f = document.getElementById('appyForm');
         // Complete the form via the "_get_" element if present
         if (action != 'cancel') completeAppyForm(f);
         f.action.value = action;
@@ -1242,20 +1357,21 @@ class Base:
              yet to close it. The timer hereafter will regularly check if the
              popup must be closed. */
           createCookie('closePopup', 'no');
-          var popup = getNode('iframePopup', true);
+          let popup = getNode('iframePopup', true);
           // Set a timer for checking when we must close the iframe popup
           popup.popupTimer = setInterval(backFromPopup, 700);
         }
         f.gotoPage.value = gotoPage;
         f.gotoLayout.value = gotoLayout;
-        f.submit(); clickOn(f.elements[action]);
+        retrieveContentEditable(f);
+        f.submit(); clickOn(f);
       }
 
       protectAppyForm = function() {
-        window.onbeforeunload = function(e){
-        var f = document.getElementById("appyForm");
+        window.onbeforeunload = function(event){
+        let f = document.getElementById("appyForm");
         if (f.action.value == "") {
-          var e = e || window.event;
+          let e = event || window.event;
           if (e) {e.returnValue = warn_leave_form;}
           return warn_leave_form;
           }
@@ -1317,7 +1433,7 @@ class Base:
         if popup: return Iframe.goBack(self.tool, initiator)
         # Determine, if not defined yet, the URL to go back to
         if self.gotoSet(): return
-        self.goto(url or self.req.referer)
+        self.goto(url or (self.req.referer if isTemp else self.getGotoUrl()))
 
     traverse['save'] = 'perm:write'
     def save(self):
@@ -1368,10 +1484,8 @@ class Base:
             # Let the initiator choose the back URL
             url = initiator.getBackUrl(self)
         else:
-            # Go the the page as defined by request variables "gotoPage" and
-            # "gotoLayout".
-            url = self.getUrl(sub=req.gotoLayout, page=req.gotoPage,
-                              nav=req.nav or 'no')
+            # Go to p_self's page and layout as defined in the request
+            url = self.getGotoUrl()
         self.goto(url)
 
     traverse['new'] = True
@@ -1406,8 +1520,8 @@ class Base:
         # Create a temp object
         o = self.database.new(handler, className, temp=True)
         # Populate p_o's edit form fields with a template object if present
-        templateId = req.template
-        if templateId: params['template'] = templateId
+        templateId = req.template_
+        if templateId: params['template_'] = templateId
         # Call method "onCreate" if available. This method typically sets local
         # roles to the newly created object.
         if hasattr(o, 'onCreate'): o.onCreate()
@@ -1433,7 +1547,9 @@ class Base:
             if not gotoSet and handler.isPublished(self):
                 back = self.tool.computeHomePage()
         # If we are called from an Ajax request, simply return the message
-        if handler.isAjax(): return text
+        if handler.isAjax():
+            self.resp.ungoto()
+            return text
         # Go to the "back" URL (if "goto" hasn't been called yet)
         if not gotoSet:
             self.goto(url=back, message=text)
