@@ -50,59 +50,78 @@ class Batch:
         self.objects = objects
         self.length = len(objects)
 
+    def getInfo(self, condition, nb, c):
+        '''Returns info about the current navigation icon to produce'''
+        # Return info is a 2-tuple (s_cssClasses, s_onClick)
+        if not condition:
+            css = 'disabled iconS'
+            js = ''
+        else:
+            css = 'clickable iconS'
+            q = c.q
+            scrollTop = q(c.scrollTop) if c.scrollTop else 'null'
+            js = 'askBunch(%s,%s,%s,%s)' % (q(self.hook), q(nb), q(self.size),
+                                            scrollTop)
+        return css, js
+
     # Input field for going to element number x
     pxGotoNumber = Px('''
      <x var2="label=_('goto_number');
               gotoName='%d_%s_goto' % (o.iid, field.name)">
-      <span style="padding-left: 5px">:label</span>
-      <input type="text" size=":len(str(total)) or 1" onclick="this.select()"
-             onkeydown=":'if (event.keyCode==13) document.getElementById' \
-                         '(%s).click()' % q(gotoName)"/><img
-             id=":gotoName" name=":gotoName"
-             class="clickable" src=":url('gotoNumber')" title=":label"
+      <span class="gotoLab">:label</span>
+      <input type="text" id=":gotoName" name=":gotoName" class="gotoNum"
+             size=":max(len(str(total))-2, 1)" onclick="this.select()"
+             onkeydown="if (event.keyCode==13) this.nextSibling.click()"/>
+      <img class="clickable"
              onclick=":'gotoTied(%s,%s,this.previousSibling,%s,%s)' % \
-                 (q(sourceUrl), q(field.name), total, q(popup))"/></x>''')
+                 (q(sourceUrl), q(field.name), total, q(popup))"
+             src=":url('gotoNumber')" title=":label"/></x>''',
+
+      css='''.gotoNum { text-align:center; color:white;
+                        mix-blend-mode:difference }
+             .gotoLab { padding:0 0.5em; font-size:90% }''')
 
     pxNavigate = Px('''
-     <x if="batch.total &gt; batch.size"
-        var2="hook=q(batch.hook); size=q(batch.size)">
+     <x if="batch.total &gt; batch.size">
 
       <!-- Go to the first page -->
-      <img if="(batch.start != 0) and (batch.start != batch.size)"
-           class="clickable iconS" src=":svg('arrows')"
-           title=":_('goto_first')" style="transform: rotate(90deg)"
-           onclick=":'askBunch(%s,%s,%s)'% (hook, q(0), size)"/>
+      <img var="cond=batch.start != 0 and batch.start != batch.size;
+                css,js=batch.getInfo(cond, 0, _ctx_)"
+           class=":css" src=":svg('arrows')" onclick=":js"
+           title=":_('goto_first')" style="transform:rotate(90deg)"/>
 
       <!-- Go to the previous page -->
-      <img var="sNumber=batch.start - batch.size" if="batch.start != 0"
-           class="clickable iconS" src=":svg('arrow')"
-           title=":_('goto_previous')" style="transform: rotate(90deg)"
-           onclick=":'askBunch(%s,%s,%s)' % (hook, q(sNumber), size)"/>
+      <img var="cond=batch.start != 0;
+                sNumber=batch.start - batch.size;
+                css,js=batch.getInfo(cond, sNumber, _ctx_)"
+           class=":css" src=":svg('arrow')" onclick=":js"
+           title=":_('goto_previous')" style="transform:rotate(90deg)"/>
 
       <!-- Explain which elements are currently shown -->
       <span class="navText"> 
-       <x>:batch.start + 1</x> <img src=":url('to')"/> 
+       <x>:batch.start + 1</x> ⇀
        <x>:batch.start + batch.length</x> <span class="navSep">//</span> 
-       <x>:batch.total</x>
+       <span class="btot">:batch.total</span>
       </span>
 
       <!-- Go to the next page -->
-      <img var="sNumber=batch.start + batch.size" if="sNumber &lt; batch.total"
-           class="clickable iconS" src=":svg('arrow')" title=":_('goto_next')"
-           style="transform: rotate(270deg)"
-           onclick=":'askBunch(%s,%s,%s)' % (hook, q(sNumber), size)"/>
+      <img var="sNumber=batch.start + batch.size;
+                cond=sNumber &lt; batch.total;
+                css,js=batch.getInfo(cond, sNumber, _ctx_)"
+           class=":css" src=":svg('arrow')" title=":_('goto_next')"
+           style="transform:rotate(270deg)" onclick=":js"/>
 
       <!-- Go to the last page -->
       <img var="lastPageIsIncomplete=batch.total % batch.size;
                 nbOfCompletePages=int(batch.total / batch.size);
                 nbOfCountedPages=lastPageIsIncomplete and \
                                  nbOfCompletePages or nbOfCompletePages-1;
-                sNumber= nbOfCountedPages * batch.size"
-           if="(batch.start != sNumber) and \
-               (batch.start != (sNumber-batch.size))"
-           class="clickable iconS" src=":svg('arrows')"
-           title=":_('goto_last')" style="transform: rotate(270deg)"
-           onclick=":'askBunch(%s,%s,%s)' % (hook, q(sNumber), size)"/>
+                sNumber= nbOfCountedPages * batch.size;
+                cond=(batch.start != sNumber) and
+                     (batch.start != (sNumber-batch.size));
+                css,js=batch.getInfo(cond, sNumber, _ctx_)"
+           class=":css" src=":svg('arrows')" onclick=":js"
+           title=":_('goto_last')" style="transform: rotate(270deg)"/>
 
       <!-- Go to the element number... -->
       <x var="gotoNumber=gotoNumber|False" if="gotoNumber"
@@ -123,6 +142,10 @@ class Batch:
         value = sutils.getStringFrom(value)
         return "sessionStorage.setItem('%s',JSON.stringify(%s))" % (key, value)
 
+    def isComplete(self):
+        '''Does this batch contain all objects ?'''
+        return self.length == self.total
+
     def __repr__(self):
         '''String representation'''
         data = 'start=%d,length=%d,size=%s,total=%d' % \
