@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -49,13 +49,14 @@ class Document(Base):
     def getMaxWidth(self):
         '''Get the maximum width for uploaded images. If the image is larger
            than this, it will be resized.'''
-        # The size depends on the container
+        # The size may be defined on the container
         try:
             container = self.container or self.initiator.o
         except AttributeError:
             # There is no container yet if the image is uploaded via ckeditor
             container = self.traversal.o
-        return '%dpx' % container.maxWidth
+        width = container.maxWidth if hasattr(container, 'maxWidth') else 700
+        return '%dpx' % width
 
     def getViewWidth(self):
         '''The width of the image as shown on the "view" layout varies depending
@@ -72,7 +73,7 @@ class Document(Base):
     def getCarousel(self):
         '''Returns the carousel into which this document is included'''
         container = self.container
-        if container and (container.class_.name == 'Carousel'):
+        if container and container.class_.name == 'Carousel':
             return container
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
