@@ -24,6 +24,7 @@ from appy.px import Px
 from appy import utils
 from appy.database import catalog
 from appy.model.utils import Fake
+from appy.model.totals import Totals
 from appy.tr import FieldTranslations
 from appy.utils import string as sutils
 from appy.model.utils import Object as O
@@ -84,18 +85,25 @@ class Show:
     # All layouts but "view". To use typically when, on "view", the field value
     # is already shown as a part of some custom widget.
     V_  = ('edit', 'result', 'xml')
+    V_B = ('edit', 'result', 'xml', 'buttons')
 
     # All layouts but "view" and "edit'. To use when you need both E_ and V_.
-    RX  = VE_ = ('result', 'xml')
+    RX = VE_ = ('result', 'xml')
+    RXB = VE_B = RX + ('buttons',) # The same + "buttons"
 
     # This set is used for showing workflow transitions in lists of objects
-    TR  = ('view', 'result')
+    TR = ('view', 'result')
+
+    # Show it on "pre" instead of "view", "result" also allowed
+    PR  = ('pre', 'result')
+    PRS = ('pre', 'result', 'sub')
 
     # This is, a.o., for custom widgets whose edit and view variants are not
     # that different, but that cannot be shown elsewhere (result, xml, etc).
-    VE  = ('view', 'edit')
-    VX  = ('view', 'xml')
-    EX  = ('edit', 'xml')
+    VE  = ('view'   , 'edit')
+    VX  = ('view'   , 'xml')
+    BX  = ('buttons', 'xml')
+    EX  = ('edit'   , 'xml')
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Initiator:
@@ -152,10 +160,10 @@ class Initiator:
            this initiator.'''
 
     def goBack(self):
-        '''Once the object has been created, where must we come back ? r_ can be
-           - "view"        return to the "view" page of the created object;
-           - "initiator"   return to the page, on self.obj, where self.field is.
-        '''
+        '''Once the object has been created, where must we come back ?'''
+        # r_ can be:
+        # - "view"      return to the "view" page of the created object ;
+        # - "initiator" return to the page, on p_self.o, where p_self.field is.
         return 'view'
 
     def getNavInfo(self, new):
@@ -199,7 +207,8 @@ class Field:
 
     # Some global static variables
     nullValues = (None, '', [], {}, ())
-    validatorTypes = (types.FunctionType, type(re.compile('')))
+    validatorTypes = (types.FunctionType, types.MethodType,
+                      type(re.compile('')))
     labelTypes = ('label', 'descr', 'help')
     viewLayouts = ('view', 'cell')
     cellLayouts = ('cell', 'query')
@@ -217,6 +226,7 @@ class Field:
 
     # Some fields are "outer": they are composed or inner fields
     outer = False
+
     # In most cases, outer fields store persistent lists
     outerType = PersistentList
 
@@ -226,6 +236,15 @@ class Field:
     # The name of the index class storing values of this field in the catalog
     indexType = 'Index' # For most fields, the base Index class suits
 
+    # By default, fields are supposed not to be "summable": they cannot be used
+    # to compute totals via appy/model/totals::Totals instances.
+    summable = False
+
+    # Most fields are not "freezable". Being freezable means: a field value,
+    # normally produced via some computation, can store the "hard-coded"
+    # (frozen) result of this computation.
+    freezable = False
+
     # Render a field. Optional vars:
     # * fieldName   can be given as different as field.name for fields included
     #               in outer fields: in this case, fieldName includes the row
@@ -244,8 +263,8 @@ class Field:
       name=fieldName or field.name|field.name;
       widgetName='w_%s' % name if isSearch else name;
       rawValue=field.getValueIf(o, name, layout, disable=isSearch);
-      value=not isSearch and \
-            field.getFormattedValue(o, rawValue, layout, showChanges);
+      value=field.getFormattedValue(o, rawValue, layout, showChanges) 
+            if not isSearch else None;
       requestValue=not isSearch and field.getStoredValue(o, name, True);
       inRequest=field.valueIsInRequest(o, req, name, layout);
       error=req['%s_error' % name] or handler.validator.errors.get(name)|None;
@@ -256,8 +275,7 @@ class Field:
       o=o or tool;
       tagId='%d_%s' % (o.iid, name);
       tagName=field.master and 'slave' or '';
-      layoutTarget=field">:field.getPx(minimal, \
-                                       hostLayout or layout, _ctx_)</x>''')
+      layoutTarget=field">:field.getPx(_ctx_)</x>''')
 
     def doRender(self, layout, o, name=None, minimal=False, tagCss=None,
                  specific=None):
@@ -275,7 +293,7 @@ class Field:
         # Call pxRender
         return self.pxRender(context)
 
-    # Show the field content for some object ("o") on a list of referred objects
+    # Show the field content for some object ("o") on a list of tied objects
     pxTied = Px('''
      <!-- The "title" field -->
      <x if="field.name == 'title'">
@@ -289,7 +307,8 @@ class Field:
       </x>
       <div if="not mayView">
        <img src=":svg('password')" class="iconS"/>
-       <x>:_('unauthorized')</x></div>
+       <x>:_('unauthorized')</x>
+      </div>
      </x>
 
      <!-- Any other field -->
@@ -304,29 +323,29 @@ class Field:
      <x if="field.name == 'title'"
         var2="navInfo=mode.getNavInfo(currentNumber); target=mode.target;
              showActions=not popup and uiSearch.showActions and guard.mayAct(o);
-            backHook=backHook|None">
+            backHook=backHook|None; ohook=ohook|None">
       <x if="mayView"
          var2="pageName=o.getDefaultPage('view');
                selectJs=popup and uiSearch.initiator.jsSelectOne(q,cbId) or ''">
        <x if="hasattr(o, 'getSupTitle')">::o.getSupTitle(navInfo)</x>
-       <x>::ui.Title.get(o, mode=mode.titleMode, nav=navInfo, target=target,
-          page=pageName, popup=popup, selectJs=selectJs,
-          pageLayout=uiSearch.search.pageLayoutOnView,
-          highlight=True, backHook=backHook)</x>
-       <span if="hasattr(o,'getSubTitle')"
-             class=":class_.getCssFor(o,'subTitle')"
-             style=":'display:inline' if mode.showSubTitles else 'display:none'"
-             name="subTitle">::uiSearch.highlight(o.getSubTitle())</span>
+       <x>::ui.Title.get(o, mode=uiSearch.getTitleMode(o, popup), nav=navInfo,
+         target=target, page=pageName, popup=popup, selectJs=selectJs,
+         pageLayout=uiSearch.search.pageLayoutOnView,
+         highlight=True, backHook=backHook)</x>
+       <span if="hasattr(o, 'getSubTitle')"
+             style=":'display:%s'% ('inline' if mode.showSubTitles else 'none')"
+             class="subTitle">::uiSearch.highlight(o.getSubTitle() or '')</span>
 
        <!-- Actions -->
-       <div if="showActions" class=":ui.getActionsCss(uiSearch.actionsDisplay)"
+       <div if="showActions" class=":class_.getSubCss(o, uiSearch)"
             var2="layout='sub';
                   editable=guard.mayEdit(o);
+                  iconsOnly=class_.getIconsOnly();
                   locked=o.Lock.isSet(o, user, 'main')">
 
         <!-- Fields on layout "sub" -->
         <x if="not popup and uiSearch.showActions == 'all'"
-           var2="fields=o.getFields(layout, 'main'); layout='cell'">
+           var2="fields=o.getFields(layout); layout='cell'">
          <!-- Call cell and not pxRender to avoid having a table -->
          <x for="field in fields"
             var2="name=field.name;
@@ -338,22 +357,28 @@ class Field:
            var2="workflow=o.getWorkflow()">:workflow.pxTransitions</x>
 
         <!-- Edit -->
-        <x if="editable">
-         <a if="not locked"
+        <div if="editable" class="ibutton" var2="text=_('object_edit')">
+         <a if="not locked" class="iflex1"
             var2="linkInPopup=popup or target.target != '_self'"
             target=":target.target"
             onclick=":target.getOnClick(backHook or ohook)"
             href=":o.getUrl(sub='edit', page=o.getDefaultPage('edit'), \
                             nav=navInfo, popup=linkInPopup)">
-          <img src=":svg('edit')" class="iconS" title=":_('object_edit')"/>
+          <img src=":svg('edit')" class="iconS" title=":text"/>
          </a>
          <x if="locked" var2="lockStyle='iconS'; page='main'">::o.Lock.px</x>
-        </x>
+         <div if="not iconsOnly" class="ibutton"
+              onclick="this.previousSibling.click()">:text</div>
+        </div>
 
         <!-- Delete -->
-        <img if="not locked and guard.mayDelete(o)" class="clickable iconS"
-             src=":svg('deleteS')" title=":_('object_delete')"
-             onClick=":'onDeleteObject(%s)' % q(o.url)"/>
+        <div if="not locked and guard.mayDelete(o)" class="ibutton"
+             var2="text=_('object_delete')">
+         <img class="clickable iconS" src=":svg('deleteS')" title=":text"
+              onClick=":'onDeleteObject(%s,%s)' % (q(o.iid), q(mode.hook))"/>
+         <div if="not iconsOnly" class="ibutton"
+              onclick="this.previousSibling.click()">:text</div>
+        </div>
        </div>
       </x>
       <x if="not mayView">
@@ -410,24 +435,25 @@ class Field:
 
     # Widget for filtering object values on query results
     pxFilterText = Px('''
-     <x var="name=field.name;
-             filterId='%s_%s' % (mode.hook, name);
-             filterIdIcon='%s_icon' % filterId">
+     <div var="name=field.name;
+          filterId='%s_%s' % (mode.hook, name);
+          filterIdIcon='%s_icon' % filterId" class="fhead">
       <!-- Pressing the "enter" key in the field clicks the icon (onkeydown) -->
-      <input type="text" size="7" id=":filterId"
-             value=":mode.filters.get(name, '')"
-             onkeydown=":'if (event.keyCode==13) document.getElementById(%s).
-                          click()' % q(filterIdIcon)"/>
+        <input type="text" size=":field.fwidth" id=":filterId"
+          value=":mode.filters.get(name, '')"
+          placeholder=":_(col.phLabel) if col.phLabel else ''|''"
+          onkeydown=":'if (event.keyCode==13) document.getElementById(%s).
+                       click()' % q(filterIdIcon)"/>
       <img id=":filterIdIcon" class="clickable iconS" src=":svg('funnel')"
            onclick=":'askBunchFiltered(%s,%s)' % (q(mode.hook), q(name))"/>
-     </x>''')
+     </div>''')
 
     def __init__(self, validator, multiplicity, default, defaultOnEdit, show,
-      page, group, layouts, move, indexed, mustIndex, indexValue,
+      renderable, page, group, layouts, move, indexed, mustIndex, indexValue,
       emptyIndexValue, searchable, filterField, readPermission, writePermission,
       width, height, maxChars, colspan, master, masterValue, focus, historized,
       mapping, generateLabel, label, sdefault, scolspan, swidth, sheight,
-      persist, inlineEdit, view, cell, edit, xml, translations):
+      persist, inlineEdit, view, cell, buttons, edit, xml, translations):
         # The validator restricts which values may be defined. It can be an
         # interval (1,None), a list of string values ['choice1', 'choice2'],
         # a regular expression, a custom function, a Selection instance, etc.
@@ -442,7 +468,15 @@ class Field:
         # Default value on layout "edit". If None, self.default is used instead.
         self.defaultOnEdit = defaultOnEdit
         # Must the field be visible or not ?
-        self.show = show    
+        self.show = show
+        # Is the field renderable on some layout ? If p_renderable is None,
+        # default rules apply for determining what field (depending on its type)
+        # can be rendered on what layout (see m_isRenderable below, overridden
+        # by some sub-classes). If you want to make an exception to these
+        # standard rules for some fields, set a method in p_renderable. It will
+        # accept a single arg (the current layout) and must return True or
+        # False.
+        self.renderable = renderable
         # When displaying/editing the whole object, on what page and phase must
         # this field value appear ?
         if page:
@@ -505,7 +539,9 @@ class Field:
         # If you are sure you will not use your indexed field for sorting, then,
         # specify "emptyIndexValue" being None. Indeed, it will produce a
         # smaller index: objects with no value for this field will not be
-        # present in the index.
+        # present in the index. Note that p_emptyIndexValue will also be
+        # (slighlty mis)used when sorting Ref objects. Indeed, starting with
+        # Python 3, comparing value None to any other value raises an error.
         self.emptyIndexValue = emptyIndexValue
         # If specified "searchable", the field will be added to some global
         # index allowing to perform application-wide, keyword searches.
@@ -612,13 +648,12 @@ class Field:
         # the widget to "edit" mode. If it returns 'icon', Switching to "edit"
         # will be done via an "edit" icon.
         self.inlineEdit = inlineEdit
-        # If you want to use alternate PXs than Field.view, Field.cell and
-        # Field.edit, you can specify it in parameters "view", "cell" and
-        # "edit". Instance attributes "view", "cell" and "edit" below will
-        # override their corresponding class attributes.
-        if view is not None: self.view = view
-        if cell is not None: self.cell = cell
-        if edit is not None: self.edit = edit
+        # Any alternate PX defined in the following attributes will be used in
+        # place of the standard ones.
+        if view    is not None: self.view    = view
+        if cell    is not None: self.cell    = cell
+        if buttons is not None: self.buttons = buttons
+        if edit    is not None: self.edit    = edit
         # Standard marshallers are provided for converting values of this field
         # into XML. If you want to customize the marshalling process, you can
         # define a method in "xml" that will accept a field value and will
@@ -682,19 +717,24 @@ class Field:
            search results (p_inRefs=False) or when sorting Ref fields
            (inRefs=True) ?'''
         if not inRefs:
-            return self.indexed and (self.emptyIndexValue is not None) and \
+            # An empty index value is required, because, starting with Python 3,
+            # comparison with None and something not being None raises an error.
+            return self.indexed and self.emptyIndexValue is not None and \
                    not self.isMultiValued()
         # When p_inRef is True, this method will be overridden by fields for
         # which sort is allowed.
 
-    def getFilterPx(self):
-        '''Returns the PX to use for rendering a filter for this field'''
+    def getFilterField(self):
+        '''p_self may be the variant of another field: this latter must be used
+           when it comes to filtering and sorting. This method returns this
+           potential filter companion, or p_self if none is defined, together
+           with the PX to use for filtering.'''
         target = self.filterField or self
         name = target.filterPx
         if name:
             r = target, getattr(target, name)
         else:
-            r = None, None
+            r = self, None
         return r
 
     def isShowable(self, o, layout):
@@ -718,19 +758,31 @@ class Field:
             return bool(r)
 
     def isRenderable(self, layout):
-        '''In some contexts, computing m_isShowable can be a performance
-           problem. For example, when showing some object's fields on layout
-           "buttons", there are plenty of fields that simply can't be shown on
-           this kind of layout: it is no worth computing m_isShowable for those
-           fields. m_isRenderable is meant to define light conditions to
-           determine, before calling m_isShowable, if some field has a chance to
-           be shown or not.
+        '''Returns True if p_self can be rendered on this p_layout'''
+        # In some contexts, computing m_isShowable can be a performance problem.
+        # For example, when showing some object's fields on layout "buttons",
+        # there are plenty of fields that simply can't be shown on this kind of
+        # layout: it is no worth computing m_isShowable for those fields.
+        # m_isRenderable is meant to define light conditions to determine,
+        # before calling m_isShowable, if some field has a chance to be shown
+        # or not.
 
-           In other words, m_isRenderable defines a "structural" condition,
-           independent of any object, while m_isShowable defines a contextual
-           condition, depending on some object.'''
-        # Most fields are not renderable on layout "buttons"
-        return False if layout == 'buttons' else True
+        # In other words, m_isRenderable defines a "structural" condition,
+        # independent of any object, while m_isShowable defines a contextual
+        # condition, depending on some object.
+
+        # That being said, if p_self.renderable is passed, it short-circuits
+        # this structural condition for p_self.
+        if self.renderable:
+            r = self.renderable(layout)
+        else:
+            r = self.isRenderableOn(layout)
+        return r
+
+    def isRenderableOn(self, layout):
+        '''Called by m_isRenderable when p_self.renderable is None'''
+        # This method may be overridden, but not p_isRenderable itself
+        return layout not in Layouts.restrictedTypes
 
     def isClientVisible(self, o):
         '''Returns True if p_self is visible according to master/slave
@@ -786,11 +838,12 @@ class Field:
             # Mapping is a method that must be applied to any i18n message
             return {'label':mapping, 'descr':mapping, 'help':mapping}
 
-    def getPx(self, minimal, layout, context):
+    def getPx(self, c):
         '''Returns the PX corresponding to p_layout'''
-        if minimal:
+        layout = c.hostLayout or c.layout
+        if c.minimal:
             # Call directly the layout-related PX on the field (bypass the
-            # layout PX)
+            # layout PX).
             r = getattr(self, layout)
         else:
             # Get the Layout instance related to p_layout and render its PX
@@ -799,8 +852,13 @@ class Field:
             else:
                 table = self.layouts[layout]
             # Add the layout in the context, it is required by its PX
-            context['table'] = table
+            c.table = table
             r = table.pxRender
+        # Before rendering the PX, compute totals if required
+        if self.summable and c.totals:
+            # Get the currently looped object
+            looped = c[c.totals.iterator]
+            Totals.update(c.totals, c.o, looped, self.name, c.rawValue, c.loop)
         return r
 
     def setMandatoriness(self, required):
@@ -911,7 +969,7 @@ class Field:
     def setRequestValue(self, o):
         '''Sets, in the request, field value on p_o in its "request" form (=the
            way the value is carried in the request).'''
-        # Get a copy of the field value on p_obj and put it in the request
+        # Get a copy of the field value on p_o and put it in the request
         value = self.getCopyValue(o)
         if value is not None:
             o.req[self.name] = value
@@ -1064,7 +1122,7 @@ class Field:
         # The base search widget corresponding to p_self is prefixed with "w_".
         # p_widgetName can be specified to get the sub-value of another widget
         # that is part of the complete search widget for p_self.
-        widgetName = widgetName or ('w_%s' % self.name)
+        widgetName = widgetName or 'w_%s' % self.name
         # p_req[widgetName] is the base (string) value encoded in the search
         # form. But the search value (or interval of values) can be made of
         # several form fields: overriden methods in Field sub-classes must build
@@ -1108,7 +1166,7 @@ class Field:
                   (hook, o.url, layout)
         if inlineEdit == 'icon':
             r = '<img src="%s" title="%s" class="inlineIcon iconS" %s/>%s' % \
-                (o.buildSvg('edit'), o.translate('object_edit'), onClick,
+                (o.buildSvg('editS'), o.translate('object_edit'), onClick,
                  value)
         else:
             r = '<span class="editable" %s>%s</span>' % (onClick, value)
@@ -1129,7 +1187,7 @@ class Field:
     def getIndex(self, o):
         '''Gets the Index instance corresponding to this field, or None if the
            field is not indexed.'''
-        catalog = o.getCatalog()
+        catalog = o.getCatalog(self.container)
         return catalog.get(self.name) if catalog else None
 
     def getIndexValue(self, o, searchable=False, raw=False):
@@ -1137,11 +1195,11 @@ class Field:
            purposes.'''
         # If p_searchable is True, the value must be a string because it is
         # meant to be included in index "searchable".
-        # ~~~
+        # ~
         # If p_raw is True, instead of returning the internal value ready to be
         # indexed, it returns the value before being converted to its internal
         # index representation.
-        # ~~~
+        # ~
         # Must we produce an index value ?
         if not self.getAttribute(o, 'mustIndex'): return
         # The indexed value is based on the stored field value
@@ -1169,12 +1227,13 @@ class Field:
            Those fields may define a secondary index, specifically for sorting.
            This is the case of Ref fields for example.'''
 
-    def getSortValue(self, obj):
-        '''Return the value of p_self on p_obj that must be used for sorting.
+    def getSortValue(self, o):
+        '''Return the value of p_self on p_o that must be used for sorting.
            While the raw p_value may be the value to use in most cases, it is
            not always true. For example, a string like "Gaëtan" could have
            "gaetan" as sort value.'''
-        return self.getValue(obj)
+        r = self.getValue(o)
+        return self.emptyIndexValue if r is None else r
 
     def valueIsInRequest(self, o, req, name=None, layout='view'):
         '''Is there a value corresponding to this field in the request? p_name
@@ -1194,7 +1253,7 @@ class Field:
         # as input for m_store. But this latter can further transform it. For
         # example, a value produced by p_getStorableValue for a Ref field is a
         # list of objects; method Ref.store will then go further and make
-        # cross-links between them and p_obj.
+        # cross-links between them and p_o.
         if self.isEmptyValue(o, value): return
         return value
 
@@ -1399,8 +1458,7 @@ class Field:
             # An inner field. No historization (yet) for it.
             part = ''
             # Retrieve the outer, inner fields and row number
-            name, px = req.px.rsplit(':', 1)
-            outer, inner, i = name.split('*')
+            outer, inner, i = o.traversal.parts[-2].split('*')
             i = int(i)
             outer = o.getField(outer)
             # Update the value. Clone the row to ensure the ZODB will detect the
@@ -1408,11 +1466,11 @@ class Field:
             # appy.Object.
             value = outer.getValue(o)
             row = value[i]
-            setattr(row, inner, reqValue)
+            setattr(row, inner, self.getStorableValue(o, reqValue))
             value[i] = row.clone()
             # Store the complete value again on p_o
             o.values[outer.name] = value
-        # Update obj's last modification date
+        # Update p_o's last modification date
         o.modified = DateTime()
         o.reindex()
         o.log(AJAX_EDITED % (self.name, part, o.id))
@@ -1427,7 +1485,7 @@ class Field:
         try:
             return o.H().methods.call(o, method, cache=cache)
         except TypeError as te:
-            # Try a version of the method that would accept self as an
+            # Try a version of the method that would accept p_self as an
             # additional parameter. In this case, we do not try to cache the
             # value (we do not call utils.callMethod), because the value may be
             # different depending on the parameter.
@@ -1468,24 +1526,18 @@ class Field:
                     r.append(group)
         return r
 
-    def process(self, obj):
-        '''This method is a general hook allowing a field to perform some
-           processing after an URL on an object has been called, of the form
-           <objUrl>/onProcess.'''
-        return obj.goto(obj.absolute_url())
-
-    def renderLabel(self, layoutType):
+    def renderLabel(self, layout):
         '''Indicates if the existing label (if hasLabel is True) must be
            rendered by pxLabel. For example, if field is an action, the
            label will be rendered within the button, not by pxLabel.'''
         if not self.hasLabel: return
         # Label is always shown in search forms
-        if layoutType == 'search': return True
+        if layout == 'search': return True
         # If the field is within a "tabs" group, the label will already be
         # rendered in the corresponding tab. If the field is in a "grid" group,
         # the label is already rendered in a separate column.
-        group = self.getGroup(layoutType)
-        if group and (group.style in ('tabs', 'grid')): return
+        group = self.getGroup(layout)
+        if group and group.style in ('tabs', 'grid'): return
         return True
 
     def getSelectSize(self, isSearch, isMultiple):
@@ -1540,18 +1592,21 @@ class Field:
             # attribute.
         return ';'.join(res)
 
-    def getRadioStyle(self, isSearch=False):
-        '''If this field must be rendered as radio buttons or checkboxes, and
-           its height is specified as a string, restrict the visible part of the
-           widget to this height, with a scrollbar when appropriate.'''
-        prefix = 's' if isSearch else ''
+    def getWidgetStyle(self, prefix=''):
+        '''If this field must be rendered as any widget not being a select
+           widget (managed by m_getSelectStyle hereabove), and if its width
+           and/or height are specified as strings, restrict the visible part of
+           the widget to these values, with a scrollbar when appropriate.'''
+        # Prefix can be "s" (*s*earch) or "f" (*f*ilter)
         r = []
-        height = getattr(self, '%sheight' % prefix)
-        if isinstance(height, str):
-            r.append('height:%s;overflow-y:auto' % height)
         width = getattr(self, '%swidth' % prefix)
         if isinstance(width, str):
             r.append('width:%s;overflow-x:auto' % width)
+        height = getattr(self, '%sheight' % prefix)
+        if isinstance(height, str):
+            # Use "max-height" and not "height", because, if the widget content
+            # is reduced, forcing the height will be ugly.
+            r.append('max-height:%s;overflow-y:auto' % height)
         return ';'.join(r)
 
     def getWidthInChars(self, isSearch):
@@ -1559,16 +1614,16 @@ class Field:
            number of chars shown in this field (a kind of "logical" width). If
            it contains a string, we must convert the value expressed in px
            (or in another CSS-compliant unit), to a number of chars.'''
-        prefix = isSearch and 's' or ''
+        prefix = 's' if isSearch else ''
         width = getattr(self, '%swidth' % prefix)
         if isinstance(width, int): return width
         if isinstance(width, str) and width.endswith('px'):
-            return int(width[:-2]) / 5
+            return int(int(width[:-2]) / 5)
         return 30 # Other units are currently not supported
 
-    def getListHeader(self, ctx):
+    def getListHeader(self, c):
         '''When p_self is used as inner-field, within a table-like rendered
            container field, this method returns the content of the header row
            corresponding to this inner field.'''
-        return ctx['_']('label', field=self)
+        return c._('label', field=self)
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
