@@ -25,9 +25,9 @@ from appy.model.searches import Search
 from appy.model.fields import Field, Initiator, Show
 
 # Constants  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-CONFIRM_ERROR = 'When using options, a popup will already be shown, with the ' \
-  'possibility to cancel the action, so it has no sense to ask a ' \
-  'confirmation via attribute "confirm".'
+CONF_ERR  = 'When using options, a popup will already be shown, with the ' \
+            'possibility to cancel the action, so it has no sense to ask a ' \
+            'confirmation via attribute "confirm".'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class ActionInitiator(Initiator):
@@ -52,7 +52,7 @@ class ActionInitiator(Initiator):
             self.backFromPopupUrl = msg
             r = None
         else:
-            r = msg
+            r = msg or None
         return r
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -65,10 +65,6 @@ class Multi:
         self.back = back
         # The DOM node storing the un/checked objects onto which the
         # multi-action must be executed.
-        if check and ',' in check:
-            # This is a search being embedded into another field. In that
-            # case, the hook is simply 'search'.
-            check = 'search'
         self.check = check
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -77,7 +73,7 @@ class Action(Field):
        given Appy class. An action is rendered as a button or icon.'''
 
     # Some methods will be traversable
-    traverse = {}
+    traverse = Field.traverse.copy()
 
     # Action-specific initiator class
     initiator = ActionInitiator
@@ -96,8 +92,20 @@ class Action(Field):
             '''Default layouts for this Action p_field'''
             return class_.b
 
+    # The button allowing to trigger the action
+    pxButton = Px('''
+     <!-- Real button -->
+     <input if="not isFake" type="button" class=":css" title=":inputTitle"
+            value=":inputLabel" style=":field.getIconUrl(url, asBG=True)"
+            onclick=":field.getOnClick(_ctx_)"/>
+
+     <!-- Fake button -->
+     <input if="isFake" type="button" class=":'%s fake' % css"
+            title=":isFake" value=":inputLabel"
+            style=":field.getIconUrl(url, asBG=True)"/>''')
+
     # PX for viewing the Action button
-    view = cell = query = Px('''
+    view = cell = buttons = query = Px('''
      <form var="isFake=field.isFake(o, _);  
                 formId='%d_%s_form' % (o.iid, name);
                 label=_('label', field=field);
@@ -108,8 +116,9 @@ class Action(Field):
                 onCell=layout in field.cellLayouts;
                 picto=picto|'pictoB';
                 asPicto=not onCell and (picto != 'pictoB');
-                css=ui.Button.getCss(label, onCell, field.render)"
-        id=":formId" action=":field.getFormAction(o, tool, layout)"
+                css=ui.Button.getCss(label, onCell, field.render,
+                                     iconOut=field.iconOut)"
+        id=":formId" action=":field.getFormAction(_ctx_)"
         target=":field.options and 'appyIFrame' or '_self'"
         method="post" class="inline">
 
@@ -128,7 +137,7 @@ class Action(Field):
 
       <!-- Form fields for multi-actions -->
       <x if="multi">
-       <input type="hidden" name="multi" value="True"/>
+       <input type="hidden" name="isMulti" value="True"/>
        <input type="hidden" name="searchParams"
               value=":tool.Search.encodeForReplay(req, layout)"/>
        <input type="hidden" name="checkedIds"/>
@@ -136,7 +145,7 @@ class Action(Field):
        <!-- The parameter starting with a star indicates to collect search
             criteria from the storage when present. -->
        <input type="hidden" name="_get_"
-        value=":'form:%s:multi,searchParams,checkedIds,checkedSem,*%s' % \
+        value=":'form:%s:isMulti,searchParams,checkedIds,checkedSem,*%s' % \
                 (formId, className)"/>
       </x>
 
@@ -149,20 +158,18 @@ class Action(Field):
        <div style=":'display:%s' % config.ui.pageDisplay">::inputTitle</div>
       </a>
 
-      <!-- As button -->
+      <!-- As button or icon -->
       <x if="not asPicto">
-       <!-- The button for executing the action -->
-       <input if="not isFake" type="button" class=":css" title=":inputTitle"
-              value=":inputLabel"
-              style=":url(field.sicon, base=field.siconBase,
-                          ram=field.siconRam, bg='18px 18px')"
-              onclick=":field.getOnClick(_ctx_)"/>
 
-       <!-- ... or the fake button -->
-       <input if="isFake" type="button" class=":'%s fake' % css"
-              title=":isFake" value=":inputLabel"
-              style=":url(field.sicon, base=field.siconBase,
-                          ram=field.siconRam, bg='18px 18px')"/>
+       <!-- Variant with the icon outside the button -->
+       <div if="field.iconOut" class="iflex1">
+        <img src=":field.getIconUrl(url)" onclick="this.nextSibling.click()"
+             class=":'clickable %s' % field.iconCss" title=":label"/>
+        <x>:field.pxButton</x>
+       </div>
+
+       <!-- Variant with the icon inside the button -->
+       <x if="not field.iconOut">:field.pxButton</x>
       </x>
      </form>''')
 
@@ -170,13 +177,15 @@ class Action(Field):
     edit = search = ''
 
     def __init__(self, validator=None, multiplicity=(1,1), default=None,
-      defaultOnEdit=None, show=Show.TR, page='main', group=None, layouts=None,
-      move=0, readPermission='read', writePermission='write', width=None,
-      height=None, maxChars=None, colspan=1, action=None, result='computation',
+      defaultOnEdit=None, show=Show.TR, renderable=None, page='main',
+      group=None, layouts=None, move=0, readPermission='read',
+      writePermission='write', width=None, height=None, maxChars=None,
+      colspan=1, action=None, result='computation',
       downloadDisposition='attachment', confirm=False, master=None,
       masterValue=None, focus=False, historized=False, mapping=None,
       generateLabel=None, label=None, icon=None, sicon=None, view=None,
-      cell=None, edit=None, xml=None, translations=None, render='button',
+      cell=None, buttons=None, edit=None, xml=None, translations=None,
+      render='button', iconOut=False, iconCss='iconS', iconSize=None,
       options=None, fake=False):
         # Attribute "action" must hold a method or a list/tuple of methods.
         # In most cases, every method will be called without arg, but there are
@@ -256,20 +265,36 @@ class Action(Field):
         self.sicon, self.siconBase, self.siconRam = utils.iconParts(sicon)
 
         # Call the base constructor
-        Field.__init__(self, None, (0,1), default, defaultOnEdit, show, page,
-          group, layouts, move, False, True, None, None, False, None,
-          readPermission, writePermission, width, height, None, colspan, master,
-          masterValue, focus, historized, mapping, generateLabel, label, None,
-          None, None, None, False, False, view, cell, edit, xml, translations)
+        Field.__init__(self, None, (0,1), default, defaultOnEdit, show,
+          renderable, page, group, layouts, move, False, True, None, None,
+          False, None, readPermission, writePermission, width, height, None,
+          colspan, master, masterValue, focus, historized, mapping,
+          generateLabel, label, None, None, None, None, False, False, view,
+          cell, buttons, edit, xml, translations)
         self.validable = False
 
         # There are various ways to render the action in the ui:
-        # "button"   (the default) as a button;
+        # "button"   (the default) as a button (icon included);
         # "icon"     as an icon on layouts where compacity is a priority
         #            (ie, within lists of objects) but still as a button on the
         #            "view" layout.
         self.render = render
 
+        # If p_iconOut is True, when the action is rendered as a "icon + button"
+        # (p_render is "button"), the icon will be positioned outside the
+        # button. Else, it will be integrated as a background-image inside it.
+        # Having the icon outside the button allows for more graphical
+        # possibilities (ie: the button may have a border that does not include
+        # the icon).
+        self.iconOut = iconOut
+
+        # The CSS class to apply to the button icon, in "icon out" mode
+        self.iconCss = iconCss
+
+        # The icon size, in "icon in" mode, where the image is inlaid as
+        # background.
+        self.iconSize = iconSize or '18px 18px'
+
         # An action may receive options: once the user clicks on the action's
         # icon or button, a form is shown, allowing to choose options. In order
         # to achieve this, specify an Appy class in field "options". self.action
@@ -366,37 +391,45 @@ class Action(Field):
         # options are in use, the process of executing and finalizing the action
         # is managed by the object creation mechanism, that has limitations.
         if self.options:
-            if self.confirm: raise Exception(CONFIRM_ERROR)
+            if self.confirm: raise Exception(CONF_ERR)
 
     def renderLabel(self, layoutType):
         return # Label is rendered directly within the button
 
-    def getFormAction(self, o, tool, layout):
+    def getFormAction(self, c):
         '''Get the value of the "action" parameter to the "form" tag
            representing the action.'''
         if self.options:
             # Submitting the form will lead to creating an object, in order to
             # retrieve action's options.
-            return '%s/new' % tool.url
+            return '%s/new' % c.tool.url
         else:
-            # Submitting the form will really trigger the action. Add the name
-            # of the class in the path if p_self is a multi-action.
-            part = '@%s/' % self.container.name if layout == 'query' else ''
-            return '%s/%s%s/perform' % (o.url, part, self.name)
+            # Submitting the form will really trigger the action.
+            if c.layout == 'query':
+                # We are running a multi-action from search results. Add the
+                # name of the class in the path.
+                part = '@%s/' % self.container.name
+            elif c.multi:
+                # We are running a multi-action from a Ref field. Add the
+                # initiator object in the action sub-path.
+                part = '@:%s/' % c.o.iid
+            else:
+                part = ''
+            return '%s/%s%s/perform' % (c.o.url, part, self.name)
 
-    def getOnClick(self, ctx):
+    def getOnClick(self, c):
         '''Gets the JS code to execute when the action button is clicked'''
         # Determine the ID of the form to submit
-        o = ctx.o
-        q = ctx.q
-        formId = '%d_%s_form' % (o.iid, ctx.name)
+        o = c.o
+        q = c.q
+        formId = '%d_%s_form' % (o.iid, c.name)
         # Determine the back hook and check hook (if multi)
-        multi = ctx.multi
+        multi = c.multi
         if multi:
             back = multi.back
             check = multi.check
         else:
-            back = ctx.ohook if ctx.layout == 'cell' else None
+            back = c.ohook if c.layout == 'cell' else None
             check = None
         check = q(check) if check else 'null'
         if not self.options:
@@ -444,7 +477,8 @@ class Action(Field):
             r = Search.replay(tool, searchParams)
         else:
             # Get objets from a ref
-            r = getattr(o, req.fieldName.split('*',1)[0])
+            name = req.hook.split('_')[1]
+            r = list(getattr(o, name))
         # Remove those not being checked in the UI
         Search.keepCheckedResults(req, r, unchecked=ids)
         return r
@@ -459,7 +493,7 @@ class Action(Field):
             args['comment'] = o.req.popupComment
         # Is that a multi-action ? A multi-action is an action to perform on a
         # list of objects instead of a single object.
-        if req.multi:
+        if req.isMulti:
             args['objects'] = self.getTargetObjects(o, req)
         # Call method(s) in self.action
         if type(self.action) in utils.sequenceTypes:
@@ -519,6 +553,17 @@ class Action(Field):
         if not self.hasDescr: return label
         return '%s: %s' % (label, o.translate(self.descrId))
 
+    def getIconUrl(self, url, pre='s', asBG=False):
+        '''Returns the URL to p_self's (s)icon (depending on p_pre(fix)), to be
+           used as a background image or not (depending on p_asBG).'''
+        # In "icon out" mode, no background image must be present
+        if asBG and self.iconOut: return ''
+        # If p_asBG, get the background image dimensions
+        bg = self.iconSize if asBG else False
+        return url(getattr(self, '%sicon' % pre),
+                   base=getattr(self, '%siconBase' % pre),
+                   ram=getattr(self, '%siconRam' % pre), bg=bg)
+
     def getInputLabel(self, label, layout):
         '''Returns the label to display on the button corresponding to this
            action = the content of attribute "value" for the "input" field.'''
@@ -534,8 +579,9 @@ class Action(Field):
         return _(self.labelId + '_confirm', blankOnError=True) or \
                _('action_confirm')
 
-    # Action fields can a priori be shown on every layout, "buttons" included
-    def isRenderable(self, layout): return True
+    def isRenderableOn(self, layout):
+        '''Action fields can be rendered on "button" but not on "top" layouts'''
+        return layout not in Layouts.topLayouts
 
     traverse['perform'] = 'perm:write'
     def perform(self, o, options=None, minimal=False):
@@ -548,15 +594,17 @@ class Action(Field):
         # Execute the action
         success, msg = self.execute(o, options=options)
         if not msg:
-            # Use the default i18n messages
-            suffix = 'done' if success else 'ko'
-            msg = o.translate('action_%s' % suffix)
+            # Use the default i18n message, if no message has been dumped yet on
+            # the response object.
+            if not resp.message:
+                suffix = 'done' if success else 'ko'
+                msg = o.translate('action_%s' % suffix)
             # If we had to redirect the user, we have no URL to do that; so we
             # fall back to a computation.
             result = 'computation' if result == 'redirect' else result
             r = msg
         elif result == 'file':
-            # msg does not contain a message, but a Python file handler
+            # v_msg does not contain a message, but a Python file handler
             r = msg.read()
             # If we are serving a file from the popup, close it afterwards
             if o.req.popup: resp.setCookie('closePopup', 'yes')
@@ -594,4 +642,12 @@ class Action(Field):
            defining useful info about the action, like the back and check
            hooks.'''
         return Multi(back, check)
+
+    def setTraversable(self, allow='perm:read'):
+        '''By default, the URL allowing to trigger an action (/perform) is
+           accessible only to those having permission 'write' on the
+           corresponding object. This method allows to update this and open the
+           action to anyone having this p_allow condition.'''
+        self.traverse = t = Action.traverse.copy()
+        t['perform'] = t['storeFromAjax'] = allow
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
