@@ -17,8 +17,10 @@
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 from appy.px import Px
+from appy.model.fields.text import Text
 from appy.model.fields import Field, Show
 from appy.ui.layout import Layouts, Layout
+from appy.model.fields.string import String
 from appy.model.searches import Search, UiSearch
 
 # Error messages - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -38,7 +40,14 @@ class Computed(Field):
         # Idem, but with a help icon
         gdh = Layouts('f-dhrvl')
 
-    view = cell = edit = Px('''<x if="field.plainText">:value</x><x
+    # Values produced by a Computed fields may be summable
+    summable = True
+
+    # By default, Computed values are considered to be freezable, excepted if
+    # explicitly declared as unfreezable, via instance attribute "unfreezable".
+    freezable = True
+
+    view = cell = buttons = edit = Px('''<x if="field.plainText">:value</x><x
       if="not field.plainText">::value</x>''')
 
     search = Px('''
@@ -46,15 +55,16 @@ class Computed(Field):
             size=":field.width" value=":field.sdefault"/>''')
 
     def __init__(self, multiplicity=(0,1), default=None, defaultOnEdit=None,
-      show=None, page='main', group=None, layouts=None, move=0, indexed=False,
-      mustIndex=True, indexType=None, indexValue=None, emptyIndexValue=None,
-      searchable=False, filterField=None, readPermission='read',
-      writePermission='write', width=None, height=None, maxChars=None,
-      colspan=1, method=None, formatMethod=None, plainText=False, master=None,
-      masterValue=None, focus=False, historized=False, mapping=None,
-      generateLabel=None, label=None, sdefault='', scolspan=1, swidth=None,
-      sheight=None, context=None, view=None, cell=None, edit=None, xml=None,
-      translations=None, unfreezable=False, validable=False):
+      show=None, renderable=None, page='main', group=None, layouts=None, move=0,
+      indexed=False, mustIndex=True, indexType=None, indexValue=None,
+      emptyIndexValue=None, searchable=False, filterField=None,
+      readPermission='read', writePermission='write', width=None, height=None,
+      maxChars=None, colspan=1, method=None, formatMethod=None, plainText=False,
+      master=None, masterValue=None, focus=False, historized=False,
+      mapping=None, generateLabel=None, label=None, sdefault='', scolspan=1,
+      swidth=None, sheight=None, fwidth=4, context=None, view=None, cell=None,
+      buttons=None, edit=None, xml=None, translations=None, unfreezable=False,
+      validable=False, pythonType=None):
         # The Python method used for computing the field value, or a PX
         self.method = method
         # A specific method for producing the formatted value of this field.
@@ -63,12 +73,13 @@ class Computed(Field):
         # the user interface while m_method computes the value stored in the
         # catalog.
         self.formatMethod = formatMethod
-        # Does field computation produce plain text or XHTML?
+        # If field computation produces a string, does this string value
+        # represent plain text or XHTML ?
         self.plainText = plainText
         if isinstance(method, Px):
             # When field computation is done with a PX, the result is XHTML
             self.plainText = False
-        # Determine default value for "show"
+        # Determine the default value for attribute "show"
         if show is None:
             # XHTML content in a Computed field generally corresponds to some
             # custom XHTML widget. This is why, by default, we do not render it
@@ -76,42 +87,49 @@ class Computed(Field):
             show = Show.E_ if self.plainText else Show.TR
         # If method is a PX, its context can be given in p_context
         self.context = context
-        # For any other Field subclass, the type of the index to use is
+        # For any other Field subclass, the type of the index to use (indexType)
+        # and the type of the value to store in the database (pythonType) are
         # statically determined. But for an indexed Computed field, which may
-        # hold any data of any type, the index type must be specified in
-        # p_indexType. Choose it in this list, depending on the values that your
-        # field will store.
+        # hold any data of any type, the index type (and, possibly, Python type)
+        # must be specified in p_indexType (and p_pythonType). Choose it in this
+        # list, depending on the values that your field will store.
         self.indexType = indexType or 'Index'
-        # ----------------------------------------------------------------------
-        #   Index type   | Suitable...
-        # ----------------------------------------------------------------------
-        #     "Index"    | ... in most cases, when none of the following index
-        #                |     types are appropriate ;
-        # ----------------------------------------------------------------------
+        self.pythonType = pythonType
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        #   Index type   | is suitable...
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #   "RefIndex"   | ... for storing lists of Appy objects (=instances of
         #                |     Appy classes). You should not use this type of 
         #                |     index and use a Ref field for storing lists of
         #                |     objects ;
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #   "DateIndex"  | ... if your field stores DateTime instances ;
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #   "TextIndex"  | ... if your field stores raw text and you want to
-        #                |     index words in found in it ;
-        # ----------------------------------------------------------------------
+        #                |     index words found in it ;
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #   "RichIndex"  | ... if your field stores a chunk of XHTML code and 
         #                |     you want to index text extracted from it ;
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  "FloatIndex"  | ... if your field stores float values ;
-        # ----------------------------------------------------------------------
-        # "BooleanIndex" | ... if your field stores boolean values.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # "BooleanIndex" | ... if your field stores boolean values ;
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        #     "Index"    | ... in any other case. Additionnally, if your field
+        #                |     stores an integer value, set pythonType to *int*
+        #                |     (=the Python basic type). If it stores a string,
+        #                |     specify *str*. This is because Appy uses the base
+        #                |     Index class indifferently for several fields like
+        #                |     Integer, String or Select.
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # Call the base constructor
         Field.__init__(self, None, multiplicity, default, defaultOnEdit, show,
-          page, group, layouts, move, indexed, mustIndex, indexValue,
-          emptyIndexValue, searchable, filterField, readPermission,
+          renderable, page, group, layouts, move, indexed, mustIndex,
+          indexValue, emptyIndexValue, searchable, filterField, readPermission,
           writePermission, width, height, None, colspan, master, masterValue,
           focus, historized, mapping, generateLabel, label, sdefault, scolspan,
-          swidth, sheight, False, False, view, cell, edit, xml, translations)
+          swidth, sheight, False, False, view, cell, buttons, edit, xml,
+          translations)
         # When a custom widget is built from a computed field, its values are
         # potentially editable and validable, so "validable" must be True.
         self.validable = validable
@@ -127,9 +145,14 @@ class Computed(Field):
         # o.[self.name] instead of recomputing it as usual. So if you want to
         # build a custom widget, specify the field as being unfreezable.
         self.unfreezable = unfreezable
+        # The base Python type corresponding to values computed by this field
+        self.pythonType = pythonType
         # Set a filter PX if this field is indexed with a TextIndex
-        if self.indexed and (self.indexType == 'TextIndex'):
+        if self.indexed and (self.indexType in ('TextIndex', 'RichIndex') or \
+                             self.pythonType == str):
             self.filterPx = 'pxFilterText'
+        # The *f*ilter width
+        self.fwidth = fwidth
         self.checkParameters()
 
     def checkParameters(self):
@@ -142,8 +165,7 @@ class Computed(Field):
 
     def renderPx(self, o, px):
         '''Renders the p_px and returns the result'''
-        traversal = o.traversal
-        context = traversal.context or traversal.createContext()
+        context = o.traversal.getContext()
         # Complete the context when relevant
         custom = self.context
         custom = custom if not callable(custom) else custom(o)
@@ -196,6 +218,17 @@ class Computed(Field):
             elif isinstance(r, Search): return self.renderSearch(o, r)
             return r
 
+    def getSearchValue(self, req, value=None):
+        '''Depending on p_self.indexType and p_self.pythonType, call the
+           appropriate method.'''
+        if self.indexType in ('TextIndex', 'RichIndex'):
+            fun = Text.computeSearchValue
+        elif self.pythonType == str:
+            fun = String.computeSearchValue
+        else:
+            fun = Field.getSearchValue
+        return fun(self, req, value=value)
+
     def getFormattedValue(self, o, value, layout='view', showChanges=False,
                           language=None):
         if self.formatMethod:
