@@ -38,11 +38,17 @@ class Date(Field):
                         'jscalendar/calendar-setup.js')}
 
     # Possible values for "format"
-    WITH_HOUR = 0
+    WITH_HOUR    = 0
     WITHOUT_HOUR = 1
     dateParts = ('year', 'month', 'day')
     hourParts = ('hour', 'minute')
 
+    # Types of the native HTML inputs, for every format
+    nativeWidgets = {WITH_HOUR: 'datetime-local', WITHOUT_HOUR: 'date'}
+
+    # Date formats as understood by native widgets
+    nativeFormats = {WITH_HOUR: '%Y-%m-%dT%H:%M', WITHOUT_HOUR: '%Y-%m-%d'}
+
     # Default value on the search screen
     searchDefault = (None, None, None)
 
@@ -52,13 +58,20 @@ class Date(Field):
     # The name of the index class storing values of this field in the catalog
     indexType = 'DateIndex'
 
-    view = cell = Px('''<x>:value</x>''')
+    view = cell = buttons = Px('''<x>:value or field.empty</x>''')
 
     # PX for selecting hour and minutes, kindly provided by field Hour
     pxHour = Hour.edit
 
     edit = Px('''
-     <x var="years=field.getSelectableYears(o)">
+     <!-- Native variant -->
+     <input if="field.native" name=":name" id=":name"
+            type=":field.nativeWidgets[field.format]"
+            value=":field.getNativeValue(
+             field.getInputValue(inRequest, requestValue, rawValue))"/>
+
+     <!-- Variant with one select widget for every part of the date/time -->
+     <x if="not field.native" var="years=field.getSelectableYears(o)">
       <!-- Day -->
       <select if="field.showDay" var2="days=range(1,32); part='%s_day' % name"
               name=":part" id=":part">
@@ -175,14 +188,15 @@ class Date(Field):
       defaultOnEdit=None, format=WITH_HOUR, dateFormat=None, hourFormat=None,
       calendar=True, startYear=time.localtime()[0]-10,
       endYear=time.localtime()[0]+10, reverseYears=False, minutesPrecision=5,
-      show=True, page='main', group=None, layouts=None, move=0, indexed=False,
-      mustIndex=True, indexValue=None, emptyIndexValue=0, searchable=False,
-      filterField=None, readPermission='read', writePermission='write',
-      width=None, height=None, maxChars=None, colspan=1, master=None,
-      masterValue=None, focus=False, historized=False, mapping=None,
-      generateLabel=None, label=None, sdefault=None, scolspan=1, swidth=None,
-      sheight=None, persist=True, view=None, cell=None, edit=None, xml=None,
-      translations=None, showDay=True, searchHour=False):
+      show=True, renderable=None, page='main', group=None, layouts=None, move=0,
+      indexed=False, mustIndex=True, indexValue=None, emptyIndexValue=0,
+      searchable=False, filterField=None, readPermission='read',
+      writePermission='write', width=None, height=None, maxChars=None,
+      colspan=1, master=None, masterValue=None, focus=False, historized=False,
+      mapping=None, generateLabel=None, label=None, sdefault=None, scolspan=1,
+      swidth=None, sheight=None, persist=True, view=None, cell=None,
+      buttons=None, edit=None, xml=None, translations=None, showDay=True,
+      searchHour=False, native=False, empty='-'):
         self.format = format
         self.calendar = calendar
         self.startYear = startYear
@@ -210,12 +224,21 @@ class Date(Field):
         # ("start" or "end"), if not None, must be a tuple (year, month, day).
         # Each of these sub-values can be None or an integer value. p_sdefault
         # can also be a method that produces the values in the specified format.
+        # ~
+        # [Experimental] the "native" mode makes use of HTML native input fields
+        # "date" (if p_format is WITHOUT_HOUR) or "datetime-local" (if p_format
+        # is WITH_HOUR).
+        self.native = native
+        # What to show on cell or view when there is no date ?
+        self.empty = empty
+        # Call the base constructor
         Field.__init__(self, validator, multiplicity, default, defaultOnEdit,
-          show, page, group, layouts, move, indexed, mustIndex, indexValue,
-          emptyIndexValue, searchable, filterField, readPermission,
+          show, renderable, page, group, layouts, move, indexed, mustIndex,
+          indexValue, emptyIndexValue, searchable, filterField, readPermission,
           writePermission, width, height, None, colspan, master, masterValue,
           focus, historized, mapping, generateLabel, label, sdefault, scolspan,
-          swidth, sheight, persist, False, view, cell, edit, xml, translations)
+          swidth, sheight, persist, False, view, cell, buttons, edit, xml,
+          translations)
 
     def getCss(self, o, layout, r):
         '''CSS files are only required if the calendar must be shown'''
@@ -232,11 +255,11 @@ class Date(Field):
         if self.reverseYears: r.reverse()
         return r
 
-    def validateValue(self, obj, value):
+    def validateValue(self, o, value):
         try:
             value = DateTime(value)
         except (DateError, ValueError):
-            return obj.translate('bad_date')
+            return o.translate('bad_date')
 
     def getFormattedValue(self, o, value, layout='view', showChanges=False,
                           language=None):
@@ -248,7 +271,7 @@ class Date(Field):
         # part "by hand".
         if '%Y' in dateFormat:
             dateFormat = dateFormat.replace('%Y', str(value.year()))
-        r = ui.formatDate(o.tool, value, dateFormat, withHour=False)
+        r = dates.Date.format(o.tool, value, dateFormat, withHour=False)
         if self.format == Date.WITH_HOUR:
             r += ' %s' % value.strftime(self.hourFormat or ui.hourFormat)
         return r
@@ -256,11 +279,13 @@ class Date(Field):
     def getRequestValue(self, o, requestName=None):
         req = o.req
         name = requestName or self.name
+        # Manage a native date
+        if self.native: return req[name]
         # Manage the "date" part
         value = ''
         for part in self.dateParts:
             # The "day" part may be hidden. Use "1" by default.
-            if (part == 'day') and not self.showDay:
+            if part == 'day' and not self.showDay:
                 valuePart = '01'
             else:
                 valuePart = req['%s_%s' % (name, part)]
@@ -277,6 +302,12 @@ class Date(Field):
             value = value[:-1]
         return value
 
+    def getNativeValue(self, value):
+        '''Ensure p_value is a string being understood by the native HTML
+           inputs.'''
+        return value.strftime(Date.nativeFormats[self.format]) \
+               if isinstance(value, DateTime) else value
+
     def searchValueIsEmpty(self, req):
         '''We consider a search value being empty if both "from" and "to" values
            are empty. At an individual level, a "from" or "to" value is
@@ -286,11 +317,15 @@ class Date(Field):
         return isEmpty(self, req) and \
                isEmpty(self, req, widgetName='%s_to_year' % self.name)
 
-    def getRequestSuffix(self): return '_year'
+    def getRequestSuffix(self):
+        return '' if self.native else '_year'
 
     def getStorableValue(self, o, value, single=False):
         '''Converts this string p_value too a DateTime instance'''
         if not self.isEmptyValue(o, value):
+            if self.native:
+                # Standardise this value: else, timezone may be wrong
+                value = value.replace('-', '/').replace('T', ' ')
             return DateTime(value)
 
     def getDateFromSearchValue(self, year, month, day, hour, setMin):
@@ -363,9 +398,9 @@ class Date(Field):
         return r
 
     def isSelected(self, o, part, fieldPart, dateValue, dbValue):
-        '''When displaying this field, must the particular p_dateValue be
-           selected in the sub-field p_fieldPart corresponding to the date
-           part ?'''
+        '''When displaying this field in non-native mode, must the particular
+           p_dateValue be selected in the sub-field p_fieldPart corresponding to
+           the date part ?'''
         # Get the value we must compare (from request or from database)
         req = o.req
         if part in req:
