@@ -43,45 +43,54 @@ RESIZED    = '%s resized to %spx.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class FileInfo:
-    '''A FileInfo instance holds metadata about a file on the filesystem.
+    '''A FileInfo instance holds metadata about a file on the filesystem'''
 
-       For every File field, we will store a FileInfo instance in the dabatase;
-       the real file will be stored in the Appy/ZODB database-managed
-       filesystem.
+    # For every File field, we will store a FileInfo instance in the dabatase;
+    # the real file will be stored in the Appy/ZODB database-managed filesystem.
+
+    # This is the primary usage of FileInfo instances. FileInfo instances can
+    # also be used every time we need to manipulate a file. For example, when
+    # getting the content of a Pod field, a temporary file may be generated and
+    # you will get a FileInfo that represents it.
 
-       This is the primary usage of FileInfo instances. FileInfo instances can
-       also be used every time we need to manipulate a file. For example, when
-       getting the content of a Pod field, a temporary file may be generated and
-       you will get a FileInfo that represents it.
-    '''
     BYTES = 50000
     NOT_FOUND = 'File "%s" was not found.'
 
-    def __init__(self, fsPath, inDb=True, uploadName=None):
-        '''p_fsPath is the path of the file on disk.
-           - If p_inDb is True, this FileInfo will be stored in the database and
-             will hold metadata about a File field whose content will lie in the
-             database-controlled filesystem. In this case, p_fsPath is the path
-             of the file *relative* to the root DB folder. We avoid storing
-             absolute paths in order to ease the transfer of databases from one
-             place to the other. Moreover, p_fsPath does not include the
-             filename, that will be computed later, from the field name.
+    # Max number of chars shown for a file's upload name
+    uploadNameMax = 30
 
-           - If p_inDb is False, this FileInfo is a simple temporary object
-             representing any file on the filesystem (not necessarily in the
-             db-controlled filesystem). For instance, it could represent a temp
-             file generated from a Pod field in the OS temp folder. In this
-             case, p_fsPath is the absolute path to the file, including the
-             filename. If you manipulate such a FileInfo instance, please avoid
-             using methods that are used by Appy to manipulate
-             database-controlled files (like methods getFilePath, removeFile,
-             writeFile or copyFile).'''
+    def __init__(self, fsPath, inDb=True, uploadName=None):
+        '''FileInfo constructor. p_fsPath is the path of the file on disk.'''
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # If p_inDb | This FileInfo
+        # is ...    | instance ...
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        #   True    | ... will be stored in the database and will hold metadata
+        #           | about a File field whose content will lie in the database-
+        #           | controlled filesystem. In this case, p_fsPath is the path
+        #           | of the file *relative* to the root DB folder. We avoid
+        #           | storing absolute paths in order to ease the transfer of
+        #           | databases from one place to the other. Moreover, p_fsPath
+        #           | does not include the filename, that will be computed
+        #           | later, from the field name.
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        #  False    |  ... is a simple temporary object representing any file on
+        #           | the filesystem (not necessarily in the db-controlled
+        #           | filesystem). For instance, it could represent a temp file
+        #           | generated from a Pod field in the OS temp folder. In this
+        #           | case, p_fsPath is the absolute path to the file, including
+        #           | the filename. If you manipulate such a FileInfo instance,
+        #           | please avoid using methods that are used by Appy to
+        #           | manipulate db-controlled files (like methods removeFile,
+        #           | writeFile or copyFile).
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         self.fsPath = fsPath
         self.fsName = None # The name of the file in fsPath
         self.uploadName = uploadName # The name of the uploaded file
         self.size = 0 # Its size, in bytes
         self.mimeType = None # Its MIME type
-        self.modified = None # The last modification date for this file
+        # The last modification date for this file, as a DateTime instance
+        self.modified = None
         # Complete metadata if p_inDb is False. p_inDb is not stored as is:
         # checking if self.fsName is the empty string is equivalent.
         if not inDb:
@@ -102,17 +111,39 @@ class FileInfo:
            filesystem ?'''
         return bool(self.fsName)
 
-    def getFilePath(self, o):
+    def getFilePath(self, o, checkTemp=True):
         '''Returns the Path instance corresponding to the file on disk that
            corresponds to this FileInfo instance.'''
+        # For a not-in-db file, its full path is already in p_self.fsPath
+        if not self.fsName: return self.fsPath
+        # Build the full path to this db-controlled file
         folder = o.getFolder()
         path = folder / self.fsName
-        if not path.is_file():
+        if not path.is_file() and checkTemp:
             # It may already have been deleted by a failed transaction. Try to
             # get a copy we may have made in the OS temp folder.
             path = Path(putils.getOsTempFolder()) / folder.name / self.fsName
         return path
 
+    def getUploadName(self, shorten=True):
+        '''Returns the upload name, nicely formatted'''
+        # The upload name may be missing
+        r = self.uploadName
+        if not r: return '?'
+        # Simply return the upload name if not too long
+        keep = FileInfo.uploadNameMax
+        if len(r) <= keep or not shorten: return r
+        # Return an "acronym" tag with the first chars of the name
+        return '<abbr title="%s" style="cursor:pointer">%s...</abbr>' % \
+               (r, r[:keep])
+
+    def exists(self, o):
+        '''Does the file really exist on the filesystem ?'''
+        # If the file exists but has no content, we consider it does not exist
+        if self.size == 0: return
+        path = self.getFilePath(o)
+        return path.is_file() and path.stat().st_size > 0
+
     def removeFile(self, folder, removeEmptyFolders=False):
         '''Removes the file from the filesystem'''
         path = folder / self.fsName
@@ -130,7 +161,14 @@ class FileInfo:
         '''Normalizes file p_name.'''
         return name[max(name.rfind('/'), name.rfind('\\'), name.rfind(':'))+1:]
 
-    def getShownSize(self): return putils.getShownSize(self.size)
+    def getShownSize(self):
+        '''Gets p_self's nicely formatted size'''
+        return putils.getShownSize(self.size, unbreakable=True)
+
+    def getNameAndSize(self, shorten=True):
+        '''Gets the file name and size, nicely formatted'''
+        return "%s - %s" % (self.getUploadName(shorten=shorten),
+                            self.getShownSize())
 
     def replicateFile(self, src, dest):
         '''p_src and p_dest are open file handlers. This method copies content
@@ -145,13 +183,14 @@ class FileInfo:
             dest.write(chunk)
         return size
 
-    def getBase64(self, o=None):
-        '''Returns the file content, as a base64-encoded string'''
+    def getBase64(self, o=None, asString=False):
+        '''Returns the file content, as base64-encoded bytes, or string if
+           p_asString is True.'''
         path = self.getFilePath(o) if o else self.fsPath
         f = open(path, 'rb')
         r = base64.b64encode(f.read())
         f.close()
-        return r
+        return r.decode() if asString else r
 
     def getExtension(self):
         '''Get the file extension from the MIME type or from the upload name'''
@@ -302,6 +341,11 @@ class FileInfo:
         # (re)compute p_self's attributes, after resizing
         self.setAttributes(spath)
 
+    def __repr__(self):
+        '''p_self's short string representation'''
+        return '<File @%s/%s %s>' % \
+               (self.fsPath, self.fsName, self.getShownSize())
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class File(Field):
     '''Field allowing to upload a binary file'''
@@ -331,7 +375,7 @@ class File(Field):
             '''Default layouts for this File p_field'''
             return class_.bg if field.inGrid() else class_.b
 
-    view = cell = Px('''
+    view = cell = buttons = Px('''
       <x>::field.getDownloadLink(name, o, layout)</x>''')
 
     edit = Px('''
@@ -367,13 +411,14 @@ class File(Field):
     search = ''
 
     def __init__(self, validator=None, multiplicity=(0,1), default=None,
-      defaultOnEdit=None, show=True, page='main', group=None, layouts=None,
-      move=0, readPermission='read', writePermission='write', width=None,
-      height=None, inputWidth=None, maxChars=None, colspan=1, master=None,
-      masterValue=None, focus=False, historized=False, mapping=None,
-      generateLabel=None, label=None, isImage=False, downloadAction=None,
-      sdefault='', scolspan=1, swidth=None, sheight=None, view=None, cell=None,
-      edit=None, xml=None, translations=None, render=None, icon='paperclip',
+      defaultOnEdit=None, show=True, renderable=None, page='main', group=None,
+      layouts=None, move=0, readPermission='read', writePermission='write',
+      width=None, height=None, inputWidth=None, maxChars=None, colspan=1,
+      master=None, masterValue=None, focus=False, historized=False,
+      mapping=None, generateLabel=None, label=None, isImage=False,
+      downloadAction=None, sdefault='', scolspan=1, swidth=None, sheight=None,
+      view=None, cell=None, buttons=None, edit=None, xml=None,
+      translations=None, render=None, icon='paperclip',
       disposition='attachment', nameStorer=None, cache=False, resize=False,
       thumbnail=None, viewWidth=None):
         # This boolean is True if the file is an image
@@ -430,11 +475,11 @@ class File(Field):
         self.viewWidth = viewWidth
         # Call the base constructor
         Field.__init__(self, validator, multiplicity, default, defaultOnEdit,
-          show, page, group, layouts, move, False, True, None, None, False,
-          None, readPermission, writePermission, width, height, None, colspan,
-          master, masterValue, focus, historized, mapping, generateLabel, label,
-          sdefault, scolspan, swidth, sheight, True, False, view, cell, edit,
-          xml, translations)
+          show, renderable, page, group, layouts, move, False, True, None, None,
+          False, None, readPermission, writePermission, width, height, None,
+          colspan, master, masterValue, focus, historized, mapping,
+          generateLabel, label, sdefault, scolspan, swidth, sheight, True,
+          False, view, cell, buttons, edit, xml, translations)
 
     def getRequestValue(self, o, requestName=None):
         name = requestName or self.name
@@ -450,7 +495,7 @@ class File(Field):
         info = self.getValue(o)
         if not info: return
         # Create a "not-in-DB", temporary FileInfo
-        return FileInfo(info.getFilePath(o), inDb=False,
+        return FileInfo(str(info.getFilePath(o)), inDb=False,
                         uploadName=info.uploadName)
 
     def isEmptyValue(self, o, value):
@@ -471,10 +516,11 @@ class File(Field):
            "edit" layout for this field.'''
         return 'width: %s' % self.inputWidth if self.inputWidth else ''
 
-    def isRenderable(self, layout):
+    def isRenderableOn(self, layout):
         '''A file with 'icon' rendering is potentially renderable everywhere'''
-        if self.render == 'icon': return True
-        return layout != 'buttons'
+        if self.render == 'icon':
+            return layout not in Layouts.topLayouts
+        return super().isRenderableOn(layout)
 
     def getDownloadLink(self, name, o, layout):
         '''Gets the HTML code for downloading the file as stored in field
@@ -483,28 +529,27 @@ class File(Field):
         # Display an empty value
         if not value: return '' if layout == 'cell' else '-'
         # On "edit", simply repeat the file title
-        if layout == 'edit': return value.uploadName
-        # Get the "file title", derived from the upload name and size
-        size = value.getShownSize()
-        title = "%s - %s" % (value.uploadName, size)
+        if layout == 'edit': return value.getUploadName()
         # Build the URL for downloading or displaying the file
         url = '%s/%s/download' % (o.url, name)
         # For images, display them directly
         if self.isImage:
             # Define a max width when relevant
             viewWidth = self.getAttribute(o, 'viewWidth')
-            css = ' style="max-width: %s"' % viewWidth if viewWidth else ''
-            return '<img src="%s" title="%s"%s/>' % (url, title, css)
+            css = ' style="max-width:%s"' % viewWidth if viewWidth else ''
+            return '<img src="%s" title="%s"%s/>' % \
+                   (url, value.getNameAndSize(shorten=False), css)
         # For non-images, display a link for downloading it, as an icon when
         # relevant.
         if self.render == 'icon':
             iurl = o.buildUrl(self.icon, base=self.iconBase, ram=self.iconRam)
-            content = '<img src="%s" title="%s"/>' % (iurl, title)
+            content = '<img src="%s" title="%s"/>' % \
+                      (iurl, value.getNameAndSize(shorten=False))
             # On "view", we have place, so display "title" besides the icon
             suffix = title if layout == 'view' else ''
         else:
             # Display textual information only
-            content = title
+            content = value.getNameAndSize()
             suffix = ''
         # Style the suffix
         if suffix: suffix = '<span class="refLink">%s</span>' % suffix
