@@ -28,10 +28,11 @@ G_COLS_ERR  = 'For a grid-style group, you must specify an even number of ' \
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Group:
-    '''Used for describing a group of fields within a page'''
+    '''Used for grouping fields or searches'''
+
     def __init__(self, name, columns=None, wide=True, style='section2',
       hasLabel=True, hasDescr=False, hasHelp=False, hasHeaders=False,
-      group=None, colspan=1, align='center', valign='top', css_class='',
+      group=None, colspan=1, align='center', valign='top', css=None,
       labelCss=None, master=None, masterValue=None, cellpadding=1,
       cellspacing=1, cellgap='0.6em', label=None, translated=None):
         self.name = name
@@ -97,8 +98,9 @@ class Group:
             # param combinations have no sense.
             self.hasLabel = self.hasDescr = self.hasHelp = False
             # Inner field/group labels will be used as tab labels
-        # "css_class", if specified, will be applied to the whole group
-        self.css_class = css_class
+        # p_css, if specified, is a CSS class that will be applied to the whole
+        # group.
+        self.css = css
         # "labelCss" is the CSS class that will be applied to the group label
         self._setLabelCss(labelCss, style)
         self.master = master
@@ -139,6 +141,11 @@ class Group:
             # columns)
             if len(self.columns) % 2: raise Exception(G_COLS_ERR)
 
+    def setColumns(self, columns):
+        '''Sets these p_columns instead of the current ones'''
+        self.columns = columns
+        self._setColumns(self.style)
+
     def _setLabelCss(self, labelCss, style):
         '''For "sectionX"-style groups, the applied CSS class is by default the
            p_style itself. For the other styles of groups (tabs, grids,...) it
@@ -217,6 +224,11 @@ class UiGroup:
     pxHelp = Px(lambda c: '<abbr title="%s"><img class="iconS" src="%s"/>' \
                     '</abbr>' % (c._('help', field=c.field), c.svg('help')))
 
+    # Standard CSS class to use, per group style and layout. If a style is not
+    # in this dict, it means that no deffault CSS class must be applied to its
+    # main tag.
+    byStyle = {'grid': {'edit': 'gridG gridE', None: 'gridG gridV'}}
+
     # Render the group title, description and help
     pxHeader = Px('''
      <!-- Title -->
@@ -238,6 +250,7 @@ class UiGroup:
             align=":ui.Language.flip(field.align, dir)"
             id=":tagId" name=":tagName" class=":groupCss"
             cellspacing=":field.cellspacing" cellpadding=":field.cellpadding">
+
       <!-- Title, description and help -->
       <x if="field.showHeader()">:field.pxHeader</x>
       <!-- The column headers -->
@@ -247,6 +260,7 @@ class UiGroup:
            width=":field.columnsWidths[colNb]">::field.hasHeaders and \
             _('%s_col%d' % (field.labelId, (colNb+1))) or ''</th>
       </tr>
+
       <!-- The rows of widgets -->
       <tr valign=":field.valign" for="row in field.elements">
        <td for="field in row" colspan=":field.colspan|1"
@@ -270,7 +284,7 @@ class UiGroup:
     pxTabs = Px('''
      <table width=":field.wide" class=":groupCss" id=":tagId" name=":tagName">
       <!-- First row: the tabs -->
-      <tr valign="middle"><td class="tabSep tabBottom">
+      <tr class="tabRow"><td>
        <table class="tabs" id=":'tabs_%s' % field.name">
         <tr valign="middle">
          <x for="sub in field.elements"
@@ -300,16 +314,16 @@ class UiGroup:
     pxGrid = Px('''
      <table cellpadding="0" cellspacing="0" width=":field.wide"
             class=":groupCss" id=":tagId" name=":tagName" align=":field.align">
+
       <!-- Title, description and help -->
       <x if="field.showHeader()">:field.pxHeader</x>
       <tr><th for="col in field.columns" width=":col.width"></th></tr>
-      <tr for="row in field.elements" valign="top"
-          class=":loop.row.odd and 'odd' or 'even'">
+      <tr for="row in field.elements">
        <x for="sub in row">
-        <td id="summaryCell">
+        <td>
          <label if="sub.hasLabel and \
                     (sub.type != 'Action')">::_('label', field=sub)</label></td>
-        <td var="field=sub" id="summaryCell">:field.pxRender</td>
+        <td var="field=sub">:field.pxRender</td>
        </x>
        <!-- Complete the last row when relevant -->
        <x if="loop.row.last"
@@ -317,16 +331,23 @@ class UiGroup:
          <td></td><td></td>
        </x>
       </tr>
-     </table>''')
+     </table>''',
+
+     css='''
+      .gridG>tbody>tr { vertical-align:top }
+      .gridG>tbody>tr:nth-child(odd) { background-color:|oddColor| }
+      .gridG>tbody>tr:nth-child(even) { background-color:|evenColor| }
+      .gridG>tbody>tr>td { padding:|gridGPadding| }
+      .gridV>tbody>tr>td:nth-child(odd)  { font-size:|gridVLFSize| }
+      .gridV>tbody>tr>td:nth-child(even) { font-size:|gridVFFSize| }
+      .gridE>tbody>tr>td:nth-child(odd)  { font-size:|gridELFSize| }
+      .gridE>tbody>tr>td:nth-child(even) { font-size:|gridEFFSize| }
+      .xhtml .gridV>tbody>tr>td { padding:0 }''')
 
     # PX that renders a group of fields (the group is referred as var "field")
     pxRender = Px('''
-     <x var="tagCss=field.master and ('slave*%s*%s' % \
-                    (field.master.getMasterTag(layout), \
-                    '*'.join(field.masterValue))) or '';
-             widgetCss=field.css_class;
-             groupCss=tagCss and ('%s %s' % (tagCss, widgetCss)) or widgetCss;
-             tagName=field.master and 'slave' or '';
+     <x var="groupCss=field.getCss(layout);
+             tagName='slave' if field.master else '';
              tagId='%d_%s' % (o.iid, field.name)">:field.pxFromStyle()</x>''')
 
     # PX that renders a group of searches
@@ -337,7 +358,7 @@ class UiGroup:
        <x>:field.translated or _(field.labelId)</x>
       </div>
       <!-- Group content -->
-      <div id=":collapse.id" style=":'padding-left: 10px; %s' % collapse.style">
+      <div id=":collapse.id" class="portletGC" style=":collapse.style">
        <x for="searches in field.elements">
         <x for="elem in searches">
          <!-- An inner group within this group -->
@@ -420,7 +441,7 @@ class UiGroup:
         lastRow = self.elements[-1]
         numberOfColumns = len(self.columnsWidths)
         # Grid groups span a single field on 2 columns
-        if self.style == 'grid': numberOfColumns = numberOfColumns / 2
+        if self.style == 'grid': numberOfColumns = int(numberOfColumns / 2)
         # Compute the number of columns already filled in the last row
         filledColumns = 0
         for rowElem in lastRow: filledColumns += rowElem.colspan
@@ -454,4 +475,23 @@ class UiGroup:
         parent = self.group.group
         if parent and (parent.style in ('tabs', 'grid')): return
         return self.hasLabel or self.hasDescr
+
+    def getCss(self, layout):
+        '''Returns the CSS class(es) to apply to p_self's main tag'''
+        r = []
+        # Get a potential CSS class depending on p_self's style and p_layout
+        css = self.byStyle.get(self.style)
+        if css:
+            css = css.get(layout) or css.get(None)
+            if css:
+                r.append(css)
+        # Add a potential class, depending on master/slave links
+        if self.master:
+            css = 'slave*%s*%s' % (self.master.getMasterTag(layout), 
+                                   '*'.join(self.masterValue))
+            r.append(css)
+        # Add p_self.css if defined
+        if self.css:
+            r.append(self.css)
+        return ' '.join(r)
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
