@@ -22,7 +22,12 @@ from appy.database.operators import in_
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Integer(Field):
-    view = cell = Px('''
+    '''Field storing an Integer value'''
+
+    # Integers and floats are summable
+    summable = True
+
+    view = cell = buttons = Px('''
      <x>::field.getInlineEditableValue(o, value, layout, name=name)</x>
      <input type="hidden" if="masterCss"
             class=":masterCss" value=":value" name=":name" id=":name"/>''')
@@ -67,15 +72,15 @@ class Integer(Field):
         return r
 
     def __init__(self, validator=None, multiplicity=(0,1), default=None,
-      defaultOnEdit=None, show=True, page='main', group=None, layouts=None,
-      move=0, indexed=False, mustIndex=True, indexValue=None,
+      defaultOnEdit=None, show=True, renderable=None, page='main', group=None,
+      layouts=None, move=0, indexed=False, mustIndex=True, indexValue=None,
       emptyIndexValue=None, searchable=False, readPermission='read',
       writePermission='write', width=5, height=None, maxChars=13, colspan=1,
       master=None, masterValue=None, focus=False, historized=False,
       mapping=None, generateLabel=None, label=None, sdefault=('',''),
-      scolspan=1, swidth=None, sheight=None, persist=True, inlineEdit=False,
-      view=None, cell=None, edit=None, xml=None, translations=None,
-      readonly=False, alignOnEdit='left', autoComplete=True):
+      scolspan=1, swidth=3, sheight=None, fwidth=3, persist=True,
+      inlineEdit=False, view=None, cell=None, buttons=None, edit=None, xml=None,
+      translations=None, readonly=False, alignOnEdit='left', autoComplete=True):
         # If attribute "readonly" is True (or stores a method returning True),
         # the rendered input field, on edit layouts, will have attribute
         # "readonly" set.
@@ -87,16 +92,19 @@ class Integer(Field):
         self.autoComplete = autoComplete
         # Call the base constructor
         Field.__init__(self, validator, multiplicity, default, defaultOnEdit,
-          show, page, group, layouts, move, indexed, mustIndex, indexValue,
-          emptyIndexValue, searchable, None, readPermission, writePermission,
-          width, height, maxChars, colspan, master, masterValue, focus,
-          historized, mapping, generateLabel, label, sdefault, scolspan, swidth,
-          sheight, persist, inlineEdit, view, cell, edit, xml, translations)
+          show, renderable, page, group, layouts, move, indexed, mustIndex,
+          indexValue, emptyIndexValue, searchable, None, readPermission,
+          writePermission, width, height, maxChars, colspan, master,
+          masterValue, focus, historized, mapping, generateLabel, label,
+          sdefault, scolspan, swidth, sheight, persist, inlineEdit, view, cell,
+          buttons, edit, xml, translations)
         # Define the corresponding Python type for values stored for this type
         self.pythonType = int
         # Define the filter PX when appropriate
         if self.indexed:
             self.filterPx = 'pxFilterText'
+        # The *f*ilter width
+        self.fwidth = fwidth
 
     def getFormattedValue(self, o, value, layout='view', showChanges=False,
                           language=None):
@@ -141,17 +149,20 @@ class Integer(Field):
            interval of typed values.'''
         # Get the "from" value
         value = Field.getSearchValue(self, req, value=value)
-        index = self.getIndexType(True)
         if value:
             value = self.pythonType(value)
-        # Get the "to" value
-        toValue = Field.getSearchValue(self, req, '%s_to' % self.name)
-        if toValue:
-            toValue = self.pythonType(toValue)
-        # Return an interval, excepted if both values are the same
-        if value != toValue:
-            r = in_(value, toValue)
-        else:
+        if req is None:
+            # A filter search
             r = value
+        else:
+            # A form search. Get the "to" value.
+            toValue = Field.getSearchValue(self, req, '%s_to' % self.name)
+            if toValue:
+                toValue = self.pythonType(toValue)
+            # Return an interval, excepted if both values are the same
+            if value != toValue:
+                r = in_(value, toValue)
+            else:
+                r = value
         return r
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
