@@ -21,46 +21,13 @@ from persistent.list import PersistentList
 
 from appy.px import Px
 from appy.model.fields import Field
+from appy.model.totals import Totals
 from appy.model.utils import Object as O
 from appy.ui.layout import Layout, Layouts
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-class Total:
-    '''Represents a computation that will be executed on a series of cells
-       within a List field.'''
-    def __init__(self, name, field, initValue):
-        self.name = name # The sub-field name
-        self.field = field # field.name is prefixed with the main field name
-        self.value = initValue
-
-    def __repr__(self):
-        return '<Total %s=%s>' % (self.name, str(self.value))
-
-#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-class Totals:
-    '''If you want to add rows representing totals computed from regular List
-       rows, specify it via Totals instances (see List attribute "totalRows"
-       below).'''
-    def __init__(self, id, label, subFields, onCell, initValue=0.0):
-        # "id" must hold a short name or acronym and must be unique within all
-        # Totals instances defined for a given List field.
-        self.id = id
-        # "label" is a i18n label that will be used to produce a longer name
-        # that will be shown at the start of the total row.
-        self.label = label
-        # "subFields" is a list or tuple of sub-field names for which totals
-        # must be computed.
-        self.subFields = subFields
-        # "onCell" stores a method that will be called every time a cell
-        # corresponding to a field listed in self.subFields is walked in the
-        # list. It will get these args:
-        # * row      the current row as an Object instance;
-        # * total    the Total instance (see above) corresponding to the current
-        #            column;
-        # * last     a boolean that is True if we are walking the last row.
-        self.onCell = onCell
-        # "initValue" is the initial value given to created Total instances
-        self.initValue = initValue
+RC_NO_OS  = 'Class "%s", mentioned in attribute "rowClass", must be a sub-' \
+            'class of class appy.model.utils.Object.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class List(Field):
@@ -74,7 +41,9 @@ class List(Field):
 
     class Layouts(Layouts):
         '''List-specific layouts'''
-        g = Layouts(edit=Layout('f;rv=', width=None), view='fl')
+        # No label on "edit" (*e*dit *m*inimalist)
+        em = Layouts(edit=Layout('f;rv=', width=None))
+        g = Layouts(edit=em['edit'], view='fl')
         gd = Layouts(edit=Layout('d;v=f;', width=None), view='fl')
         # Default layouts for sub-fields
         sub = Layouts(Layout('frv', width=None))
@@ -97,56 +66,50 @@ class List(Field):
      <tr valign="top" style=":'display:none' if rowId == -1 else ''"
          class=":'odd' if loop.row.odd else 'even'">
       <x if="not isCell">:field.pxFirstCell</x>
-      <td if="showTotals"></td>
-      <td for="info in subFields" if="info[1]"
-          var2="x=field.updateTotals(totals, o, info, row, loop.row.last);
-                field=info[1];
-                minimal=isCell;
+      <td for="name, field in subFields" if="field"
+          var2="minimal=isCell;
                 fieldName='%s*%d' % (field.name, rowId)">:field.pxRender</td>
       <!-- Icons -->
       <td if="isEdit" align=":dright">
-       <img class="clickable iconS" src=":svg('deleteS')"
+       <img class="clickable iconS" src=":svg('deleteL')"
             title=":_('object_delete')"
-            onclick=":field.jsDeleteConfirm(q, tableId)"/>
+            onclick=":field.jsDeleteConfirm(_ctx_)"/>
        <img class="clickable iconS" src=":svg('arrow')"
             style="transform: rotate(180deg)" title=":_('move_up')"
-            onclick=":'moveRow(%s,%s,this)' % (q('up'), q(tableId))"/>
+            onclick=":'moveRow(%s,%s,%s,this)' %
+                       (q('up'), q(tableId), q(tagId))"/>
        <img class="clickable iconS" src=":svg('arrow')"
             title=":_('move_down')"
-            onclick=":'moveRow(%s,%s,this)' % (q('down'), q(tableId))"/>
+            onclick=":'moveRow(%s,%s,%s,this)' %
+                       (q('down'), q(tableId), q(tagId))"/>
        <input class="clickable" type="image" tabindex="0"
               src=":url('addBelow')" title=":_('object_add_below')"
-              onclick=":'insertRow(%s,this); return false' % q(tableId)"/>
+              onclick=":'insertRow(%s,%s,this);
+                         return false' % (q(tableId), q(tagId))"/>
       </td></tr>''')
 
-    # Display totals (on view only) when defined
-    pxTotals = Px('''
-     <tr for="totalRow in field.totalRows" var2="total=totals[totalRow.id]">
-      <th>:_(totalRow.label)</th>
-      <th for="info in subFields">:field.getTotal(total, info)</th>
-     </tr>''')
-
     # PX for rendering the list (shared between pxView, pxEdit and pxCell)
     pxTable = Px('''
      <table var="isEdit=layout == 'edit';
                  isCell=layout == 'cell';
-                 tableId='list_%s' % name" if="isEdit or value"
-            id=":tableId" width=":field.width" class="grid"
-            var2="subFields=field.getSubFields(o, layout);
+                 tableId='list_%s' % name"
+            if="isEdit or value"
+            id=":tableId" width=":field.width"
+            class=":field.getTableCss(_ctx_)"
+            var2="Totals=field.Totals;
+                  subFields=field.getSubFields(o, layout);
                   swidths=field.getWidths(subFields);
-                  totals=field.createTotals(isEdit);
-                  showTotals=not isEdit and totals">
+                  totals=Totals.initialise(field, 'row', subFields, isEdit)">
       <!-- Header -->
       <thead>
-       <tr valign="middle">
-        <th if="showTotals"></th>
+       <tr valign=":field.headerAlign">
         <th for="name, sub in subFields" if="sub"
             width=":swidths[loop.name.nb]">
          <x var="field=sub">::field.getListHeader(_ctx_)</x></th>
         <!-- Icon for adding a new row -->
         <th if="isEdit">
          <img class="clickable" src=":url('plus')" title=":_('object_add')"
-              onclick=":'insertRow(%s)' % q(tableId)"/>
+              onclick=":'insertRow(%s,%s)' % (q(tableId), q(tagId))"/>
         </th>
        </tr>
       </thead>
@@ -159,17 +122,17 @@ class List(Field):
          for="row in rows" var2="rowId=loop.row.nb">:field.pxRow</x>
 
       <!-- Totals -->
-      <x if="showTotals">:field.pxTotals</x>
+      <x if="totals">:totals.px</x>
      </table>''')
 
-    view = cell = Px('''<x>:field.pxTable</x>''')
+    view = cell = buttons = Px('''<x>:field.pxTable</x>''')
     edit = Px('''<x>
      <!-- This input makes Appy aware that this field is in the request -->
      <input type="hidden" name=":name" value=""/><x>:field.pxTable</x>
     </x>''',
 
      js='''
-       updateRowNumber = function(row, rowIndex, action) {
+       updateRowNumber = function(tagId, row, rowIndex, action) {
          /* Within p_row, we must find tables representing fields. Every such
             table has an id of the form [objectId]_[field]*[subField]*[i].
             Within this table, for every occurrence of this string, the "[i]"
@@ -177,13 +140,13 @@ class List(Field):
             p_rowIndex is this index. If p_action is 'add', the index must
             become [i] + p_rowIndex. */
 
-         // Browse tables representing fields
-         var fields = row.getElementsByTagName('table'),
-             tagTypes = ['input', 'select', 'img', 'textarea', 'a', 'script'],
-             newIndex = -1, id, old, elems, neww, val, w, oldIndex;
+         // Browse elements representing fields
+         let fields = row.querySelectorAll('[id^="'+tagId+'"]'),
+             newIndex = -1, id, old, elems, new_, val, w, oldIndex,
+             tagTypes = ['a', 'script'], widgets=null;
 
          // Patch fields
-         for (var i=0; i<fields.length; i++) {
+         for (let i=0; i<fields.length; i++) {
            // Extract, from the table ID, the field identifier
            id = fields[i].id;
            if ((!id) || (id.indexOf('_') == -1)) continue;
@@ -198,27 +161,32 @@ class List(Field):
              oldIndex = parseInt(elems[2]);
              newIndex = (action == 'set')? rowIndex: oldIndex + rowIndex;
            }
-           neww = elems[0] + '*' + elems[1] + '*' + newIndex;
+           new_ = elems[0] + '*' + elems[1] + '*' + newIndex;
            // Replace the table ID with its new ID
-           fields[i].id = fields[i].id.replace(old, neww);
-           // Find sub-elements mentioning "old" and replace it with "neww"
+           fields[i].id = fields[i].id.replace(old, new_);
+           // Find widgets mentioning v_old and replace it with v_new_
            val = w = null;
-           for (var j=0; j<tagTypes.length; j++) {
-             var widgets = fields[i].getElementsByTagName(tagTypes[j]);
-             for (var k=0; k<widgets.length; k++) {
-               w = widgets[k];
-               // Patch id
-               val = w.id;
-               if (val) w.id = val.replace(old, neww);
-               // Patch name
-               val = w.name;
-               if (val) w.name = val.replace(old, neww);
+           widgets = fields[i].querySelectorAll('[id^="'+elems[0]+'\*"]');
+           for (let j=0; j<widgets.length; j++) {
+             w = widgets[j];
+             // Patch id
+             val = w.id;
+             if (val) w.id = val.replace(old, new_);
+             // Patch name
+             val = w.name;
+             if (val) w.name = val.replace(old, new_);
+           }
+           // Find other sub-tags and perform a similar replacement
+           for (let k=0; k<tagTypes.length; k++){
+             widgets = fields[i].getElementsByTagName(tagTypes[k]);
+             for (var l=0; l<widgets.length; l++) {
+               w = widgets[l];
                // Patch href
                if ((w.nodeName == 'A') && w.href)
-                 { w.href = w.href.replace(old, neww); }
+                 { w.href = w.href.replace(old, new_); }
                // Patch (and reeval) script
                if (w.nodeName == 'SCRIPT') {
-                 w.text = w.text.replace(old, neww);
+                 w.text = w.text.replace(old, new_);
                  eval(w.text);
                }
              }
@@ -226,7 +194,7 @@ class List(Field):
          }
        }
 
-       insertRow = function(tableId, previous) {
+       insertRow = function(tableId, tagId, previous) {
          /* Add a new row in table with ID p_tableId, after p_previous row or at
             the end if p_previous is null. */
          var table = document.getElementById(tableId),
@@ -241,21 +209,21 @@ class List(Field):
            // We must insert the new row before it
            body.insertBefore(newRow, next);
            // The new row takes the index of "next" (- the 2 unsignificant rows)
-           updateRowNumber(newRow, newIndex-2, 'set');
+           updateRowNumber(tagId, newRow, newIndex-2, 'set');
            // Row numbers for "next" and the following rows must be incremented
            for (var i=newIndex+1; i < rows.length; i++) {
-             updateRowNumber(rows[i], 1, 'add');
+             updateRowNumber(tagId, rows[i], 1, 'add');
            }
          }
          else {
            // We must insert the new row at the end
            body.appendChild(newRow);
            // Within newRow, incorporate the row nb within field names and ids
-           updateRowNumber(newRow, rows.length-3, 'set');
+           updateRowNumber(tagId, newRow, rows.length-3, 'set');
          }
        }
 
-       deleteRow = function(tableId, deleteImg, ask, rowIndex) {
+       deleteRow = function(tableId, tagId, deleteImg, ask, rowIndex) {
          var table = document.getElementById(tableId),
              rows = table.rows,
              row = (deleteImg)? deleteImg.parentNode.parentNode: rows[rowIndex];
@@ -263,33 +231,33 @@ class List(Field):
          // Must we ask the user to confirm this action ?
          if (ask) {
              askConfirm('script',
-                        'deleteRow("'+tableId+'",null,false,'+rowIndex+')');
+                'deleteRow("'+tableId+'","'+tagId+'",null,false,'+rowIndex+')');
              return;
          }
          // Decrement higher row numbers by 1 because of the deletion
          for (var i=rowIndex+1; i < rows.length; i++) {
-           updateRowNumber(rows[i], -1, 'add');
+           updateRowNumber(tagId, rows[i], -1, 'add');
          }
          table.deleteRow(rowIndex);
        }
 
-       moveRow = function(direction, tableId, moveImg) {
+       moveRow = function(direction, tableId, tagId, moveImg) {
          var row = moveImg.parentNode.parentNode,
              body = document.getElementById(tableId).tBodies[0], sibling;
          // Move up or down
          if (direction == 'up') {
            sibling = row.previousElementSibling;
            if (sibling && sibling.style.display != 'none') {
-             updateRowNumber(row, -1, 'add');
-             updateRowNumber(sibling, 1, 'add');
+             updateRowNumber(tagId, row, -1, 'add');
+             updateRowNumber(tagId, sibling, 1, 'add');
              body.insertBefore(row, sibling);
            }
          }
          else if (direction == 'down') {
            sibling = row.nextElementSibling;
            if (sibling) {
-             updateRowNumber(row, 1, 'add');
-             updateRowNumber(sibling, -1, 'add');
+             updateRowNumber(tagId, row, 1, 'add');
+             updateRowNumber(tagId, sibling, -1, 'add');
              // If sibling is null, row will be added to the end
              sibling = sibling.nextElementSibling;
              body.insertBefore(row, sibling);
@@ -300,18 +268,21 @@ class List(Field):
     search = ''
 
     def __init__(self, fields, validator=None, multiplicity=(0,1), default=None,
-      defaultOnEdit=None, show=True, page='main', group=None, layouts=None,
-      move=0, readPermission='read', writePermission='write', width='',
-      height=None, maxChars=None, colspan=1, master=None, masterValue=None,
-      focus=False, historized=False, mapping=None, generateLabel=None,
-      label=None, subLayouts=Layouts.sub, widths=None, view=None, cell=None,
-      edit=None, xml=None, translations=None, deleteConfirm=False,
-      totalRows=None):
+      defaultOnEdit=None, show=True, renderable=None, page='main', group=None,
+      layouts=None, move=0, readPermission='read', writePermission='write',
+      width='', height=None, maxChars=None, colspan=1, master=None,
+      masterValue=None, focus=False, historized=False, mapping=None,
+      generateLabel=None, label=None, subLayouts=Layouts.sub, widths=None,
+      view=None, cell=None, buttons=None, edit=None, xml=None,
+      translations=None, deleteConfirm=False, totalRows=None, rowClass=O,
+      headerAlign='middle', listCss=None):
+        # Call the base constructor
         Field.__init__(self, validator, multiplicity, default, defaultOnEdit,
-         show, page, group, layouts, move, False, True, None, None, False, None,
-         readPermission, writePermission, width, height, None, colspan, master,
-         masterValue, focus, historized, mapping, generateLabel, label, None,
-         None, None, None, True, False, view, cell, edit, xml, translations)
+         show, renderable, page, group, layouts, move, False, True, None, None,
+         False, None, readPermission, writePermission, width, height, None,
+         colspan, master, masterValue, focus, historized, mapping,
+         generateLabel, label, None, None, None, None, True, False, view, cell,
+         buttons, edit, xml, translations)
         self.validable = True
         # Tuple of elements of the form (name, Field instance) determining the
         # format of every element in the list. It can also be a method returning
@@ -331,9 +302,31 @@ class List(Field):
             self.initSubFields(fields)
         # When deleting a row, must we display a popup for confirming it ?
         self.deleteConfirm = deleteConfirm
-        # If you want to specify additional rows representing totals, give in
-        # "totalRows" a list of Totals instances (see above).
-        self.totalRows = totalRows or []
+        # If you want to specify additional rows representing totals, set, in
+        # "totalRows", a list of appy.model.totals::Totals instances.
+        self.totalRows = totalRows
+        # By default, every row of data that is stored in a List value is an
+        # instance of class Object, from package appy.model.utils. You can
+        # choose an alternate class, by defining it in attribute p_rowClass
+        # hereafter. The class you specify in p_rowClass must be a sub-class of
+        # class appy.model.utils.Object. Use this at your own risk. Take a close
+        # look at class appy.model.utils.Object, because the way you define your
+        # class could lead to Appy malfunctions.
+        self.rowClass = rowClass
+        # Header row vertical alignment
+        self.headerAlign = headerAlign
+        # CSS class to apply to the main table tag. If you let None, default
+        # will be "grid" on layout "edit" and "small" on other layouts (those
+        # CSS classes come from appy.css). In p_listCss, if you define a string,
+        # it will be understood as one (or more) CSS class(es) to apply on all
+        # layouts. If you define a dict, you can specify one (or more) CSS
+        # class(es) per layout, like in this example:
+        #
+        #                   {'edit': 'grid', 'view':'small'}
+        #
+        self.listCss = listCss
+        # Check parameters
+        self.checkParameters()
 
     def initSubFields(self, subFields):
         '''Initialises sub-fields' attributes'''
@@ -360,6 +353,22 @@ class List(Field):
         if not callable(subs):
             self.lazyInitSubFields(subs, class_)
 
+    def checkParameters(self):
+        '''Ensures this List is correctly defined'''
+        if not issubclass(self.rowClass, O):
+            raise Exception(RC_NO_OS % self.rowClass)
+
+    def getTableCss(self, c):
+        '''Return the CSS classe(s) to apply to the main table tag'''
+        default = 'grid' if c.isEdit else 'small'
+        r = self.listCss
+        if not r:
+            # Get the default class, that depends on the current layout
+            r = default
+        elif not isinstance(r, str):
+            r = r.get(c.layout) or default
+        return r
+
     def getWidths(self, subFields):
         '''Get the widths to appply to these p_subFields'''
         return self.widths or [''] * len(subFields)
@@ -399,7 +408,7 @@ class List(Field):
         for key in req.keys():
             if not key.startswith(prefix): continue
             # I have found a row: get its index
-            row = O()
+            row = self.rowClass()
             rowId = key.split('*')[-1]
             if rowId == '-1': continue # Ignore the template row
             for subName, subField in self.getSubFields(o):
@@ -445,7 +454,7 @@ class List(Field):
     def getStorableRowValue(self, o, requestValue):
         '''Gets a ready-to-store Object instance representing a single row,
            from p_requestValue.'''
-        r = O()
+        r = self.rowClass()
         for name, field in self.getSubFields(o):
             if not hasattr(requestValue, name) and \
                not field.isShowable(o, 'edit'):
@@ -457,8 +466,7 @@ class List(Field):
                 continue
             subValue = getattr(requestValue, name, None)
             try:
-                setattr(r, name,
-                        field.getStorableValue(o, subValue, single=True))
+                setattr(r, name, field.getStorableValue(o, subValue))
             except ValueError:
                 # The value for this field for this specific row is incorrect.
                 # It can happen in the process of validating the whole List
@@ -471,10 +479,13 @@ class List(Field):
 
     def getStorableValue(self, o, value, single=False):
         '''Gets p_value in a form that can be stored in the database'''
-        return [self.getStorableRowValue(o, v) for v in value]
+        r = PersistentList()
+        for v in value:
+            r.append(self.getStorableRowValue(o, v))
+        return r
 
     def getCopyValue(self, o):
-        '''Return a (deep) copy of field value on p_obj''' 
+        '''Return a (deep) copy of field value on p_o''' 
         r = getattr(o, self.name, None)
         if r: return copy.deepcopy(r)
 
@@ -490,10 +501,11 @@ class List(Field):
             if field:
                 field.getJs(o, layout, r, config)
 
-    def jsDeleteConfirm(self, q, tableId):
+    def jsDeleteConfirm(self, c):
         '''Gets the JS code to call when the user wants to delete a row'''
         confirm = 'true' if self.deleteConfirm else 'false'
-        return 'deleteRow(%s,this,%s)' % (q(tableId), confirm)
+        q = c.q
+        return "deleteRow(%s,%s,this,%s)" % (q(c.tableId), q(c.tagId), confirm)
 
     def subValidate(self, o, value, errors):
         '''Validates inner fields'''
@@ -504,35 +516,4 @@ class List(Field):
                 message = subField.validate(o, getattr(row, name, None))
                 if message:
                     setattr(errors, '%s*%d' % (subField.name, i), message)
-
-    def createTotals(self, isEdit):
-        '''When rendering the List field, if total rows are defined, create a
-           Total instance for every sub-field for which a total must be
-           computed.'''
-        if isEdit or not self.totalRows: return
-        r = {} # Keyed by Totals.name
-        for totals in self.totalRows:
-            subTotals = O()
-            for name in totals.subFields:
-                totalObj = Total(name, self.getField(name), totals.initValue)
-                setattr(subTotals, name, totalObj)
-            r[totals.id] = subTotals
-        return r
-
-    def updateTotals(self, totals, o, info, row, last):
-        '''Every time a cell is encountered while rendering the List, this
-           method is called to update totals when needed'''
-        if not totals: return
-        # Browse Totals instances
-        for totalRow in self.totalRows:
-            # Are there totals to update ?
-            total = getattr(totals[totalRow.id], info[0], None)
-            if total:
-                totalRow.onCell(o, row, total, last)
-
-    def getTotal(self, totals, info):
-        '''Get the total for the field p_info if available'''
-        total = getattr(totals, info[0], None)
-        if total: return total.value
-        return ''
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
