@@ -52,7 +52,7 @@ class Multilingual:
     # every language, and assembling the result according to the "languages
     # layout". The "Uni" PX receives the current language as variable "lg". If
     # the field is unilingual, the received "lg" variable is None.
-    view = edit = cell = Px('''
+    view = edit = cell = buttons = Px('''
      <x var="languages=field.getAttribute(o, 'languages');
              multi,value=field.renderMultilingual(o, languages, value, layout);
              pxUni=getattr(field, '%sUni' % layout)">
@@ -74,7 +74,7 @@ class Multilingual:
      </x>''',
 
      css='''
-      .lgMulti { display:inline-flex; column-gap:20px }
+      .lgMulti { display:inline-flex; column-gap:20px; margin:|lgMargin| }
       .lgMulti > div { display:flex; flex-wrap:nowrap; align-items:flex-start }
       .language {
         color:|darkColor|; font-size:7pt; border:|darkColor| 1px solid;
@@ -122,12 +122,12 @@ class Multilingual:
                     # Best choice: choose the content corresponding to the user
                     # language.
                     r = value.get(o.guard.userLanguage)
-                if not r:
-                    # Get the first non-empty value
-                    for val in value.values():
-                        if val:
-                            r = val
-                            break
+                    if not r:
+                        # Get the first non-empty value
+                        for val in value.values():
+                            if val:
+                                r = val
+                                break
                 return False, r
             else:
                 # ... and all languages must be rendered
