@@ -113,11 +113,11 @@ class Ogone(Field):
 
     # The "view" and "cell" layouts are used to display a button for performing
     # an on-line payment.
-    # ~~~
+    # ~
     # Note that variable "value" may not be present in the context, because, if
     # the field is shown on layout "buttons", on lists of objects, caller fields
     # like Ref calls directly field.cell instead of field.pxRender.
-    view = cell = Px('''
+    view = cell = buttons = Px('''
      <!-- The form for sending the payment request to Ogone.
           Variable "value" is misused and contains payment parameters. -->
 
@@ -147,16 +147,19 @@ class Ogone(Field):
 
     edit = search = ''
 
-    def __init__(self, orderMethod, responseMethod, show='view', page='main',
-      group=None, layouts=None, move=0, readPermission='read',
-      writePermission='write', width=None, height=None, colspan=1, master=None,
-      masterValue=None, focus=False, mapping=None, generateLabel=None,
-      label=None, view=None, cell=None, edit=None, xml=None, translations=None):
-        Field.__init__(self, None, (0,1), None, None, show, page, group,
-          layouts, move, False, True, None, None, False, None, readPermission,
-          writePermission, width, height, None, colspan, master, masterValue,
-          focus, False, mapping, generateLabel, label, None, None, None, None,
-          False, False, view, cell, edit, xml, translations)
+    def __init__(self, orderMethod, responseMethod, show='view',
+      renderable=None, page='main', group=None, layouts=None, move=0,
+      readPermission='read', writePermission='write', width=None, height=None,
+      colspan=1, master=None, masterValue=None, focus=False, mapping=None,
+      generateLabel=None, label=None, view=None, cell=None, buttons=None,
+      edit=None, xml=None, translations=None):
+        # Call the base constructor
+        Field.__init__(self, None, (0,1), None, None, show, renderable, page,
+          group, layouts, move, False, True, None, None, False, None,
+          readPermission, writePermission, width, height, None, colspan, master,
+          masterValue, focus, False, mapping, generateLabel, label, None, None,
+          None, None, False, False, view, cell, buttons, edit, xml,
+          translations)
         # orderMethod must contain a method returning a dict containing info
         # about the order. Following keys are mandatory, among others:
         #   * orderID   An identifier for the order. Don't use the object ID
@@ -188,7 +191,7 @@ class Ogone(Field):
         self.responseMethod = responseMethod
 
     # The Ogone field can a priori be shown on every layout, "buttons" included
-    def isRenderable(self, layout): return True
+    def isRenderableOn(self, layout): return layout != 'query'
 
     def createShaDigest(self, values, passphrase):
         '''Creates an Ogone-compliant SHA-512 digest based on key-value pairs in
