@@ -243,9 +243,9 @@ class UiPhase:
           if="phase.showPages(singlePhase, isEdit)">
 
       <!-- Siblings navigation -->
-      <div var="nav=req.nav"
-        if="not isEdit and phase.name == 'main' and nav and nav != 'no'"
-        var2="self=ui.Siblings.get(nav,tool,_ctx_)">:self.pxNavigate|'⚠'</div>
+      <div if="phase.showNavigation(_ctx_)"
+           var2="snav=ui.Siblings.get(req.nav, tool, _ctx_);
+                 gotoSource=True">:snav.pxNavigate|'⚠'</div>
 
       <!-- Page(s) within the phase -->
       <div if="not isEdit" for="pag in phase.pages.values()"
@@ -267,9 +267,9 @@ class UiPhase:
         </div>
 
         <!-- Page name and icons -->
-        <x var="label=Px.truncateText(pag.getLabel(), 25)">
+        <x var="label=Px.truncateText(pag.getLabel(), 25, css='clickable')">
          <a if="viewUrl" href=":viewUrl">::label</a>
-         <x if="not viewUrl">::label</x>
+         <a if="not viewUrl" class="unallowed">::label</a>
         </x>
 
         <!-- The edit or lock icon -->
@@ -285,16 +285,16 @@ class UiPhase:
       </div>
 
       <!-- Additional object controls -->
-      <x>:o.pxButtons</x>
+      <x var="inPhase=True">:o.pxButtons</x>
      </div>''',
 
      # Class names ending with "C" are used when controls are *C*ompact
      css='''
-      .phase { margin:0 0 30px -30px; display:flex; flex-wrap:wrap;
+      .phase { margin:|phaseMargin|; display:flex; flex-wrap:wrap;
                gap:|pageGap|; width:fit-content }
 
       .phase > div { height:|pageHeight|; width:|pageWidth|; text-align:center;
-        text-transform:|pageTextTransform|; font-weight:bold;
+        text-transform:|pageTextTransform|; font-weight:|pageTextWeight|;
         display:flex !important; align-items:center; justify-content:center;
         padding:|pagePadding|; background-color:|phaseBgColor|;
         position:relative; padding:0 5px; color:|phaseColor|;
@@ -303,26 +303,30 @@ class UiPhase:
         border-right:1px solid |phaseBorderColor|;
         border-top:1px solid |phaseBorderColor| }
 
-      .phase a { color:|phaseColor| }  
+      .phase a { color:|phaseColor| }
       .pagePicto, .phase form { display:|pageDisplay| }
-      .phaseC { margin:0 0 20px 0; border-left:1px solid |phaseBorderColor| }
+      .phaseC { margin:|phaseCMargin|; border-left:1px solid |phaseBorderColor|}
       .phaseC > div { height:|pageCHeight|; width:|pageCWidth|;
         font-size:|pageCFontSize|; padding:|pageCPadding|;
         text-transform:|pageCTextTransform|;
         border-radius:|pageCRadius| }
 
+      .phase .snav { flex-wrap:wrap }
+      .phase .navSib { padding:|navsPadding| }
+      .phase .navText { padding:0; color:|phaseColor| }
+
       .picto { width:|pictoSize|; height:|pictoSize|; margin:|pictoMargin| }
-      .pictoT { position:|epictoPosition|; top:8px; right:8px; width:22px;
-                padding: |epictoPadding| }
-      .pictoC, .pictoB { width:|pictoCSize|; height:|pictoCSize|;
+      .pictoT { position:|epictoPosition|; top:8px; right:8px;
+                width:|epictoWidth|; padding: |epictoPadding| }
+      .pictoC, .pictoB { width:|pictoCWidth|; height:|pictoCHeight|;
                          margin:|pictoCMargin| }
       .filtered { filter:invert(66%) }
 
       .currentPage { background-color:|phaseBgcColor| !important;
                      color:|phaseCcolor| }
       .currentPage img { filter:|phaseFilter| }
-      .currentPage div { color:|brightColor| }
-      .currentPage a { color:|phaseCcolor| }
+      .currentPage div { color:|phaseCcolor| }
+      .phase .currentPage a, .phase .currentPage span { color:|phaseCcolor| }
 
       // Animation for the "edit" icon
       @keyframes shakeImage {
@@ -379,6 +383,17 @@ class UiPhase:
                 return page.showOnEdit
         return True
 
+    def showNavigation(self, c):
+        '''Must siblings navigation be shown within this phase ?'''
+        # There may be no siblings to navigate to
+        nav = c.req.nav
+        if not nav or nav == 'no': return
+        # Navigation must not be shown on "edit", or on a phase not being "main"
+        if c.isEdit or self.name != 'main': return
+        # Navigation may already be shown in the page preamble, if present
+        if 'p' in c.pageLayout.layoutString: return
+        return True
+
     def __repr__(self):
         if self.pages:
             pages = ': page(s) %s' % ','.join(list(self.pages.keys()))
@@ -405,7 +420,7 @@ class UiPhases:
       <x if="not singlePhase and not isEdit">
        <table cellpadding="0" cellspacing="0">
         <!-- First row: the tabs -->
-        <tr><td class="tabSep">
+        <tr class="tabRowP"><td>
          <table class="tabs" id=":'tabs_%d' % id">
           <tr valign="middle">
            <x for="phase in phases.all.values()"
@@ -422,7 +437,7 @@ class UiPhases:
         <!-- Other rows: the fields -->
         <tr for="phase in phases.all.values()"
             id=":'tabcontent_%d_%s' % (id, phase.name)"
-          style=":(loop.phase.nb==0) and 'display:table-row' or 'display:none'">
+          style=":'display:%s' % ('table-row' if loop.phase.nb==0 else 'none')">
          <td>:phase.view</td>
         </tr>
        </table>
