@@ -17,7 +17,11 @@
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 import io
+from DateTime import DateTime # Just to have it in the interpreter context
 from contextlib import redirect_stdout
+from persistent.list import PersistentList
+from persistent.mapping import PersistentMapping
+# Some of these imports are just there to put names in the interpreter context
 
 from appy.px import Px
 from appy.utils import Traceback
@@ -38,7 +42,7 @@ class Python(Field):
 
     # All layouts produce the same rendering. That being said, in 99% of all
     # situations, only the "view" layout will be used.
-    view = cell = edit = Px('''
+    view = cell = buttons = edit = Px('''
      <div class="python"
           var="prefix='%s_%s' % (o.iid, field.name);
                hook='%s_output' % prefix">
@@ -99,16 +103,18 @@ class Python(Field):
     # There is no possibility to render this field on layout "search"
     search = ''
 
-    def __init__(self, show='view', page='main', group=None, layouts=None,
-      move=0, readPermission='read', writePermission='write', width=None,
-      height=None, colspan=1, master=None, masterValue=None, focus=False,
-      mapping=None, generateLabel=None, label=None, view=None, cell=None,
-      edit=None, xml=None, translations=None):
-        Field.__init__(self, None, (0,1), None, None, show, page, group,
-          layouts, move, False, True, None, None, False, None, readPermission,
-          writePermission, width, height, None, colspan, master, masterValue,
-          focus, False, mapping, generateLabel, label, None, None, None, None,
-          False, False, view, cell, edit, xml, translations)
+    def __init__(self, show='view', renderable=None, page='main', group=None,
+      layouts=None, move=0, readPermission='read', writePermission='write',
+      width=None, height=None, colspan=1, master=None, masterValue=None,
+      focus=False, mapping=None, generateLabel=None, label=None, view=None,
+      cell=None, buttons=None, edit=None, xml=None, translations=None):
+        # Call the base constructor
+        Field.__init__(self, None, (0,1), None, None, show, renderable, page,
+          group, layouts, move, False, True, None, None, False, None,
+          readPermission, writePermission, width, height, None, colspan, master,
+          masterValue, focus, False, mapping, generateLabel, label, None, None,
+          None, None, False, False, view, cell, buttons, edit, xml,
+          translations)
 
     def getAjaxData(self, hook, o):
         '''Initializes an AjaxData object on the DOM node corresponding to this
