@@ -44,7 +44,7 @@ class Rich(Multilingual, Field):
        editor "ckeditor".'''
 
     # Some elements are traversable
-    traverse = {}
+    traverse = Field.traverse.copy()
 
     # Required Javascript files
     cdnUrl = 'https://cdn.ckeditor.com/%s/%s/ckeditor.js'
@@ -54,10 +54,11 @@ class Rich(Multilingual, Field):
 
     class Layouts(Layouts):
         '''Rich-specific layouts'''
-        baseEdit = Layout('lrv-d-f', css_class='bottomSpaceS')
+        baseEdit = Layout('lrv-d-f', css='bottomSpaceS')
         b = Layouts(edit=baseEdit, view='l-f')
         c = Layouts(edit=baseEdit, view='lc-f') # Idem but with history
         f = Layouts(edit=baseEdit, view='f') # Field only (no label) on view
+        ft = Layouts(edit=baseEdit, view=Layout('f', css='topSpace'))
         g = Layouts(edit=Layout('d2-f;rv=', width='99%'),
                     view=Layout('fl', width='99%'))
         gc = Layouts(edit=Layout('drv-f', width='99%'),
@@ -119,7 +120,7 @@ class Rich(Multilingual, Field):
 
     # Unilingual edit
     editUni = Px('''
-     <textarea var="inputId=not lg and name or '%s_%s' % (name, lg)"
+     <textarea var="inputId=name if not lg else ('%s_%s' % (name, lg))"
        id=":inputId" name=":inputId" cols=":field.getTextareaCols()"
        style=":field.getTextareaStyle()"
        rows=":field.height">:field.getInputValue(inRequest, requestValue, value)
@@ -131,17 +132,18 @@ class Rich(Multilingual, Field):
             value=":field.sdefault" name=":widgetName"/>''')
 
     def __init__(self, validator=None, multiplicity=(0,1), default=None,
-      defaultOnEdit=None, show=True, page='main', group=None, layouts=None,
-      move=0, indexed=False, mustIndex=True, indexValue=None, searchable=False,
-      filterField=None, readPermission='read', writePermission='write',
-      width=None, height=None, maxChars=None, colspan=1, master=None,
-      masterValue=None, focus=False, historized=False, mapping=None,
-      generateLabel=None, label=None, sdefault='', scolspan=1, swidth=None,
-      sheight=None, persist=True, styles=defaultStyles, customStyles=None,
-      documents=False, spellcheck=False, languages=('en',),
+      defaultOnEdit=None, show=True, renderable=None, page='main', group=None,
+      layouts=None, move=0, indexed=False, mustIndex=True, indexValue=None,
+      searchable=False, filterField=None, readPermission='read',
+      writePermission='write', width=None, height=None, maxChars=None,
+      colspan=1, master=None, masterValue=None, focus=False, historized=False,
+      mapping=None, generateLabel=None, label=None, sdefault='', scolspan=1,
+      swidth=None, fwidth=10, sheight=None, persist=True, styles=defaultStyles,
+      customStyles=None, documents=False, spellcheck=False, languages=('en',),
       languagesLayouts=None, viewSingle=False, inlineEdit=False,
-      toolbar='Standard', view=None, cell=None, edit=None, xml=None,
-      translations=None, inject=False, valueIfEmpty='', viewCss='xhtml'):
+      toolbar='Standard', view=None, cell=None, buttons=None, edit=None,
+      xml=None, translations=None, inject=False, valueIfEmpty='',
+      viewCss='xhtml'):
         # The list of styles that the user will be able to select in the styles
         # dropdown (within CKEditor) is defined hereafter.
         self.styles = styles
@@ -151,15 +153,15 @@ class Rich(Multilingual, Field):
         # entries; every entry represents a custom style and must be a dict
         # having the following keys and values. Every key and value must be a
         # string.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #    "id"    | An identifier for your style, that must be different from
         #            | any standard CKEditor style like h1, h2, p, div, etc;
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #   "name"   | A translated name for your style, that will appear in the
         #            | dropdown. Do not use any special (ie, accentuated) char
         #            | in the value: prefer the use of an HTML entity for
         #            | defining such char.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  "element" | The HTML tag that will surround the text onto which the
         #            | style will be applied. Do not use "blockquote", it does
         #            | not work. Non-standard tag "address" works well, or any
@@ -168,7 +170,7 @@ class Rich(Multilingual, Field):
         #            | activated it in attribute "style" hereabove, you won't be
         #            | able to make the difference in the result between both
         #            | styles because they will produce a similar result.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         self.customStyles = customStyles or Rich.customStyles
         # By mentioning, in attribute "documents", the name of a Ref field
         # pointing to Document instances (see appy/model/document.py), images
@@ -189,46 +191,46 @@ class Rich(Multilingual, Field):
         # These links will not be rendered as-is: they will represent as many
         # placeholders for injecting other content.
         #
-        #                            ------------
-        #                            | Case (a) |
-        #                            ------------
+        #                            - - - - - - -
+        #                            |  Case (a) |
+        #                            - - - - - - -
         #
         # Each URL must point to a pure textual resource containing Python code.
         # When rendering this link, Appy will download the code file and extract
         # from it the part corresponding to <part_specifier>. Possible code
         # specifiers are the following.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  c_<name>    | This will extract the definition of *c*lass named
         #              | <name>, including its subsequent docstring and comment
         #              | lines, until the first line of not-commented code is
         #              | reached, or a comment starting with "# ~" is
         #              | encountered.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  s_<cn>.<mn> | This will extract the *s*ignature of method named <mn>
         #              | in class named <cn>, its subsequent docstring and
         #              | comment lines, until the first not-commented line of
         #              | code is reached, or a comment starting with "# ~" is
         #              | encountered.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  m_<cn>.<mn> | This will extract the complete method named <mn> (as
         #              | defined in class named <cn>), including its body.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #
-        #                            ------------
-        #                            | Case (b) |
-        #                            ------------
+        #                            - - - - - - -
+        #                            |  Case (b) |
+        #                            - - - - - - -
         # 
         # This case is used to inject the content of a PX. The content of the
         # "a" tag must start with "px:", followed by a specifier made of 2
         # parts, separated by an asterisk (*):
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # "object"  | must be the name of an object, class or module being
         #           | available in the current PX context ;
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #   "px"    | is the name of the PX to use. It must correspond to an
         #           | attribute, available on "object", and be an instance of
         #           | class appy.px.Px.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #
         # Note that, when using POD to dump content of Rich fields via the xhtml
         # function, special "a" tags as described hereabove (cases (a) and (b))
@@ -245,15 +247,18 @@ class Rich(Multilingual, Field):
         # class named "xhtml" is applied. If you want to specify another class,
         # use the following attribute. A method can also be passed.
         self.viewCss = viewCss
-        # ~~~
-        # Call the base constructors
+        # ~
+        # Call base constructors
         Multilingual.__init__(self, languages, languagesLayouts, viewSingle)
         Field.__init__(self, validator, multiplicity, default, defaultOnEdit,
-          show, page, group, layouts, move, indexed, mustIndex, indexValue,
-          None, searchable, filterField, readPermission, writePermission, width,
-          height, maxChars, colspan, master, masterValue, focus, historized,
-          mapping, generateLabel, label, sdefault, scolspan, swidth, sheight,
-          persist, inlineEdit, view, cell, edit, xml, translations)
+          show, renderable, page, group, layouts, move, indexed, mustIndex,
+          indexValue, None, searchable, filterField, readPermission,
+          writePermission, width, height, maxChars, colspan, master,
+          masterValue, focus, historized, mapping, generateLabel, label,
+          sdefault, scolspan, swidth, sheight, persist, inlineEdit, view, cell,
+          buttons, edit, xml, translations)
+        # The *f*ilter width
+        self.fwidth = fwidth
         # No max chars by default
         if maxChars is None:
             self.maxChars = Rich.NO_MAX
@@ -322,10 +327,14 @@ class Rich(Multilingual, Field):
             r = value
         return r
 
+    def getXhtmlCleaner(self):
+        '''Returns a Cleaner instance tailored to p_self'''
+        return Cleaner()
+
     def validateUniValue(self, o, value):
         '''Ensure p_value as will be stored (=cleaned) is valid XHTML'''
         try:
-            Cleaner().clean(value)
+            self.getXhtmlCleaner().clean(value)
         except SAXParseException as err:
             # Try after removing problematic chars
             value = StringCleaner.clean(value)
@@ -345,25 +354,28 @@ class Rich(Multilingual, Field):
         r = Field.getHistoryValue(self, o, value, i, language, empty=None)
         return empty if not r else self.getDiffValue(o, i, language)
 
-    def getUniStorableValue(self, o, value):
-        '''Gets the p_value as can be stored in the database within p_obj'''
+    def getUniStorableValue(self, o, value, wrap=True):
+        '''Gets the p_value as can be stored in the database within p_o'''
         if not value: return value
         # Clean the value. When image upload is enabled, ckeditor inserts some
         # "style" attrs (ie for image size when images are resized). So in this
         # case we can't remove style-related information.
+        cleaner = self.getXhtmlCleaner()
         try:
-            value = Cleaner().clean(value)
+            value = cleaner.clean(value, wrap=wrap)
         except SAXParseException as err:
             # Try to remove chars known for producing SAX errors ...
             value = StringCleaner.clean(value)
             # ... then, re-try
             try:
-                value = Cleaner().clean(value)
+                value = self.getXhtmlCleaner().clean(value, wrap=wrap)
             except SAXParseException as err:
                 # Errors while parsing p_value can't prevent the user from
                 # storing it.
                 o.log(XML_ERROR % (self.name, o.id, str(err)), type='warning')
-        # Manage maxChars
+        # Manage maxChars. Truncating p_value will produce a corrupted chunk of
+        # XHTML: the objective here, in allowing size restrictions, is to avoid
+        # people or robots to inject large, insidious, content.
         max = self.maxChars
         if max and (len(value) > max): value = value[:max]
         return value
