@@ -129,10 +129,12 @@ class Select(Field):
         d  = Layouts(Layout('d2-l-f;rv=', width=None))
         # With a description, but no label
         dn = Layouts(Layout('d-frv=', width=None))
+        # Within grid groups
         g  = Layouts(edit=Layout('f;rv=', width=None),
                      view=Layout('fl', width=None))
-        gd = Layouts(edit=Layout('drv-f=', width=None),
+        gd = Layouts(edit=Layout('d;rv=-f2=', width=None),
                      view=Layout('fl', width=None))
+        gh = Layouts(edit=Layout('f;hrv=', width=None), view=gd['view'])
 
         @classmethod
         def getDefault(class_, field):
@@ -151,11 +153,11 @@ class Select(Field):
      <input type="hidden" if="masterCss" class=":masterCss" value=":rawValue"
             name=":name" id=":name"/>''')
 
-    # More compact representation on the cell layout
-    cell = Px('''
-     <x var="multiple=value and isMultiple">
-      <x if="multiple">:', '.join(value)</x>
-      <x if="not multiple">:field.view</x>
+    # More compact representation on cell and button layouts
+    cell = buttons = Px('''
+     <x var="isMultiple=value and field.isMultiValued(); masterCss=None">
+      <x if="isMultiple">:', '.join(value)</x>
+      <x if="not isMultiple">:field.view</x>
      </x>''')
 
     edit = Px('''
@@ -175,7 +177,7 @@ class Select(Field):
      </select>
 
      <!-- As radio buttons or checkboxes -->
-     <div if="not isSelect" style=":field.getRadioStyle()">
+     <div if="not isSelect" style=":field.getWidgetStyle()">
 
       <!-- (Un)check all -->
       <div if="field.render == 'checkbox' and len(possibleValues) &gt; 1"
@@ -226,29 +228,37 @@ class Select(Field):
 
     # Widget for filtering object values on search results
     pxFilter = Px('''
-     <select var="name=field.name;
-                  filterId='%s_%s' % (mode.hook, name);
-                  charsWidth=field.getWidthInChars(True)"
-        id=":filterId" name=":filterId" class="discreet"
-          onchange=":'askBunchFiltered(%s,%s)' % (q(mode.hook), q(name))">
-      <option for="val, text in field.getPossibleValues(tool, \
-                withTranslations=True, withBlankValue='forced', \
-                blankLabel='everything', className=className)"
-              selected=":mode.inFilter(name, val)" value=":val"
-              title=":text">:Px.truncateValue(text, charsWidth)</option>
-     </select>''')
+     <div class="dropdownMenu fdrop"
+          var="name=field.name;
+               charsWidth=field.getWidthInChars(True)"
+          onmouseover="toggleDropdown(this)"
+          onmouseout="toggleDropdown(this,'none')">
+        <img src=":svg('fdrop')"/>
+        <span if="name in mode.filters">›</span>
+
+       <!-- The menu -->
+       <div class="dropdown fdown" style=":field.getWidgetStyle('f')">
+        <div for="val, text in field.getPossibleValues(tool,
+                   withTranslations=True, withBlankValue='forced',
+                   blankLabel='everything', className=className)">
+         <x if="mode.inFilter(name, val)">→ </x>
+         <a onClick=":'askBunchFiltered(%s,%s,%s)' % (q(mode.hook), q(name),
+              q(val))" title=":text">::Px.truncateValue(text, charsWidth)</a>
+        </div>
+       </div>
+     </div>''')
 
     def __init__(self, validator=None, multiplicity=(0,1), default=None,
-      defaultOnEdit=None, show=True, page='main', group=None, layouts=None,
-      move=0, indexed=False, mustIndex=True, indexValue=None,
+      defaultOnEdit=None, show=True, renderable=None, page='main', group=None,
+      layouts=None, move=0, indexed=False, mustIndex=True, indexValue=None,
       emptyIndexValue='-', searchable=False, filterField=None,
       readPermission='read', writePermission='write', width=None, height=None,
       maxChars=None, colspan=1, master=None, masterValue=None, focus=False,
       historized=False, mapping=None, generateLabel=None, label=None,
-      sdefault='', scolspan=1, swidth=None, sheight=None, persist=True,
-      inlineEdit=False, view=None, cell=None, edit=None, xml=None,
-      translations=None, noValueLabel='choose_a_value', render='select',
-      svalidator=None):
+      sdefault='', scolspan=1, swidth=None, sheight=None, fwidth='7em',
+      fheight=None, persist=True, inlineEdit=False, view=None, cell=None,
+      buttons=None, edit=None, xml=None, translations=None,
+      noValueLabel='choose_a_value', render='select', svalidator=None):
         # When choosing a value in a select widget, the entry representing no
         # value is translated according the label defined in attribute
         # "noValueLabel". The default one is something like "[ choose ]", but if
@@ -258,28 +268,28 @@ class Select(Field):
         # A Select field, is, by default, rendered as a HTML select widget, but
         # there are alternate render modes. Here are all the possible render
         # modes.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  render    | The field is rendered as...
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # "select"   | an HTML select widget, rendered as a dropdown list if
         #            | max multiplicity is 1 or as a selection box if several
         #            | values can be chosen.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # "radio"    | radio buttons. This mode is valid for fields with a max
         #            | multiplicity being 1.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # "checkbox" | checkboxes. This mode is valid for fields with a max
         #            | multiplicity being higher than 1.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         self.render = render
         # Call the base constructor
         Field.__init__(self, validator, multiplicity, default, defaultOnEdit,
-          show, page, group, layouts, move, indexed, mustIndex, indexValue,
-          emptyIndexValue, searchable, filterField, readPermission,
+          show, renderable, page, group, layouts, move, indexed, mustIndex,
+          indexValue, emptyIndexValue, searchable, filterField, readPermission,
           writePermission, width, height, maxChars, colspan, master,
           masterValue, focus, historized, mapping, generateLabel, label,
           sdefault, scolspan, swidth, sheight, persist, inlineEdit, view, cell,
-          edit, xml, translations)
+          buttons, edit, xml, translations)
         # Default value on the *s*earch form. It must be a list of values.
         self.sdefault = sdefault or []
         # Validator to use on the *s*earch form. This has sense only if you use
@@ -295,6 +305,9 @@ class Select(Field):
             self.filterPx = 'pxFilter'
         self.swidth = self.swidth or self.width
         self.sheight = self.sheight or self.height
+        # The *f*ilter width and height
+        self.fwidth = fwidth
+        self.fheight = fheight
         self.checkParameters()
 
     def checkParameters(self):
@@ -472,8 +485,8 @@ class Select(Field):
 
     def getStorableValue(self, o, value, single=False):
         '''Get a multivalued value when appropriate'''
-        if value and not single and self.isMultiValued() and \
-           (type(value) not in utils.sequenceTypes):
+        if value and self.isMultiValued() and \
+           type(value) not in utils.sequenceTypes:
             value = [value]
         return value
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
