@@ -208,17 +208,17 @@ class String(Multilingual, Field):
         return True
 
     def __init__(self, validator=None, multiplicity=(0,1), default=None,
-      defaultOnEdit=None, show=True, page='main', group=None, layouts=None,
-      move=0, indexed=False, mustIndex=True, indexValue=None,
+      defaultOnEdit=None, show=True, renderable=None, page='main', group=None,
+      layouts=None, move=0, indexed=False, mustIndex=True, indexValue=None,
       emptyIndexValue='-', searchable=False, filterField=None,
       readPermission='read', writePermission='write', width=None, height=None,
       maxChars=None, colspan=1, master=None, masterValue=None, focus=False,
       historized=False, mapping=None, generateLabel=None, label=None,
-      sdefault='', scolspan=1, swidth=None, sheight=None, persist=True,
-      transform='none', placeholder=None, languages=('en',),
+      sdefault='', scolspan=1, swidth=None, fwidth=10, sheight=None,
+      persist=True, transform='none', placeholder=None, languages=('en',),
       languagesLayouts=None, viewSingle=False, inlineEdit=False, view=None,
-      cell=None, edit=None, xml=None, translations=None, readonly=False,
-      stripped=True):
+      cell=None, buttons=None, edit=None, xml=None, translations=None,
+      readonly=False, stripped=True):
         # Does this field store an URL ?
         self.isUrl = validator == String.URL
         # "placeholder", similar to the HTML attribute of the same name, allows
@@ -243,12 +243,12 @@ class String(Multilingual, Field):
         # Call the base constructors
         Multilingual.__init__(self, languages, languagesLayouts, viewSingle)
         Field.__init__(self, validator, multiplicity, default, defaultOnEdit,
-          show, page, group, layouts, move, indexed, mustIndex, indexValue,
-          emptyIndexValue, searchable, filterField, readPermission,
+          show, renderable, page, group, layouts, move, indexed, mustIndex,
+          indexValue, emptyIndexValue, searchable, filterField, readPermission,
           writePermission, width, height, maxChars, colspan, master,
           masterValue, focus, historized, mapping, generateLabel, label,
           sdefault, scolspan, swidth, sheight, persist, inlineEdit, view, cell,
-          edit, xml, translations)
+          buttons, edit, xml, translations)
         # Default width, height and maxChars
         if width is None:
             self.width  = 30
@@ -260,10 +260,13 @@ class String(Multilingual, Field):
             self.filterPx = 'pxFilterText'
         self.swidth = self.swidth or self.width
         self.sheight = self.sheight or self.height
+        # The *f*ilter width
+        self.fwidth = fwidth
 
-    def isRenderable(self, layout):
-        '''A value being an URL is potentially renderable everywhere'''
-        return self.isUrl or (layout != 'buttons')
+    def isRenderableOn(self, layout):
+        '''A value being an URL can be rendered everywhere'''
+        if self.isUrl: return True
+        return super().isRenderableOn(layout)
 
     def isSortable(self, inRefs=False):
         '''Can this field be sortable ?'''
@@ -279,25 +282,39 @@ class String(Multilingual, Field):
                                        language)
 
     def validateUniValue(self, o, value): return
-        
-    def getSearchValue(self, req, value=None):
-        '''Potentially apply a transform to search value in p_req'''
-        r = Field.getSearchValue(self, req, value=value)
-        if r and self.transform:
-            r = self.applyTransform(r)
+
+    @classmethod
+    def getRange(class_, value):
+        '''If p_value ends with a star, returns a range. Else, it returns
+           p_value unchanged.'''
+        # Leave the value untouched if already correct
+        if isinstance(value, in_) or not value.endswith('*'): return value
+        # Build and return a range
+        prefix = value[:-1]
+        return in_(prefix, prefix + 'z')
+
+    @classmethod
+    def computeSearchValue(class_, field, req, value=None):
+        '''Potentially apply a transform to search value in p_req and possibly
+           define an interval of search values.'''
+        r = Field.getSearchValue(field, req, value=value).strip()
+        if not r: return r
+        # Potentially apply a transform to the search value
+        if getattr(field, 'transform', None):
+            r = field.applyTransform(r)
         # Define a range if the search term ends with a *
-        if r and isinstance(r, str) and r.endswith('*'):
-            prefix = r[:-1]
-            r = in_(prefix, prefix+'Z')
-        return r
+        return class_.getRange(r)
+
+    def getSearchValue(self, req, value=None):
+        '''See called method's docstring'''
+        return String.computeSearchValue(self, req, value=value)
 
     def getSortValue(self, o):
-        '''Return the value of p_self on p_obj that must be used for sorting.
-           While the raw p_value may be the value to use in most cases, it is
-           not always true. For example, a string like "Gaëtan" could have
-           "gaetan" as sort value.'''
-        return Normalize.text(self.getValue(o) or '',
-                              keepDash=False, keepBlank=True)
+        '''Return p_self' value on p_o that must be used for sorting'''
+        # While the raw field value, as produced by method "getValue", may be
+        # the value to use in most cases, it is not always true. For example, a
+        # string like "Gaëtan" could have "gaetan" as sort value.
+        return Normalize.sortable(self.getValue(o) or '')
 
     def getPlaceholder(self, o):
         '''Returns a placeholder for the field if defined'''
