@@ -26,7 +26,7 @@ class Switch(Field):
        The selected sub-set depends on a master/slave relationship that must be
        established between a Switch field and some other master field.
     '''
-    view = edit = cell = Px('''
+    view = edit = cell = buttons = Px('''
      <x var="fieldset,fields=field.getChosenFields(o)" if="fields">
       <!-- Remember the chosen fieldset in this hidden input field -->
       <input if="layout == 'edit'" type="hidden" name=":field.name"
@@ -38,12 +38,12 @@ class Switch(Field):
 
     search = ''
 
-    def __init__(self, fields, validator=None, show=True, page='main',
-      group=None, layouts=None, move=0, readPermission='read',
+    def __init__(self, fields, validator=None, show=True, renderable=None,
+      page='main', group=None, layouts=None, move=0, readPermission='read',
       writePermission='write', width=None, height=None, maxChars=None,
       colspan=1, master=None, masterValue=None, focus=False, mapping=None,
       generateLabel=None, label=None, scolspan=1, swidth=None, sheight=None,
-      inlineEdit=False, view=None, cell=None, edit=None, xml=None,
+      inlineEdit=False, view=None, cell=None, buttons=None, edit=None, xml=None,
       translations=None):
         # p_fields must be a tuple of fieldsets of the form
         #                        ~((s_name, fields),)~
@@ -51,11 +51,12 @@ class Switch(Field):
         #                         ~((s_name, Field),)~
         self.fields = fields
         # Call the base Field constructor
-        Field.__init__(self, validator, (0,1), None, None, show, page, group,
-          layouts, move, False, True, None, None, False, None, readPermission,
-          writePermission, width, height, None, colspan, master, masterValue,
-          focus, False, mapping, generateLabel, label, False, scolspan, swidth,
-          sheight, True, inlineEdit, view, cell, edit, xml, translations)
+        Field.__init__(self, validator, (0,1), None, None, show, renderable,
+          page, group, layouts, move, False, True, None, None, False, None,
+          readPermission, writePermission, width, height, None, colspan, master,
+          masterValue, focus, False, mapping, generateLabel, label, False,
+          scolspan, swidth, sheight, True, inlineEdit, view, cell, buttons,
+          edit, xml, translations)
 
     def init(self, class_, name):
         '''Switch-specific lazy initialisation'''
@@ -66,12 +67,10 @@ class Switch(Field):
 
     def getChosenFields(self, o, layout='view', fieldset=None):
         '''Returns, among self.fields, the chosen sub-set, as a "flat" list of
-           Field instances.
-
-           More precisely, r_ is a tuple (name, fields), "name" being the name
-           of the chosen fieldset and "fields" being the flat list of
-           corresponding fields.
-        '''
+           Field instances.'''
+        # More precisely, r_ is a tuple (name, fields), "name" being the name of
+        # the chosen fieldset and "fields" being the flat list of corresponding
+        # fields.
         req = o.req
         # Determine the name of the chosen fieldset. Get it from p_fieldset or
         # from the request.
