@@ -34,14 +34,15 @@ from appy.database.indexes.text import TextIndex
 from appy.model.fields.multilingual import Multilingual
 
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-INLINE_EDIT_MULTILINGUAL = 'Is is currently not possible to inline-edit ' \
-                           'multilingual Text fields.'
+IN_ED_MLG  = 'Is is currently not possible to inline-edit multilingual Text ' \
+             'fields.'
 
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Replacements:
     '''Used by class Text2Html below, this class is used to define replacements
        for characters found in structured text, in order to recreate formatting
        from structured text written with some conventions.'''
+
     # Default replacement conventions
     default = {
       # Replace double quotes by "guillemets" (angle quotes). The sub-dict has
@@ -56,17 +57,20 @@ class Replacements:
       '{': '<span style="background-color: yellow">', '}': '</span>',
       '&': '&amp;' # XML escaping
     }
+
     # Among "default", some keys represent opening and ending delimiters
     delimiters = {'[': ']', '<': '>', '{': '}'}
     endDelimiters = flipDict(delimiters)
+
     # Prefix (semi)colons, question and exclamation marks with a non-breakable
     # space, excepted if such a space is already present.
     nbspChars = (':', ';', '!', '?', '%')
     for c in nbspChars:
         default[c] = {' ': c, 0: ' %s' % c}
     whitespace = (' ', ' ')
+
     # Regular expression for replacing special markers (see "replacementFun"
-    # attribute below.
+    # attribute below).
     regex = re.compile('\*(.+?)\*')
 
     def __init__(self, replacements=None):
@@ -82,7 +86,7 @@ class Replacements:
         '''Return the replacement char for p_char. p_previous is the previous
            char encountered before p_char.'''
         r = self.replacements[char]
-        if not isinstance(r, basestring):
+        if not isinstance(r, str):
             # The replacement depends on "previous"
             if previous in r:
                 r = r[previous]
@@ -94,10 +98,10 @@ class Replacements:
         '''Apply replacements on p_text and return a tuple
                            (s_result, [s_toReopen])
         '''
-        # ----------------------------------------------------------------------
-        #  s_result  | Is the resulting unicode string, whose special chars have
-        #            | been replaced with their replacements.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        #  s_result  | Is the resulting string, whose special chars have been
+        #            | replaced with their replacements.
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # [toReopen] | Among special chars to replace, there are opening and
         #            | ending chars, used as delimiters for applying style, ie,
         #            | bold and italic. After having processed p_text, for some
@@ -112,10 +116,7 @@ class Replacements:
         #            | of p_text, he will possibly give us this list of tags
         #            | p_toReopen. Indeed, the real ending delimiters may be
         #            | present in the next p_text.
-        # ----------------------------------------------------------------------
-        # Ensure p_text is Unicode
-        isStr = isinstance(text, str)
-        if isStr: text = text.decode('utf-8')
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # Replace special markers when appropriate
         if fun: text = self.regex.sub(fun, text)
         # Build the result as a list
@@ -160,9 +161,7 @@ class Replacements:
                 add(self.getReplacementFor(delimiters[stack[i]], None))
                 i -= 1
         # Join the result to a string
-        r = u''.join(r)
-        # Encode the result when relevant
-        if isStr: r = r.encode('utf-8')
+        r = ''.join(r)
         return r, stack
 
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -170,6 +169,7 @@ class Text2Html:
     '''Converts a chunk of structured text into XHTML. Some conventions apply,
        ie, rows of text starting with a dash are converted to bulleted lists;
        rows starting with char "|" are considered to be table rows, etc.'''
+
     defaultReplacements = Replacements()
 
     def __init__(self, p='p', prefix='', replacements='default',
@@ -182,17 +182,17 @@ class Text2Html:
         # The result, will be a list that'll be joined at the end of the process
         self.res = None
         # If p_replacements is:
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #   "default"    | A default Replacements instance will be used, in
         #                | order to format text according to some conventions
         #                | (see class hereabove)
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #   None         | Text replacement is disabled
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #   an instance  | A Replacements instance containing a set of
         #                | formatting options that may de different from the
         #                | defaults.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         if replacements == 'default':
             self.replacements = Text2Html.defaultReplacements
         else:
@@ -237,8 +237,14 @@ class Text2Html:
                 self.add('</%s>' % tag, '')
                 self.add('</%s>' % self.p)
         else:
-            t = not start and '/' or ''
-            self.add('<%s%s>' % (t, tag))
+            suffix = '\n'
+            if start:
+                t = ''
+                if tag == self.p:
+                    suffix = ''
+            else:
+                t = '/'
+            self.add('<%s%s>' % (t, tag), suffix)
 
     def convertText(self, text):
         '''Converts, within p_text, some chars according to self.replacements
@@ -256,7 +262,7 @@ class Text2Html:
 
     def applyPreListClass(self):
         '''Apply the CSS class in self.preListClass to the last paragraph in
-           self.res.'''
+           p_self.res.'''
         r = self.res
         if not r: return
         i = len(r) - 1
@@ -290,13 +296,14 @@ class Text2Html:
                 # Is this the last bullet ?
                 isLast = self.isLast(rows, i, '- ')
                 if isLast:
-                    start= '<li class="%s">' % self.lastLiClass
-                    end = '</li>'
                     inList = False
+                    if self.lastLiClass:
+                        css = ' class="%s"' % self.lastLiClass
+                    else:
+                        css = ''
                 else:
-                    start = '<li>'
-                    end = '</li>'
-                self.add('%s%s%s' % (start, self.convertText(row[2:]), end))
+                    css = ''
+                self.add('<li%s>%s</li>' % (css, self.convertText(row[2:])))
                 # Add the end list tag when relevant
                 if isLast: self.addTag('ul', start=False)
             elif row.startswith('|'):
@@ -333,7 +340,7 @@ class Html2Text(Parser):
     '''Converts a chunk HTML into structured text'''
 
     # This parser uses conventions for default replacements only
-    # ~~~
+    # ~
     # Conversion of HTML start tags
     startTags = {'b':'[', 'strong':'[', 'i':'<', 'em':'<', 'li':'- '}
 
@@ -367,15 +374,15 @@ class Icon:
     '''An icon from the toolbar when the Text field is used in "structured"
        mode.'''
 
-    def __init__(self, name, type,
-                 label=None, icon=None, data=None, shortcut=None):
+    def __init__(self, name, type, label=None, icon=None, data=None,
+                 shortcut=None):
         # A short, unique name for the icon
         self.name = name
         # The following type of icons exist. Depending on the type, p_data
         # carries a specific type of information.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # p_type      | p_data
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # "wrapper"   | the icon corresponds to a portion of text that will be
         #             | wrapped around a start and end char. p_data contains 2
         #             | chars: the start and end wrapper chars.
@@ -383,15 +390,15 @@ class Icon:
         #             | For example, icon "bold" is of type "wrapper", with data
         #             | being "[]". When applied to selected text "hello", it
         #             | becomes "[hello]".
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # "char"      | the icon corresponds to a char to insert into the field.
         #             | p_data is the char to insert.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # "action"    | the icon corresponds to some action that is not
         #             | necessarily related to the field content. In that case,
         #             | p_data may be None or its sematincs may be specific to
         #             | the action.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # "sentences" | a clic on the icon will display a menu containing
         #             | predefined sentences. Selecting one of them will inject
         #             | it in the target field, where the cursor is currently
@@ -399,7 +406,7 @@ class Icon:
         #             | method that must exist on the current object. This
         #             | method will be called without arg and must return a list
         #             | of sentences, each one being a string.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         self.type = type
         # The i18n label for the icon's tooltip. Should include the keyboard
         # shortcut when present. If None, defaults to "icon_<name>"
@@ -442,7 +449,7 @@ class Icon:
     def get(self, o):
         '''Returns the HTML chunk representing this icon'''
         shortcut = str(self.shortcut) if self.shortcut else ''
-        r = '<img class="icon" src="%s" title="%s" name="%s"' \
+        r = '<img class="iconTB" src="%s" title="%s" name="%s"' \
             ' onmouseover="switchIconBack(this, true)"' \
             ' onmouseout="switchIconBack(this, false)"' \
             ' data-type="%s" data-data="%s" data-shortcut="%s" ' \
@@ -468,6 +475,7 @@ Icon.all = [Icon('bold',      'wrapper', data='[]',       shortcut=66),
 class Text(Multilingual, Field):
     '''Field allowing to encode a text made of several paragraphs, implemented
        by a HTML textarea tag.'''
+
     # Make some classes available here
     Icon = Icon
     Replacements = Replacements
@@ -489,8 +497,9 @@ class Text(Multilingual, Field):
     defaultLanguagesLayouts = {
       'edit': 'horizontal', 'view': 'vertical', 'cell': 'vertical'}
 
-    # The name of the index class storing values of this field in the catalog
-    indexType = 'TextIndex'
+    # Alternative language layout
+    verticalLanguagesLayouts = {
+      'edit': 'vertical', 'view': 'vertical', 'cell': 'vertical'}
 
     # Unilingual view
     viewUni = cellUni = Px('''
@@ -513,8 +522,8 @@ class Text(Multilingual, Field):
       .toolbar { height: 24px; margin: 2px 0 }
       .sentenceContainer { position: relative; display: inline }
       .sentence { padding: 3px 0 }
-      .icon { padding: 3px; border-width: 1px; border: 1px transparent solid }
-      .iconSelected { background-color: #dbdbdb; border-color: #909090 }
+      .iconTB { padding: 3px; border-width: 1px; border: 1px transparent solid }
+      .iconTBSel { background-color: #dbdbdb; border-color: #909090 }
      ''',
 
      js='''
@@ -539,7 +548,7 @@ class Text(Multilingual, Field):
         target['icons'] = getIconsMapping(toolbar);
       }
       switchIconBack = function(icon, selected) {
-        icon.className = (selected)? 'icon iconSelected': 'icon';
+        icon.className = (selected)? 'iconTB iconTBSel': 'iconTB';
       }
       lengthenArea = function(area, percentage) {
         // Lengthen some text p_area by some p_percentage
@@ -612,11 +621,11 @@ class Text(Multilingual, Field):
                 fdir='row' if inToolbar else 'column'"
            style=":'float:%s;display:flex;flex-direction:%s' % (align, fdir)">
        <div>
-        <img id=":'%s_save' % tid" src=":svg('save')"
+        <img id=":'%s_save' % tid" src=":svg('saveS')"
              class=":'iconS %s' % ('clickable' if inToolbar else 'inlineIcon')"
              title=":_('object_save')"/></div>
        <div>
-        <img id=":'%s_cancel' % tid" src=":svg('cancel')"
+        <img id=":'%s_cancel' % tid" src=":svg('cancelS')"
              class=":'iconS %s' % ('clickable' if inToolbar else 'inlineIcon')"
              title=":_('object_cancel')"/></div>
       </div>
@@ -650,14 +659,15 @@ class Text(Multilingual, Field):
             value=":field.sdefault" name=":widgetName"/>''')
 
     def __init__(self, validator=None, multiplicity=(0,1), default=None,
-      defaultOnEdit=None, show=True, page='main', group=None, layouts=None,
-      move=0, indexed=False, mustIndex=True, indexValue=None, searchable=False,
-      filterField=None, readPermission='read', writePermission='write',
-      width=60, height=5, maxChars=None, colspan=1, master=None,
-      masterValue=None, focus=False, historized=False, mapping=None,
-      generateLabel=None, label=None, sdefault='', scolspan=1, swidth=20,
-      sheight=None, persist=True, inlineEdit=False, view=None, cell=None,
-      edit=None, xml=None, translations=None,
+      defaultOnEdit=None, show=True, renderable=None, page='main', group=None,
+      layouts=None, move=0, indexed=False, mustIndex=True, indexValue=None,
+      searchable=False, filterField=None, readPermission='read',
+      writePermission='write', width=60, height=5, maxChars=None, colspan=1,
+      master=None, masterValue=None, focus=False, historized=False,
+      mapping=None, generateLabel=None, label=None, sdefault='', scolspan=1,
+      swidth=20, fwidth=10, sheight=None, persist=True, inlineEdit=False,
+      view=None, cell=None, buttons=None, edit=None, xml=None,
+      translations=None, indexType='TextIndex',
       # Specific attributes
       placeholder=None, languages=('en',), languagesLayouts=None,
       viewSingle=False, structured=False, readonly=False):
@@ -674,14 +684,24 @@ class Text(Multilingual, Field):
         # the rendered textarea tag, on edit layouts, will have attribute
         # "readonly" set.
         self.readonly = readonly
-        # Call the base constructors
+        # The default index type for Text fields is "TextIndex": it is meant to
+        # split the field content into words one can search. If you prefer to
+        # have a simple index that will store a single string for sorting
+        # purposes, use index type "SortIndex". If you want sorting to be based
+        # on values not being normalized, use index "Index" instead of
+        # "SortIndex".
+        self.indexType = indexType
+        # Call base constructors
         Multilingual.__init__(self, languages, languagesLayouts, viewSingle)
         Field.__init__(self, validator, multiplicity, default, defaultOnEdit,
-          show, page, group, layouts, move, indexed, mustIndex, indexValue,
-          None, searchable, filterField, readPermission, writePermission, width,
-          height, maxChars, colspan, master, masterValue, focus, historized,
-          mapping, generateLabel, label, sdefault, scolspan, swidth, sheight,
-          persist, inlineEdit, view, cell, edit, xml, translations)
+          show, renderable, page, group, layouts, move, indexed, mustIndex,
+          indexValue, None, searchable, filterField, readPermission,
+          writePermission, width, height, maxChars, colspan, master,
+          masterValue, focus, historized, mapping, generateLabel, label,
+          sdefault, scolspan, swidth, sheight, persist, inlineEdit, view, cell,
+          buttons, edit, xml, translations)
+        # The *f*ilter width
+        self.fwidth = fwidth
         # Specify a filter PX if the field content is indexed
         if self.indexed: self.filterPx = 'pxFilterText'
         self.checkParameters()
@@ -691,7 +711,7 @@ class Text(Multilingual, Field):
         # Currently, it is not possible to inline edit multilingual Texts
         langs = self.languages
         if (callable(langs) or (len(langs) > 1)) and self.inlineEdit:
-            raise Exception(INLINE_EDIT_MULTILINGUAL)
+            raise Exception(IN_ED_MLG)
 
     def getTextareaStyle(self):
         '''Get the content of textarea's "style" attribute'''
@@ -772,8 +792,8 @@ class Text(Multilingual, Field):
     def validateUniValue(self, o, value): return
 
     def getFilterValue(self, value):
-        '''Ensure the value is suffixed with a star'''
-        return value if value.endswith('*') else '%s*' % value
+        '''Normalizes p_value and ensures it ends with a star'''
+        return '%s*' % sutils.Normalize.text(value).strip()
 
     @classmethod
     def computeSearchValue(class_, field, req, value=None):
@@ -786,7 +806,7 @@ class Text(Multilingual, Field):
             r = r[:-1]
             if not r: return r
             r = r.split()[-1]
-            r = in_(r, r+'z') # 'z' is higher than 'Z'
+            r = in_(r, r + 'z') # 'z' is higher than 'Z'
         else:
             # Perform a search based on keywords. No need to normalize it: it
             # will be done by the index afterwards.
@@ -803,14 +823,14 @@ class Text(Multilingual, Field):
         '''See called method's docstring'''
         return Text.computeSearchValue(self, req, value=value)
 
-    def getListHeader(self, ctx):
+    def getListHeader(self, c):
         '''If this field is structured, when used as an inner field, the
            toolbar must be rendered only once, within the container field's
            header row corresponding to this field.'''
         # Inject the toolbar when appropriate
-        if (ctx.layout == 'edit') and self.showToolbar(ignoreInner=True):
-            bar = self.pxToolbar(ctx)
+        if c.layout == 'edit' and self.showToolbar(ignoreInner=True):
+            bar = self.pxToolbar(c)
         else:
             bar = ''
-        return '%s%s' % (Field.getListHeader(self, ctx), bar.encode('utf-8'))
+        return '%s%s' % (Field.getListHeader(self, c), bar)
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
