@@ -30,11 +30,15 @@ from appy.model.fields.group import Group as FieldGroup
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Group(Base):
     '''Base class representing a group'''
-    workflows.Owner
+
+    workflow = workflows.Owner
 
     m = {'group': FieldGroup('main', style='grid', hasLabel=False),
          'width': 25, 'indexed': True, 'layouts': Layouts.g, 'label': 'Group'}
 
+    # CSS class to apply to every group title in a list of groups
+    styles = {'title': 'titlet'}
+
     @staticmethod
     def update(class_):
         title = class_.fields['title']
