@@ -22,7 +22,6 @@ import collections
 import persistent
 from persistent.mapping import PersistentMapping
 
-
 from appy.px import Px
 from appy.model import Model
 
@@ -36,12 +35,16 @@ from appy.model.group import Group
 from appy.ui.layout import Layouts
 from appy.model.fields import Field
 from appy.model.workflow import Role
+from appy.model.fields.ref import Ref
+from appy.model.fields.pod import Pod
 from appy.model.searches import Search
 from appy.utils import string as sutils
+from appy.model.fields.text import Text
 from appy.model.document import Document
 from appy.model.utils import Object as O
 from appy.model.fields.phase import Phase
 from appy.model.fields.string import String
+from appy.model.fields.select import Select
 from appy.model.translation import Translation
 from appy.model.workflow.history import History
 from appy.model.searches.gridder import Gridder
@@ -57,6 +60,7 @@ DUP_S_NAME  = '%s: duplicate search "%s".'
 S_F_HOM     = '%s: "%s" is both used as search and field name, this is not ' \
               'allowed.'
 CUS_ID_KO   = 'The custom ID produced by method "generaeId" must be a string.'
+S_FIELD_KO  = 'Field "%s", mentioned as search field on class "%s", not found.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Class(Meta):
@@ -80,24 +84,25 @@ class Class(Meta):
     def __init__(self, class_, isBase=False, appOnly=False):
         Meta.__init__(self, class_, appOnly)
         # p_class_ may be of 3 types:
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # "base" | A class from the base Appy model (from appy.model) that has
         #        | not been overridden by an app's class.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # "app"  | An original class from the app, that does not exist in the
         #        | base model.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # "over" | A class defined in the app but that overrides a class defined
         #        | in the base Appy model.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         self.type = 'base' if isBase else 'app'
+
         # Create the concrete class from which instances will be created.
         # Indeed, In order to bring all Appy functionalities to p_class_, we
         # need to create a new class inheriting from:
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #   p_class_  | The class defined by the developer or being part of the
         #             | base Appy model.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #    Base*    | Class appy.model.base.Base, the base class for any Appy
         #             | class, or one if its sub-classes, depending on p_class_
         #             | name. For example, if the user defines a class named
@@ -111,10 +116,10 @@ class Class(Meta):
         #             | Note that for not-overridden classes from appy.model (=
         #             | classes of type "base"), there is no base class: the
         #             | class is already a base class.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  Persistent | Class persistent.Persistent allows to store instances in
         #             | the database (ZODB).
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # Creating the concrete class is not necessary when p_self.appOnly is
         # True.
         if not appOnly:
@@ -188,7 +193,7 @@ class Class(Meta):
             # To get a field value is done via method Field::getValue
             getter = lambda self, field=field: field.getValue(self)    
             # To set a value is done via method Field::store
-            if field.type == 'Ref':
+            if isinstance(field, Ref):
                 setter = lambda self, v, field=field: \
                                 field.store(self, v, secure=False)
             else:
@@ -230,6 +235,7 @@ class Class(Meta):
         Err = Model.Error
         # Find field in p_class_ and base classes
         for aClass in self.getFieldClasses(class_):
+            tfield = sfield = None
             for name, field in aClass.__dict__.items():
                 if name.startswith('__'): continue
                 if isinstance(field, Field):
@@ -239,20 +245,28 @@ class Class(Meta):
                        Meta.unallowedName(name):
                         raise Err(NAME_KO % (name, self.name))
                     # A field was found.
-                    # ~~~
-                    # Special fields "title" and "record" must be duplicated.
-                    # Else, all app classes will get the modifications that the
-                    # developer can apply to it via "update" methods.
-                    if name == 'title':
-                        field = String(**Base.titleAttributes)
+                    # ~
+                    # Special fields "title", "record", "searchable" and "state"
+                    # must be duplicated. Else, all app classes will get the
+                    # modifications that the developer can apply to it via
+                    # "update" methods.
+                    if name == 'searchable':
+                        field = sfield = Text(**Base.searchableAttributes)
+                    elif name == 'title':
+                        field = tfield = String(**Base.titleAttributes)
                     elif name == 'record':
                         field = Computed(**Base.recordAttributes)
                         field.page = field.page.clone()
+                    elif name == 'state':
+                        field = Select(**Base.stateAttributes)
                     # Late-initialize it
                     field.init(self, name)
                     r[name] = field
                     if field.page:
                         self.readPage(field.page)
+            # Reify the link betweeen fields "title" and "searchable" on their
+            # duplicates for this v_aClass.
+            if tfield and sfield: tfield.filterField = sfield
         self.fields = r
 
     def setField(self, name, field):
@@ -308,7 +322,7 @@ class Class(Meta):
         return r
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    # Instance-creation method
+    #                         Instance-creation method
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     def new(self, iid, id, login, initialComment=None, initialState=None):
@@ -327,8 +341,10 @@ class Class(Meta):
         # first values in it.
         o.values = PersistentMapping() # ~{s_fieldName: fieldValue}~
         # Initialise object's history by triggering the _init_ transition
-        tr = self.workflow.initialTransition
-        state = initialState or tr.states[1]
+        workflow = self.workflow
+        tr = workflow.initialTransition
+        # Cheat with the initial state if p_initialState is there
+        state = workflow.states[initialState] if initialState else tr.states[1]
         tr.trigger(o, initialComment, doSay=False, forceTarget=state)
         return o
 
@@ -354,18 +370,12 @@ class Class(Meta):
     # class-dependent elements.
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-    def getListCss(self, layout='view'):
-        '''Returns the CSS class(es) to apply when displaying a list (=a table)
-           containing instances of this class.'''
-        css = getattr(self.python, 'listCss', None)
-        return css if css else 'list'
-
-    def getListTitle(self, o):
+    def getListTitle(self, o, nav):
         '''Gets p_o's title as it must appear within lists of objects'''
         if hasattr(self.python, 'listTitle'):
             r = self.python.listTitle(o, nav)
         else:
-            r = o.getShownValue('title')
+            r = o.getShownValue()
         return r
 
     def getPageLayouts(self, o):
@@ -392,16 +402,70 @@ class Class(Meta):
         # This value can be a single value or a tuple/list of values
         return layout == r if isinstance(r, str) else layout in r
 
-    def getCssFor(self, o, elem):
-        '''Gets the name of the CSS class to use for styling some p_elem on
+    # Default CSS classes by UI zone
+    defaultCssClasses = {
+      'list' : 'list', # [class level] The table tag containing a list of this
+                       # class' objects (tied objects from a Ref field, results
+                       # from a search, etc.)
+      'title': None,   # [object level] The "title" "a" tag (or span if
+                       # unclickable) referring to an instance of this class,
+                       # shown within lists of objects.
+      'pside': None  , # [class level] *p*ortlet side: a zone within the
+                       # portlet, on the right side, aimed at becoming a
+                       # colored strip if desired.
+      'tbot' : None  , # [class level] *t*itle *bot*tom: a zone, on search
+                       # results, below the search title, aimed at becoming a
+                       # colored sub-strip, when desired.
+      'sep'  : None  , # [class level] Still on search results, within the row
+                       # located under the search title, this zope represents
+                       # the *sep*arator between controls and the count of
+                       # objects.
+      'sub'  : None  , # [object level] The zone displaying an object's actions
+                       # on a list of objects, under or besides its title.
+    }
+
+    def getCssFor(self, o, zone, add=None):
+        '''Gets the name of the CSS class to use for styling some p_zone on
            instance p_o. If this class does not define a dict or method named
-           "styles", the defaut CSS class to use will be named p_elem.'''
-        styles = getattr(self.python, 'styles', None)
-        if not styles: return elem
-        elif callable(styles): # A method
-            return styles(o, elem) or elem
-        else: # A dict or a appy.Object instance
-            return styles.get(elem) or elem
+           "styles", a defaut CSS class may be retrieved from
+           p_self.defaultCssClasses.'''
+        r = getattr(self.python, 'styles', None)
+        if r:
+            # v_r can be method, a dict or an Object
+            r = r(o, zone) if callable(r) else r.get(zone)
+        # Get the default CSS for the zone, when relevant
+        r = r or Class.defaultCssClasses.get(zone)
+        # There may be some CSS class to p_add
+        if add:
+            r = add if not r else ('%s %s' % (r, add))
+        return r
+
+    def getSubCss(self, o, context, index=None, inPickList=False):
+        '''Gets the CSS class to apply to the "sub" layout (=object actions on
+           a list of objects), in the p_context of a field or search, under or
+           besides p_o's title. p_index is o's p_index within the current list
+           of objects.'''
+        # Within a Ref with render="menus", a specific CSS class must be used
+        if isinstance(context, Ref) and index is None and not inPickList:
+            # Currently, no custom CSS is possible for menus
+            return 'menuActions'
+        # For any other case: there is a mandatory standard CSS class + a
+        # potential custom class.
+        display = context.actionsDisplay
+        if display == 'block':
+            r = 'object'
+        elif display == 'right':
+            r = 'aligned'
+        else:
+            r = 'inline'
+        # Complete this standard CSS class with a custom one, if any
+        return self.getCssFor(o, 'sub', add='%sActions' % r)
+
+    def getIconsOnly(self):
+        '''For base actions (edit, delete, etc), must icons only appear, or
+           complete buttons containing an icon + a label ?'''
+        # By default, base actions are rendered as simple icons
+        return getattr(self.python, 'iconsOnly', True)
 
     def getCreateVia(self, tool):
         '''How to create instances of this class ? The answer can be: via a
@@ -411,14 +475,14 @@ class Class(Meta):
         r = getattr(self.python, 'createVia', 'form')
         return r if not callable(r) else r(tool)
 
-    def getCreateLink(self, tool, create, formName,
-                      sourceField=None, insert=None):
+    def getCreateLink(self, tool, createVia, formName, sourceField=None,
+                      insert=None):
         '''When instances of p_self must be created from a template (from the
            UI), this method returns the link allowing to search such templates
            from a popup.'''
         r = '%s/Search/results?className=%s&search=fromSearch&popup=True&' \
             'fromClass=%s&formName=%s' % \
-            (tool.url, create.container.name, self.name, formName)
+            (tool.url, createVia.container.name, self.name, formName)
         # When object creation occurs via a Ref field, its coordinates are given
         # in p_sourceField as a string: "sourceObjectId:fieldName".
         if sourceField: r += '&sourceField=%s' % sourceField
@@ -477,14 +541,17 @@ class Class(Meta):
         return getattr(self.concrete, 'indexable', True)
 
     def getListPods(self, tool):
-        '''Finds, among p_self' fields, those being Pod fields showable on
+        '''Finds, among p_self's fields, those being Pod fields showable on
            search results.'''
-        r = []
+        # r is a dict {p_field: [visibleTemplate]}
+        r = {}
         for field in self.fields.values():
-            if field.type != 'Pod' or not field.multiObjects: continue
-            show = field.show(tool) if callable(field.show) else field.show
-            if show == 'query':
-                r.append(field)
+            if isinstance(field, Pod) and field.multiObjects:
+                show = field.show(tool) if callable(field.show) else field.show
+                if show == 'query':
+                    visible = field.getVisibleTemplates(tool)
+                    if visible:
+                        r[field] = visible
         return r
 
     def getDynamicSearches(self, tool):
@@ -571,7 +638,13 @@ class Class(Meta):
             cols = getattr(self.python, 'numberOfSearchColumns', 1)
         # Get fields from names
         if names:
-            fields = [self.fields.get(name) for name in names]
+            fields = []
+            for name in names:
+                field = self.fields.get(name)
+                if field is None:
+                    tool.log(S_FIELD_KO % (name, self.name), type='warning')
+                else:
+                    fields.append(field)
         else:
             # If fields are not explicitly listed, take all indexed fields,
             # "title" excepted, "searchable" and "title" being mostly redundant.
@@ -583,7 +656,8 @@ class Class(Meta):
             field.getCss(None, 'edit', css)
             field.getJs(None, 'edit', js, config)
         # Returns a gridder allowing to render the search field
-        gridder = Gridder(cols=cols)
+        gridder = Gridder(cols=cols, colsAuto=True,
+                         justifyContent='left', gap='1em 3em')
         return fields, css, js, gridder
 
     def getResultModes(self):
@@ -596,12 +670,6 @@ class Class(Meta):
         method = getattr(self.python, 'getResultsTop', None)
         return method(tool, search, mode) if method else None
 
-    def getResultCss(self, layout='view'):
-        '''Gets the CSS class(es) to apply when displaying a table containing
-           p_self's instances.'''
-        r = getattr(self.python, 'listCss', None)
-        return r if r else 'list'
-
     def mayEditEventComment(self, o, event, user, isManager):
         '''May p_user edit the comment on this p_event, coming from p_o's
            history ?'''
@@ -623,10 +691,10 @@ class Class(Meta):
         '''Must the mechanism for toggling sub-titles be enabled on p_self ?'''
         # Sub-titles may appear, within lists or grids of p_self's instances,
         # under the title of every instance.
-        # ~~~
+        # ~
         # p_from may hold a Ref field. In that case, toggling sub-titles occurs
         # in the context of a list of objects being tied via this field.
-        # ~~~
+        # ~
         # Check first if the mechanism is explicitly disabled
         if from_ and from_.toggleSubTitles is not None:
             toggle = from_.toggleSubTitles
@@ -668,24 +736,32 @@ class Class(Meta):
         return r
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    # PXs
+    #                                  PXs
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     # Icon for creating instances of this class from a template
     pxAddFrom = Px('''
+     <!-- The image outside the button, if "iconOut" -->
+     <img if="iconOut" src=":field.getIconUrl(url)"
+          class=":'clickable %s' % field.iconCss"
+          onclick="this.nextSibling.click()"/>
+
+     <!-- The button in itself -->
      <a target="appyIFrame" id=":addFormName + '_from'"
-        href=":class_.getCreateLink(tool, create, addFormName, sourceField)">
+        href=":class_.getCreateLink(tool, createVia, addFormName, sourceField)">
       <input var="css='Small' if fromRef else 'Portlet';
-                  label=_('object_add_from')"
-         type="button" value=":label" class=":'button%s button' % css"
-         onclick="openPopup('iframePopup')" style=":svg('add', bg=True)"/>
+                  cssOut='noIcon ' if iconOut else '';
+                  label=_(field.addFromLabel if field else 'object_add_from')"
+         type="button" value=":label" onclick="openPopup('iframePopup')"
+         class=":'%sbutton%s button' % (cssOut, css)"
+         style=":'' if iconOut else svg('add', bg=True)"/>
      </a>''')
 
     # Style characteristics to apply to the "add" button, depending on its type
     addStyles = {
       # The "add" button as it appears in the portlet
       'portlet': O(css='buttonPortlet button', bgIcon='portletAdd',
-                   bgSize='24px 24px'),
+                   bgSize=True),
       # The "add" button as it may appear elsewhere (ie, on search results)
       'small'  : O(css='buttonSmall button', bgIcon='add', bgSize='18px 18px'),
       # When using pxAdd from a custom PX, let this latter manage button style
@@ -694,14 +770,14 @@ class Class(Meta):
 
     # Form for creating instances of this class from the portlet or a search
     pxAdd = Px('''
-     <form var="create=class_.getCreateVia(tool); className=class_.name"
-           if="create" class="addForm" name=":'%s_add' % className"
+     <form var="createVia=class_.getCreateVia(tool); className=class_.name"
+           if="createVia" class="addForm" name=":'%s_add' % className"
            var2="styles=class_.addStyles[buttonType];
                  target=ui.LinkTarget(class_.python, popup=viaPopup);
                  text=_(label or 'object_add')"
            action=":'%s/new' % tool.url" target=":target.target">
       <input type="hidden" name="className" value=":className"/>
-      <input type="hidden" name="template" value=""/>
+      <input type="hidden" name="template_" value=""/>
       <input type="hidden" name="insert" value=""/>
       <input type="hidden" name="nav" value=":nav"/>
       <input type="hidden" name="popup"
@@ -713,10 +789,10 @@ class Class(Meta):
         onclick=":target.getOnClick('searchResults')"/>
 
       <!-- Create from a pre-filled form when relevant -->
-      <div if="create != 'form'" class="addFrom"
-         var2="fromRef=False; sourceField=None;
+      <div if="createVia != 'form'" class="addFrom"
+         var2="fromRef=False; sourceField=None; iconOut=False;
                addFormName='%s_add' % className">:class_.pxAddFrom</div>
      </form>''',
 
-     css='''.addFrom { margin-top: 5px }''')
+     css='''.addFrom input { margin-left:0 }''')
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
