@@ -202,6 +202,17 @@ class Workflow(Meta):
         return [(state.name, o.translate(state.labelId)) \
                 for state in self.states.values() if not state.isIsolated()]
 
+    def changeInitialState(self, state):
+        ''''Sets this p_state as the new initial state for this workflow'''
+        # Replace the old one...
+        current = self.initialState
+        current.initial = False
+        # ... by the new one
+        state.initial = True
+        self.initialState = state
+        # Update the initial transition, too
+        self.initialTransition.states = [state, state]
+
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     # PXs
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -210,7 +221,8 @@ class Workflow(Meta):
     pxTransitions = Px('''
      <x var="transitions=transitions|workflow.getTransitions(o)"
         if="transitions" var2="formId='trigger_%d' % o.iid">
-      <form id=":formId" method="post" class="inline" data-baseurl=":o.url">
+      <form id=":formId" method="post" style="display:none"
+            data-baseurl=":o.url">
        <input type="hidden" name="popup" value=":popup"/>
        <input type="hidden" name="nav" value=":req.nav or 'no'"/>
        <input type="hidden" name="page" value=":req.page or 'main'"/>
@@ -220,7 +232,7 @@ class Workflow(Meta):
       </form>
 
       <!-- Render a transition or a group of transitions -->
-      <div for="transition in transitions"
-           var2="trGroup=transition">:transition.px</div>
+      <div for="transition in transitions" class="flex1"
+         var2="trGroup=transition">:transition.px</div>
      </x>''')
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
