@@ -35,11 +35,13 @@ from appy.model.fields.string import String
 from appy.model.fields.boolean import Boolean
 from appy.model.fields.integer import Integer
 from appy.model.fields.ref import Ref, autoref
+from appy.model.fields.computed import Computed
 from appy.model.fields.phase import Page as FPage
 from appy.model.workflow.standard import Anonymous
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 EXPR_ERR = 'Page "%s" (%s): error while evaluating page expression "%s" (%s).'
+EDITED   = 'Page %d %s within %s.'
 DELETED  = 'Web page %s deleted.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -57,6 +59,7 @@ class Page(Base):
         if tool.user.isAnon(): return ('title',)
         return ('title', 'expression', 'next', 'selectable*100px|',
                 'podable*80px|', 'viewCss*100px|')
+
     listColumns = getListColumns
 
     # An alternate, shorter list of columns
@@ -93,6 +96,12 @@ class Page(Base):
                          % (esc(label), o.url, esc(o.getShownValue())))
         return '\n'.join(r)
 
+    def getTitle(self, maxChars=None):
+        '''Returns p_self's title, shortened'''
+        r = self.getShownValue()
+        maxChars = maxChars or self.config.ui.pageChars
+        return Px.truncateValue(r, width=maxChars)
+
     # A warning: image upload is impossible while the page is temp
     warning = Info(show=lambda o: 'edit' if o.isTemp() else None,
                    focus=True, layouts=Info.Layouts.n, **pa)
@@ -260,6 +269,7 @@ class Page(Base):
       composite=True, back=Ref(attribute='parent', show=False, **pa),
       showHeaders=True, actionsDisplay='inline', show=showSubPages,
       numbered=True, checkboxes=lambda o: o.allows('write'),
+      titleMode=lambda o: 'plink' if o.container.inPublic(True) else 'link',
       toggleSubTitles=False, **pa)
 
     def getParents(self, includeMyself=True):
@@ -315,6 +325,21 @@ class Page(Base):
                 page.getMergedContent(r, level=level+1)
         return r
 
+    # Link allowing, when a sub-page is shown in the "public" PX, to go back to
+    # the parent page.
+
+    def showToParent(self):
+        '''Show link "to parent" on for a sub-page being rendered via the
+           "public" PX.'''
+        return 'view' if self.parent and self.inPublic() else None
+
+    def getToParent(self):
+        '''Get a link allowing to navigate to p_selg's parent page'''
+        text = self.getLabel('toParent')
+        return '<a href="%s">↥ %s</a>' % (self.parent.getPublicUrl(), text)
+
+    toParent = Computed(show=showToParent, method=getToParent, layouts='f')
+
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     #                                Editors
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -353,9 +378,13 @@ class Page(Base):
     #  Main methods
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
 
+    def getPublicUrl(self):
+        '''Return the URL for this page, when shown via the tool's public PX'''
+        return '%s/public?rp=%s' % (self.tool.url, self.iid)
+
     def inPortlet(self, selected, level=0):
         '''Returns a chunk of XHTML code for representing this page in the
-           portlet.'''
+           portlet, in a root class zone.'''
         # Is this page currently selected ? A selected page is the page being
         # currently consulted on the site or any of its parent pages.
         isSelected = self in selected
@@ -377,10 +406,27 @@ class Page(Base):
         container = self.container
         return not container or container.class_.name == 'Tool'
 
+    def inPublic(self, orParent=False):
+        '''Is p_self (or one of its parents if p_orParent is True) currently
+           shown on the "public" page ?'''
+        # Get the ID of the page being currently shown on the "public" page
+        rp = self.req.rp
+        if not rp: return
+        # Does v_rp correspond to p_self ?
+        r = rp == str(self.iid)
+        if r: return r
+        # [only if p_orParent is True] Does p_rp correspond to some ancestor of
+        #                              p_self ?
+        if orParent and self.parent:
+            r = self.parent.inPublic(orParent=True)
+        return r
+
     def onEdit(self, created):
         '''Link the page among root pages if created from the portlet'''
         if created and not self.initiator:
             self.tool.link('pages', self)
+        verb = 'created' if created else 'edited'
+        self.log(EDITED % (self.id, verb, self.container.iid))
 
     def onDelete(self):
         '''Log the page deletion'''
@@ -392,13 +438,22 @@ class Page(Base):
 
     # This selector allows to choose one root page among published tool.pages
     pxSelector = Px('''
-      <select onchange="gotoURL(this)">
+      <a for="page in pagesL" title=":page.getShownValue()"
+         href=":page.url if inlaidHeader else page.getPublicUrl()">
+       <span class=":'upage' if page.inPublic() else
+                      ('current' if o==page else '')">:page.getTitle()</span>
+      </a>
+      <select onchange="gotoURL(this)" if="pagesO">
        <option value="">:_('goto_link')</option>
-       <option for="page in pages"
-               value=":'%s/public?rp=%s' % (tool.url, page.iid)"
-               selected=":req.rp==str(page.iid)">:page.getShownValue()</option>
+       <option for="page in pagesO"
+               value=":page.url if inlaidHeader else page.getPublicUrl()"
+               selected=":o == page if inlaidHeader else page.inPublic()"
+               title=":page.getShownValue()">:page.getTitle(
+                 maxChars=config.ui.pageCharsS)</option>
       </select>''',
 
+     css = '''.upage { text-decoration:underline #afb8d4;
+                       text-underline-offset:6px }''',
      js='''
        function gotoURL(select) {
          var url = select.value;
@@ -407,7 +462,8 @@ class Page(Base):
 
     # PX showing all root pages in the portlet, when shown for pages
     portletBottom = Px('''
-     <div class="topSpaceS" var="pages=tool.OPage.getRoot(tool)">
+     <div class="topSpaceS"
+          var="pages=tool.OPage.getRoot(tool, splitted=False)">
       <x if="pages"
          var2="selected=o.getParents() if o.class_.name == 'Page' else {}">
        <x for="page in pages">::page.inPortlet(selected)</x>
@@ -420,16 +476,50 @@ class Page(Base):
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
 
     @classmethod
-    def getRoot(class_, tool):
-        '''Return the pages being visible by the logged user, among the site's
-           root pages from p_tool.pages.'''
-        # Return the cached version, if available
+    def getRoot(class_, tool, splitted=True):
+        '''Return the root, selectable pages being visible by the logged user,
+           among the site's root pages from p_tool.pages.'''
+        # If p_splitted is True, pages will be returned in 2 distinct lists:
+        # - the first one will contain those that must be rendered as "a" tags;
+        # - the second one will contain those that must be rendered as options
+        #   in a HTML "select" control.
+        #
+        # Choosing which page to put in which list is determined by parameter
+        # p_tool.config.expandedRootPages.
+        #
+        # Return the cached version (for the p_splitted variant only), if
+        # available.
+        key = 'appyRootPagesS' if splitted else 'appyRootPages'
         cache = tool.H().cache
-        if 'appyRootPages' in cache: return cache.appyRootPages
-        # Compute it
-        r = [page for page in tool.pages \
-             if page.selectable and tool.guard.mayView(page)]
-        cache.appyRootPages = r
+        if key in cache: return cache[key]
+        if splitted:
+            # Compute 2 sub-lists
+            links = []
+            options = []
+            # Determine the maximum number of pages to render as links
+            linksCount = 0 if tool.H().isMobile() \
+                              else tool.config.ui.expandedRootPages
+        else:
+            # Return a single list
+            r = []
+        # Walk root pages
+        guard = tool.guard
+        for page in tool.pages:
+            # Ignore the page, if unselectable or unviewable
+            if not page.selectable or not guard.mayView(page): continue
+            if splitted:
+                # Put the page in v_links or v_options
+                if linksCount == 0:
+                    options.append(page)
+                else:
+                    links.append(page)
+                    linksCount -= 1
+            else:
+                r.append(page)
+        # Cache and return the result
+        if splitted:
+            r = (links, options)
+        cache[key] = r
         return r
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
