@@ -23,6 +23,7 @@ from DateTime import DateTime
 
 from appy.px import Px
 from appy.model.batch import Batch
+from appy.model.totals import Totals
 from appy.ui.criteria import Criteria
 from appy.ui.template import Template
 from appy.utils import string as sutils
@@ -58,8 +59,11 @@ class UiSearch:
              batch=mode.batch;
              empty=mode.empty;
              search=uiSearch.search;
+             class_=search.container;
              showNewSearch=showNewSearch|True;
              showHeaders=showHeaders|True;
+             totals=search.Totals.initialise(search, 'o', mode.columns)
+                    if mode.isSummable() else None;
              specific=uiSearch.getResultsTop(mode, ajax)">
 
      <!-- Application, class-specific code before displaying results -->
@@ -69,60 +73,75 @@ class UiSearch:
      <div id=":mode.hook">
       <script>::mode.getAjaxData()</script>
 
-      <!-- Pod templates -->
-      <x if="not empty and not popup and search.showPods">
-       <table var="fields=class_.getListPods(tool)"
-              if="fields and mode.objects" align=":dright">
-        <tr>
-         <td var="o=mode.objects[0]"
-             for="field in fields" var2="fieldName=field.name"
-             class=":not loop.field.last and 'pod' or ''">:field.pxRender</td>
-        </tr>
-       </table>
-      </x>
+      <!-- The top zone -->
+      <div class="sTop" if="not popup">
 
-      <!-- Title -->
-      <div if="not popup and search.showTitle" class="pageTitle topSpaceS">
-       <x>::uiSearch.translated</x>
-       <x if="mode.batch and not empty">
-        <img src=":svg('arrowsA')" class="iconS"
-             style="transform: rotate(270deg)"/>
-        <x>:mode.batch.total</x>
-       </x>
-       <x if="search.add and guard.mayInstantiate(class_)"
-          var2="buttonType='small'; viaPopup=search.viaPopup; label=None;
-                nav=mode.getNavInfo(batch.total)">:class_.pxAdd</x>
-      </div>
-      <table width="100%">
-       <tr valign="top">
+       <!-- Title -->
+       <div if="search.showTitle" class="pageTitle">
+        <x>::uiSearch.translated</x>
+        <x if="mode.batch and not empty">
+         <span var="css=class_.getCssFor(tool, 'sep')"
+               class=":'navSep %s' % css if css else 'navSep'">//</span>
+         <span class="btot">:mode.batch.total</span>
+        </x>
         <!-- Search description -->
-        <td>
-         <x if="search.showTitle">
-          <div class="discreet"
-               if="uiSearch.translatedDescr">:uiSearch.translatedDescr</div>
-          <!-- Perform a new search -->
-          <a if="showNewSearch and mode.newSearchUrl"
-             href=":mode.newSearchUrl">:_('search_new')</a>
-          <x if="mode.fromRef and not mode.inField">&nbsp;&mdash;&nbsp; 
-           <a href=":mode.getRefUrl()">:_('goto_source')</a></x>
-         </x>
-        </td>
-        <!-- (Top) navigation -->
-        <td if="mode.batch and uiSearch.showTopNav"
-            align=":search.navAlign">:mode.batch.pxNavigate</td>
-       </tr>
-      </table>
+        <img if="uiSearch.translatedDescr" src=":svg('detail')"
+             class="sdetail" title=":uiSearch.translatedDescr"/>
+
+        <!-- Class-specific colored border (*t*itle *bot*tom) -->
+        <div var="css=class_.getCssFor(tool, 'tbot')"
+             if="css" class=":'tbot %s' % css"></div>
+       </div>
+
+       <!-- Icons -->
+       <x if="search.showTitle">
+
+        <!-- Switch mode -->
+        <div if="not mode.empty and len(resultModes) &gt; 1" class="smodes">
+         <img for="cls in mode.getClasses(resultModes)"
+              if="mode.name != cls.name" name=":cls.name"
+              title=":cls.getText(_)" src=":svg(cls.icon)"
+              class="clickable iconM" onclick="switchMode(this)"/>
+        </div>
+
+        <!-- Add a new object -->
+        <div if="search.add and guard.mayInstantiate(class_)"
+           var2="buttonType='small'; viaPopup=search.viaPopup;
+                 nav=mode.getNavInfo(batch.total);
+                 label=search.addLabel">:class_.pxAdd</div>
+
+        <!-- Perform a new search -->
+        <div if="showNewSearch and mode.newSearchUrl">
+         <a href=":mode.newSearchUrl">:_('search_new')</a>
+         <x if="mode.fromRef and not mode.inField">&nbsp;&mdash;&nbsp; 
+          <a href=":mode.getRefUrl()">:_('goto_source')</a></x>
+        </div>
+       </x>
+
+       <!-- (Top) navigation -->
+       <div if="mode.batch and uiSearch.showTopNav" class="snav"
+            style=":search.getNavMargins()">:mode.batch.pxNavigate</div>
+
+       <!-- Pod templates -->
+       <x if="not empty and not popup and search.showPods and mode.objects">
+        <div var="info=class_.getListPods(tool)" class="spod" if="info">
+          <x var="o=mode.objects[0]" for="field, visible in info.items()"
+             var2="fieldName=field.name">:field.pxRender</x>
+        </div>
+       </x>
+      </div>
 
       <!-- Results -->
       <x if="not empty" var2="currentNumber=0">:mode.px</x>
 
       <!-- (Bottom) navigation -->
-      <div if="mode.batch and uiSearch.showBottomNav"
+      <div if="not empty and mode.batch and uiSearch.showBottomNav"
+           class="snab" var2="scrollTop='payload'"
            align=":search.navAlign">:mode.batch.pxNavigate</div>
 
       <!-- No result -->
-      <div if="empty" class="topSpace">::_('query_no_result')</div>
-    </div></x>''')
+      <div if="empty">::_(search.noResultLabel)</div>
+     </div></x>''')
 
     def __init__(self, search, tool, ctx, initiator=None, name=None):
         self.search = search
@@ -186,7 +205,7 @@ class UiSearch:
     def getCbJsInit(self, hook):
         '''Returns the code that creates JS data structures for storing the
            status of checkboxes for every result of this search.'''
-        default = self.checkboxesDefault and 'unchecked' or 'checked'
+        default = 'unchecked' if self.checkboxesDefault else 'checked'
         return '''var node=findNode(this, '%s');
                   node['_appy_objs_cbs'] = {};
                   node['_appy_objs_sem'] = '%s';''' % (hook, default)
@@ -203,7 +222,7 @@ class UiSearch:
         '''Gets the current mode'''
         return Mode.get(self)
 
-    def getTitleMode(self, popup):
+    def getTitleMode(self, o, popup):
         '''How titles to search results, being instances of p_class_, must be
            rendered ?'''
         # Possible r_return values are the following.
@@ -212,6 +231,9 @@ class UiSearch:
         #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # "link"   | links allowing to go to instances' view pages;
         # "dlink"  | links, as for "link", but the link is wrapped in a "div";
+        # "plink"  | links, as for "link", but *p*ublic links = a special URL
+        #          | allowing to render object content in a specific way, on the
+        #          | "public" PX.
         # "select" | objects, that can be selected from a popup;
         # "text"   | simple, unclickable text.
         #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -219,7 +241,7 @@ class UiSearch:
         # Check if the title mode is specified on the container class
         mode = self.container.getTitleMode()
         if not mode: return 'link'
-        return mode if not callable(mode) else mode(self.tool)
+        return mode(o) if callable(mode) else mode
 
     def getResultsTop(self, mode, ajax):
         '''If p_class_ defines something to display on the results page just
@@ -238,9 +260,11 @@ class Search:
     '''Used for specifying a search for a given class'''
 
     traverse = {}
+    Totals = Totals
+
     # Special initiator in use when an object is being created from a Search. It
     # has nothing to do with popup-oriented initiator as defined hereafter.
-    initiator = initiators.SearchInitiator
+    initiator = initiators.SearchInitiator    
 
     # Parameters needed to replay a search (see m_replay)
     replayParams = ('className', 'search', 'sortKey', 'sortOrder', 'filters')
@@ -249,12 +273,14 @@ class Search:
                  maxPerPage=30, default=False, colspan=1, viaPopup=None,
                  show=True, showActions='all', actionsDisplay='block',
                  showPods=True, showTitle=True, showFilters=True,
-                 showNav='both', navAlign='right', translated=None,
-                 translatedDescr=None, checkboxes=False, checkboxesDefault=True,
-                 container=None, add=False, resultModes=None, shownInfo=None,
-                 actions=None, rowAlign='top', pageLayoutOnView=None,
-                 thumbnailCss='thumbnail', switchModeAlign='left',
-                 gridFiltersAlign='center', **fields):
+                 showNav='both', navAlign='right', listCss=None,
+                 translated=None, translatedDescr=None, checkboxes=False,
+                 checkboxesDefault=True, container=None, add=False,
+                 addLabel='object_add', addFromLabel='object_add_from',
+                 resultModes=None, shownInfo=None, actions=None, rowAlign='top',
+                 pageLayoutOnView=None, thumbnailCss='thumbnail',
+                 gridFiltersAlign='center', totalRows=None,
+                 noResultLabel='query_no_result', **fields):
         # "name" is mandatory, excepted in some special cases (ie, when used as
         # "select" param for a Ref field).
         self.name = name
@@ -282,10 +308,18 @@ class Search:
         self.actionsDisplay = actionsDisplay
         # Various parts of search results may be shown or not
         self.showPods = showPods
-        self.showTitle = showTitle # The title, description, results count...
+        self.showTitle = showTitle # Determines when to show the zone including
+                                   # the search title, its description, results
+                                   # count, etc.
         self.showFilters = showFilters # Filters, in grid mode
-        self.showNav = showNav # May be 'top', 'bottom', 'both' or None
-        self.navAlign = navAlign # Can be 'left', 'center' or 'right'
+        self.showNav = showNav # May be "top", "bottom", "both" or None
+        # Alignment for the navigation panel: can be "left", "center" or "right"
+        self.navAlign = navAlign
+        # For search results shown in list mode, you may specify here a default
+        # CSS class to apply to the main table tag that will hold search
+        # results. If p_listCss is None, the CSS class declared on the class
+        # itself, in its "styles" static attribute, will be used.
+        self.listCss = listCss
         # In dict "fields", keys are names of indexed fields and values are
         # simple search values or terms built with database operators.
         self.fields = fields
@@ -301,6 +335,9 @@ class Search:
         self.container = container
         # Is it possible to create new "container" instances from this search ?
         self.add = add
+        # Labels for the "add" and "add from" buttons
+        self.addLabel = addLabel
+        self.addFromLabel = addFromLabel
         # There can be various ways to display query results
         self.resultModes = resultModes
         # Similar to the homonym Ref attribute, "shownInfo" defines the columns
@@ -310,7 +347,7 @@ class Search:
         # Specify here Action fields that must be shown as custom actions that
         # will be triggered on search results.
         self.actions = actions
-        # When search resulrs are rendered as a list, this attribute specifies
+        # When search results are rendered as a list, this attribute specifies
         # each row's vertical alignment.
         self.rowAlign = rowAlign
         # You may override the page layout used when accessing, on the "view"
@@ -319,12 +356,16 @@ class Search:
         # The name of the CSS class to use for rendering thumbnails, when result
         # mode is "grid".
         self.thumbnailCss = thumbnailCss
-        # Alignment for the widget allowing to switch from one mode to another.
-        # Can be any valid value for property "justify-content" in a flex
-        # layout, a.o.: "left", "right" and "center".
-        self.switchModeAlign = switchModeAlign
         # Alignment of filters on a grid layout
         self.gridFiltersAlign = gridFiltersAlign
+        # If you want to compute some totals based on search results, set, in
+        # p_totalRows, a list of appy.model.totals::Totals instances. Note that
+        # these rows will only be shown if all search results are present (ie,
+        # the current batch matches all results).
+        self.totalRows = totalRows
+        # The i18n label to use for building the message to show when there is
+        # no result.
+        self.noResultLabel = noResultLabel
 
     def init(self, class_):
         '''Lazy search initialisation'''
@@ -341,6 +382,13 @@ class Search:
         return r if not callable(r) else \
                tool.H().methods.call(tool, self.show, class_=class_)
 
+    def getShownInfo(self, o):
+        '''Returns p_self.shownInfo (call it if it is a method) or None if not
+           set.'''
+        r = self.shownInfo
+        if r is None: return
+        return o.H().methods.call(o, r) if callable(r) else r
+
     def getSessionKey(self, name=None):
         '''Returns the name of the key storing siblings of a given object in the
            context of this search.'''
@@ -350,8 +398,7 @@ class Search:
 
     mergeAttributes = ('sortBy', 'sortOrder', 'showActions', 'actionsDisplay',
                        'actions', 'maxPerPage', 'resultModes', 'shownInfo',
-                       'checkboxes', 'checkboxesDefault', 'switchModeAlign',
-                       'gridFiltersAlign')
+                       'checkboxes', 'checkboxesDefault', 'gridFiltersAlign')
 
     def merge(self, other):
         '''Merge parameters from another search in p_other'''
@@ -371,10 +418,20 @@ class Search:
                 r.append(action)
         return r
 
-    def getField(self, name, handler):
+    def getField(self, name):
         '''Gets the field whose name is p_name'''
-        className = self.container.name
-        return handler.server.model.classes.get(className).fields.get(name)
+        return self.container.fields.get(name)
+
+    def getNavMargins(self):
+        '''Return the CSS margins used to align top navigation as required'''
+        align = self.navAlign
+        if align == 'left':
+            r = 'initial'
+        elif align == 'right':
+            r = 'margin-left:auto'
+        elif align == 'center':
+            r = 'margin-left:auto;margin-right:auto'
+        return r
 
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
     #  The "run" method
@@ -382,35 +439,35 @@ class Search:
 
     def run(self, handler, batch=True, start=0, ids=False, secure=True,
             sortBy=None, sortOrder='asc', filters=None, refObject=None,
-            refField=None, other=None):
+            refField=None, other=None, maxPerPage=None):
         '''Based on p_self's parameters, and this method's attributes, perform a
            search in the corresponding catalog ans return the results.'''
         # If p_batch is True, the result is a appy.model.batch.Batch instance
         # returning only a subset of the results starting at p_start and
-        # containing at most p_self.maxPerPage results. Else, it is a list of
-        # objects (or object ids if p_ids is True) representing the complete
-        # result set. If p_ids is True, p_batch is ignored and implicitly
-        # considered being False.
-        # ~~~
+        # containing at most p_maxPerPage (or p_self.maxPerPage if None)
+        # results. Else, it is a list of objects (or object ids if p_ids is
+        # True) representing the complete result set. If p_ids is True, p_batch
+        # is ignored and implicitly considered being False.
+        # ~
         # p_secure is transmitted and documented in called method
         # appy.database.catalog.Catalog.search (called via method
         # appy.database.Database.search).
-        # ~~~
+        # ~
         # The result is sorted according to the potential sort key and order
         # ('asc'ending or 'desc'ending) as defined in p_self.sortBy and
         # p_self.sortOrder. If p_sortBy is not None, p_sortBy and p_sortOrder
         # override p_self's homonym attributes.
-        # ~~~
+        # ~
         # p_filters alter search parameters according to selected filters in the
         # ui.
-        # ~~~
+        # ~
         # If p_refObject and p_refField are given, the search is limited to the
         # objects being referenced from p_refObject through p_refField.
-        # ~~~
+        # ~
         # If p_other is not None, it is another Search instance whose parameters
         # will be merged with p_self's parameters (including sortBy and
         # sortOrder).
-        # ~~~
+        # ~
         # Prepare search parameters
         className = self.container.name
         params = self.fields.copy()
@@ -427,9 +484,9 @@ class Search:
         if filters:
             for name, value in filters.items():
                 if not value: continue
-                field = self.getField(name, handler)
+                field = self.getField(name)
                 value = field.getSearchValue(None, value=value)
-                if value: params[name] = value
+                if value is not None: params[name] = value
         # Manage a potential Ref field
         if refObject and refField.back:
             backName = refField.back.name
@@ -449,7 +506,8 @@ class Search:
         # Return the complete set of matched objects if p_batch is False
         if not batch: return database.getObjects(handler, r, className)
         # Create a Batch and populate it with the appropriate subset of objects
-        batch = Batch(total=len(r), size=self.maxPerPage, start=start)
+        maxPerPage = maxPerPage or self.maxPerPage
+        batch = Batch(total=len(r), size=maxPerPage, start=start)
         batch.setObjects(database.getObjects(handler, r, className,
                                             start=batch.start, size=batch.size))
         return batch
@@ -598,7 +656,7 @@ class Search:
         # callers (like POD fields) may need to get info about the search as
         # stored on the UiSearch instance, like its translated name and
         # description.
-        # ~~~
+        # ~
         # Get search parameters as an object
         params = O()
         i = 0
@@ -637,11 +695,11 @@ class Search:
         # also denote referred objects.
         # 
         # The method returns a tuple (ids, unchecked):
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #   "ids"     | is a dict of object IIDs;
         # "unchecked" | is True if these objects are unchecked, or False if
         #             | they are checked.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
         ids = {int(v) for v in req.checkedIds.split(',')} if req.checkedIds \
                                                           else {}
@@ -674,39 +732,12 @@ class Search:
     # Display results of a search whose name is in the request
     results = Px('''
      <div var="hook=uiSearch.getRootHook()" id=":hook">
-      <script>:uiSearch.getCbJsInit(hook)</script>
+      <script>:uiSearch.getCbJsInit(mode.outerHook)</script>
       <x if="not mode.inField">::ui.Globals.getForms(tool)</x>
 
-      <!-- Switch mode -->
-      <div if="not mode.empty and len(resultModes) &gt; 1" class="modes"
-           style=":'justify-content:%s' % uiSearch.search.switchModeAlign">
-       <img for="cls in mode.getClasses(resultModes)"
-            name=":cls.name" title=":cls.getText(_)"
-            src=":svg(cls.icon)" class="clickable iconL"
-            style=":'display:%s' % ('block' if mode.name!=cls.name else 'none')"
-            onclick="switchResultMode(this)"/>
-      </div>
-
       <!-- Search results -->
       <x>:uiSearch.pxResult</x>
-     </div>''',
-
-     css='''.modes{ display:flex; flex-wrap:wrap; grid-gap:15px;
-                    margin:5px 0 } ''',
-
-     js='''
-       function switchResultMode(img, hook) {
-         // Hide the mode corresponding to p_img
-         img.style.display = 'none';
-         // Display any other mode
-         var images=img.parentNode.children;
-         for (var i=0; i<images.length; i++) {
-           if (images[i] != img) images[i].style.display = 'block';
-         }
-         askAjax('searchResults', null, {'resultMode': img.name});
-       }''',
-
-     template=Template.px, hook='content')
+     </div>''', template=Template.px, hook='content')
 
     # Same as PX "results" hereabove, but without template, to be used as inner
     # PX from another one (ie, within a field).
@@ -716,8 +747,8 @@ class Search:
     @classmethod
     def jresults(class_, tool):
         '''Returns, as a JSON dict, in the context of a search whose name is in
-           the request, the indexes of the siblings object of the one whose
-           index is in the request.'''
+           the request, the indexes of the objects being siblings of the one
+           whose index is in the request.'''
         req = tool.req
         # The index of the current object
         index = int(req.index)
@@ -730,7 +761,9 @@ class Search:
         start = index - int(size / 2)
         if start < 0: start = 0
         # Perform the search
-        ids = search.run(tool.H(), ids=True)
+        ids = search.run(tool.H(), ids=True, sortBy=req.sortKey or None,
+                         sortOrder=req.sortOrder or 'asc',
+                         filters=modelClass.getFilters(tool))
         r = {}
         i = start
         end = start + size
@@ -746,19 +779,19 @@ class Search:
     # Display results of a live search
     liveResults = Px('''
      <div var="search=uiSearch.search;
-               filters=search.container.getFilters(tool);
-               objects=search.run(handler,filters=filters).objects;
-               io,ifield=search.getRefInfo(tool, pre, nameOnly=False, sep='_')"
+              filters=search.container.getFilters(tool);
+              objects=search.run(handler,filters=filters,maxPerPage=10).objects;
+              io,ifield=search.getRefInfo(tool, pre, nameOnly=False, sep='_')"
           id=":'%s_LSResults' % pre">
       <p if="not objects">::_('query_no_result')</p>
-      <div for="o in objects" style="padding: 3px 5px">
+      <div for="o in objects">
        <a href=":search.onSelectLiveResult(o, io, ifield)"
           var="title=o.getShownValue(layout='xml');
                content=Px.truncateValue(title, width=80)"
           title=":title">:content</a>
       </div>
       <!-- Go to the page showing all results -->
-      <div if="objects and not io" align=":dright" style="padding: 3px">
+      <div if="objects and not io" align=":dright">
        <a href=":'javascript:allLSResults(%s,%s)' % (q(pre), q(className))"
           class="lsAll">:_('search_results_all') + '...'</a>
       </div>
@@ -775,6 +808,29 @@ class Search:
         return "javascript:addLSResult('%s','%s','%s',%s)" % \
                (io.iid, ifield.name, o.iid, multi)
 
+    @classmethod
+    def getAdvancedUrls(class_, tool, className):
+        '''Get (a) the URL to go to when posting the advanced search form and
+           (b) the URL to go back to the initiator Ref, if the search is
+           triggered from a Ref.'''
+        # URL (a) will contain base GET parameters: this will be a kind of mixed
+        # GET/POST request. That way, if the user is redirected to a referer
+        # URL corresponding to this URL, there will be no error: the "GET" part
+        # is "replayable" (even if, of course, it will not produce exactly the
+        # same results, the POST parameters being missing).
+        r = '%s/Search/results?className=%s&source=searchForm&search=' \
+            'customSearch' % (tool.url, className)
+        rinfo = tool.req.ref
+        if rinfo:
+            # Compute URL (b)
+            id, name = rinfo.split(':')
+            o = tool.getObject(id)
+            field = o.getField(name)
+            back = '%s/view?page=%s' % (o.url, field.page.name)
+        else:
+            back = None
+        return r, back
+
     # The advanced search form
     advanced = Px('''
      <x var="refInfo=req.ref;
@@ -793,11 +849,9 @@ class Search:
       </div>
 
       <!-- Form for searching objects of the requested class -->
-      <form name="search" action=":'%s/Search/results' % tool.url"
-            method="post">
-       <input type="hidden" name="className" value=":className"/>
-       <input type="hidden" name="source" value="searchForm"/>
-       <input type="hidden" name="search" value="customSearch"/>
+      <form name="search" id="search_form" method="post"
+            var="postUrl,backUrl=tool.Search.getAdvancedUrls(tool,className)"
+            action=":postUrl">
        <input if="refInfo" type="hidden" name="ref" value=":refInfo"/>
 
        <!-- The search fields -->
@@ -806,18 +860,30 @@ class Search:
             style=":'grid-column:span %d' %field.scolspan">:field.pxRender</div>
        </div>
 
-       <!-- Submit button -->
-       <input var="label=_('search_button');
-                   css=ui.Button.getCss(label, small=False)"
-              type="submit" class=":css" value=":label"
-              style=":svg('search', bg='18px 18px')"/>
+       <!-- Submit -->
+       <div>
+        <input type="submit" style=":svg('search', bg='18px 18px')"
+               var="label=_('search_button');
+                    css=ui.Button.getCss(label, small=False)"
+               class=":css" value=":label"/>
+       </div>
+
+       <!-- Back -->
+       <div if="backUrl">
+        <input type="button" style=":svg('back', bg='18px 18px')"
+               var="label=_('go_back');
+                    css=ui.Button.getCss(label, small=False)"
+               class=":css" value=":label"
+               onclick=":'window.location=%s' % q(backUrl)"/>
+       </div>
       </form>
      </x>''', template=Template.px, hook='content')
 
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     # The live search PX. It requires variables:
-    # --------------------------------------------------------------------------
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     #  className | The name of the class whose instances will be searched
-    # --------------------------------------------------------------------------
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     #    pre     | The prefix used to determine identifiers for tags in use
     #            | within live search PXs. Indeed, if several live searches are
     #            | present on the same pages, these identifiers will allow to
@@ -825,9 +891,9 @@ class Search:
     #            | is one live search per root class, pre == className. When
     #            | called from a Ref field with link='dropdown', pre is of the
     #            | form <Object IID>_<field name>, for example "666_managers".
-    # --------------------------------------------------------------------------
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     #   liveCss  | The CSS class(es) to assign to the main PX tag
-    # --------------------------------------------------------------------------
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     live = Px('''
      <div var="searchLabel=_('search_button')" class=":liveCss">
@@ -842,7 +908,7 @@ class Search:
               onblur=":'onLiveSearchEvent(event,%s,%s,%s)' % \
                        (q(pre), q(className), q('hide'))"/>
        <!-- Dropdown containing live search results -->
-       <div id=":'%s_LSDropdown' % pre" class="dropdown">
+       <div id=":'%s_LSDropdown' % pre" class="dropdown sdrop">
         <div id=":'%s_LSResults' % pre"></div>
        </div>
       </div>
@@ -850,18 +916,21 @@ class Search:
 
      css='''.lsSelected { background-color:#d9d7d9 }
             .lsRef { display:flex }
-            .lsSearch { display:flex; margin-top:5px; padding:|lsPadding|;
-                        background-color:|lsBgColor| }
+            .lsSearch { display:flex; align-items:center; margin-top:5px;
+                        padding:|lsPadding|; background-color:|lsBgColor| }
+            .lsSearch > img { width:|portletISize| }
             .lsSearch input[type=text] { background-color:|lsInputBgColor|;
-               border-bottom: |lsBottom| |lsBottomColor|;
-               width:|lsInputWidth|; height:|lsInputHeight| }
+               border-bottom: |lsBottom| |lsBottomColor|; width:|lsInputWidth|;
+               height:|lsInputHeight|; font-size:|lsFSize|; margin:|lsMargin|;
+               padding:|lsInputPadding| }
             .lsSearch .dropdown { background-color:|lsrBgColor| }
+            .sdrop > div > div { padding:3px 5px }
             .lsAll { font-size:80%; font-style:italic }''',
 
      # Live-search-related functions (LS)
      js='''
        function cleanKeywords(keywords) {
-         return keywords.replace(/['",:]/g, '').trim();
+         return keywords.replace(/['",:+]/g, '').trim();
        }
 
        function detectEventType(event) {
@@ -881,13 +950,13 @@ class Search:
          }
        }
 
-       /* Function that selects the search result within the dropdown, after the
-          user has pressed the 'up' od 'down' key (p_direction). */
+       /* Select the search result within the dropdown, after the user has
+          pressed the 'up' od 'down' key (p_direction). */
        function selectLSResult(dropdown, direction){
-         var results = dropdown.children[0].getElementsByTagName('div');
+         let results = dropdown.children[0].getElementsByTagName('div');
          if (results.length == 0) return;
-         var j; // The index of the new element to select
-         for (var i=0, len=results.length; i<len; i++) {
+         let j; // The index of the new element to select
+         for (let i=0, len=results.length; i<len; i++) {
            if (results[i].className == 'lsSelected') {
              if (direction == 'up') {
                if (i > 0) j = i-1;
@@ -905,16 +974,19 @@ class Search:
          if (isNaN(j)) results[0].className = 'lsSelected';
        }
 
-       // Function that allows to go to a selected search result
-       function gotoLSLink(dropdown) {
-         var a, results = dropdown.children[0].getElementsByTagName('div');
-         for (var i=0, len=results.length; i<len; i++) {
+       // Go to a selected search result
+       function gotoLSLink(dropdown, pre, className) {
+         let a, results = dropdown.children[0].getElementsByTagName('div');
+         for (let i=0, len=results.length; i<len; i++) {
            if (results[i].className == 'lsSelected') {
              a = results[i].children[0];
              if (a.href) window.location = a.href;
              else eval(a.onclick);
+             return;
            }
          }
+         // If we are here, no link was selected: display all results
+         allLSResults(pre, className);
        }
     
        function hideLSDropdown(dropdown, timeout) {
@@ -927,11 +999,11 @@ class Search:
        // Retrieve all parameters for performing a live search
        function getLSParams(pre, className) {
          // Retrieve search keywords
-         var keywords = document.getElementById(pre + '_LSinput').value;
+         let keywords = document.getElementById(pre + '_LSinput').value;
          keywords = cleanKeywords(keywords);
          if (keywords[keywords.length-1] != '*') keywords = keywords + '*';         
          // Define base parameters
-         var r = {'className':className, 'pre':pre,
+         let r = {'className':className, 'pre':pre,
                   'filters':'searchable:' + keywords};
          // Add parameters depending on the type of search to run
          if (pre.indexOf('_') == -1) {
@@ -948,24 +1020,24 @@ class Search:
 
        // Manage an p_event triggered from the LS input field
        function onLiveSearchEvent(event, pre, class_, action) {
-         var dropdown = document.getElementById(pre + '_LSDropdown');
+         let dropdown = document.getElementById(pre + '_LSDropdown');
          if (lsTimeout) clearTimeout(lsTimeout);
          // Hide the dropdown if action is forced to 'hide'
          if (action == 'hide') { hideLSDropdown(dropdown, true); return; }
          // Detect if the dropdown must be shown or hidden
-         var input = document.getElementById(pre + '_LSinput'),
+         let input = document.getElementById(pre + '_LSinput'),
              keywords = cleanKeywords(input.value || '');
          if (keywords.length > 2) {
-           var eventType = detectEventType(event);
+           let eventType = detectEventType(event);
            if (!eventType) return;
-           if (eventType == 'hide') { hideLSDropdown(dropdown, false); return;}
-           if (eventType == 'go') { gotoLSLink(dropdown); return; }
+           if (eventType == 'hide') { hideLSDropdown(dropdown, false); return; }
+           if (eventType == 'go') { gotoLSLink(dropdown, pre, class_); return; }
            if (eventType == 'search') {
              // Trigger an Ajax search and refresh the dropdown content
-             var params = getLSParams(pre, class_);
+             let params = getLSParams(pre, class_);
              lsTimeout = setTimeout(function() {
-               askAjaxChunk(siteUrl + '/tool/Search/liveResults', 'GET', params,
-                            pre + '_LSResults');
+               askAjaxChunk(siteUrl + '/tool/Search/liveResults', 'POST',
+                            params, pre + '_LSResults');
                dropdown.style.display = 'block';}, 400);
              }
            else { // Move up or down within results
@@ -977,19 +1049,19 @@ class Search:
 
        // Get all search results
        function allLSResults(pre, className) {
-         var params = getLSParams(pre, className);
+         let params = getLSParams(pre, className);
          post(siteUrl + '/tool/Search/results', params);
        }
 
        // Add the selected object to the initiator Ref field
        function addLSResult(oid, ifield, id, multi) {
-         var hook = oid + '_' + ifield,
+         let hook = oid + '_' + ifield,
              url = siteUrl + '/' + oid;
          if (multi) {
            // Also get the already selected objects in ifield
-           var sel = document.getElementById(ifield),
+           let sel = document.getElementById(ifield),
                selected=sel.selectedOptions, ids=[];
-           for (var i=0; i<selected.length; i++) {
+           for (let i=0; i<selected.length; i++) {
              ids.push(selected[i].value);
            }
            if (ids.indexOf(id) == -1) ids.push(id);
