@@ -22,8 +22,8 @@ class Gridder:
        SomeAppyClass.gridder.'''
 
     def __init__(self, width='350px', gap='20px', justifyContent='space-evenly',
-                 alignContent='space-evenly', cols=None, margin=None,
-                 showFields=True, flex=False):
+                 alignContent='space-evenly', cols=None, colsAuto=False,
+                 margin=None, showFields=True, flex=False):
         # ~~~
         # If a p_width is specified, there will not be a fixed number of
         # columns: this number will depend on available space and this width.
@@ -46,6 +46,10 @@ class Gridder:
         self.alignContent = alignContent
         # The maximum number of columns
         self.cols = cols
+        # If colsAuto is True, the columns will be rendered with default
+        # settings. Else, columns will have grid-template-columns with:
+        # repeat(X,1fr)
+        self.colsAuto = colsAuto
         # The gridder's margins
         self.margin = margin
         # By default, for every object rendered in a grid, the following
@@ -73,7 +77,10 @@ class Gridder:
             display = 'grid'
             # Determine grid-specific parameters
             if self.cols:
-                columns = 'repeat(%d,1fr)' % self.cols
+                if self.colsAuto:
+                    columns = ' '.join(['auto']*self.cols)
+                else:
+                    columns = 'repeat(%d,1fr)' % self.cols
             else:
                 columns = 'repeat(auto-fill,minmax(%s,1fr))' % self.width
             specific = '; grid-template-columns:%s' % columns
