@@ -29,6 +29,9 @@ from appy.model.utils import Object as O
 from appy.ui import LinkTarget, Columns, Title
 from appy.model.searches.gridder import Gridder
 
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+E_CUST   = 'This must be a sub-class from appy.model.searches.modes.Custom.'
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Mode:
     '''Abstract base class for search modes. A concrete Mode instance is created
@@ -44,7 +47,7 @@ class Mode:
     pxActions = Px('''
      <table>
       <tr><td for="action in actions"
-            var2="multi=action.getMulti('queryResult', req.search);
+            var2="multi=action.getMulti('searchResults', mode.outerHook);
                   field=action; fieldName=field.name;
                   layout='query'">:action.pxRender</td></tr>
      </table>''')
@@ -56,24 +59,33 @@ class Mode:
         name = uiSearch.req.resultMode or uiSearch.getModes()[0]
         # Determine the concrete mode class
         if name in Mode.concrete:
-            custom = False
             concrete = eval(name.capitalize())
         else:
-            custom = True
-            concrete = Custom
+            # Get the concrete mode on the Appy class
+            for mode in uiSearch.container.python.resultModes:
+                if not isinstance(mode, str):
+                    concrete = mode
+                    break
         # Create the Mode instance
         r = concrete(uiSearch)
-        if custom:
-            # The custom PX is named "name" on the model class
-            r.px = getattr(self.class_, name)
         r.init()
         return r
 
     @classmethod
     def getClasses(class_, names):
         '''Return the Mode sub-classes corresponding to their p_names'''
-        # Todo: manage custom modes
-        return [eval(name.capitalize()) for name in names]
+        r = []
+        for name in names:
+            if isinstance(name, str):
+                # A standard mode
+                mode = eval(name.capitalize())
+            else:
+                # A custom Mode
+                mode = name
+                if not issubclass(mode, Custom):
+                    raise Exception(E_CUST)
+            r.append(mode)
+        return r
 
     @classmethod
     def getText(class_, _):
@@ -119,8 +131,8 @@ class Mode:
         # The target for "a" tags
         self.target = LinkTarget(self.class_.python,
                                  popup=uiSearch.search.viaPopup)
-        # How to render links to result objects ?
-        self.titleMode = uiSearch.getTitleMode(self.popup)
+        # The notion of "column" does not apply to any mode
+        self.columns = None
 
     def init(self):
         '''Lazy mode initialisation. Can be completed by sub-classes.'''
@@ -130,6 +142,7 @@ class Mode:
         self.criteria = req.criteria
         ui = self.uiSearch
         search = ui.search
+        self.rootHook = ui.getRootHook()
         # The search may be triggered via a Ref field
         io, ifield = search.getRefInfo(tool, nameOnly=False)
         if io:
@@ -146,6 +159,13 @@ class Mode:
         # This search may lie within another field (a Ref or a Computed)
         self.inField = (ifield and ('search*' in req.search)) or \
                        (req.search and ',' in req.search)
+        if self.fromRef and self.inField:
+            # The search is a simple view of objects from a Ref field: inner
+            # elements (like checkboxes) must have IDs being similar to the
+            # standard view of these objects via the Ref field.
+            self.outerHook = '%s_%s' % (self.refObject.iid, self.refField.name)
+        else:
+            self.outerHook = self.rootHook
 
     def getAjaxData(self):
         '''Initializes an AjaxData object on the DOM node corresponding to the
@@ -160,7 +180,7 @@ class Mode:
             if initatorParams: params.update(initatorParams)
         # Add custom search criteria
         if self.criteria:
-            params['criteria+'] = self.criteria
+            params['criteria-'] = self.criteria
         # Add the "ref" key if present
         ref = self.tool.req.ref
         if ref:
@@ -202,6 +222,10 @@ class Mode:
            the URL allowing to navigate back to the page where this Ref field
            lies.'''
 
+    def isSummable(self):
+        '''May totals be computed on searh results using this mode ?'''
+        return # To be overridden by child classes
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class List(Mode):
     '''Displays search results as a table containing one row per object'''
@@ -213,49 +237,26 @@ class List(Mode):
     name = 'list'
 
     px = Px('''
-     <table class=":class_.getResultCss(layout)" width="100%">
-      <!-- Headers, with filters and sort arrows -->
-      <tr if="showHeaders">
-       <th for="column in mode.columns"
-           var2="field=column.field" width=":column.width" align=":column.align"
-           class=":mode.cbClass if field == '_checkboxes' else ''">
-        <x if="column.header">
-         <img if="field == '_checkboxes'" src=":svg('checkAll')"
-              class="clickable iconM" title=":_('check_uncheck')"
-              onclick=":'toggleAllCbs(%s)' % q(mode.checkboxesId)"/>
-         <x if="not column.special">
-          <span class="htitle">::Px.truncateText(_(field.labelId))</span>
-          <!-- Sort icons -->
-          <div if="field.isSortable() and mode.batch.total &gt; 1"
-               class="thIcons">
-           <img if="(mode.sortKey != field.name) or (mode.sortOrder == 'desc')"
-                onclick=":'askBunchSorted(%s, %s, %s)' % \
-                          (q(mode.hook), q(field.name), q('asc'))"
-                src=":svg('asc')" class="clickable iconS"/>
-           <img if="(mode.sortKey != field.name) or (mode.sortOrder == 'asc')"
-                onclick=":'askBunchSorted(%s, %s, %s)' % \
-                          (q(mode.hook), q(field.name), q('desc'))"
-                src=":svg('asc')" class="clickable iconS"
-                style="transform:rotate(180deg)"/>
-          </div>
-          <!-- Filter widget -->
-          <x var="field,filterPx=field.getFilterPx()"
-             if="filterPx and mode.showFilters">:filterPx</x>
-          <x if="ui.Title.showToggleSub(class_, field, tool, \
-                   from_=mode.refField)">:ui.Title.pxToggleSub</x>
-         </x>
-        </x>
-       </th>
-      </tr>
+     <x var="genCss=mode.columns.getCss(showHeaders, uiSearch.search.rowAlign);
+             genName=mode.columns.name if genCss else None">
+      <style if="genCss">::genCss</style>
+      <table class=":mode.getMainCss(_ctx_)">
 
-      <!-- Results -->
-      <tr if="not mode.objects">
-       <td colspan=":len(mode.columns)+1">::_('query_no_result')</td>
-      </tr>
-      <x for="o in mode.objects"
-         var2="rowCss=loop.o.odd and 'even' or 'odd';
-               @currentNumber=currentNumber + 1">:o.pxResult</x>
-     </table>
+       <!-- Headers, with filters and sort arrows -->
+       <tr if="showHeaders">
+        <x for="col in mode.columns" var2="field=col.field">:col.pxHeader</x>
+       </tr>
+
+       <!-- Results -->
+       <tr if="not mode.objects">
+        <td colspan=":len(mode.columns)+1">::_('query_no_result')</td>
+       </tr>
+       <x for="o in mode.objects"
+          var2="@currentNumber=currentNumber + 1">:o.pxResult</x>
+       <!-- Totals -->
+       <x if="totals">:totals.px</x>
+      </table>
+     </x>
 
      <!-- The button for selecting objects and closing the popup -->
      <div if="popup and mode.cbShown" align=":dleft">
@@ -296,27 +297,24 @@ class List(Mode):
         self.batch.hook = self.hook
         self.objects = self.batch.objects
         # Show sub-titles ? Show filters ?
-        self.showSubTitles = req.showSubTitles in ('True', None)
-        self.showFilters = self.filters or self.batch.total > 1
-        # Every matched object may be selected via a checkbox
-        self.rootHook = ui.getRootHook()
-        self.checkboxes = ui.checkboxes
-        if self.fromRef and self.inField:
-            # The search is a simple view of objects from a Ref field:
-            # checkboxes must have IDs similar to the standard view of these
-            # objects via the Ref field.
-            prefix = '%s_%s' % (self.refObject.iid, self.refField.name)
+        if not self.class_.toggleSubTitles(self.tool, None):
+            # When toggling is disabled, always show sub-titles, do not depend
+            # on the cookie value.
+            self.showSubTitles = True
         else:
-            prefix = self.rootHook
-        self.checkboxesId = prefix + '_objs'
+            # Show sub-titles depending on the cookie value
+            self.showSubTitles = req.showSubTitles in ('True', None)
+        # Every matched object may be selected via a checkbox
+        self.checkboxes = ui.checkboxes
+        self.checkboxesId = self.outerHook + '_objs'
         self.cbShown = ui.showCheckboxes()
         self.cbClass = '' if self.cbShown else 'hide'
+        self.cbChecked = eval(req.cbChecked or 'False')
         # Determine result emptiness
         self.empty = not self.objects and not self.filters
         # Compute info related to every column in the list
-        self.columnLayouts = self.getColumnLayouts()
-        self.columns = Columns.get(tool, self.class_, self.columnLayouts,
-                                   ui.dir, addCheckboxes=self.checkboxes)
+        self.columns = Columns.get(tool, self.class_, self.getColumnInfos(),
+                                   addCB=self.checkboxes)
         # Get the Javascript code allowing to store IDs from batch objects in
         # the browser's session storage.
         self.store = self.batch.store(search, name=ui.name)
@@ -330,14 +328,31 @@ class List(Mode):
            'total': self.batch.total,
            'resultMode': self.__class__.__name__.lower()})
 
-    def getColumnLayouts(self):
-        '''Returns the column layouts'''
+    def getMainCss(self, c):
+        '''Get the CSS class(es) to apply to the main tag'''
+        r = self.uiSearch.search.listCss
+        if r:
+            # Use the CSS class(es) defined at the Search level, plus a
+            # potential CSS class generated from column layouts.
+            if c.genName:
+                r = '%s %s' % (r, c.genName)
+        else:
+            # Use the CSS class(es) as defined on the tied class
+            r = c.class_.getCssFor(c.tool, 'list', add=c.genName)
+        return r
+
+    def getColumnInfos(self):
+        '''Returns infos about search results' columns'''
         r = None
         tool = self.tool
         name = self.uiSearch.name
+        search = self.uiSearch.search
         # Try first to retrieve this info from a potential source Ref field
         o = self.refObject
-        if o:
+        if o and not search.shownInfo:
+            # When a Ref field and the search both define columns, those from
+            # the search are preferred. Indeed, colums may define search-
+            # specific elements (ie, configuring filters).
             field = self.refField
             r = field.getAttribute(o, 'shownInfo')
         elif ',' in name:
@@ -345,11 +360,10 @@ class List(Mode):
             o = tool.getObject(id)
             field = o.getField(fieldName)
             if field.type == 'Ref':
-                r = field.getAttribute(o, 'shownInfo')
+                r = search.getShownInfo(o) or field.getAttribute(o, 'shownInfo')
         if r: return r
         # Try to retrieve this info via search.shownInfo
-        search = self.uiSearch.search
-        r = search.shownInfo
+        r = search.getShownInfo(o or tool)
         return r if r else self.class_.getListColumns(tool)
 
     def getFiltersString(self):
@@ -366,6 +380,8 @@ class List(Mode):
            currently set p_self.filters.'''
         values = self.filters.get(name)
         if not values: return
+        # Normalize the value to a string
+        value = str(value) if isinstance(value, int) else value
         if isinstance(values, str):
             r = value == values
         else:
@@ -383,6 +399,10 @@ class List(Mode):
         o = self.refObject
         return '%s/view?page=%s' % (o.url, self.refField.page.name)
 
+    def isSummable(self):
+        '''May totals be computed on searh results using this mode ?'''
+        return self.batch.isComplete()
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Grid(List):
     '''Displays search results as a table containing one cell per object'''
@@ -395,14 +415,11 @@ class Grid(List):
 
     px = Px('''
      <!-- Filters -->
-     <div class="gridFilters" if="uiSearch.search.showFilters"
-          style=":'justify-content:%s' % uiSearch.search.gridFiltersAlign">
-      <x for="column in [c for c in mode.columns if not c.special]"
-         var2="field,filterPx=column.field.getFilterPx()">
-       <div if="filterPx and mode.showFilters">
-        <div><label>:_(field.labelId)</label></div><x>:filterPx</x>
-       </div>
-      </x>
+     <div var="search=uiSearch.search" if="search.showFilters" class="gfilters"
+          style=":'justify-content:%s' % search.gridFiltersAlign">
+      <div for="field, filterPx in mode.columns.getFiltered()">
+       <label>:_(field.labelId)</label><x>:filterPx</x>
+      </div>
      </div>
 
      <!-- The grid itself -->
@@ -442,9 +459,12 @@ class Grid(List):
       <div if="not mode.objects">::_('query_no_filtered_result')</div>
      </div>''',
 
-     css='''.gridFilters { display:flex; flex-wrap:wrap;
-                           margin:|gridFiltersMargin|}
-            .gridFilters > div { margin: 10px 20px }''',
+     css='''
+      .gfilters { display:flex; flex-wrap:wrap; margin:|gridFiltersMargin|}
+      .gfilters > div { margin: 10px 20px; display:inline-flex;
+                        align-items:center }
+      .gfilters input[type=text] { margin:0 0 0 0.4em }
+      .gfilters label { padding:0 0.4em 0 0 }''',
 
      js='''
       followTitleLink = function(img) {
@@ -467,6 +487,11 @@ class Grid(List):
         # Disable the possibility to show/hide sub-titles
         self.showSubTitles = True
 
+    def isSummable(self):
+        '''Although some List modes may be summable, for the moment, Grids are
+           not.'''
+        return
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Calendar(Mode):
     '''Displays search results in a monthly calendar view'''
@@ -490,29 +515,29 @@ class Calendar(Mode):
         Mode.__init__(self, *args)
         # For this calendar view to work properly, objects to show inside it
         # must have the following fields:
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # date    | an indexed and required Date field with format =
         #         | Date.WITH_HOUR storing the object's start date and hour;
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # endDate | a not necessarily required Date field with format =
         #         | Date.WITH_HOUR storing the object's end date and hour.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # Optionnally, if you want to use index "months" (we advice you to do
         # so), an indexed Computed field with this name must also be defined
         # (see details in m_init below).
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # Optionally, too, if objects define the following attributes, special
         # icons indicating repeated events will be shown.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # successor   | Ref field with multiplicity = (0,1), allowing to
         #             | navigate to the next object in the list (for a repeated
         #             | event);
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # predecessor | Ref field with multiplicity = (0,1), allowing to
         #             | navigate to the previous object in the list.
         #             | "predecessor" must be the back ref of field "successor"
         #             | hereabove.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     def getMonthIndex(self):
         '''Deduce, from p_self.class_, what is the month index to use'''
@@ -531,20 +556,20 @@ class Calendar(Mode):
         # The matched objects, keyed by day. For every day, a list of entries to
         # show. Every entry is a 2-tuple (s_entryType, Object) allowing to
         # display an object at this day. s_entryType can be:
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  "start"  | the object starts and ends at this day: display its start
         #           | hour and title;
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  "start+" | the object starts at this day but ends at another day:
         #           | display its start hour, title and some sign indicating
         #           | that it spans on another day;
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  "end"    | the object started at a previous day and ends at this day.
         #           | Display its title and a sign indicating this fact;
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  "pass"   | The object started at a previous day and ends at a future
         #           | day.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         self.objects = {} # ~{s_YYYYmmdd: [(s_entryType, Object)]}~
         # If filters are defined from a list mode, get it
         self.filters = self.class_.getFilters(self.tool)
@@ -552,7 +577,7 @@ class Calendar(Mode):
         # shown month. If p_self.klass defines a Computed indexed TextIndex
         # field named "months", this index will be used. Else, index "date" will
         # be used. These 2 indexes are described hereafter.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # "date"   | This index is expected to define the (start) date of the
         #          | objects to search. So, the search will match any object
         #          | whose (start) date is included in the range of dates
@@ -567,7 +592,7 @@ class Calendar(Mode):
         #          | among the range of visible dates, will not be part of the
         #          | result. This is why an alternate index named "month' is
         #          | also proposed (see below).
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # "months" | When using index "months", every object is expected to
         #          | hold, in a Computed indexed TextIndex field named "months",
         #          | the space-separated list of months the object crosses
@@ -603,7 +628,7 @@ class Calendar(Mode):
         #          |
         #          |     months  = Computed(method=computeMonths,
         #          |                        indexed='TextIndex', show=False)
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         self.monthIndex = self.getMonthIndex()
 
     def updateAjaxParameters(self, params):
@@ -614,13 +639,13 @@ class Calendar(Mode):
 
     # For every hereabove-defined entry type, this dict stores info about how
     # to render events having this type. For every type:
-    # --------------------------------------------------------------------------
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     # start      | bool | Must we show the event's start hour or not ?
     # end        | bool | Must we show the event's end hour or not ?
     # css        | str  | The CSS class to add the table event tag
     # past       | bool | True if the event spanned more days in the past
     # future     | bool | True if the event spans more days in the future
-    # --------------------------------------------------------------------------
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     entryTypes = {
      'start':  O(start=True,  end=True,  css=None,      past=None, future=None),
      'start+': O(start=True,  end=False, css='calMany', past=None, future=True),
@@ -770,9 +795,9 @@ class Custom(Mode):
 
     def init(self):
         '''By default, the Custom mode performs full (unpaginated) searches'''
-        r = self.tool.executeQuery(self.className, search=self.uiSearch.search,
-                                   maxResults='NO_LIMIT')
+        Mode.init(self)
+        # Run the search
+        self.objects = self.uiSearch.search.run(self.tool.H(), batch=False)
         # Initialise Mode's mandatory fields
-        self.objects = r.objects
         self.empty = not self.objects
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
