@@ -24,21 +24,23 @@ from appy.tr import po
 from appy.px import Px
 from appy import Config
 from appy.server import Server
+from appy.utils import Function
 from appy.model.base import Base
 from appy.model.user import User
 from appy.data import nativeNames
-from appy.utils.dates import Date
 from appy.model.group import Group
 from appy.model.fields import Show
 from appy.model.query import Query
 from appy.model.mover import Mover
 from appy.database import Database
 from appy.ui.layout import Layouts
-from appy.utils.mail import sendMail
+from appy.xml.escape import Escape
+from appy.database.log import Viewer
 from appy.ui.template import Template
 from appy.model.fields.ref import Ref
 from appy.server.backup import Backup
 from appy.model.searches import Search
+from appy.utils import dates as dutils
 from appy.model.fields.rich import Rich
 from appy.model.fields import Initiator
 from appy.model.carousel import Carousel
@@ -52,6 +54,7 @@ from appy.test.monitoring import Monitoring
 from appy.model.translation import Translation
 from appy.model.fields.calendar import Calendar
 from appy.model.fields.computed import Computed
+from appy.utils.mail import sendMail, sendMailIf
 from appy.server.context import AuthenticationContext
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -68,8 +71,8 @@ class Tool(Base):
 
     # Make some modules and classes available and/or traversable via the tool
     ui = appy.ui
-    Date = Date
     OPage = OPage
+    Date = dutils.Date
     AuthContext = AuthenticationContext
 
     # While most users have generally the read permission on the tool and
@@ -116,7 +119,7 @@ class Tool(Base):
         if servers:
             self.log(SRV_CONF % ', '.join(servers))
 
-    def init(self, handler, poFiles):
+    def init(self, handler, poFiles, method=None):
         '''Ensure all required tool sub-objects are present and coherent'''
         # Get the root database object from the current request p_handler
         root = handler.dbConnection.root
@@ -170,11 +173,6 @@ class Tool(Base):
         # Check connected servers
         self.checkServers(config)
 
-        # Import an Appy 0 site when relevant
-        peers = config.peers
-        if peers and peers.appy0:
-            peers.appy0.pump(self)
-
         # Define the home page
         Tool.default = getattr(Tool, config.ui.home)
 
@@ -182,6 +180,14 @@ class Tool(Base):
         # an app to execute code on server initialisation.
         if hasattr(self, 'onInstall'): self.onInstall()
 
+        # Call this custom p_method on the tool
+        if method: Function.scall(self, method)
+
+        # Import an Appy 0 site when relevant
+        peers = config.peers
+        if peers and peers.appy0:
+            peers.appy0.pump(self)
+
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
     #  Page "main"
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
@@ -354,6 +360,20 @@ class Tool(Base):
       page=Page('database', phase='admin', show=forAdmin,
                 label='Tool_page_database'), **ta)
 
+    # View site's config.py from the UI
+    def getConfigPy(self):
+        '''Display the content of this Appy site's config.py file'''
+        with open('%s/config.py' % self.config.server.sitePath, 'r') as f:
+            return '<pre>%s</pre>' % Escape.xhtml(f.read())
+
+    configPy = Computed(method=getConfigPy,
+      page=Page('configPy', phase='admin', show=forAdmin,
+                label='Tool_page_configPy'), **ta)
+
+    logsViewer = Computed(method=Viewer.run,
+      page=Page('logsViewer', phase='admin', show=forAdmin,
+                label='Tool_page_logsViewer'), **ta)
+
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
     #  Monitoring
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
@@ -372,26 +392,35 @@ class Tool(Base):
         return sendMail(self.config.mail, to, subject, body,
                         attachments=attachments, log=self.log, replyTo=replyTo)
 
+    def sendMailIf(self, o, privilege, subject, body, attachments=None,
+                   privilegeType='permission', excludeExpression='False',
+                   userMethod=None, replyTo=None):
+        '''Sends a mail concerning this p_o(bject), to people having this
+           p_privilege.'''
+        return sendMailIf(o.config.mail, o, privilege, subject, body,
+                          attachments=attachments, privilegeType=privilegeType,
+                          excludeExpression=excludeExpression,
+                          userMethod=userMethod, log=o.log, replyTo=replyTo)
+
     def computeHomePage(self):
         '''Compute the page that is shown to the user when he hits the app'''
-        # Get the currently logged user
-        user = self.user
-        isAnon = user.isAnon()
-        if isAnon and self.config.ui.discreetLogin:
-            return '%s/public' % self.url
-        # If the app defines a tool method names "getHomePage", call it
+        # Does the app's tool define a custom method for getting the home page ?
+        # If yes, call it.
         r = None
         if hasattr(self, 'getHomePage'):
             r = self.getHomePage()
         if not r:
+            # Get the currently logged user
+            user = self.user
             # Bring anonymous users to tool/home, Managers to tool/view and
-            # others to user/home.
-            if isAnon:
-                r = '%s/home' % self.url
+            # others to user/view.
+            if user.isAnon():
+                suffix = 'public' if self.config.ui.discreetLogin else 'home'
+                r = '%s/%s' % (self.url, suffix)
             elif user.hasRole('Manager'):
                 r = '%s/view' % self.url
             else:
-                r = user.url
+                r = '%s/view' % user.url
         return r
 
     def computeHomeObject(self, user, popup=False):
@@ -414,7 +443,7 @@ class Tool(Base):
     def formatDate(self, date, format=None, withHour=True, language=None):
         '''Check doc in called method'''
         if not date: return
-        return self.config.ui.formatDate(self, date, format, withHour, language)
+        return dutils.Date.format(self, date, format, withHour, language)
 
     def restart(self, wait=5):
         '''Restarts the currently running Appy server (S), by forking another
@@ -432,7 +461,7 @@ class Tool(Base):
             args.append('-w')
             args.append(str(wait))
         self.log(RESTART_S % wait)
-        subprocess.Popen(args, stdout=None, stdin=None, stderr=None)
+        subprocess.Popen(args, stdin=None, stdout=None, stderr=None)
 
     def backup(self):
         '''Method allowing to trigger the backup of a Appy site'''
@@ -487,7 +516,9 @@ class Tool(Base):
      css='''.homes { display:flex; width:100%; height:100% }
             .homes div { background-size: cover }
             .homesTop { position:absolute; text-align:center; left:0; right:0;
-                        top:15px; z-index:10 }''')
+               top:15px; z-index:10 }
+            .homesTop .topIcons { background-color:|topIconsHomesBg|;
+              display:inline-flex; flex-direction:|topIconsHomesDir| }''')
 
     # PX "public" shows one of the root pages (rp), or the default one if none
     # of them is specified. When discreet logins are enabled, the public page
@@ -512,7 +543,7 @@ class Tool(Base):
      css='''
       .publicTop { position:fixed; top:20px; right:20px; color:|headerColor| }
       .publicTop .img { width:27px }
-      .publicTop .topIcons { display:inline }
+      .publicTop .topIcons { display:inline-flex }
      ''',
 
      template=Template.px, hook='content', name='public')
