@@ -16,7 +16,7 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-import time
+import time, base64
 
 from appy.px import Px
 from appy.model.base import Base
@@ -85,7 +85,7 @@ class User(Base):
     def showPassword(self):
         '''Field "password" must be shown only for the local user allowed to
            modify it.'''
-        mayEdit = (self.source == 'zodb') and (self.user.login == self.login)
+        mayEdit = self.source == 'zodb' and self.user.login == self.login
         return 'edit' if mayEdit else None
 
     def showCancel(self):
@@ -155,6 +155,17 @@ class User(Base):
         '''Sets a title for this user'''
         self.title = self.getTitle(nameFirst=True)
 
+    def getSupTitle(self, nav):
+        '''Display a specific icon if the user is a local copy of an external
+           (ldap/sso) user.'''
+        if self.source == 'zodb': return
+        # For external users, note the source and last synchronization date in
+        # the icon's title.
+        sdate = self.syncDate
+        suffix = (' - sync@ %s' % self.tool.formatDate(sdate)) if sdate else ''
+        return '<img src="%s" title="%s%s" class="help"/>' % \
+               (self.buildUrl('external'), self.source.upper(), suffix)
+
     def ensureAdminIsManager(self):
         '''User "admin" must always have role "Manager"'''
         if self.id != 'admin': return
@@ -189,7 +200,7 @@ class User(Base):
         if self.isTemp(): return 'edit'
         # The manager has the possibility to change the login itself (local
         # users only).
-        if self.user.hasRole('Manager') and (self.source == 'zodb'):
+        if self.user.hasRole('Manager') and self.source == 'zodb':
             return True
         return Show.E_
 
@@ -198,7 +209,7 @@ class User(Base):
         # 2 cases: (1) The user is being created and has no login yet, or
         #          (2) The user is being edited and has already a login, that
         #              can potentially be changed.
-        if not self.login or (login != self.login):
+        if not self.login or login != self.login:
             # A new p_login is requested. Check if it is valid and free.
             # Some logins are not allowed.
             if login in self.specialUsers:
@@ -223,7 +234,7 @@ class User(Base):
         '''
         guard = guard or self.guard
         # Return the cached value on the guard when appropriate
-        if not compute and not groupsOnly and (self.login == guard.userLogin):
+        if not compute and not groupsOnly and self.login == guard.userLogin:
             return guard.userLogins
         # Compute it
         r = [group.login for group in self.groups or ()]
@@ -321,24 +332,30 @@ class User(Base):
         if user.hasRole('Owner', self): return Show.E_
 
     roles = Select(show=showRoles, indexed=True, width=40, height=10,
-                   validator=Selection(lambda o: o.model.getGrantableRoles(o)),
-                   render='checkbox', **pm)
+      fwidth='3em', validator=Selection(lambda o: o.model.getGrantableRoles(o)),
+      render='checkbox', **pm)
+
+    def addRole(self, role):
+        '''Adds p_role among p_self.roles'''
+        r = self.roles or []
+        r.append(role)
+        self.values['roles'] = r
 
     def getRoles(self, compute=False, guard=None):
-        '''Returns all the global roles granted to this user. Not simply
-           self.roles, but also "ungrantable roles" (like Anonymous or
-           Authenticated) and roles inherited from group membership.
-
-           If p_compute is False and p_self is the currently logged user, roles
-           are retrieved from the guard, that caches it. Else, they are really
-           computed.
-        '''
+        '''Returns all the global roles granted to this user'''
+        # The method returns not simply self.roles, but also "ungrantable roles"
+        # (like Anonymous or Authenticated) and roles inherited from group
+        # membership.
+        #
+        # If p_compute is False and p_self is the currently logged user, roles
+        # are retrieved from the guard, that caches it. Else, they are really
+        # computed.
         guard = guard or self.guard
         # Return the cached value on the guard when appropriate
         if not compute and self.login == guard.userLogin:
             return guard.userRoles
         # Compute it
-        r = list(self.roles)
+        r = list(self.roles or ())
         # Add ungrantable roles
         r.append('Anonymous' if self.isAnon() else 'Authenticated')
         # Add group global roles
@@ -427,8 +444,9 @@ class User(Base):
         # authentication cookie with its new password.
         if self == user:
             self.guard.Cookie.updatePassword(self.H(), password)
-        return True, self.translate('new_password_text',
+        message = self.translate('new_password_text',
                                     mapping={'password': password})
+        self.say(message, fleeting=False)
 
     def showResetPassword(self):
         '''Action "reset password" is available to anyone having write access to
@@ -499,13 +517,19 @@ class User(Base):
     #  PXs
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
 
-    # Display, in the user strip, links to the User instance of the logged user
-    pxUserLink = Px('''
-     <a if="cfg.userLink" href=":user.url">
-      <img src=":svg('user')" class="icon"/>
-      <span class="topText">:user.getFirstName()</span>
-     </a>
-     <x if="not cfg.userLink">:user.getFirstName()</x>''')
+    # Display, in the user strip, a link to the User instance of the logged user
+    pxLink = Px('''
+     <!-- The variant with a clickable username -->
+     <div if="cfg.userLink" class="textIcon">
+       <a if="showUserIcon|True" href=":user.url"><img src=":svg('user')"
+          class="icon"/></a>
+       <div>
+        <a href=":user.url"><span>:cfg.getUserText(user)</span></a>
+       </div>
+     </div>
+
+     <!-- The variant with the name not being clickable -->
+     <span if="not cfg.userLink">:cfg.getUserText(user)</span>''')
 
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
     #  Class methods
@@ -546,16 +570,13 @@ class User(Base):
             handler.log('app', 'error', 'Unreadable cookie (%s)' % str(e))
         if login: place = 'cookie'
         # b. Identify the user from HTTP basic authentication
-        if not login and ('_basic_auth_' in req):
-            # Credentials from HTTP basic authentication are present, decode it
-            creds = req._basic_auth_
+        if not login and 'Authorization' in handler.headers:
+            creds = handler.headers['Authorization']
             if creds.lower().startswith('basic '):
-                try:
-                    creds = creds.split(' ')[-1]
-                    login, password = base64.decodestring(creds).split(':', 1)
-                    if login: place = 'basic'
-                except Exception:
-                    pass
+                # Decode these credentials from HTTP basic authentication
+                creds = base64.decodebytes(creds[6:].encode()).decode()
+                login, password = creds.split(':', 1)
+                if login: place = 'basic'
         # c. Identify the user from the authentication form
         if not login:
             login = req.login
@@ -565,10 +586,10 @@ class User(Base):
             ctx = req.context
             if login: place = 'form'
         # d. Identify the user from a SSO reverse proxy
-        if not login or (login == '_s_s_o_'):
+        if not login or login == '_s_s_o_':
             sso = handler.config.security.sso
             if sso:
-                login = sso.extractUserLogin(guard)
+                login = sso.extractUserLogin(handler.headers)
                 if login: place = 'sso'
         # e. If all identification methods failed, identify the user as "anon"
         return login or 'anon', password, ctx, place
@@ -586,7 +607,7 @@ class User(Base):
             # instance normally exists, may be requested before it is created
             # (on database initialization). This is why we may create here a
             # fake one.
-            if hasattr(handler, 'connection'):
+            if hasattr(handler, 'dbConnection'):
                 user = handler.dbConnection.root.objects.get('system') or \
                        User.System()
             else:
@@ -602,6 +623,12 @@ class User(Base):
             # Sign-On) reverse proxy: its credentials were carried in HTTP
             # headers. Simply return its local copy, or create it if not found.
             user = config.sso.getUser(tool, login, createIfNotFound=True)
+            # If authentication contexts are in use, retrieve the potentially
+            # defined default context when relevant.
+            if ctx is None and authContext:
+                ctx = authContext.getDefaultContext(tool, user)
+                # Update the cookie with this default context
+                if ctx: Cookie.update(handler, ctx)
         else:
             # In any other case, authentication must be performed
             user = None
@@ -615,17 +642,22 @@ class User(Base):
             # Authentication fails (and user "anon" is returned) if the user was
             # not found or inactive, its password was invalid or the required
             # authentication context was not found.
-            if user is None or user.isAnon() or user.state == 'inactive' or \
+            if user is None or user.isAnon() or \
+               user.state in config.noLoginStates or \
                not user.getField('password').check(user, password) or \
-               (not ctx and authContext and authContext.isMandatory(tool,True))\
-               and place != 'basic':
+               (not ctx and authContext and authContext.isMandatory(tool, True)\
+                and place != 'basic'):
                 # Disable the authentication cookie
                 guard.Cookie.disable(handler)
                 user = handler.dbConnection.root.objects.get('anon')
             else:
                 # The user is authenticated. Create an authentication cookie for
-                # him. Set a default context when appropriate.
-                if authContext and not ctx:
+                # him. Set a default context when appropriate (v_ctx being None
+                # involves searching for a default context, while p_ctx being
+                # the empty string means that selecting no context at all was a
+                # positive action performed by the user: no default context must
+                # be searched in that case).
+                if authContext and ctx is None:
                     ctx = authContext.getDefaultContext(tool, user)
                 guard.Cookie.write(handler, user.login, password, ctx)
         # Set, on the guard, when present, attributes related to the
@@ -640,7 +672,7 @@ class User(Base):
         # every connected user. The user will be part of the result only if the
         # expression evaluates to True. The user is passed to the expression as
         # variable "user".
-        # ~~~
+        # ~
         # Get and count connected users
         users = list(tool.H().onlineUsers.items())
         users.sort(key=lambda u: u[1], reverse=True) # Sort by last access date
@@ -668,7 +700,7 @@ class User(Base):
         else:
             count = len(rows)
         # Build the entire table
-        r = '<table class="list"><tr><th>(%d)</th><th>%s</th></tr>' % \
+        r = '<table class="small"><tr><th>(%d)</th><th>%s</th></tr>' % \
             (count, tool.translate('last_user_access'))
         return r + '\n'.join(rows) + '</table>'
 
@@ -701,6 +733,7 @@ class User(Base):
             else:
                 # Return the clear password to the UI
                 message = _('new_password_text', mapping={'password': password})
+            self.resp.fleetingMessage = False
             # The user will need to change it at next login
             self.changePasswordAtNextLogin = True
         page = self.req.page or 'main'
