@@ -31,6 +31,31 @@ class Object:
     '''At every place we need an object, but without any requirement on its
        class (methods, attributes,...) we will use this minimalist class.'''
 
+    # Warning - This class is particular: attribute access has been modified in
+    #           such a way that AttributeError is never raised.
+    #
+    # Example:                 o = Object(a=1)
+    #
+    # >>> o.a
+    # >>> 1
+    # >>> o.b
+    # >>>                      (None is returned)
+    # >>> hasattr(o, 'a')
+    # >>> True
+    # >>> hasattr(o, 'b')
+    # >>> True                 It's like if any attribute was defined on it
+    # >>> 'a' in o
+    # >>> True
+    # >>> 'b' in o
+    # >>> False
+
+    # While this behaviour may be questionable and dangerous, it can be very
+    # practical for objects that can potentially contain any attribute (like an
+    # object representing HTTP request parameters or form values). Code getting
+    # attribute values can be shorter, freed from verbose "hasattr" calls.
+    # Conclusion: use this class at your own risk, if you know what you are
+    # doing (isn't it the Python spirit?)
+
     def __init__(self, **fields):
         for k, v in fields.items(): setattr(self, k, v)
 
@@ -55,7 +80,7 @@ class Object:
 
     def __bool__(self): return bool(self.__dict__)
     def d(self): return self.__dict__
-    def get(self, name, default=None): return getattr(self, name, default)
+    def get(self, name, default=None): return self.__dict__.get(name, default)
     def __contains__(self, k): return k in self.__dict__
     def keys(self): return self.__dict__.keys()
     def values(self): return self.__dict__.values()
@@ -67,7 +92,7 @@ class Object:
 
     def __getitem__(self, k):
         '''Dict-like access self[k] must return None if key p_k doesn't exist'''
-        return getattr(self, k, None)
+        return self.__dict__.get(k)
 
     def __delitem__(self, k):
         '''Dict-like attribute removal'''
@@ -91,10 +116,14 @@ class Object:
         other = other.__dict__ if isinstance(other, Object) else other
         for k, v in other.items(): setattr(self, k, v)
 
-    def clone(self):
+    def clone(self, **kwargs):
         '''Creates a clone from p_self'''
-        r = Object()
+        r = self.__class__() # p_self's class may be an Object sub-class
         r.update(self)
+        # Cloning can be altered by specifying values in p_kwargs, that will
+        # override those from p_self.
+        for k, v in kwargs.items():
+            r[k] = v
         return r
 
     # Allow this highly manipulated Object class to be picklable
@@ -132,18 +161,17 @@ class Hack:
     '''This class proposes methods for patching some existing code with
        alternative methods.'''
 
+    H_PATCHED = '%d method(s) patched from %s to %s: %s.'
+    H_ADDED   = '%d method(s) and/or attribute(s) added from %s to %s: %s.'
+
     @staticmethod
-    def patch(method, replacement, class_=None):
-        '''This method replaces m_method with a p_replacement method, but
-           keeps p_method on its class under name
-           "_base_<initial_method_name>_". In the patched method, one may use
-           Hack.base to call the base method. If p_method is static, you must
-           specify its class in p_class_.'''
-        # Get the class on which the surgery will take place
-        isStatic = class_
-        class_ = class_ or method.im_class
-        # On this class, store m_method under its "base" name
-        name = isStatic and method.func_name or method.im_func.__name__
+    def patch(class_, method, replacement, isStatic):
+        '''This method replaces, on p_class_, this m_method with a p_replacement
+           method (being static if p_isStatic is True), but keeps p_method on
+           p_class_ under name "_base_<initial_method_name>_".'''
+        # In the patched method, one may use method Hack.base to call the base
+        # method.
+        name = method.__name__
         baseName = '_base_%s_' % name
         if isStatic:
             # If "staticmethod" isn't called hereafter, the static functions
@@ -157,26 +185,26 @@ class Hack:
     def base(method, class_=None):
         '''Allows to call the base (replaced) method. If p_method is static,
            you must specify its p_class_.'''
-        isStatic = class_
-        class_ = class_ or method.im_class
-        name = isStatic and method.func_name or method.im_func.__name__
-        return getattr(class_, '_base_%s_' % name)
+        class_ = class_ or method.__self__.__class__
+        return getattr(class_, '_base_%s_' % method.__name__)
 
     @staticmethod
     def inject(patchClass, class_, verbose=False):
-        '''Injects any method or attribute from p_patchClass into class_.'''
+        '''Injects any method or attribute from p_patchClass into p_class_'''
         # As a preamble, inject methods and attributes from p_patchClass's base
         # classes, if any.
         for base in patchClass.__bases__:
+            if base == object: continue
             Hack.inject(base, class_, verbose=verbose)
         patched = []
-        added = []
+        added = []  
         # Inject p_patchClass' own methods and attributes
         for name, attr in patchClass.__dict__.items():
-            if name.startswith('__'): continue # Ignore special methods
+            # Ignore special methods
+            if name.startswith('__'): continue
             # Unwrap functions from static methods
-            className = attr.__class__.__name__
-            if className == 'staticmethod':
+            typeName = attr.__class__.__name__
+            if typeName == 'staticmethod':
                 attr = attr.__get__(attr)
                 static = True
             else:
@@ -188,13 +216,10 @@ class Hack:
             else:
                 hasAttr = False
                 classAttr = None
-            if hasAttr and (className != 'type') and \
-               callable(attr) and callable(classAttr):
+            if hasAttr and typeName != 'type' and callable(attr) and \
+               callable(classAttr):
                 # Patch this method via Hack.patch
-                if static:
-                    Hack.patch(classAttr, attr, class_)
-                else:
-                    Hack.patch(classAttr, attr)
+                Hack.patch(class_, classAttr, attr, static)
                 patched.append(name)
             else:
                 # Simply replace the static attr or add the new static
@@ -204,10 +229,9 @@ class Hack:
         if verbose:
             pName = patchClass.__name__
             cName = class_.__name__
-            print('%d method(s) patched from %s to %s (%s)' % \
-                  (len(patched), pName, cName, str(patched)))
-            print('%d method(s) and/or attribute(s) added from %s to %s (%s)'%\
-                  (len(added), pName, cName, str(added)))
+            print(Hack.H_PATCHED % (len(patched), pName, cName,
+                                    ', '.join(patched)))
+            print(Hack.H_ADDED % (len(added), pName, cName, ', '.join(added)))
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 def notFromPortlet(tool):
