@@ -81,7 +81,7 @@ class EventIterator:
         if self.condition is None: return True
         # Update the evaluation context when appropriate
         if self.context:
-            locals().update(context)
+            locals().update(self.context)
         return eval(self.condition)
 
     def __iter__(self): return self
@@ -272,14 +272,15 @@ class Change(Event):
      </table>''')
 
     def __init__(self, login, state, date, **params):
-        # Extract changed fields from p_params. Attribute "changes" stores, in a
-        # dict, the previous values of fields whose values have changed. If the
-        # dict's key is of type...
-        # ----------------------------------------------------------------------
-        # str   | it corresponds to the name of the field;
+        '''Extract changed fields from p_params'''
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # Attribute "changes" stores, in a dict, the previous values of fields
+        # whose values have changed. If the dict's key is of type...
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # str   | it corresponds to the name of the field; 
         # tuple | it corresponds to a tuple (s_name, s_language) and stores the
         #       | part of a multilingual field corresponding to s_language.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         self.changes = PersistentMapping(params.pop('changes'))
         Event.__init__(self, login, state, date, **params)
 
@@ -439,7 +440,7 @@ class History(PersistentList):
       }''')
 
     view = Px('''
-     <div if="not o.isTemp()"
+     <div if="not o.isTemp() and layout !='xml'"
        var2="history=o.history; hasHistory=not history.isEmpty();
              createComment=history[-1].comment">
       <table width="100%" class="header" cellpadding="0" cellspacing="0">
@@ -450,7 +451,7 @@ class History(PersistentList):
           <x>:user.getTitleFromLogin(o.creator)</x> 
 
          <!-- Creation and last modification dates -->
-         <x>:_('Base_created')</x> 
+         <x>:_('date_on')</x>
          <x var="created=o.created; modified=o.modified">
           <x>:tool.Date.format(tool, created, withHour=True)</x>
           <x if="modified != created">&mdash;
@@ -474,7 +475,7 @@ class History(PersistentList):
      </div>''',
 
      css='''
-      .header { margin-bottom:5px; background-color:#f3f3f7;
+      .header { margin:|histMargin|; background-color:#f3f3f7;
                 border:3px solid white }
       .by { padding:7px }
       .history>tbody>tr>td { padding:8px }
@@ -643,6 +644,17 @@ class History(PersistentList):
                 condition = ename in name
             if condition: return event
 
+    def getEventLogins(self, transitions):
+        '''Returns the list of user logins responsible for having triggered, for
+           the last time, p_transitions on p_self.'''
+        r = set()
+        for transition in transitions:
+            # Get the last event with this transition
+            event = self.getLast(transition)
+            if event:
+                r.add(event.login)
+        return r
+
     def setEvents(self, other):
         '''Replace p_self's events with clones from p_other's events'''
         # Empty p_self's history first
@@ -684,7 +696,8 @@ class History(PersistentList):
     xhtmlDefaultExclude = ('change', '_init_')
 
     def asXhtml(self, exclude=xhtmlDefaultExclude, userExpression=None,
-                noLabelFor=('comment',), escapeComments=True):
+                noLabelFor=('comment',), escapeComments=True,
+                chronological=False):
         '''Produce a XHTML version of this history, in the form of a discussion
            thread, suitable to be injected as various places: in methods
            getSubTitle, in PODs, etc.'''
@@ -705,7 +718,7 @@ class History(PersistentList):
         esc = Escape.xhtml
         tool = o.tool
         fmt = tool.Date.format
-        for event in self:
+        for event in self.iter(chronological=chronological):
             # Ignore inappropriate events
             type = event.getTypeName(short=True)
             if exclude and type in exclude: continue
