@@ -16,6 +16,8 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+from DateTime import DateTime
+
 from appy.px import Px
 from appy.utils import iconParts
 from appy.model.fields.group import Group
@@ -28,12 +30,14 @@ UNTRIGGERABLE = 'Transition "%s" on %s can\'t be triggered.'
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Transition:
     '''Represents a workflow transition'''
+
     class Error(Exception): pass
     traverse = {}
 
     def __init__(self, states, condition=True, action=None, show=True,
                  confirm=False, group=None, icon=None, sicon=None,
-                 redirect=None, historizeActionMessage=False, iconOnly=False):
+                 redirect=None, historizeActionMessage=False, iconOnly=False,
+                 iconOut=False, iconCss='iconS', updateModified=False):
         # In its simpler form, "states" is a list of 2 states:
         # (fromState, toState). But it can also be a list of several
         # (fromState, toState) sub-lists. This way, you may define only 1
@@ -82,6 +86,18 @@ class Transition:
         # If p_iconOnly is True, the transition will be rendered as an icon (and
         # not a button), even on the "buttons" layout.
         self.iconOnly = iconOnly
+        # If p_iconOut is True, when the transition is rendered as a
+        # "icon + button" (p_iconOnly is False), the icon will be positioned
+        # outside the button. Else, it will be integrated as a background-image
+        # inside it. Having the icon outside the button allows for more
+        # graphical possibilities (ie: the button may have a border that does
+        # not include the icon).
+        self.iconOut = iconOut
+        # The CSS class to apply to the button icon, in "icon out" mode
+        self.iconCss = iconCss
+        # If p_updateModified is True, triggering this transition is considered
+        # as a change on the object: his last modification date will be updated.
+        self.updateModified = updateModified
 
     def init(self, workflow, name):
         '''Lazy initialisation'''
@@ -294,16 +310,16 @@ class Transition:
     def trigger(self, o, comment=None, doAction=True, doHistory=True,
                 doSay=True, reindex=True, secure=True, data=None,
                 forceTarget=None):
-        '''This method triggers this transition on some p_o(bject). If
-           p_doAction is False, the action that must normally be executed after
-           the transition has been triggered will not be executed. If
-           p_doHistory is False, there will be no trace from this transition
-           triggering in p_o's history. If p_doSay is False, we consider the
-           transition as being triggered programmatically, and no message is
-           returned to the user. If p_reindex is False, object reindexing will
-           be performed by the caller method. If p_data is specified, it is a
-           dict containing custom data that will be integrated into the history
-           event.'''
+        '''This method triggers this transition on some p_o(bject)'''
+        # If p_doAction is False, the action that must normally be executed
+        # after the transition has been triggered will not be executed. If
+        # p_doHistory is False, there will be no trace from this transition
+        # triggering in p_o's history. If p_doSay is False, we consider the
+        # transition as being triggered programmatically, and no message is
+        # returned to the user. If p_reindex is False, object reindexing will be
+        # performed by the caller method. If p_data is specified, it is a dict
+        # containing custom data that will be integrated into the history event.
+        # ~
         # Is that the special _init_ transition ?
         name = self.name
         isInit = name == '_init_'
@@ -316,8 +332,12 @@ class Transition:
         # common action.
         fromState = o.state if not isInit else None
         # Add the event in the object history
-        event = o.history.add('Trigger', target.name, transition=name,
-                              comment=comment)
+        history = o.history
+        event = history.add('Trigger', target.name, transition=name,
+                            comment=comment)
+        # Update the object's last modification date when relevant
+        if self.updateModified:
+            history.modified = DateTime()
         # Execute the action that is common to all transitions, if defined. It
         # is named "onTrigger" on the workflow class by convention. This common
         # action is executed before the transition-specific action (if any).
@@ -330,8 +350,9 @@ class Transition:
         if doHistory and msg and self.historizeActionMessage:
             event.completeComment(msg)
         # Reindex the object if required. Not only security-related indexes
-        # (Allowed, State) need to be updated here.
-        if reindex and not isInit and not o.isTemp(): o.reindex()
+        # (allowed, state) need to be updated here.
+        if reindex and not isInit and not o.isTemp() and o.class_.isIndexable():
+            o.reindex()
         # Return a message to the user if needed
         if not doSay: return
         return msg or o.translate('object_saved')
@@ -357,6 +378,8 @@ class Transition:
         # If we are viewing the object and if the logged user looses the
         # permission to view it, redirect the user to its home page.
         if msg: o.say(msg)
+        # The developer may already have defined an URL to return to
+        if o.gotoSet(): return
         # Return to o/view, excepted if the object is not viewable anymore
         if o.guard.mayView(o):
             if self.redirect == 'referer':
@@ -392,49 +415,60 @@ class Transition:
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class UiTransition:
-    '''Represents a widget that displays a transition'''
+    '''Widget that displays a transition'''
+
+    # The button in itself, real or fake
+    pxButton = Px('''
+     <!-- Real button -->
+     <input if="mayTrigger" type="button" class=":css" value=":label"
+            var="back=transition.getBackHook(_ctx_)"
+            name=":tr.name" style=":transition.getIconUrl(asBG=True)"
+            onclick=":'triggerTransition(%s,this.name,%s,%s)' % \
+                       (q(formId), q(transition.confirm), back)"/>
+     <!-- Fake button -->
+     <input if="not mayTrigger" type="button" class=":'%s fake' % css"
+            style=":transition.getIconUrl(asBG=True)"
+            value=":label" title=":transition.reason"/>''')
 
     px = Px('''
      <x var="label=transition.title;
              mayTrigger=transition.mayTrigger;
              inSub=layout=='sub';
-             asIcon=not inSub or transition.iconOnly">
+             tr=transition.transition;
+             asIcon=not inSub or tr.iconOnly">
 
-      <!-- Picto or icon -->
-      <a if="asIcon" class=":'clickable' if mayTrigger else 'help fake'"
+      <!-- As picto or icon -->
+      <a if="asIcon" class=":'clickable' if mayTrigger else 'fake'"
          var="back=transition.getBackHook(_ctx_);
               iconAttr='sicon' if inSub else 'icon';
               iconBase='siconBase' if inSub else 'iconBase';
               iconRam='siconRam' if inSub else 'iconRam'"
-         name=":transition.name"
-         title=":transition.getIconTitle()"
+         name=":tr.name" title=":transition.getIconTitle()"
          onclick=":'triggerTransition(%s,this.name,%s,%s)' % 
                     (q(formId), q(transition.confirm), back) 
                     if mayTrigger else ''">
-       <img src=":url(getattr(transition, iconAttr),
-                      base=getattr(transition, iconBase),
-                      ram=getattr(transition,iconRam))"
-            class=":picto|'iconS'"/>
+       <img src=":url(getattr(tr, iconAttr), base=getattr(tr, iconBase),
+                      ram=getattr(tr,iconRam))"
+            class=":picto|tr.iconCss"/>
        <div style=":'display:%s' % config.ui.pageDisplay"
             if="not inSub">::label</div>
       </a>
 
-      <!-- Real button -->
-      <x if="not asIcon" var2="css=ui.Button.getCss(label, inSub)">
-       <input if="mayTrigger" type="button" class=":css" value=":label"
-              var="back=transition.getBackHook(_ctx_)"
-              name=":transition.name"
-              style=":url(transition.sicon, base=transition.siconBase,
-                          ram=transition.siconRam, bg='18px 18px')"
-              onclick=":'triggerTransition(%s,this.name,%s,%s)' % \
-                         (q(formId), q(transition.confirm), back)"/>
+      <!-- As button -->
+      <x if="not asIcon"
+         var2="css=ui.Button.getCss(label, inSub, iconOut=tr.iconOut)">
 
-       <!-- Fake button, explaining why the transition can't be triggered -->
-       <input if="not mayTrigger" type="button"
-              class=":'%s fake' % css"
-              style=":url(transition.sicon, base=transition.siconBase,
-                          ram=transition.siconRam, bg='18px 18px')"
-              value=":label" title=":transition.reason"/></x></x>''',
+       <!-- Variant with the icon outside the button -->
+       <div if="tr.iconOut" class="iflex1">
+        <img src=":transition.getIconUrl()"
+             class=":'%s %s'%('clickable' if mayTrigger else 'fake',tr.iconCss)"
+             onclick="this.nextSibling.click()"/>
+        <x>:transition.pxButton</x>
+       </div>
+
+       <!-- Variant with the icon inside the button -->
+       <x if="not tr.iconOut">:transition.pxButton</x>
+      </x></x>''',
 
      js='''
       // Function used for triggering a workflow transition
@@ -444,28 +478,21 @@ class UiTransition:
         submitForm(formId, msg, true, back);
       }''')
 
-    # Fields to copy from Transition to UiTransition
-    copied = ('name' , 'iconOnly' ,
-              'icon' , 'iconBase' , 'iconRam',
-              'sicon', 'siconBase', 'siconRam')
-
     def __init__(self, transition, o, mayTrigger):
+        self.o = o
         _ = o.translate
         # The tied p_transition
         self.transition = transition
-        # Copy p_transitions fields here
-        for name in UiTransition.copied:
-            setattr(self, name, getattr(transition, name))
         self.type = 'transition'
         labelId = transition.labelId
         self.title = _(labelId)
         if transition.confirm:
-            msg = o.translate('%s_confirm' % labelId, blankOnError=True) or \
-                  o.translate('action_confirm')
+            msg = _('%s_confirm' % labelId, blankOnError=True) or \
+                  _('action_confirm')
             self.confirm = msg
         else:
             self.confirm = ''
-        # May this transition be triggered via the UI?
+        # May this transition be triggered via the UI ?
         self.mayTrigger = True
         self.reason = ''
         if not mayTrigger:
@@ -474,13 +501,13 @@ class UiTransition:
         # Required by the UiGroup
         self.colspan = 1
 
-    def getBackHook(self, ctx):
+    def getBackHook(self, c):
         '''If, when the transition has been triggered, we must ajax-refresh some
            part of the page, this method will return the ID of the corresponding
            DOM node. Else (ie, the entire page needs to be refreshed), it
            returns None.'''
-        if ctx.inSub and not self.transition.redirect:
-            r = ctx.q(ctx.backHook or ctx.ohook)
+        if c.inSub and not self.transition.redirect:
+            r = c.q(c.backHook or c.ohook)
         else:
             r = 'null'
         return r
@@ -490,9 +517,21 @@ class UiTransition:
            latter is represented by an icon only.'''
         if not self.mayTrigger:
             r = self.reason
-        elif self.iconOnly:
+        elif self.transition.iconOnly:
             r = self.title
         else:
             r = ''
         return r
+
+    def getIconUrl(self, pre='s', asBG=False):
+        '''Returns the URL to p_self's (s)icon (depending on p_pre(fix)), to be
+           used as a background image or not (depending on p_asBG).'''
+        tr = self.transition
+        # In "icon out" mode, no background image must be present
+        if asBG and tr.iconOut: return ''
+        # If p_asBG, get the background image dimensions
+        bg = '18px 18px' if asBG else False
+        return self.o.buildUrl(getattr(tr, '%sicon' % pre),
+                               base=getattr(tr, '%siconBase' % pre),
+                               ram=getattr(tr, '%siconRam' % pre), bg=bg)
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
