@@ -66,9 +66,10 @@ class Peer:
     # Separator for the object counter. Allows to easily view the counter
     counterSep = '-' * 5
 
-    # Peer constructor - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+    # Constructor  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+
     def __init__(self, url, login=None, password=None, id=None, name=None,
-                 myId=None, myName=None, folder=None, timeout=10):
+                 myId=None, myName=None, folder=None, timeout=20):
         # The URL of the distant site
         self.url = url
         # An optional ID for this distant site
@@ -102,9 +103,9 @@ class Peer:
         # tricky case: an autoref being add=False,link=False,composite=True
         # will be considered addable but this could be wrong.
         self.addable = {} # ~{s_className: {s_refFieldName: b_addable}}~
-        # ~~~
+        # ~
         # The following attributes store run-time temporary data structures
-        # ~~~
+        # ~
         # Create the proxy instance representing the distant server. The passed
         # p_timeout, expressed in seconds, will be used for every HTTP request
         # sent to the peer.
@@ -138,14 +139,24 @@ class Peer:
         # considered to have the same name on both sites.
         self.classNames = {}
 
-    def get(self, url=None, params=None, post=False, raiseIfString=True):
+    def getInfo(self, check=False):
+        '''Info about a Peer instance is made from its self.url, self.id and
+           self.name.'''
+        r = [self.url]
+        for name in ('id', 'name'):
+            info = getattr(self, name)
+            if info not in r: r.append(info)
+        return r[0] if len(r) == 1 else ('%s (%s)' % (r[0], ', '.join(r[1:])))
+
+    def get(self, url=None, params=None, post=False, raiseIfString=True,
+            timeout=None):
         '''Sends a HTTP GET request (or POST if p_post is True) to p_url
            (or p_self.url if not given) with p_params.'''
         try:
             if post:
-                r = self.server.post(uri=url, data=params)
+                r = self.server.post(path=url, data=params, timeout=timeout)
             else:
-                r = self.server.get(uri=url, params=params)
+                r = self.server.get(path=url, params=params, timeout=timeout)
         except Resource.Error as re:
             raise self.Error('%s: %s' % (url or self.url, str(re)))
         if r.code == 404:
@@ -188,11 +199,11 @@ class Peer:
         distantName = self.classNames.get(className) or className
         return '%s/searchAll?className=%s' % (tool.url, distantName)
 
-    def printSymbol(self, nb):
+    def printSymbol(self, nb, total):
         '''Don't get bored while observing script execution'''
         if (nb % 100) == 0:
             # Every 100 elements, show the current element's p_nb
-            r = '%s=%d=%s' % (Peer.counterSep, nb, Peer.counterSep)
+            r = '%s=%d/%d=%s' % (Peer.counterSep, nb, total, Peer.counterSep)
         else:
             # Print a random char
             r = random.choice(self.printSymbols)
@@ -212,10 +223,11 @@ class Peer:
         class_ = tool.model.classes[className]
         importer = self.getImporter(class_)
         counts = O(current=0, created=0)
-        tool.log(ROOT_RUN % (len(urls), className))
+        total = len(urls)
+        tool.log(ROOT_RUN % (total, className))
         for url in urls:
             counts.current += 1
-            self.printSymbol(counts.current)
+            self.printSymbol(counts.current, total)
             # Get the distant object and its ID
             distant = self.get(url)
             distantId = self.getIdFromUrl(url)
@@ -223,9 +235,11 @@ class Peer:
             action, detail = importer.getAction(distant)
             if action != importer.CREATE: continue
             # Create a local object corresponding to the distant one
-            o = tool.create(className, executeMethods=False, indexIt=False)
+            id = distantId if importer.mustKeepDistantId(distantId) else None
+            o = tool.create(className, id=id, executeMethods=False,
+                            indexIt=False)
             # Fill local object's attributes from distant ones
-            importer(self, o, distant).run()
+            importer(tool, self, o, distant).run()
             # Add it to the dict of already imported objects
             self.localObjects[distantId] = o
             # Count it
@@ -242,7 +256,7 @@ class Peer:
             url = '%s/%s/xml' % (url, self.distantToolName)
             dtool = self.get(url)
             importer = self.getImporter(tool.class_)
-            importer(self, tool, dtool,
+            importer(tool, self, tool, dtool,
                      importHistory=False, importLocalRoles=False).run()
             # After the tool and its dependent objects, import root classes and
             # their dependent objects.
@@ -256,5 +270,7 @@ class Peer:
         else:
             # Resolve missing links
             self.unresolved.resolve(tool)
+            # Perform misc final tasks, like reindexing fields requiring the
+            # complete model will all Refs being completely reified.
             tool.log(IMPORT_END % (time.time() - start))
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
