@@ -16,9 +16,9 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+from appy.pod import PodError
 from appy.pod.elements import *
 from appy.model.utils import Object
-from appy.pod import PodError, Evaluator
 from appy.utils import Traceback, commercial, CommercialError
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -108,14 +108,15 @@ class Action:
            in the form "normalExpr|errorExpr". If it is the case, if the
            "normal" expression raises an error, the "error" expression is
            evaluated instead.'''
+        eval = context['_eval_'].run
         if '|' not in expr:
-            r = Evaluator.run(expr, context)
+            r = eval(expr, context)
         else:
             expr, errorExpr = expr.rsplit('|', 1)
             try:
-                r = Evaluator.run(expr, context)
+                r = eval(expr, context)
             except Exception:
-                r = Evaluator.run(errorExpr, context)
+                r = eval(errorExpr, context)
         return r
 
     def evaluateExpression(self, result, context, expr):
@@ -176,7 +177,7 @@ class Action:
             fromRes = None
             error = False
             try:
-                fromRes = Evaluator.run(self.fromExpr, context)
+                fromRes = context['_eval_'].run(self.fromExpr, context)
             except Exception as e:
                 msg = FROM_EVAL_ERROR % (self.fromExpr,self.getExceptionLine(e))
                 self.manageError(result, context, msg, e)
