@@ -122,8 +122,12 @@ class Buffer:
         self.parent = parent
         self.subBuffers = {} # ~{i_bufferIndex: Buffer}~
         self.env = env
-        # Are we computing for pod (True) or px (False)
+        # Are we computing for pod (True) or px (False) ?
         self.pod = env.__class__.__name__ != 'PxEnvironment'
+        # If the name of this buffer's main tag must be resolved via an
+        # expression (tag "z"), store this expression here: it will be reused to
+        # define the same expression when adding the end tag to the buffer.
+        self.tagExpr = None
 
     def addSubBuffer(self, subBuffer=None):
         if not subBuffer:
@@ -162,11 +166,11 @@ class Buffer:
     def patchElement(self, elem, attrs):
         '''Modifies p_elem's attributes when relevant and make them dependent of
            some variable definitions.'''
+        attrs = attrs if isinstance(attrs, dict) else attrs._attrs
         if elem == 'table:table':
             # If a variable "tableName" is defined in the context, the current
             # table will get this name instead of its predefined name. Useful,
             # ie, for naming calc sheets (which are implemented as tables).
-            attrs = attrs._attrs
             name = 'table:name'
             currentName = attrs.get(name) or getUuid(removeDots=True,prefix='T')
             attrs[name] = ':tableName|"%s"' % currentName
@@ -174,13 +178,11 @@ class Buffer:
             # Convert attribute "number-columns-repeated" of every table column
             # (or add it if it does not exist) to let the user define how he
             # will repeat table columns via variable "columnsRepeated".
-            attrs = attrs._attrs
             key = 'table:number-columns-repeated'
             columnNumber = self.env.getTable().nbOfColumns -1
             nb = (key in attrs) and attrs[key] or '1'
             attrs[key] = ':columnsRepeated[%d]|%s' % (columnNumber, nb)
         elif elem == 'table:table-cell':
-            attrs = attrs._attrs
             key = 'table:style-name'
             style = attrs.get(key)
             if style and style.startswith('S'):
@@ -196,9 +198,8 @@ class Buffer:
             elif self.env.parser.caller.protection:
                 self.patchProtected(elem, attrs)
         elif elem == 'text:section' and self.env.parser.caller.protection:
-            self.patchProtected(elem, attrs._attrs)
+            self.patchProtected(elem, attrs)
         elif elem == 'text:list-item' and 'text:start-value' in attrs:
-            attrs = attrs._attrs
             val = int(attrs['text:start-value'])
             attrs['text:start-value'] = ':loop._all_[-1].nb+%d|%d' % (val, val)
 
@@ -217,12 +218,18 @@ class Buffer:
                    returns False, must not be dumped at all. In this case,
                    p_hook must be a tuple (s_attrName, s_expr).
         '''
-        self.write('<%s' % elem)
+        # With PX, the special "z" tag is being computed by an expression
+        if not self.pod and elem == 'z':
+            self.write('<')
+            self.tagExpr = tagExpr = attrs.get('z', "'div'") or "'div'"
+            self.addExpression(tagExpr)
+        else:
+            self.write('<%s' % elem)
         # Some table elements must be patched (pod only)
         if self.pod: self.patchElement(elem, attrs)
         for name, value in attrs.items():
-            if ignoreAttrs and (name in ignoreAttrs): continue
-            if renamedAttrs and (name in renamedAttrs): name=renamedAttrs[name]
+            if ignoreAttrs and name in ignoreAttrs: continue
+            if renamedAttrs and name in renamedAttrs: name=renamedAttrs[name]
             # If the value begins with ':', it is a Python expression. Else,
             # it is a static value.
             if not value.startswith(':'):
@@ -243,7 +250,15 @@ class Buffer:
         return res
 
     def dumpEndElement(self, elem):
-        self.write('</%s>' % elem)
+        '''Dump this end p_elem into p_self'''
+        if not self.pod and elem == 'z':
+            # For a "z" tag, the effective tag name depends on an expression
+            self.write('</')
+            self.addExpression(self.tagExpr)
+            self.tagExpr = None # Remove it: it has been "consumed"
+            self.write(' >')
+        else:
+            self.write('</%s>' % elem)
 
     def dumpElement(self, elem, content=None, attrs={}):
         '''For dumping a whole element at once'''
@@ -381,7 +396,7 @@ class MemoryBuffer(Buffer):
         buf = i = None
         if afterClosing:
             buf, i = self.getInsertIndex('</%s>' % afterClosing, opening=False)
-        if after and (buf is None):
+        if after and buf is None:
             buf, i = self.getInsertIndex('<%s' % after, opening=True)
         if buf is None:
             # Insert p_subBuffer after the main element
@@ -410,7 +425,7 @@ class MemoryBuffer(Buffer):
 
     def getLength(self): return len(self.content)
 
-    def write(self, thing): self.content += thing
+    def write(self, thing): self.content += thing or ''
 
     def getIndex(self, podElemName):
         res = -1
@@ -462,7 +477,7 @@ class MemoryBuffer(Buffer):
         del self.elements[elemIndex]
 
     def pushSubBuffer(self, subBuffer):
-        '''Sets p_subBuffer at the very end of the buffer.'''
+        '''Sets p_subBuffer at the very end of the buffer'''
         subIndex = None
         for index, aSubBuffer in self.subBuffers.items():
             if aSubBuffer == subBuffer:
@@ -517,14 +532,14 @@ class MemoryBuffer(Buffer):
         # Get the meta-condition if found
         if elem and elem.attrs:
             metaCondition = elem.attrs.get('text:condition')
-            if metaCondition and (metaCondition.lower() != 'ooow:true'):
-                metaCondition = metaCondition.split(':', 1)[1]
-                metaWrap = metaCondition[0]
-                if (metaWrap != metaCondition[-1]) or \
-                   (metaWrap not in expr.metaWraps):
-                    raise ParsingError(MTA_C_KO % metaCondition)
-                expr.metaWrap = metaWrap
-                expr.metaCondition = metaCondition.strip('"\'"')
+            if metaCondition:
+                meta = metaCondition.lstrip('ooow:')
+                if meta and meta.lower() != 'true':
+                    metaWrap = meta[0]
+                    if metaWrap != meta[-1] or metaWrap not in expr.metaWraps:
+                        raise ParsingError(MTA_C_KO % meta)
+                    expr.metaWrap = metaWrap
+                    expr.metaCondition = meta.strip('"\'"')
         if tiedHook: tiedHook.tiedExpression = expr
         self.elements[self.getLength()] = expr
         # To be sure that an expr and an elem can't be found at the same index
@@ -898,8 +913,7 @@ class MemoryBuffer(Buffer):
                         else:
                             raise actions.EvaluationError(e, EVAL_KO % \
                                         (evalEntry.expr, '\n'+Traceback.get(5)))
-                elif isinstance(evalEntry, Attributes) or \
-                     isinstance(evalEntry, Attribute):
+                elif isinstance(evalEntry, (Attributes, Attribute)):
                     result.write(evalEntry.evaluate(context))
                 else: # It is a subBuffer
                     if evalEntry.action:
