@@ -630,11 +630,17 @@ class ImageImporter(DocImporter):
         else:
             styleInfo = ''
         name = getUuid(prefix='I', removeDots=True)
-        image = '<%s:frame %s%s:name="%s" %s:z-index="0" ' \
-                '%s:anchor-type="%s"%s><%s:image %s:type="simple" ' \
-                '%s:show="embed" %s:href="%s" %s:actuate="onLoad"/>' \
-                '</%s:frame>' % (d, styleInfo, d, name, d, t, self.anchor,
-                self.getImageSize(), d, x, x, x, imagePath, x, d)
+        # p_self.anchor is specific to an ODT template. If the image is to be
+        # dumped into an ODS result, it will have implicit "to-cell" anchor.
+        if self.renderer.templateType == 'ods':
+            anchor = ''
+        else:
+            anchor = ' %s:anchor-type="%s"' % (t, self.anchor)
+        image = '<%s:frame %s%s:name="%s" %s:z-index="0" %s%s>' \
+                '<%s:image %s:type="simple" %s:show="embed" %s:href="%s" ' \
+                '%s:actuate="onLoad"/></%s:frame>' % \
+                (d, styleInfo, d, name, d, anchor, self.getImageSize(), d, x, x,
+                 x, imagePath, x, d)
         if hasattr(self, 'wrapInPara') and self.wrapInPara:
             style = isinstance(self.wrapInPara, str) and \
                     (' text:style-name="%s"' % self.wrapInPara) or ''
