@@ -23,11 +23,11 @@ import zipfile, shutil, xml.sax, os, os.path, re, mimetypes, time
 import appy.pod
 from appy import utils
 from appy.xml import Tag
-from appy.pod import PodError
 from appy.xml.escape import Escape
 from appy.pod.lo_pool import LoPool
 from appy.utils.zip import unzip, zip
 from appy.pod.buffers import FileBuffer
+from appy.pod import PodError, Evaluator
 from appy.utils.path import FolderDeleter
 from appy.pod.converter import FILE_TYPES
 from appy.pod import styles_manager as sm
@@ -121,14 +121,14 @@ class Renderer:
 
     def __init__(self, template, context, result, pythonWithUnoPath=None,
       ooServer='localhost', ooPort=2002, stream='auto', stylesMapping={},
-      html=False, forceOoCall=False, finalizeFunction=None,
-      overwriteExisting=False, raiseOnError=False, findImage=None,
-      rotateImages=False, stylesTemplate=None, optimalColumnWidths=False,
-      distributeColumns=False, script=None, managePageStyles=None,
-      resolveFields=False, expressionsHolders=defaultExpressionsHolders,
-      metadata=True, pdfOptions='ExportNotes=True', csvOptions=None,
-      deleteTempFolder=True, protection=False, pageStart=1, tabbedCR=False,
-      fonts=None, loPool=None):
+      stylesOutlineDeltas=None, html=False, forceOoCall=False,
+      finalizeFunction=None, overwriteExisting=False, raiseOnError=False,
+      findImage=None, rotateImages=False, stylesTemplate=None,
+      optimalColumnWidths=False, distributeColumns=False, script=None,
+      evaluator=None, managePageStyles=None, resolveFields=False,
+      expressionsHolders=defaultExpressionsHolders, metadata=True,
+      pdfOptions='ExportNotes=True', csvOptions=None, deleteTempFolder=True,
+      protection=False, pageStart=1, tabbedCR=False, fonts=None, loPool=None):
         '''Base on a document template (whose path is in p_template), which is
            an ODT or ODS file containing special expressions and statements
            written in Python, this renderer generates an ODT file (whose path is
@@ -157,31 +157,34 @@ class Renderer:
         # up-and-running.
 
         # p_stream dictates the communication between the renderer and LO.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # p_stream can hold the following values.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # "auto"     | if p_ooServer is "localhost", exchanging the ODT result
         # (default)  | produced by the renderer (=the input) and the converted
         #            | file produced by LO (=the output) will be made via files
         #            | on disk. Else, the renderer will consider that LO runs on
         #            | another machine and the exchange will be made via streams
         #            | over the network.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # True       | If you want to force communication of the input and
         #            | output files as streams, use stream=True.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # False      | If you want to force communication of the input and
         #            | output files via the disk, use stream=False.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # "in"       | The input is streamed and the output is written on disk.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # "out"      | The input is read from disk and the output is streamed.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-        # If you plan to make "XHTML to OpenDocument" conversions (via the POD
-        # function "xhtml"), you may specify a "styles mapping" in
-        # p_stylesMapping. Moreover, if the input(s) to this function is valid
-        # HTML but not valid XHTML, specify p_html being True.
+        # If you plan to make "XHTML to OpenDocument" conversions, via the POD
+        # function "xhtml", you may specify (a) a "styles mapping" in
+        # p_stylesMapping. You can also (b) alter the outline level of every
+        # style found in your POD template, via dict p_stylesOutlineDeltas,
+        # whose keys are style names and values are deltas to apply to their
+        # outline levels. Finally, (c), if the input(s) to function "xhtml" is
+        # valid HTML but not valid XHTML, specify p_html being True.
 
         # If you specify a function in p_finalizeFunction (or a list/tuple of
         # functions), it will be called by the renderer before re-zipping the
@@ -246,16 +249,28 @@ class Renderer:
         # information, see appy/pod/converter.py, option "-s". Note that when
         # such p_script is specified, p_forceOoCall is forced to True.
 
+        # By default, POD expressions and statement parts are evaluated using
+        # the Python built-in "eval" function. If you consider this to be a
+        # security problem (are POD templates written by external people ? by
+        # advanced users ?), you can use pod in conjunction with the third-party
+        # module named RestrictedPython:
+        #
+        #              https://restrictedpython.readthedocs.io
+        #
+        # In order to do so, pass, in attribute p_evaluator, an instance of
+        # class appy.pod.restricted.Evaluator. For more information and options,
+        # consult appy.pod.restricted.py.
+
         # If this document is a sub-document to be included in a master one, it
         # has sense to set a specific value for parameter p_managePageStyles:
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  "rename" | will rename all page styles with unique names. This way,
         #           | when imported into a master document, no name clash will
         #           | occur and elements tied to page styles (like headers and
         #           | footers) will be correctly imported for all included
         #           | sub-documents. The "do... pod" statement automatically
         #           | sets this parameter to "rename";
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #   <int>   | will rename page styles similarly to "rename" and will
         #           | also restart, in the sub-document, page numbering at this
         #           | integer value. It will only work if page styles are
@@ -266,7 +281,7 @@ class Renderer:
         #           | "default style". To check if it has worked, click inside
         #           | the first paragraph in the page, open its properties and
         #           | check that a page break has been defined on it.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
         # If you want to replace (some) fields with their "hard-coded" values,
         # use p_resolveFields. It can be useful, for instance, if the POD result
@@ -366,6 +381,7 @@ class Renderer:
         self.optimalColumnWidths = optimalColumnWidths
         self.distributeColumns = distributeColumns
         self.script = script
+        self.evaluator = evaluator
         self.managePageStyles = managePageStyles
         self.resolveFields = resolveFields
         self.expressionsHolders = expressionsHolders
@@ -394,6 +410,7 @@ class Renderer:
         self.contentXml = info['content.xml']
         self.stylesXml = info['styles.xml']
         # Manage the styles defined into the ODT template
+        self.stylesOutlineDeltas = stylesOutlineDeltas
         self.stylesManager = sm.StylesManager(self)
         # From LibreOffice 3.5, it is not possible anymore to dump errors into
         # the resulting ods as annotations. Indeed, annotations can't reside
@@ -424,7 +441,7 @@ class Renderer:
     # Attributes to clone to a sub-renderer when using m_clone hereafter
     cloneAttributes = ('html', 'raiseOnError', 'findImage', 'rotateImages',
       'stylesTemplate', 'optimalColumnWidths', 'distributeColumns',
-      'expressionsHolders', 'protection', 'tabbedCR', 'fonts')
+      'expressionsHolders', 'protection', 'tabbedCR', 'fonts', 'evaluator')
 
     def clone(self, template, context, result, **params):
         '''Creates another Renderer instance, similar to p_self, but for
@@ -471,8 +488,11 @@ class Renderer:
           'NumberedProperties': sm.NumberedProperties,
           'pageBreak': self.insertPageBreak,
           'columnBreak': self.insertColumnBreak,
+          '_eval_': self.evaluator or Evaluator,
           # Variables to use for representing pod-reserved chars
           'PIPE': '|', 'SEMICOLON': ';'})
+        # Add Evaluator-specific entries
+        r['_eval_'].updateContext(r)
         # Insert the context as an entry of itself
         if '_ctx_' not in r: r['_ctx_'] = r
         return r
@@ -520,13 +540,13 @@ class Renderer:
         # signature, even if there is still space available on the previous one.
 
         # p_keepWithNext may hold the following values.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  0  | (the default) Keep-with-next functionality is disabled.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  1  | Keep-with-next is enabled as described hereabove: p_s's last
         #     | paragraph receives LibreOffice property "keep-with-next". It
         #     | also works if the last paragraph is a bulleted or numbered item.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  >1 | If p_keepWithNext is higher than 1, it represents a number of
         #     | characters to "keep-with-next". Indeed, in some cases, keeping
         #     | only the last paragraph may not be sufficient: a bit more text
@@ -563,7 +583,7 @@ class Renderer:
         #     | tables that way requires LibreOffice running in server mode, as
         #     | soon as you specify p_keepWithNext > 1, POD wil assume
         #     | LibreOffice runs in server mode.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
         # If p_keepImagesRatio is True, while importing images from "img" tags
         # within p_s, their width/height ratio will be kept. Note that in most
@@ -620,14 +640,14 @@ class Renderer:
         # the same name is found on the ODF template, it will be applied to the
         # corresponding paragraph. The CSS class:
 
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # defined in      | will be
         # attribute ...   | applied to ...
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # p_firstCss      | the ODF result's first paragraph;
         # p_lastCss       | the ODF result's last paragraph;
         # p_otherCss      | any other paragraph in between.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
         # Moreover, because, internally, p_s is converted to a chunk of XHTML
         # code, and passed to method m_renderXhtml, a p_stylesMapping can be
@@ -687,22 +707,29 @@ class Renderer:
         # p_anchor, p_wrapInPara, p_size, p_sizeUnit, p_style and p_keepRatio
         # are only relevant for images:
         # * p_anchor defines the way the image is anchored into the document;
-        #       Valid values are 'page', 'paragraph', 'char' and 'as-char';
+        #       valid values are 'page', 'paragraph', 'char' and 'as-char', but
+        #       apply only in ODT templates. In an ODS template, implicit anchor
+        #       will be "to cell" and p_anchor will be ignored. In an ODS
+        #       template, anchoring "to the page" is currently not supported ;
         # * p_wrapInPara, if true, wraps the resulting 'image' tag into a 'p'
-        #       tag;
+        #       tag. Do NOT use this when importing an image into an ODS result
+        #       via this method. Here is an example of image import ito an ODS
+        #       file that will work:
+        #         do cell
+        #         from+ document(at='/some/path/a.png', wrapInPara=False)
         # * p_size, if specified, is a tuple of float or integers (width,
         #       height) expressing size in p_sizeUnit (see below). If not
-        #       specified, size will be computed from image info;
+        #       specified, size will be computed from image info ;
         # * p_sizeUnit is the unit for p_size elements, it can be "cm"
         #       (centimeters), "px" (pixels) or "pc" (percentage). Percentages,
-        #       in p_size, must be expressed as integers from 1 to 100.
+        #       in p_size, must be expressed as integers from 1 to 100 ;
         # * if p_maxWidth is specified (as a float value representing cm),
         #       image's width will not be greater than it. If p_maxWidth is
         #       "page" (the default), p_maxWidth will be computed as the width
-        #       of the main document's page style, margins excluded.
+        #       of the main document's page style, margins excluded ;
         # * if p_style is given, it is a appy.shared.css.CssStyles instance,
         #       containing CSS attributes. If "width" and "heigth" attributes
-        #       are found there, they will override p_size and p_sizeUnit.
+        #       are found there, they will override p_size and p_sizeUnit ;
         # * If p_keepRatio is True, the image width/height ratio will be kept
         #       when p_size is specified.
 
@@ -732,7 +759,7 @@ class Renderer:
         # your function does not return a string containing the convert options,
         # no conversion will occur.
 
-        # ~~~
+        # ~
         # Is there something to import ?
         if not content and not at: raise PodError(DOC_KO)
         # Convert p_at to a string if it is not the case
@@ -747,6 +774,7 @@ class Renderer:
             # If format is a mimeType, convert it to an extension
             if format in utils.mimeTypesExts:
                 format = utils.mimeTypesExts[format]
+        format = format.lower()
         isImage = isOdt = False
         importer = None
         if format in self.ooFormats:
@@ -816,7 +844,7 @@ class Renderer:
         # By default, if p_forceOoCall is True for p_self, a sub-renderer ran by
         # a c_PodImporter will inherit from this attribute, excepted if
         # parameter p_forceOoCall is different from INHERIT.
-        # ~~~
+        # ~
         # Is there a pod template defined ?
         if not content and not at: raise PodError(DOC_KO)
         imp = importers.PodImporter(content, at, format, self)
@@ -1013,7 +1041,7 @@ class Renderer:
         pageStyles = None
         mps = self.managePageStyles
         if mps is not None:
-            pageStyles = self.stylesManager.pageStyles.init(mps)
+            pageStyles = self.stylesManager.pageStyles.init(mps, self.template)
         # Patch styles.xml and content.xml
         dynamic = self.stylesManager.dynamicStyles
         for name in ('styles', 'content'):
