@@ -18,12 +18,13 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+import re, os.path
 from collections import UserDict
-import re, os.path, random, string
 
 from appy import utils
 from appy.pod import *
 from appy.utils import css
+from appy.utils.string import randomName
 from appy.pod.odf_parser import OdfEnvironment, OdfParser
 
 # Possible states for the styles parser
@@ -32,6 +33,9 @@ PARSING_STYLE    = 1 # Parsing a style definition
 PARSING_M_STYLES = 2 # Parsing section "master-styles"
 PARSING_P_LAYOUT = 3 # Parsing a page layout
 
+# Dict types
+dictTypes = (dict, UserDict)
+
 # Error-related constants  - - - - - - - - - - - - - - - - - - - - - - - - - - -
 MAP_KO     = 'The styles mapping must be a dictionary or a UserDict instance.'
 KEY_STR_KO = "The styles mapping dictionary's keys must be strings."
@@ -54,6 +58,10 @@ P_ODT_TXT  = 'For XHTML element "%s", you must associate a paragraph-wide ' \
 TXT_ODT_P  = 'For XHTML element "%s", you must associate an OpenDocument ' \
              '"text" style (that applies to only a chunk of text within a ' \
              'paragraph). "%s" is a paragraph-wide style.'
+IMPLIC_PS  = '%s: no explicit page style is defined in it, but attribute ' \
+             '"managePageStyles" has been set with an integer value. In that ' \
+             'case, you must explicitly define the default page style on the ' \
+             'first page of your POD template.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Properties:
@@ -329,6 +337,14 @@ class Style:
         self.fontSizeUnit = None # May be pt, %, ...
         # Were the styles lies within styles and substyles hierarchy
         self.outlineLevel = outlineLevel
+        # A possible delta to apply to the outline level. Do not confuse this
+        # notion of delta, that applies when the outline level needs to be
+        # dumped as attribute to a "text:h" paragraph whose style is p_self,
+        # with the notion of delta being in use for a styles mapping (key "h*"),
+        # allowing to fine-tune the mapping between outline levels implicitly
+        # expressed on XHTML tags ("h1", "h2", etc) and ODF styles's outline
+        # levels.
+        self.outlineDelta = 0
         # Namespace for the ODF "style-name" attribute corresponding to this
         # style
         self.styleNameNs = (family == 'table-cell') and 'table' or 'text'
@@ -362,11 +378,12 @@ class Style:
             res = ' %s:style-name="%s"' % (self.styleNameNs, self.name)
         # Outline level when relevant
         if self.outlineLevel is not None:
-            res += ' text:outline-level="%d"' % self.outlineLevel
+            level = self.outlineLevel + self.outlineDelta
+            res += ' text:outline-level="%d"' % level
         # Colspan and rowspan when relevant
-        if attrs and ('colspan' in attrs):
+        if attrs and 'colspan' in attrs:
             res += ' table:number-columns-spanned="%s"' % attrs['colspan']
-        if attrs and ('rowspan' in attrs):
+        if attrs and 'rowspan' in attrs:
             res += ' table:number-rows-spanned="%s"' % attrs['rowspan']
         # Additional parameters as stored in self.defaults
         if withDefaults and self.defaults:
@@ -478,8 +495,10 @@ class PageStyle:
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class PageStyles:
     '''Stores all page styles found in a document'''
+
     # Rex for parsing a page style definition in styles.xml
     STYLES_DEF = re.compile('<style:master-page\s+style:name="(.*?)"\s+.*?>')
+
     # Rex for parsing a tag, in content.xml, that refers to a page style
     CONTENT_USE = re.compile('<style:style([^<]+)style:master-page-name='\
                              '"([^"]+)"(.*?)</style:style>')
@@ -493,15 +512,18 @@ class PageStyles:
         # the same renderer several times (ie, when re-importing the same
         # sub-pod), we must avoid calling m_init more than once.
         self.initialised = False
+        # The name of the current pod template, for use in error messages
+        self.template = None
 
     def add(self, style):
         '''Adds a newly parsed page p_style'''
         self.styles[style.name] = style
 
-    def init(self, managePageStyles):
+    def init(self, managePageStyles, template):
         '''Post-initialisation'''
         # Do not perform this if initialisation already occurred
         if self.initialised: return
+        self.template = template
         # Take p_managePageStyles into account
         if isinstance(managePageStyles, int):
             self.startNumber = managePageStyles
@@ -550,6 +572,10 @@ class PageStyles:
             self.first = True
             # Replace style uses with their renamed versions
             r = self.CONTENT_USE.sub(self.getNewStyleUse, r)
+            # Raise an exception if no replacement has been made, but a start
+            # number was specified.
+            if self.startNumber is not None and self.first:
+                raise Exception(IMPLIC_PS % self.template)
         return r
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -986,11 +1012,14 @@ class DynamicStyles:
 class StylesGenerator:
     '''Analyse, for a given XHTML tag, its attributes (including CSS attributes
        within the "style" attribute) and possibly generate a custom style.'''
+
     suffix = 'pod'
+
     # Map HTML tags to ODF style families. Any tag not present in this list is
     # supposed to have family "text".
     styleFamilies = {'p': 'paragraph', 'div': 'paragraph', 'td': 'table-cell',
                      'th': 'table-cell'}
+
     # For paragraph styles, there are 2 kinds of properties: paragraph and text
     # properties. Any property not being listed as a text property below will be
     # considered as a paragraph property.
@@ -1003,6 +1032,7 @@ class StylesGenerator:
       'fo:background-color': {'para': False, 'list': False, 0: True},
       'style:text-line-through-style': True, 'style:text-overline-style': True,
       'style:text-position': True}
+
     # Properties applying to table cells and that are not transferred to inner
     # paragraphs.    
     cellProperties = ('fo:padding', 'fo:border', 'fo:background-color',
@@ -1017,9 +1047,7 @@ class StylesGenerator:
         # From one renderer execution to another, we need to produce styles
         # having unique names. Indeed, when LO incorporates sub-pods into a
         # master pod, homonym syles are considered to be exactly the same.
-        aA = string.ascii_letters
-        choice = random.choice
-        self.prefix = '%s%s%s' % (choice(aA), choice(aA), choice(aA))
+        self.prefix = randomName()
         # The number attributed to the last generated style
         self.last = 0
         # The names of the styles that were already generated, keyed by some
@@ -1257,6 +1285,7 @@ class StylesManager:
     # Valid value types for some keys within style mappings
     mappingValueTypes = {'h*': int, 'table': TableProperties,
                          'ol': NumberedProperties, 'ul': BulletedProperties}
+
     def __init__(self, renderer):
         self.renderer = renderer
         self.stylesString = renderer.stylesXml
@@ -1277,6 +1306,12 @@ class StylesManager:
         self.stylesGenerator = StylesGenerator(self)
         # Dynamically-generated styles
         self.dynamicStyles = DynamicStyles()
+        # Apply outline deltas, if specified
+        if renderer.stylesOutlineDeltas:
+            styles = self.styles
+            for name, delta in renderer.stylesOutlineDeltas.items():
+                if name in styles:
+                    styles[name].outlineDelta = delta
 
     def checkStylesAdequation(self, htmlStyle, odtStyle):
         '''Checks that p_odtStyle may be used for style p_htmlStyle'''
@@ -1308,9 +1343,9 @@ class StylesManager:
            internal representation of it.'''
 
         # p_stylesMapping is a dict.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # Every key can be:
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # (1) | the name of a XHTML 'paragraph-like' tag (p, h1, h2...) or the
         #     | meta-name "para", representing "p", "div", "blockquote" or
         #     | "address";
@@ -1319,27 +1354,27 @@ class StylesManager:
         # (4) | string 'h*';
         # (5) | 'table';
         # (6) | 'ol' or 'ul'.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # Every value must be:
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # (a) | if the key is (1), (2) or (3), value must be the display name of
         #     | an ODT style;
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # (b) | if the key is (4), value must be an integer indicating how to
         #     | map the outline level of outlined styles (ie, for mapping XHTML
         #     | tag "h1" to the OD style with outline-level=2, value must be
         #     | integer "1". In that case, h2 will be mapped to the ODT style
         #     | with outline-level=3, etc.). Note that this value can also be
         #     | negative;
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # (c) | if key is "table", the value must be an instance of
         #     | appy.pod.styles_manager.c_TableProperties;
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # (d) | if key is "ol", the value must be an instance of
         #     | appy.pod.styles_manager.c_NumberedProperties;
         #     | if key is "ul", the value must be an instance of
         #     | appy.pod.styles_manager.c_BulletedProperties.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # Some precisions now about about keys. If key is (1) or (2), parameters
         # can be given between square brackets. Every such parameter represents
         # a CSS attribute and its value, with 2 possible operators: = and !=.
@@ -1371,33 +1406,32 @@ class StylesManager:
         #
         # The method returns a dict which is the internal representation of the
         # styles mapping.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # Every key can be:
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # (I)   | the name of a XHTML tag, corresponding to (1), (2), (5) or (6)
         #       | whose potential parameters have been removed;
         # (II)  | the name of a CSS class (=(3));
         # (III) | string 'h*' (=(4)).
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # Every value can be:
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # (i)   | a Style instance that was found from the specified ODT style
         #       | display name in p_stylesMapping, if key is (I) and if only one
         #       | non-parameterized XHTML tag was defined in p_stylesMapping;
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # (ii)  | a list of the form [ (params, Style), (params, Style),...] if
         #       | key is (I) and if one or more parameterized (or not) XHTML
         #       | tags representing the same tag were found in p_stylesMapping.
         #       | "params", which can be None, is a dict whose pairs are of the
         #       | form (cssAttribute, cssValue);
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # (iii) | an integer value (=(b));
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # (iv)  | [x]Properties instance if cases (5) or (6).
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         res = {}
-        if not isinstance(stylesMapping, dict) and \
-           not isinstance(stylesMapping, UserDict):
+        if not isinstance(stylesMapping, dictTypes):
             raise PodError(MAP_KO)
         for xhtmlStyleName, odtStyleName in stylesMapping.items():
             if not isinstance(xhtmlStyleName, str):
