@@ -6,7 +6,7 @@ xhtmlInput = '''
   <p>- sub-paragraph, row 1</p>
   <p>- sub-paragraph, row 2</p>
  </li>
- <li><p>row 3;</p></li>
+ <li><p class="Signature">row 3 (Signature style applied);</p></li>
  <li>row 4:
    <ul>
      <li>Sub list:<p>aa</p><div>bb</div><div>aa</div></li>
