@@ -1334,19 +1334,19 @@ class XhtmlPreprocessor:
                (class_.prePara, content.replace('\n', class_.preParaN))
 
     @classmethod
-    def preprocess(class_, s, html=False, pre=True, inject=False):
+    def preprocess(class_, s, html=False, pre=True, inject=False, root='p'):
         '''Converts string p_s to valid XHTML and r_eturns it'''
 
         # If p_html is True, p_s is supposed to be valid HTML: void tags must be
         # converted to XHTML's self-closing start tags. If p_pre is True, "pre"
         # tags are converted to one-cell tables. If p_inject is True,
-        # injections, as defined by the appy Rich field, are performed.
+        # injections, as defined by the appy Rich or Poor field, are performed.
 
         if s is None: s = ''
         # Surround p_s with a tag in order to get a XML-compliant file (we need
         # a root tag). We also remove special blank chars that produce SAX
         # parsing errors.
-        s = '<p>%s</p>' % s.replace('\f', '').replace('\v', '')
+        s = '<%s>%s</%s>' % (root, s.replace('\f', '').replace('\v', ''), root)
         # Convert, when required, HTML void tags to XHTML self-closing tags
         if html: s = class_.voidTag.sub(class_.patchTag, s)
         # Integrate, when required, chunks of code files
