@@ -23,6 +23,7 @@
 import xml.sax
 
 from appy.utils import css
+from appy.pod import Evaluator
 from appy.model.utils import Object as O
 from appy.pod.buffers import MemoryBuffer
 from appy.xml import xmlPrologue, xhtmlPrologue
@@ -172,28 +173,25 @@ class Px:
         parsingError._msg += '\n%s' % '\n'.join(lines)
 
     def __call__(self, context, applyTemplate=True, isTemplate=False):
-        '''Renders the PX.
-
-           If the PX is based on a template PX, we have 2 possibilities.
-           1. p_applyTemplate is True. This case corresponds to the initial
-              call to the current PX. In this case we call the template with a
-              context containing, in the hook variable, the current PX.
-           2. p_applyTemplate is False. In this case, we are currently executing
-              the PX template, and, at the hook, we must include the current PX,
-              as is, without re-applying the template (else, an infinite
-              recursion would occur).
-        '''
-        # Developer, forget the following line
-        if '_ctx_' not in context: context['_ctx_'] = context
-
+        '''Renders the PX'''
+        # If the PX is based on a template PX, we have 2 possibilities.
+        # 1. p_applyTemplate is True. This case corresponds to the initial call
+        #    to the current PX. In this case we call the template with a context
+        #    containing, in the hook variable, the current PX.
+        # 2. p_applyTemplate is False. In this case, we are currently executing
+        #    the PX template, and, at the hook, we must include the current PX,
+        #    as is, without re-applying the template (else, an infinite
+        #    recursion would occur).
+        # ~
+        # Ensure PX-specific keys are in the context
+        context['_ctx_'] = context
+        context['_eval_'] = Evaluator
         # Include, in the context, variable names for "reserved" PX chars
-        if 'PIPE' not in context:
-            context['PIPE'] = '|'
-            context['SEMICOLON'] = ';'
+        context['PIPE'] = '|'
+        context['SEMICOLON'] = ';'
 
         # The first PX to be called will be stored as key "_px_", while the
-        # currently called (sub-)PX will be stored at key "_cpx_" ("c" stands
-        # for "current").
+        # *c*urrently called (sub-)PX will be stored at key "_cpx_".
         px = context.get('_px_')
         if not isTemplate:
             if px is None: # This is the first PX to be called
@@ -233,13 +231,19 @@ class Px:
             else:
                 rt[pxId] = 1
                 # This is the first time we execute it: include CSS and JS code
-                # if present.
-                if self.js: r = ('<script>%s</script>\n' % self.js) + r
-                css = self.css
-                if css:
-                    # Patch CSS if a CSS patcher is found
-                    css = context['_css_'](css) if '_css_' in context else css
-                    r = ('<style>%s</style>\n' % css) + r
+                # if present, excepted if the are executing the first PX in the
+                # context of an Ajax request: in that case, JS and CSS, if any,
+                # are already present, just above the ajax-refreshed zone.
+                if px is None and context.get('ajax'):
+                    pass
+                else:
+                    if self.js: r = ('<script>%s</script>\n' % self.js) + r
+                    css = self.css
+                    if css:
+                        # Patch CSS if a CSS patcher is found
+                        css = context['_css_'](css) if '_css_' in context \
+                                                    else css
+                        r = ('<style>%s</style>\n' % css) + r
             # Include the prologue
             if self.prologue:
                 r = self.prologue + r
@@ -391,13 +395,15 @@ class Px:
         return value
 
     @staticmethod
-    def truncateText(text, width=20, suffix='...', tag='abbr'):
+    def truncateText(text, width=20, suffix='...', tag='abbr', css=None):
         '''Truncates p_text to max p_width chars. If the text is longer than
-           p_width, the truncated part is put in a "abbr" xhtml tag.'''
+           p_width, the truncated part is put in a xhtml p_tag.'''
+        # Optional p_css class(es) are applied to the p_tag
         width = width or 20 # p_width could be None
+        css = ' class="%s"' % css if css else ''
         if len(text) > width:
-            text = '<%s title="%s">%s%s</%s>' % \
-                   (tag, text, text[:width], suffix, tag)
+            text = '<%s title="%s"%s>%s%s</%s>' % \
+                   (tag, text, css, text[:width], suffix, tag)
         return text
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
