@@ -16,9 +16,8 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-from appy.xml import Environment, Parser
 from appy.pod.buffers import MemoryBuffer
-from appy.pod.xhtml.tags import SELF_CLOSING
+from appy.xml import Environment, Parser, XHTML_SC
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class PxEnvironment(Environment):
@@ -64,8 +63,16 @@ class PxEnvironment(Environment):
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class PxParser(Parser):
     '''PX parser that is specific for parsing PX data'''
+
+    # XML attributes being specific to PX
     pxAttributes = ('var', 'for', 'if', 'var2')
-    noDumpTags = ('selected', 'checked', 'disabled', 'multiple', 'readonly')
+
+    # PX attributes that must not be part of the reslt
+    pxIgnorable = pxAttributes + ('z',)
+
+    # XHTML attributes that could not be dumped, depending on their value
+    noDump = ('selected', 'checked', 'disabled', 'multiple', 'readonly')
+
     # The following dict allows to convert attrs "lfor" to "for". Indeed,
     # because tags "label" can have an attribute named "for", it clashes with
     # the "for" attribute added by PX. The solution is to force users to write,
@@ -103,15 +110,15 @@ class PxParser(Parser):
             # manage special attributes that could not be dumped at all, like
             # "selected" or "checked".
             hook = None
-            ignorableAttrs = self.pxAttributes
+            ignorableAttrs = PxParser.pxIgnorable
             buffer = e.currentBuffer
-            for name in self.noDumpTags:
+            for name in self.noDump:
                 if name in attrs and attrs[name].startswith(':'):
                     hook = (name, buffer.crunchExpr(attrs[name][1:]))
                     ignorableAttrs += (name,)
                     break
             buffer.dumpStartElement(elem, attrs, ignoreAttrs=ignorableAttrs,
-                                    noEndTag=elem in SELF_CLOSING, hook=hook,
+                                    noEndTag=elem in XHTML_SC, hook=hook,
                                     renamedAttrs=self.renamedAttributes)
 
     def endElement(self, elem):
@@ -124,7 +131,7 @@ class PxParser(Parser):
             buffer.addExpression(content)
             e.currentContent = ''
         # Dump the end element into the current buffer
-        if elem != 'x' and elem not in SELF_CLOSING:
+        if elem != 'x' and elem not in XHTML_SC:
             e.currentBuffer.dumpEndElement(elem)
         # If this element is the main element of the current buffer, we must
         # pop it and continue to work in the parent buffer.
