@@ -121,9 +121,9 @@ class Config:
         self.traversers = {}
         # The path to the site. Will be set by m_set below.
         self.sitePath = None
-        # ~~~
+        # ~
         # Options for the pool of threads
-        # ~~~
+        # ~
         # The initial number of threads to run
         self.threads = 5
         # The maximum number of requests a worker thread will process before
@@ -158,16 +158,16 @@ class Config:
         self.hungCheckPeriod = 100
         # On server shutdown, threads may still be busy. Determine here the
         # maximum number of seconds the server will wait for all threads to
-        # terminate, before brutally killing them. By default, wait 1 minute.
+        # terminate, before brutally killing them. By default, wait 10 seconds.
         # Set it to a higher value if your app frequently performs longer
         # transactions and you want to give more chances to these latters to end
         # up gracefully.
-        self.maxWait = 60
+        self.maxWait = 10
 
     def isIPv6(self):
         '''Is IP v6 in use ?'''
         host = self.address
-        return (host.count(':') > 1)  or ('[' in host)
+        return host.count(':') > 1  or '[' in host
 
     def set(self, appFolder, sitePath):
         '''Sets site-specific configuration elements'''
@@ -230,7 +230,7 @@ class Config:
                 return True
         s.close()
 
-    def check(self):
+    def check(self, messages):
         '''Ensure this config is valid'''
         # At least one thread must be specified
         assert (self.threads > 1), MIN_THR_KO
@@ -256,7 +256,7 @@ class Server:
         # If an ext is there, load it
         if ext: __import__(ext.name)
         # Ensure the config is valid
-        config.check()
+        warnings = config.check()
         # p_mode can be:
         #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # "fg"     | Server start, in the foreground (debug mode)
@@ -286,10 +286,14 @@ class Server:
         self.loggers = O(site=cfg.getLogger('site'),
                          app=cfg.getLogger('app', mode != 'bg'))
         self.logStart(method)
+        # Now that we have loggers, log configuration warnings, if any
+        logger = self.loggers.app
+        for warning in warnings:
+            logger.warn(warning)
         try:
             # Load the application model. As a side-effect, the app's po files
             # were also already loaded.
-            self.model, poFiles = config.model.get(config, self.loggers.app)
+            self.model, poFiles = config.model.get(config, logger)
             # Initialise the HTTP server
             cfg = config.server
             self.pool = None
@@ -305,8 +309,8 @@ class Server:
             config.database.getDatabase(self, handler, poFiles, method=method)
             # Initialise the static configuration
             counts = cfg.static.init(config.ui)
-            self.loggers.app.info(RAM_LOAD % (counts.css, counts.svg,
-                                              getShownSize(counts.size)))
+            logger.info(RAM_LOAD % (counts.css, counts.svg,
+                                    getShownSize(counts.size)))
             # Unregister the virtual handler
             VirtualHandler.remove()
         except (Model.Error, Database.Error) as err:
@@ -319,7 +323,7 @@ class Server:
         self.scheduler = Scheduler(self)
         # The server is ready
         if self.classic:
-            self.loggers.app.info(READY % (cfg.address, cfg.port, os.getpid()))
+            logger.info(READY % (cfg.address, cfg.port, os.getpid()))
 
     def init(self, config):
         '''Initialise the server socket'''
@@ -391,11 +395,16 @@ class Server:
         self.logShutdown()
         # Shutdown the loggers
         logging.shutdown()
-        # Delete the "pid" file when it exists
+        # -= Warning =- hereafter, we enter a "blind" zone :: no log
         if self.mode == 'bg':
+            # Delete the "pid" file when it exists
             pidFile = self.config.database.getPidPath()
             if pidFile.exists():
-                pidFile.unlink()
+                try:
+                    pidFile.unlink()
+                except Exception as err:
+                    pass
+        sys.exit(0)
 
     def abort(self, error=None):
         '''Server shutdown following an error'''
