@@ -41,7 +41,7 @@ class Cookie:
             cookie = handler.resp.headers['Cookies'][Cookie.name]
         if not cookie: return None, None, None
         unquoted = urllib.parse.unquote(cookie).encode('utf-8')
-        cookieValue = base64.decodestring(unquoted).decode('utf-8')
+        cookieValue = base64.decodebytes(unquoted).decode('utf-8')
         if ':' not in cookieValue: return None, None, None
         # Extract the context from the cookieValue
         r, context = cookieValue.rsplit(':', 1)
@@ -53,8 +53,8 @@ class Cookie:
     def write(class_, handler, login, password, ctx=None):
         '''Encode p_login, p_password and p_ctx into the authentication
            cookie.'''
-        r = '%s:%s:%s' % (login, password, ctx or '')
-        r = base64.encodestring(r.encode('utf-8')).rstrip()
+        r = '%s:%s:%s' % (login, password or '', ctx or '')
+        r = base64.encodebytes(r.encode('utf-8')).rstrip()
         handler.resp.setCookie(Cookie.name, urllib.parse.quote(r))
 
     @classmethod
