@@ -66,6 +66,9 @@ class Config:
         # When a login is encoded by the user, a transform can be applied (one
         # among Config.transformValues).
         self.loginTransform = None
+        # When the user is in one of the following states, loggin in will be
+        # blocked.
+        self.noLoginStates = ('inactive',)
         # When using a LDAP for authenticating users, place an instance of class
         # appy.server.ldap.Config in the field below.
         self.ldap = None
@@ -97,7 +100,7 @@ class Config:
                 r = sso.ldap, sso
         return r
 
-    def check(self):
+    def check(self, messages):
         '''Check this config'''
         # Check attribute "loginTransform"
         transform = self.loginTransform
@@ -136,9 +139,15 @@ class Guard:
         # Unwrap some useful objects
         self.model = handler.server.model
         self.config = handler.server.config
+        # Get the current user and store user-related attributes on p_self
+        self.initUser()
+
+    def initUser(self):
+        '''Initialise guard attributes related to the current user'''
         # Authenticate the currently logged user and get its User instance
         self.user = User.authenticate(self)
         # Remember that this user hits the server now
+        handler = self.handler
         if not handler.fake:
             handler.onlineUsers[self.user.login] = DateTime()
         # Cache info about this user
@@ -164,7 +173,7 @@ class Guard:
     # "the currently logged user".
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    # Security checks on classes
+    #                       Security checks on classes
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     def mayInstantiate(self, class_, checkInitiator=False, initiator=None,
@@ -204,7 +213,7 @@ class Guard:
                 return True
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    # Security checks on objects
+    #                       Security checks on objects
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     def stringInfo(self, info):
@@ -273,7 +282,7 @@ class Guard:
         return r
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    # Login / logout
+    #                              Login / logout
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     traverse['enter'] = True
@@ -326,8 +335,7 @@ class Guard:
         # Log the action
         handler.log('app', 'info', 'Logged out.')
         # Redirect the user to the app's home page
-        back = '%s/public' % tool.url if self.config.ui.discreetLogin \
-                                      else self.config.server.getUrl(handler)
+        back = self.config.server.getUrl(handler)
         handler.resp.goto(back, message=tool.translate('logout_ok'))
 
     def getLogoutUrl(self, tool, user):
@@ -358,7 +366,7 @@ class Guard:
                "'User', 'nav': 'ref.tool.users.0.0'})" % url
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    # Password reinitialisation
+    #                         Password reinitialisation
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     traverse['askPasswordReinit'] = True
@@ -432,24 +440,30 @@ class Guard:
         return tool.goto(siteUrl, _('wrong_password_reinit'))
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    # PXs
+    #                                    PXs
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     # A custom zone, at the bottom of pxLogin, allowing an app to add custom
     # content.
     pxLoginBottom = Px('')
 
-    def getLoginLogo(self, discreet, url):
+    def getLoginLogo(self, tool, discreet, url):
         '''Returns the possibly clickable logo to include in the login box'''
         # Compute the "img" tag representing the logo
         r = '<img src="%s"/>' % url('loginLogo')
         # Wrap it, when appropriate, in a link allowing to go back to the public
-        # page.
-        if discreet == 'home':
+        # or home page.
+        if discreet == 'home' and tool.defaultPage:
             r = '<a onclick="goto(siteUrl+\'/tool/public\')" ' \
                 'class="clickable">%s</a>' % r
+        elif discreet:
+            r = '<a onclick="goto(siteUrl)" class="clickable">%s</a>' % r
         return r
 
+    def getLoginBoxCss(self, discreet):
+        '''Get the CSS class(es) for the main login box tag'''
+        return 'loginBox boxD' if discreet == True else 'loginBox'
+
     # The login form
     pxLogin = Px('''
      <!-- Popup for reinitializing the password -->
@@ -470,10 +484,11 @@ class Guard:
      </div>
 
      <!-- The login box -->
-     <div class="loginBox"
-          var="cfg=config.security;
-               discreet=config.ui.discreetLogin;
+     <div var="cfg=config.security;
+               discreet=config.ui.discreetLogin or
+                        (isAnon and _px_.name == 'public');
                display='none' if discreet and not req.stay else 'block'"
+          class=":guard.getLoginBoxCss(discreet)"
           style=":'display:%s' % display">
 
       <!-- Allow to hide the box when relevant -->
@@ -481,7 +496,7 @@ class Guard:
            onclick="toggleLoginBox(false)" class="clickable iconXS"/>
 
       <!-- A (possibly clickable) logo -->
-      <div align="center">::guard.getLoginLogo(discreet, url)</div>
+      <div align="center">::guard.getLoginLogo(tool, discreet, url)</div>
 
       <h1>::_('app_name')</h1>
       <center class="sub">:_('please_connect')</center>
@@ -549,17 +564,20 @@ class Guard:
          top:|loginTop|; left:50%; z-index:10; transform:translate(-50%,-50%);
          box-shadow:|loginShadow|; border-radius: |loginBorderRadius|
       }
+      .boxD { border:|loginBorderD| |loginColorPh|; top:|loginTopD|;
+              box-shadow:|loginShadowD| }
       .loginBox a, .loginBox a:visited {
         font-weight:bold; letter-spacing:1px; color:|loginColor| }
-      .loginBox h1 { padding-bottom:0; margin-bottom:3px; text-align:center;
-                     font-size:160%; font-weight: |loginTitleWeight| }
+      .loginBox h1 {
+         padding-bottom:0; margin:|loginTitleMargin|; text-align:center;
+         font-size:|loginTitleSize|; font-weight:|loginTitleWeight|;
+         text-transform:|loginTitleTransform| }
       .loginBox .sub { font-size:120% }
       #loginForm input {
         background-color:|loginBgColor|; width:|loginWidth|; border:none;
         padding:|loginPadding|; margin:|loginMargin|; color:|loginColor|;
-        border-radius:|loginRadius|; }
-      .submitLogin { text-align:|submitAlign|
-      }
+        border-radius:|loginRadius|; font-size:|loginInputSize| }
+      .submitLogin { text-align:|submitAlign| }
       #loginForm input[type=submit] {
         color:|submitColor|; width:|submitWidth|; height:|submitHeight|;
         background:|submitBgColor| |submitBgUrl| no-repeat center left;
@@ -588,16 +606,14 @@ class Guard:
        <input type="button" value=":_('cookie_accept')"
               onclick="setAcceptCookie(this)"/>
       </div>
-     </div>
-    ''',
+     </div>''',
 
     css='''
       .cookieWarn { background-color:lightgrey; width:100%; position:absolute;
                     bottom:0; font-size:90%; z-index: 100 }
       .cookieWarn div { padding: 7px 15px }
       .cookieWarn a { margin: 0 10px; text-decoration: underline }
-      .cookieWarn input { display:inline; background-color:#b5b5b5 }
-    ''',
+      .cookieWarn input { display:inline; background-color:#b5b5b5 }''',
 
     js='''
       function setAcceptCookie(button) {
