@@ -20,8 +20,9 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 from http import HTTPStatus
 from http import client as hc
-import socket, threading, urllib.parse, transaction
+import re, socket, threading, urllib.parse, transaction
 
+from appy.utils import Function
 from appy.model.base import Base
 from appy.server.guard import Guard
 from appy.server.error import Error
@@ -88,6 +89,9 @@ class MethodsCache(dict):
         # name.
         if not cheat:
             prefix = class_.__name__ if class_ else str(o.iid)
+            # Prefix the method name with the name of its class. Indeed,
+            # p_method may not belong to p_o's class.
+            name = Function.getQualifiedName(method)
         key = '%s:%s' % (prefix, name)
         # Return the cached value if present in the method cache
         if key in self: return self[key]
@@ -207,6 +211,10 @@ class Handler:
         # log p_level.
         getattr(logger, level)(cfg.sep.join(r))
 
+    def isMobile(self):
+        '''Was the currently handled HTTP request initiated from a mobile
+           device ?'''
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 # Inject class Handler on class Base to avoid circular package dependencies
 Base.Handler = Handler
@@ -224,6 +232,9 @@ class HttpHandler(Handler):
     # Supported HTTP methods
     supportedHttpMethods = asDict(['GET', 'POST'])
 
+    # Regex used to detect a mobile browser from the User-Agent header
+    mobileRex = re.compile('Android|iPhone|iPod')
+
     def __init__(self, clientSocket, server):
         # The global config
         self.config = server.config
@@ -271,12 +282,12 @@ class HttpHandler(Handler):
     def getLayout(self):
         '''Try to deduce the current layout from the traversal, if present'''
         traversal = getattr(self, 'traversal', None)
-        if traversal: return traversal.context.layout
+        if traversal: return getattr(traversal.context, 'layout', None)
 
     def isAjax(self):
         '''Is this handler handling an Ajax request ?'''
-        traversal = getattr(self, 'traversal', None)
-        if traversal and traversal.context: return traversal.context.ajax
+        req = self.req
+        return req and req.ajax == 'True'
 
     def isPublished(self, o):
         '''Is object p_o the currently published object ?'''
@@ -287,6 +298,14 @@ class HttpHandler(Handler):
         if not path.endswith('/'): path = '%s/' % path
         return ('/%s/' % o.id) in path
 
+    def isMobile(self):
+        '''Was the currently handled HTTP request initiated from a mobile
+           device ?'''
+        headers = self.headers
+        if not headers: return
+        agent = headers.get('User-Agent')
+        return False if not agent else bool(HttpHandler.mobileRex.search(agent))
+
     def manageGuardError(self, resp, traversal, error=None):
         '''A security-related error has occurred: the logged user is not allowed
            to access the desired URL. If the user is anon, redirect him to the
@@ -295,13 +314,20 @@ class HttpHandler(Handler):
         # Return the response content. In the case of a redirect, None will be
         # returned.
         if self.guard.user.isAnon():
-            config = self.config
-            siteUrl = config.server.getUrl(self)
-            gotoUrl = urllib.parse.quote('%s%s' % (siteUrl, self.path))
-            resp.goto(url='%s/tool/%s?goto=%s&stay=1' % \
-                      (siteUrl, config.ui.home, gotoUrl),
-                      message=self.tool.translate('please_authenticate'))
-            r = None
+            if resp.contentType == 'html':
+                # This is (supposedly) a user behind a browser
+                config = self.config
+                siteUrl = config.server.getUrl(self)
+                gotoUrl = urllib.parse.quote('%s%s' % (siteUrl, self.path))
+                resp.goto(url='%s/tool/%s?goto=%s&stay=1' % \
+                          (siteUrl, config.ui.home, gotoUrl),
+                          message=self.tool.translate('please_authenticate'))
+                r = None
+            else:
+                # Return a 403 error, marshalled
+                resp.code = HTTPStatus.FORBIDDEN
+                tag = error.__class__.__name__ if error is not None else 'Error'
+                r = traversal.marshall(error or '', rootTag=tag)
         else:
             # Log and return a 403 error: forbidden
             resp.code = HTTPStatus.FORBIDDEN
@@ -437,7 +463,7 @@ class HttpHandler(Handler):
                 # Extract parameters and create a Request instance in p_self.req
                 self.req = Request.create(self)
                 # Compute dynamic content
-                r = self.attemptDatabaseRequest()   
+                r = self.attemptDatabaseRequest()
                 # Build the HTTP response (if not done yet)
                 resp.build(r)
             finally:
@@ -539,6 +565,7 @@ class VirtualHandler(Handler):
     requestVersion = 'HTTP/1.1'
     headers = {'User-Agent': 'system'}
     fake = True
+    traversal = None
 
     # Entries logged from a virtual handler will be prefixed with this char
     logChar = '>'
