@@ -37,7 +37,7 @@ LDAP_ACT      = '%d user(s) %sactivated: %s.'
 LDAP_NO_DEACT = 'No user was deactivated.'
 LDAP_QRY_ERR  = 'LDAP query error %s: %s'
 PRIV_SET      = '%s: automatic privileges set.'
-SYNC_SUMMARY  = 'users sync from %s: %d local user(s) created, %d updated, ' \
+SYNC_SUMMARY  = 'Users sync from %s: %d local user(s) created, %d updated, ' \
                 '%d untouched and %d invalid entries. %s local user(s) ' \
                 'deactivated and %d reactivated.'
 
@@ -178,19 +178,19 @@ class Config:
         '''Formats the user-related p_ldapData retrieved from the LDAP, as a
            dict of params usable for creating or updating the corresponding
            Appy user.'''
-        res = {}
+        r = {}
         # Retrieve base params
         for name, appyName in self.ldapAttributes.items():
             if not appyName: continue
             # Get the name of the attribute as known in the LDAP
             ldapName = getattr(self, name)
             if not ldapName: continue
-            self.setAppyValue(ldapName, ldapData, appyName, res)
+            self.setAppyValue(ldapName, ldapData, appyName, r)
         # Retrieve custom params
         for ldapName, appyName in self.customAttributes.items():
-            if ldapData.has_key(ldapName):
-                self.setAppyValue(ldapName, ldapData, appyName, res)
-        return res
+            if ldapName in ldapData:
+                self.setAppyValue(ldapName, ldapData, appyName, r)
+        return r
 
     def setLocalUser(self, tool, attrs, login, password=None, sso=None):
         '''Creates or updates the local User instance corresponding to a LDAP
@@ -387,7 +387,7 @@ class Config:
         # Log the operation and return a message
         message = SYNC_SUMMARY % (serverUri, counts.created, counts.updated,
                                   counts.untouched, counts.wrong,
-                                  counts.deactivated, counts.reactivated) 
+                                  counts.deactivated, counts.reactivated)
         tool.log(message)
         return message
 
