@@ -416,7 +416,7 @@ class Config:
         #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         self.all = []
 
-    def check(self):
+    def check(self, messages):
         '''Ensures the Scheduler config is correct'''
         # Check attribute "minutes"
         minutes = self.minutes
@@ -443,7 +443,7 @@ class Scheduler:
         # Store, in attribute "last", the last minute corresponding to the last
         # scheduler execution. It allows to prevent several executions of the
         # same job(s) at the same minute.
-        # ~~~
+        # ~
         self.last = time.localtime().tm_min
         # This attribute is initialised to the current minute. It means that, as
         # soon as the Appy server is started, the scheduler will not be able to
