@@ -28,8 +28,8 @@ from appy.xml.marshaller import Marshaller
 from appy.model.workflow.transition import Transition
 
 # Errors - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-UNTRAVERSABLE = 'Untraversable element "%s" while traversing %s.'
-TRAVERSE_UNKNOWN = 'Unknown traversal type "%s".'
+UN_TRAV   = 'Untraversable element "%s" while traversing %s.'
+TRAV_KO   = 'Unknown traversal type "%s".'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class NotFound(Exception):
@@ -41,6 +41,10 @@ class Traversal:
     '''Traverses the path of an URL and executes the corresponding code'''
     Error = NotFound
 
+    # Correspondence between some URL parts and their PXs
+    layouts = {'default': 'view', 'view': 'view', 'edit': 'edit', 'xml' : 'xml',
+               'pxTied': 'cell', 'pxResult': 'cell', 'save': 'edit'}
+
     # Let's define an URL as
     #               <protocol>://<domain name><path>
     # "Traversing" is the process of splitting <path> into parts and finding a
@@ -74,28 +78,40 @@ class Traversal:
         if other:
             self.init(other)
             self.parent = other
+            other.child = self
             return
         # Perform standard initialisation
         self.handler = handler
         self.req = handler.req
+        self.resp = handler.resp
         self.tool = handler.tool
         # The path that will be traversed, splitted into parts
         self.parts = handler.parts
         # The currently logged user
         self.user = handler.guard.user
+        # Set additional attributes via m_reset
+        self.reset(None)
+        # The PX context
+        self.context = None
+        # The parent traversal, for a child traversal
+        self.parent = None
+        # The child traversal, for a parent traversal
+        self.child = None
+
+    def reset(self, o):
+        '''This p_o(bject) has just been encountered: reset the traversal,
+           starting with this new object as base.'''
+        # If p_o is None, this method is used to initialise a base traversal
+        # ~
         # The current result of traversing
-        self.r = None
-        # The last encountered object
-        self.o = None
+        self.r = o
+        # The current object being traversed
+        self.o = o
         # The last encountered field. For inner fields, the field name is
         # explicitly mentioned in attribute p_self.fieldName.
         self.field = self.fieldName = None
         # The last encountered transition
         self.transition = None
-        # The PX context
-        self.context = None
-        # The parent traversal, for a child traversal
-        self.parent = None
         # The traversal mode
         self.mode = Traversal.STANDARD
 
@@ -111,10 +127,26 @@ class Traversal:
             r += ' (from %s)' % self.parent.getPath()
         return r
 
+    def __repr__(self):
+        '''p_self's short string representation'''
+        return '<Traversal @%s>' % self.getPath()
+
     def createContext(self, layout='view'):
         '''Create and return a PX context for rendering p_o on p_layout'''
         return Px.createContext(self, layout, Quote)
 
+    def getContext(self):
+        '''Returns the existing context or create one if it does not exist'''
+        r = self.context
+        if r: return r
+        # Determine the layout
+        if self.parts:
+            layout = Traversal.layouts.get(self.parts[-1], 'view')
+        else:
+            layout = 'view'
+        self.context = r = self.createContext(layout)
+        return r
+
     def asObjectId(self, part):
         '''Interpret p_part as an object ID and return this ID as
            * a positive integer (for a definitive object having a standard ID);
@@ -142,12 +174,18 @@ class Traversal:
 
     def getSpecialObject(self, name):
         '''Gets the special object named p_name'''
-        # Currenly, the only type of supported "special object" is an app class
-        return self.handler.server.model.classes.get(name).python
+        # A special object can be a class or an object
+        if name[0] == ':':
+            # It is an object
+            r = self.tool.getObject(name[1:])
+        else:
+            # It is a class
+            r = self.handler.server.model.classes.get(name).python
+        return r
 
     def getLayout(self, pxName):
         '''Determine the layout type from the currently traversed PX'''
-        return 'view' if pxName == 'default' else pxName
+        return Traversal.layouts.get(pxName) or 'view'
 
     def getPermission(self, permission):
         '''If a field is currently traversed, a "read" or "write" permission
@@ -182,7 +220,7 @@ class Traversal:
                     # The user must be the one whose login is in "info"
                     r = self.user.login == info
                 else:
-                    raise self.handler.guard.Error(TRAVERSE_UNKNOWN % type)
+                    raise self.handler.guard.Error(TRAV_KO % type)
         else:
             # A tuple or list of logically OR'ed infos. Having at least one of
             # the listed prerogatives allows the user to traverse.
@@ -194,7 +232,7 @@ class Traversal:
         # Return the result or raise an Unauthorized exception if p_raiseError
         # is True and the object can't be traversed.
         if raiseError and not r:
-            message = UNTRAVERSABLE % (name, self.getPath())
+            message = UN_TRAV % (name, self.getPath())
             raise self.handler.guard.Error(message)
         return r
 
@@ -254,19 +292,10 @@ class Traversal:
 
     def executePx(self, name, px):
         '''Renders this p_px, whose p_name is given'''
-        # Firstly, create or update the PX context
-        layout = self.getLayout(name)
-        if not self.context:
-            self.context = self.createContext(layout)
-        else:
-            # Update the existing context
-            self.context.o = self.o
-            self.context.field = self.field
-            self.context.layout = layout
-        # For inner fields, ensure the full name is in the context
-        if self.fieldName:
-            self.context.fieldName = self.fieldName
-        # Secondly, execute the tied action if defined
+        # Firstly, execute the tied action if defined. Indeed, this action may
+        # have an impact on the data possibly shown by the PX. For example, the
+        # action may delete an object matching a search whose results are shown
+        # in the PX.
         actionName = self.req.action
         if actionName:
             # Execute this action by performing a sub-traversal
@@ -280,7 +309,19 @@ class Traversal:
             msg = sub.run()
             if msg:
                 # Set a special cookie in the response containing the message
-                handler.resp.addMessage(msg)
+                self.resp.addMessage(msg)
+        # Then, create or update the PX context
+        layout = self.getLayout(name)
+        if not self.context:
+            self.context = self.createContext(layout)
+        else:
+            # Update the existing context
+            self.context.o = self.o
+            self.context.field = self.field
+            self.context.layout = layout
+        # For inner fields, ensure the full name is in the context
+        if self.fieldName:
+            self.context.fieldName = self.fieldName
         # Finally, call the PX
         return px(self.context)
 
@@ -342,6 +383,21 @@ class Traversal:
             self.field = r
         return r
 
+    def marshall(self, r, rootTag=None):
+        '''Depending on the response type, the traversal p_r(esult) may need to
+           be marshalled.'''
+        if r is None: return r
+        if self.resp.contentType == 'xml':
+            # Currently, only XML marshalling is there
+            if isinstance(r, Base):
+                tag = r.class_.name
+                appy = True
+            else:
+                tag = self.parts[-1]
+                appy = False
+            r = Marshaller(rootTag=rootTag or tag).marshall(r, appy=appy)
+        return r
+
     def run(self):
         '''Traverses a given path whose parts are splitted into p_self.parts'''
         parts = self.parts
@@ -379,23 +435,24 @@ class Traversal:
                         self.r = self.o = self.getObject(part)
                 # Manage any not-empty part
                 elif part:
-                    # Try to get the attribute or method named "part" on self.r
-                    self.r = self.getPart(part)
-                    # Manage this part
-                    self.managePart(previous, part, self.r)
+                    if part.isdigit(): # An object (again): reset the traversal.
+                        self.reset(self.tool.getObject(part))
+                    else:
+                        # Try to get attribute or method named "part" on self.r
+                        self.r = self.getPart(part)
+                        # Manage this part
+                        self.managePart(previous, part, self.r)
                 i += 1
                 previous = self.r
-        # If self.r is an object, call the default PX on it
-        if isinstance(self.r, Persistent):
+        # If self.r is an object and the response must be HTML, call the default
+        # PX on this object.
+        if isinstance(self.r, Persistent) and self.resp.contentType == 'html':
             self.managePart(self.r, 'default', self.r.default)
+        # Potentially marshall the result
+        r = self.marshall(self.r)
         # If the result of the traversal is not a string, it is invalid. Produce
         # a 404 error in that case.
-        r = self.r
         if r is not None and not isinstance(r, str):
-            # Can v_r be marshalled ? Currently, only XML marshalling is there
-            if self.handler.resp.contentType == 'xml':
-                r = Marshaller(rootTag=self.parts[-1]).marshall(r)
-            else:
-                raise NotFound(self.getPath())
+            raise NotFound(self.getPath())
         return r
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
