@@ -21,7 +21,8 @@
 import os.path, re
 
 from appy.px import Px
-from appy.ui.layout import ColumnLayout
+from appy.utils import asDict
+from appy.utils.string import Variables
 from appy.model.utils import Object as O
 
 # Make classes from sub-packages available here  - - - - - - - - - - - - - - - -
@@ -31,6 +32,7 @@ from appy.ui.iframe import Iframe
 from appy.ui.message import Message
 from appy.ui.portlet import Portlet
 from appy.ui.globals import Globals
+from appy.ui.columns import Columns
 from appy.ui.template import Template
 from appy.ui.navigate import Siblings
 from appy.ui.includer import Includer
@@ -48,10 +50,6 @@ class Config:
     # Defaults fonts used in the web user interface
     defaultFonts = "Rajdhani, sans-serif"
 
-    # Regular expression for variables possibly defined in some (almost static)
-    # files like CSS and SVG.
-    variable = re.compile('\|(\w+?)\|', re.S)
-
     def __init__(self):
         '''Constructor for the UI configuration'''
 
@@ -189,28 +187,35 @@ class Config:
         # Attribute "headerShow" determines when and where the page header must
         # be shown. Possible values are the following.
         #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-        # "top" | The header will be placed on top of the page, in order to
-        #       | produce this global page schema:
-        #       |
-        #       |      H         e         a         d         e          r
-        #       |
-        #       |      Portlet   P  a  g  e   c  o  n  t  e  n  t   Sidebar
-        #       |
-        #       |      F         o         o         t         e          r
-        #       |
+        #   "top"   | The header will be placed on top of the page, in order to
+        #           | produce this global page schema:
+        #           |
+        #           |   H         e         a         d         e          r
+        #           |
+        #           |   Portlet   P  a  g  e   c  o  n  t  e  n  t   Sidebar
+        #           |
+        #           |   F         o         o         t         e          r
+        #           |
         #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-        # "sub" | The header will be placed on top of the page content, in
-        #       | between the portlet and sidebar, to produce this global page
-        #       | schema:
-        #       |
-        #       |      Portlet   H     e     a     d     e       r  Sidebar
-        #       |
-        #       |                P  a  g  e    c  o  n  t  e  n  t
-        #       |
-        #       |      F         o         o         t         e          r
-        #       |
+        #   "sub"   | The header will be placed on top of the page content, in
+        #           | between the portlet and sidebar, to produce this global
+        #           | page schema:
+        #           |
+        #           |   Portlet   H     e     a     d     e       r  Sidebar
+        #           |
+        #           |             P  a  g  e    c  o  n  t  e  n  t
+        #           |
+        #           |   F         o         o         t         e          r
+        #           |
         #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-        # None  | The header will be invisible
+        # "portlet" | The header will be placed on top and within the portlet:
+        #           |
+        #           |   Header    P  a  g  e    c  o  n  t  e  n  t  Sidebar
+        #           |   Portlet
+        #           |
+        #           |   F         o         o         t         e          r
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        #   None    | The header will be invisible
         #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # If you place a function in this attribute, it will be called with, as
         # args, the current root PX and its context, and must return one of the
@@ -257,9 +262,22 @@ class Config:
         self.altColorLight = '#e9f2f3'
         # Color for some selected text
         self.selectColor = self.brightColor
+        # Background color for fields with focus=True
+        self.focusColorBg = '#ced0d8'
         # Row background colors for most list-based data
         self.evenColor = 'rgba(243,243,247,0.7)'
         self.oddColor = 'rgba(255,255,255,0.5)'
+        # Background, border and font colors for some dark table headers
+        self.thColor = self.brightColor
+        self.thBdColor = self.darkColor
+        self.thBgColor = self.darkColor
+        # Styling groups with style "grid". It corresponds to CSS class "ggrid",
+        # not to be confused with CSS class "grid".
+        self.gridGPadding = '8px' # Padding, on all layouts
+        self.gridVLFSize = '100%' # Font size, on *v*iew (*l*abels)
+        self.gridVFFSize = '100%' # Font size, on *v*iew (*f*ield values)
+        self.gridELFSize = '100%' # Font size, on *e*dit (*l*abels)
+        self.gridEFFSize = '100%' # Font size, on *e*dit (*f*ield values)
         # In the popup, everything is smaller. If you want to achieve the same
         # result in the main window, set "compact" to True.
         self.compact = False
@@ -273,8 +291,7 @@ class Config:
         # Background color for the header
         self.headerBgColor = self.darkColor
         self.headerSpanColor = self.brightColor # Color for standard text
-                                                # rendered within in the header
-        self.headerSpanWeight = 'bold'
+                                                # rendered within in the header.
 
         # Text color for links
         self.linkColor = self.darkColor
@@ -296,6 +313,10 @@ class Config:
 
         # Styling the login box
         self.loginTitleWeight = 'normal'
+        self.loginTitleSize = '160%'
+        self.loginTitleTransform = 'none'
+        self.loginTitleMargin = '0.7em 0 3px 0'
+        self.boxBgColor = self.brightColor
         self.loginBgColor = '#f5f3f2'
         self.loginBoxPadding = '35px'
         self.loginColor = self.darkColor
@@ -305,15 +326,20 @@ class Config:
                                             # placeholder. Other possible values
                                             # include "none" or "capitalize".
         self.loginWidth = '240px'
+        self.loginInputSize = '100%' # Font size within input fields
         self.loginPadding = '12px'
         self.loginRadius = '0'
         self.loginMargin = '8px 0'
         self.loginBorder = '0px solid' # Background excepted
+        self.loginBorderD = self.loginBorder # For the *d*iscreet variant of
+                                             # the login box
         self.loginShadow = '0 4px 8px 0 rgba(160, 104, 132, 0.2), ' \
                            '0 6px 20px 0 rgba(0, 0, 0, 0.19)'
+        self.loginShadowD = self.loginShadow
         self.loginBorderRadius = '0'
         self.loginAlign = 'center'
         self.loginTop = '50%' # Position of the login box on the y axis
+        self.loginTopD = self.loginTop
 
         # Styling the "connect" button
         self.submitTop = '30px' # Vertical space between login/password inputs
@@ -329,40 +355,103 @@ class Config:
         self.submitHeight = 'inherit'
         self.submitWeight = 'normal'
         self.submitPadding = '12px'
-        self.boxBgColor = self.brightColor
 
         # Popups
         self.popupColor = self.brightColor
 
         # Portlet
         self.portletShowLogo = True # Logo on top of the portlet
+        self.portletShowFooter = False # Portlet-specific footer
         self.portletWidth = '250px'
         self.portletMinWidth = '140px'
         self.portletBgColor = 'transparent'
+        # The following attribute allows to colorize the background of the
+        # portlet zone corresponding to the currently shown page.
+        self.portletBgColorS = 'transparent'
         self.portletTextColor = self.darkColor
         self.portletShadow = '2px 2px 5px #002039' # Set "unset" to remove  it
         self.portletSep = '1px solid #002039' # Set "none" to remove it
-        self.pgMargin = '0.5em 2px 0.3em' # *p*ortlet *group margins
+        self.portletSepGap = '1.5em' # Vertical gap between portlet zones
+        self.portletPadding = '30px 50px 0 0'
+        self.portletTPadding = '5px 0' # Portlet title: padding
+        self.portletTWeight = 'normal' # Portlet title boldness: could be "bold"
+        self.portletTSize = '100%' # Portlet title font-size
+        self.portletISize = '24px' # Size of most *i*cons in the portlet
+        self.portletAHeight = '24px' # Height for *A*dd buttons. If the imahe
+                                     # image height is < text height, space can
+                                     # be reduced using value 'min-content'.
+        self.portletBgPos = '0' # Background position for portlet buttons
+        self.portletFBgColor = self.darkColor # *F*ooter background color
+        self.portletFWidth = self.portletWidth # *F*ooter width
+        self.portletFHeight = '100px' # *F*ooter height
+        self.pgMargin = '0.5em 0 0.3em' # *p*ortlet *group margins
+        self.pgPadding = '5px 0 0 0' # *p*ortlet *group padding
+        self.pgTransform = 'uppercase' # Set "none" to disable it
+        self.pgLineHeight = 'normal'
+        self.pgGap = '0.4em'
+        self.gcPadding  = '0 0 0 10px' # *G*roup *c*ontent paddings
+        self.gcFSize = '100%' # *F*ont size
+        self.gcFStyle = 'normal' # *F*ont style, can be "italic"
+
+        # The sidebar (sb)
+        self.sbPadding = '28px 8px 30px 0px'
+
+        # Advanced search link
         self.asMargin = '3px 0 10px 30px' # For link "*a*dvanced *s*earch"
+        self.asFSize = '90%'
 
         # Live search (ls)
         self.lsBgColor = 'transparent' # Bg color for zone "search field + icon"
         self.lsInputBgColor = 'transparent' # Bg color for the search field only
         self.lsInputWidth = '7.5em'
         self.lsInputHeight = '22px'
+        self.lsInputPadding = '0'
         self.lsrBgColor = self.brightColor # Bg color for search *r*esults
         self.lsPadding = '0'
+        self.lsMargin = '0' # Needs !important suffix
+        self.lsFSize ='100%' # *F*ont size
         self.lsBottom = '1px solid' # Border bottom
         self.lsBottomColor = '#afb8d4'
+        self.sdropWidth = '11em' # *s*earch results dropdown width
+        self.sdropMargin = '0.6em 0 0 0'
+
+        # The search "top" zone (above search results)
+        self.sTopFSize = '100%'
+        self.sTopPadding = '0 0 1em 0'
+        self.sTopMargin = 'initial'
+        self.sTopBorderB = 'none' # Border bottom
 
         # Other elements
         self.gridFiltersMargin = '20px 0 0 0' # Filter in grid search results
         self.weekEndBg = '#797979' # Background for calendar week-end cells
+        self.ecWidth = '12px' # Width for *e*xpand / *c*ollapse icons
+        self.podIWidth = '18px' # Width for the POD icons
+        self.podPWidth = '50px' # Width for the POD icons in phases
+        self.podPWidthC = '35px' # Width for the POD icons in *c*ompact phases
+        self.podITopC = '21px' # *I*con's text top position on *c*ell layouts
+        self.podIRightC = '3px' # *I*con's text right position on *c*ell layouts
+        self.podSelFSize = '70%' # *F*ont size for pod *sel*ectors
+        self.sfDirection = 'row' # *s*earch *f*ilter direction: set
+                                 # "row-reverse" to position the search/funnel
+                                 # icon before the search field.
+        self.gridPadding = '10px' # Padding for tables having CSS class "grid"
+        self.histMargin = 'margin:0 0 5px 0' # Margins for zone "history"
+        self.lgMargin = '0' # Margin for a multi*l*in*g*ual block
 
         # Header
         self.headerHeight = '60px'
         self.burgerMargin = '0 5px'
-        self.topIconsMargin = '0 10px 0 0px'
+        self.topIconsMargin = '0 0.5em 0 0'
+        self.topIconsGap = '0.5em 1em'
+        self.topIconsHomesDir = 'row' # On "homes", if the number of icons/links
+                                      # is too high, it may be convenient to
+                                      # display them in flex-direction "column"
+                                      # instead of "row".
+        self.topIconsHomesBg = 'transparent'
+        self.topTextPadding = '0 0.5em' # Padding for links or chunks of text
+                                        # shown in the header.
+        self.topTextSpacing = '1px' # Letter sppacing
+        self.topSpanWeight = 'bold'
 
         # Footer
         self.footerHeight = '20px'
@@ -372,25 +461,18 @@ class Config:
         self.footerFontSz = '65%'
 
         # Tabs
-        self.tabSep = '1px' # Space between tabs and content
+        self.tabMargin = '0 0 1px 0' # Space between tabs and content
+        self.tabMarginP = '0' # Same space, for tabs within *p*hases
+        self.tabColor = self.linkColor # Color for clickable labels
+        self.tabFSize = '95%' # *F*ont size
+        self.tabBg = '#f1eeee' # background color for an unselected tab
+        self.tabBgS = '#fbfbfb' # BG for the selected tab
         self.tabBorder = '1px solid %s' % self.linkColor # Top, l, r borders
+        self.tabTransform = 'none' # Could be "uppercase"
         self.tabBorderBottom = self.tabBorder # Bottom border
+        self.tabBorderBottomP = 'unset' # Bottom border for tabs within *p*hases
         self.tabBorderRadius = '5px 5px 0 0'
-        self.tabsBorderBottom = 'unset' # Define a border bottom for tabs only
-                                        # if you define a tabSep > 0.
 
-        # Basically, you may produce 2 families of tabs:
-        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-        # (1) | (the default) Tabs and content are glued: tabSep is 1px, a
-        #     | border bottom is set for every tab (tabBorderBottom) and there
-        #     | is no global border bottom for all tabs (tabsBorderBottom is
-        #     | unset).
-        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-        # (2) | There is space between tabs and content. In that case, tabSep
-        #     | is > 1px, no border bottom must be set for every tab
-        #     | (tabBorderBottom must be 'unset') and a global border bottom
-        #     | must be defined (set tabsBorderBottom to tabBorder).
-        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # Styling phases (containing a picto per page)
         self.phaseBgColor = self.brightColor # When unselected
         self.phaseBgcColor = self.darkColor # When selected (*c*urrent)
@@ -398,6 +480,8 @@ class Config:
         self.phaseBorderColor = self.linkColor # Set "transparent" to hide it
         self.phaseColor = self.linkColor # Font color unselected
         self.phaseCcolor = self.brightColor # Font color, selected (*c*urrent)
+        self.phaseMargin = '0 0 30px -30px' # Margins for phases
+        self.phaseCMargin = '0 0 20px 0' # Margins for phases, *c*ompact variant
         self.pageGap = '0 0' # Gap between pages (vertical & horizontal).
                              # Example: "0.6em ​1em"
 
@@ -413,8 +497,12 @@ class Config:
         self.pageCRadius = 'unset' # Example: "0 0 5px 5px"
         self.pageTextTransform = 'uppercase'  # "none", "lowercase",
         self.pageCTextTransform = 'uppercase' # "capitalize" ...
+        self.pageTextWeight = 'bold'
         self.pageFontSize = '90%'
         self.pageCFontSize = '9pt'
+        self.pageChars = 27  # Max # of shown chars for a page title
+        self.pageCharsS = 31 # Max # of shown chars for a page title, when shown
+                             # in a *s*elect widget.
 
         #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # If attribute          | The name of the page 
@@ -428,17 +516,20 @@ class Config:
         self.navsPadding = '10px 0 0 0'
         # Size of pictos (width and height)
         self.pictoSize = '55px'
-        self.pictoCSize = '35px'
+        self.pictoCWidth = '35px'
+        self.pictoCHeight = '35px'
         self.pictoMargin = '0 0 6px 0'
         self.pictoCMargin = '0 0 4px 0'
 
-        # Buttons. Most buttons appear on top of dark backgrounds. Buttons
-        # appearing in standard bright backgrounds use p_self.darkColor and not
-        # this p_self.buttonColor.
+        # Standard buttons, generally appearing on top of dark backgrounds.
+        # Buttons appearing in standard bright backgrounds use p_self.darkColor
+        # and not this p_self.buttonColor.
         self.buttonColor = self.brightColor
         self.buttonCase = 'uppercase'
-        self.buttonBgColor = 'transparent' # Standard buttons outside phases
-        self.sbuttonBgColor = 'transparent' # *S*mall buttons
+
+        # *A*ction buttons (fields being actions and workflow transitions)
+        self.abuttonBgColor = 'transparent' # Standard buttons outside phases
+        self.abuttonBgColorS = 'transparent' # *S*mall buttons
 
         # Bottom space for every field
         self.fieldBottom = '0.7em'
@@ -447,6 +538,7 @@ class Config:
         # If you want to use standard positioning, set value "inherit".
         self.epictoPosition = 'absolute'
         self.epictoPadding = '0'
+        self.epictoWidth = '22px'
 
         # Application-wide default formats for hours and dates
         self.dateFormat = '%d/%m/%Y'
@@ -496,7 +588,7 @@ class Config:
         # config.js, contents.css and styles.js stored in
         # appy/ui/static/ckeditor will be compatible with the version you want
         # to use.
-        self.ckVersion = '4.17.1'
+        self.ckVersion = '4.17.2'
         # ckDistribution can be "basic", "standard", "standard-all", "full" or
         # "full-all" (see doc in http://cdn.ckeditor.com).
         # CK toolbars are not configurable yet. So toolbar "Appy", defined in
@@ -511,30 +603,50 @@ class Config:
         # showability of the tool's icon in the UI. This function will receive
         # the tool as unique arg.
         self.toolShow = True
-        # Attribute "showRootPages" determines if the selector containing root
-        # pages, located in the page header, must be shown or not. It can be a
-        # function accepting the tool as unique arg.
+        # Attribute "showRootPages" determines if the "links" allowing to hit
+        # root, selectable pages (=every page being in tool.pages, flagged as
+        # "selectable"), located in the page header, must be shown or not. It
+        # can be a function accepting the tool as unique arg.
         self.showRootPages = True
+        # "links" to root pages can be rendered in 2 ways: either as explicit
+        # "a" tags, or in a "select" HTML control, as options. The number
+        # defined hereafter determines how many "links" will be rendered as
+        # explicit "a" tags. The remaining pages (if any) will thus appear as
+        # options in a "select" HTML control. Note that, on a mobile device,
+        # this number is ignored: in order to save space, all pages will appear
+        # in a "select" HTML control.
+        self.expandedRootPages = 3
         # Attribute "showUserLink" determines if the link to logged user's
         # profile, located in the page header, must be show or not. It can be a
         # function accepting the tool as unique arg.
         self.showUserLink = True
+        # When "showUserLink" is True, but the following attribute "userLink" is
+        # False, the user's first name is unclickable.
+        self.userLink = True
+        # If you want to prefix the user's first name by some word, like
+        # "welcome", set here a i18n label that will be used to produce this
+        # word (like standard label "welcome").
+        self.userPrefix = None
+
+        # If you want to suffix the user's first name link with the user's full
+        # title, set the following attribute to True. Works only when
+        # p_self.headerShow is "portlet".
+        self.userTitle = False
         # In the page header, there is an icon allowing to go back to the user's
         # home page, as defined by method tool.computeHomePage(). You can define
         # the visibility of this icon via the following attribute. It can hold a
-        # function that will receive the tool as unique arg.
+        # function that will receive the tool as unique arg. Note that, if
+        # p_self.headerShow is "portlet", the home icon is not shown, because
+        # replaced with the portlet logo.
         self.showHomeIcon = True
         # When link "login" is shown (i, discreet login is enabled), must an
         # icon be shown besides the link ?
         self.showLoginIcon = True
-        # When "showUserLink" is True, but the following attribute "userLink" is
-        # False, the user's first name is unclickable.
-        self.userLink = True
         # Must text "Disconnect" be renderer besides the "logout" icon ?
         # Defaults to "no".
         self.logoutText = False
         # Attribute "discreetLogin" determines the app's home page and
-        # visibility of the login box. If the attribute is...
+        # visibility of the login box. If its value is...
         #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # False   | (the default), users will hit the app on the default home
         #         | page (see attribute "home") containing a login box inviting
@@ -555,34 +667,6 @@ class Config:
         #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         self.discreetLogin = False
 
-    def formatDate(self, tool, date, format=None, withHour=True, language=None):
-        '''Returns p_date formatted as specified by p_format, or self.dateFormat
-           if not specified. If p_withHour is True, hour is appended, with a
-           format specified in self.hourFormat.'''
-        fmt = format or self.dateFormat
-        # Resolve Appy-specific formatting symbols used for getting translated
-        # names of days or months:
-        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-        #  %dt  | translated name of day
-        #  %DT  | translated name of day, capitalized
-        #  %mt  | translated name of month
-        #  %MT  | translated name of month, capitalized
-        #  %dd  | day number, but without leading '0' if < 10
-        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-        if ('%dt' in fmt) or ('%DT' in fmt):
-            day = tool.translate('day_%s' % date._aday, language=language)
-            fmt = fmt.replace('%dt', day.lower()).replace('%DT', day)
-        if ('%mt' in fmt) or ('%MT' in fmt):
-            month = tool.translate('month_%s' % date._amon, language=language)
-            fmt = fmt.replace('%mt', month.lower()).replace('%MT', month)
-        if '%dd' in fmt: fmt = fmt.replace('%dd', str(date.day()))
-        # Resolve all other, standard, symbols
-        r = date.strftime(fmt)
-        # Append hour
-        if withHour and (date._hour or date._minute):
-            r += ' (%s)' % date.strftime(self.hourFormat)
-        return r
-
     def getHeaderText(self, tool):
         '''Get the permanent text that must appear in the page header'''
         # Get the text via config attribute "test"
@@ -689,8 +773,14 @@ class Config:
     def patchCss(self, css):
         '''Replaces variables possibly defined in this p_css code with values
            from p_self.'''
-        fun = lambda match: getattr(self, match.group(1))
-        return self.variable.sub(fun, css)
+        return Variables.replace(css, self)
+
+    def getUserText(self, user):
+        '''Get the text to show within the user link'''
+        r = user.getFirstName()
+        if self.userPrefix:
+            r = '%s %s' % (user.translate(self.userPrefix), r)
+        return r
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class LinkTarget:
@@ -773,10 +863,10 @@ class Collapsible:
           onclick=":'toggleCookie(%s,%s,%s,%s,%s)' % (q(coll.id), \
                     q(coll.display), q(coll.default), \
                     q(icons.expand), q(icons.collapse))"
-       src=":coll.expanded and url(icons.collapse) or url(icons.expand)"/>''',
+       src=":coll.expanded and svg(icons.collapse) or svg(icons.expand)"/>''',
 
      css='''
-      .expandCollapse { padding-right: 4px; cursor: pointer }
+      .expandCollapse { cursor:pointer; width:|ecWidth| }
       .showHide { position: absolute; top: 10px; left: 0px; cursor: pointer }
       .showHideInv { position: absolute; top: 10px; right: 0px; cursor: pointer}
      ''')
@@ -799,11 +889,13 @@ class Collapsible:
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Sidebar:
+    '''The Appy sidebar'''
+
     @classmethod
     def show(class_, tool, o, layout, popup):
         '''The sidebar must be shown when p_o declares to use the sidebar. If
            it must be shown, its width is returned.'''
-        if not o: return
+        if not o or popup: return
         sidebar = getattr(o.class_.python, 'sidebar', None)
         if not sidebar: return
         if callable(sidebar): sidebar = sidebar(tool)
@@ -811,7 +903,7 @@ class Sidebar:
         if sidebar.show in (True, layout):
             # Complete user info
             sidebar.width = sidebar.width or '320px'
-            sidebar.minWidth = sidebar.minWidth or '220px'
+            sidebar.minWidth = sidebar.minWidth or '320px'
             return sidebar
 
     @classmethod
@@ -829,8 +921,8 @@ class Sidebar:
       <x var="layout='view'">:o.pxFields</x>
      </div>''',
 
-     css='''.sidebar { padding: 28px 8px 30px 0px; position: sticky; top: 0;
-                       overflow-y:auto; overflow-x: auto }''')
+     css='''.sidebar { padding:|sbPadding|; position:sticky; top:0;
+                       overflow-y:auto; overflow-x:auto }''')
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Breadcrumb:
@@ -852,7 +944,7 @@ class Breadcrumb:
         # - "url"   is the URL to this object.
         self.parts = None
         # The CSS classes to apply to the main breacrumb tag
-        self.css = 'pageTitle bottomCell'
+        self.css = 'pageTitle breadTitle'
         if popup: self.css += ' pageTitleP'
         # No breadcrumb is computed for the tool
         if o != o.tool:
@@ -900,19 +992,23 @@ class Button:
     '''Manages rendering of XHTML buttons'''
 
     @classmethod
-    def getCss(class_, label, small=True, render='button'):
+    def getCss(class_, label, small=True, render='button', iconOut=False):
         '''Gets the CSS class(es) to set on a button, given its l_label, size
            (p_small or not) and rendering (p_render).'''
-        # CSS for a small button. No minimum width applies: small buttons are
-        # meant to be small.
+        # p_iconOut being True means that the button icon is rendered outside
+        # the button. If it is the case, an additional CSS class must be applied
+        # to it.
+        prefix = 'noIcon ' if iconOut else ''
         if small:
+            # CSS for a small button. No minimum width applies: small buttons
+            # are meant to be small.
             part = 'Icon' if render == 'icon' else 'Small'
-            return 'button%s button' % part
+            return '%sbutton button%s' % (prefix, part)
         # CSS for a normal button. A minimum width (via buttonFixed) is defined
         # when the label is small: it produces ranges of buttons of the same
         # width (excepted when labels are too large), which is more beautiful.
-        if len(label) < 15: return 'buttonFixed button'
-        return 'button'
+        if len(label) < 15: return '%sbuttonFixed button' % prefix
+        return '%sbutton' % prefix
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Footer:
@@ -931,8 +1027,29 @@ class Footer:
 class Browser:
     '''Determines if the browser is compatible with Appy'''
 
-    ieRex = re.compile('MSIE\s+\d\.\d')
-    ieRex2 = re.compile('Trident.*rv\:')
+    # Main regex, allowing to extract browser name & version from the user agent
+    rex = re.compile('([a-zA-Z]+)(?:/|\s+)(\d+\.\d+)')
+
+    # Here are some examples. Trident corresponds to IE 11.
+    #   MSIE 6.0
+    #   Trident/7.0
+    #   Firefox/103.0
+    #   Chrome/103.0
+    #   ...
+
+    # Internet Explorer is not supported, any version
+    unsupported = asDict(('Trident', 'MSIE'))
+
+    # Supported browsers
+    supported = asDict(('Firefox', 'Seamonkey', 'Chrome', 'Chromium', 'Safari',
+                        'OPR', 'Opera'))
+
+    # If several browser names are included in the user agent, these ones take
+    # precedence over others.
+    precedes = asDict(('Firefox', 'Chrome', 'Chromium'))
+
+    # Browser names and minimal versions as supported by Appy
+    versions = {'Chrome': 87.0, 'Chromium': 87.0, 'Firefox': 90.0}
 
     @classmethod
     def getIncompatibilityMessage(class_, tool, handler):
@@ -941,85 +1058,29 @@ class Browser:
         # Get the "User-Agent" request header
         agent = handler.headers.get('User-Agent')
         if not agent: return
-        if class_.ieRex.search(agent) or class_.ieRex2.search(agent):
-            return tool.translate('wrong_browser')
-
-#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-class Columns:
-    '''Create special UI objects containing information about rendering objects
-       in table columns.'''
-
-    # Standard columns
-    standard = O(
-      number     = O(special=True, field='_number', width='15px',
-                     align='left', header=False),
-      checkboxes = O(special=True, field='_checkboxes', width='10px',
-                     align='center', header=True)
-    )
-
-    # Error messages
-    FIELD_NOT_FOUND = 'field "%s", used in a column specifier, not found.'
-
-    @classmethod
-    def get(class_, tool, modelClass, columnLayouts, dir='ltr',
-            addNumber=False, addCheckboxes=False):
-        '''Extracts and returns, from a list of p_columnLayouts, info required
-           for displaying columns of field values for modelClass's instances.
-        '''
-        # p_columnLayouts are specified for each field whose values must be
-        # shown. 2 more, not-field-related, column layouts can be specified with
-        # these names:
-        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-        # "_number"     | if the listed objects must be numbered by Appy, this
-        #               | string represents the column containing that number;
-        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-        # "_checkboxes" | if Appy must show checkboxes for the listed objects,
-        #               | this string represents the column containing the
-        #               | checkboxes.
-        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-        # If columns "_number" and "_checkboxes" are not among p_columnLayouts
-        # but are required (by p_addNumber and p_addCheckboxes), they are added
-        # to the result. Specifying them within p_columnLayouts allows to give
-        # them a precise position among all columns. When automatically added,
-        # they will appear before any other column (which is desirable in most
-        # cases).
-        r = []
-        numberFound = checkboxesFound = False
-        for info in columnLayouts:
-            name, width, align, header = ColumnLayout(info).get()
-            # It that a special column name ?
-            special = True
-            if name == '_number': numberFound = True
-            elif name == '_checkboxes': checkboxesFound = True
-            else: special = False
-            align = Language.flip(align, dir)
-            # For non-special columns, get the corresponding field
-            if not special:
-                field = modelClass.fields.get(name)
-                if not field:
-                    tool.log(class_.FIELD_NOT_FOUND % name, type='warning')
-                    continue
-            else:
-                # Let the column name in attribute "field"
-                field = name
-            r.append(O(special=special, field=field, width=width, align=align,
-                       header=header))
-        # Add special columns if required and not present
-        if addCheckboxes and not checkboxesFound:
-            r.insert(0, class_.standard.checkboxes)
-        if addNumber and not numberFound:
-            r.insert(0, class_.standard.number)
-        return r
-
-#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-def getActionsCss(display):
-    '''Gets the CSS class to apply to the zone rendering object actions, having
-       the specified p_display.'''
-    if display == 'block':
-        r = 'objectActions'
-    elif display == 'right':
-        r = 'alignedActions'
-    else:
-        r = 'inlineActions'
-    return r
+        # Get all (name, version) pairs as carried by the user agent
+        pairs = class_.rex.findall(agent)
+        if not pairs: return
+        # Keep only the relevant pair
+        name = version = None
+        for n, v in pairs:
+            # Directly dismiss unsupported browsers
+            if n in class_.unsupported:
+                return tool.translate('wrong_browser')
+            elif n in class_.supported:
+                # A supported browser has been found
+                if name:
+                    if n in class_.precedes:
+                        # We have 2 names: keep the most appropriate one
+                        name = n
+                        version = v
+                    break
+                name = n
+                version = v
+        # Issue a warning if the browser version is unsupported
+        if version:
+            minimal = class_.versions.get(name)
+            if minimal and float(version) < minimal:
+                map = {'name': name, 'version': version, 'minimal':minimal}
+                return tool.translate('old_browser', mapping=map)
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
