@@ -51,24 +51,43 @@ class Criteria:
         handler.cache.criteria = r.copy()
         return r
 
+    # The HTML tag representing an highlighted zone
+    highlighted = '<span class="highlight">%s</span>'
+
+    # The regex template representing highlithed text
+    highlightedRex = '(?<= |\(|\>)?%s'
+
+    @classmethod
+    def getKeywords(class_, handler):
+        '''Get the potential keywords to highlight'''
+        # Are such keywords present within search criteria ?
+        criteria = class_.readFromRequest(handler)
+        if criteria and 'searchable' in criteria:
+            r = criteria['searchable']
+            if isinstance(r, str):
+                # Highlighting operators is not supported yet
+                return handler.req.w_searchable or r
+        # Are such keywords present within search filters ?
+        filters = handler.req.filters
+        if filters:
+            filters = sutils.getDictFrom(filters)
+            if 'searchable' in filters:
+                return filters['searchable'].rstrip('*')
+
     @classmethod
     def highlight(class_, handler, text):
         '''Highlights parts of p_text if we are in the context of a search whose
            keywords must be highlighted.'''
         if not text: return ''
-        # Must we highlight something ?
-        criteria = class_.readFromRequest(handler)
-        if not criteria or ('searchable' not in criteria): return text
-        # Highlighting operators is not supported yet
-        keywords = criteria['searchable']
-        if not isinstance(keywords, str): return text
-        # Highlight every variant of every keyword. Use preferably the words as
-        # encoded by the user instead of their normalized version.
-        keywords = handler.req.w_searchable or keywords
+        # Must we highlight something ? Keywords to highlight may come, either
+        # from search criteria, or from search filters.
+        keywords = class_.getKeywords(handler)
+        if not keywords: return text
+        # Highlight every variant of every keyword
         for word in keywords.strip().split():
             for variant in (word, word.capitalize(), word.lower()):
-                highlighted = '<span class="highlight">%s</span>' % variant
-                text = re.sub('(?<= |\(|\>)?%s' % variant, highlighted, text)
+                text = re.sub(class_.highlightedRex % variant,
+                              class_.highlighted % variant, text)
         return text
 
     def getFromRequest(self, class_):
