@@ -71,7 +71,7 @@ class Globals:
        <input type="hidden" name="podFormat"/>
        <input type="hidden" name="action" value="upload"/>
        <input type="file" name="uploadedFile"/><br/><br/>
-       <input type="submit" value="%s"/>
+       <input type="button" onclick="this.form.submit()" value="%s"/>
        <input type="button" onclick="closePopup('uploadPopup')" value="%s"/>
       </form>
      </div>
@@ -116,7 +116,6 @@ class Globals:
       <input type="hidden" name="queryData"/>
       <input type="hidden" name="criteria"/>
       <input type="hidden" name="customParams"/>
-      <input type="hidden" name="showSubTitles" value="True"/>
       <input type="hidden" name="checkedIds"/>
       <input type="hidden" name="checkedSem"/>
       <input type="hidden" name="freezeAction"/>
