@@ -1,7 +1,7 @@
 '''Global elements to include in HTML pages'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -41,8 +41,8 @@ class Iframe:
      </div>'''
 
     # HTML page to render for closing the popup
-    back = '<html><head>%s%s</head><body><script>backFromPopup()</script>' \
-           '</body></html>'
+    back = '<!DOCTYPE html>\n<html><head>%s%s</head><body>' \
+           '<script>backFromPopup()</script></body></html>'
 
     @classmethod
     def goBack(class_, tool, initiator=None):
@@ -60,6 +60,12 @@ class Iframe:
         resp = tool.resp
         resp.setCookie('closePopup', close)
         # Include appy.js and call a Javascript function that will do the job
-        return class_.back % (Includer.js(tool.buildUrl('appy.js')),
-                              Includer.vars(tool)) 
+        version = tool.config.server.static.versions['appy.js']
+        # If the version of appy.js is not mentioned, it will be impossible to
+        # debug inside it. Adding statements like: "console.log" or "debugger"
+        # will not work, because the browser will consider "appy.js" not being
+        # the effective appy.js. In short, for the browser, appy.js is not
+        # appy.js?6.
+        return class_.back % (Includer.js(tool.buildUrl('appy.js?%d'% version)),
+                              Includer.vars(tool))
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
