@@ -77,16 +77,15 @@ class Language:
 
     # PX for selecting the language in the ui
     pxSelector = Px('''
-      <select var="languages=config.ui.selectableLanguages or \
-                     config.ui.languages;
-                   defaultLanguage=languages[0]"
-              onchange=":'switchLanguage(this,%s)' % q(siteUrl)">
+      <select onchange="switchLanguage(this)"
+          var="languages=config.ui.selectableLanguages or config.ui.languages;
+               defaultLanguage=languages[0]">
        <option for="lg in languages" value=":lg"
                selected=":lang == lg">:ui.Language.getName(lg)</option>
       </select>''',
 
       js='''
-       function switchLanguage(select, siteUrl) {
+       function switchLanguage(select) {
          goto(siteUrl + '/tool/ui/Language/switch?language=' + select.value);}
       ''')
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
