@@ -19,6 +19,7 @@
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 from appy.px import Px
+from appy.utils import asDict
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 # A "layout" defines how to dispose, in the user interface, parts of some
@@ -58,7 +59,7 @@ class Cell:
         '''Renders p_value (one element among self.content) for a given
            p_layoutTarget (a field or object) on some p_layout.'''
         # Do not render "r" if we must render a not-required object
-        if (value == 'pxRequired') and not layoutTarget.required: return
+        if value == 'pxRequired' and not layoutTarget.required: return
         # The name of the PX is the layout or p_value
         name = layout if value == 'f' else value
         r = layoutTarget
@@ -107,20 +108,24 @@ class Layout:
     '''Represents a logical table defining how to dispose graphical parts of
        some UI element.'''
 
-    # --------------------------------------------------------------------------
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     # Defining parts of a layouted object and how they will be rendered is done
     # via a "layout string". Such a string is made of:
     # - letters, corresponding to object parts;
     # - symbols, allowing to define column or row breaks, alignment, etc.
 
-    # Letters for a page -------------------------------------------------------
+    # Letters for a page - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+
     #  e - The object h*e*ader, containing its title, breadcrumb, siblings and
     #      navigation.
-    #  b - The range of *b*uttons (pages and phases, save, delete, actions and
+    #  b - The range of *b*uttons: pages and phases, save, delete, actions and
     #      workflow transitions.
     #  w - The fields (= *w*idgets) to render on some page for some object
+    #  p - The object *p*reamble, containing its class name, navigation and pod
+    #      fields
+
+    # Letters for a field  - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-    # Letters for a field ------------------------------------------------------
     #  l - "label"        The field label
     #  d - "description"  The field description
     #  h - "help"         Help for the field (typically rendered as an icon,
@@ -136,25 +141,27 @@ class Layout:
     # hereabove letters.
     pxs = {
       # Page-related elements
-      'e': 'pxHeader', 'w': 'pxFields', 'b': 'pxControls',
+      'e': 'pxHeader', 'w': 'pxFields', 'b': 'pxControls', 'p': 'pxPreamble',
       # Field-related elements
       'l': 'pxLabel', 'd': 'pxDescription', 'h': 'pxHelp', 'v': 'pxValidation',
       'r': 'pxRequired', 'c': 'pxChanges'
     }
 
-    # Symbols to use within a layout string ------------------------------------
+    # Symbols to use within a layout string  - - - - - - - - - - - - - - - - - -
+
     # A row delimiter is to be used at the end of a row. The symbol used defines
     # alignment for the previously defined row.
     rowDelimiters =  {'-':'middle', '=':'top', '_':'bottom'}
     rowDelms = ''.join(rowDelimiters.keys())
+
     # A cell delimiter is to be used at the end of a cell. The symbol used
     # defines alignment for the previouly defined cell.
     cellDelimiters = {'|': 'center', ';': 'left', '!': 'right'}
     cellDelms = ''.join(cellDelimiters.keys())
 
     # Base layout attributes
-    baseAttributes = ('style', 'css_class', 'cellpadding', 'cellspacing',
-                      'width', 'align')
+    baseAttributes = ('style', 'css', 'cellpadding', 'cellspacing', 'width',
+                      'align')
 
     # A layout can be derived from another one. The following dict defines the
     # letters to remove to define a simpler layout (view) from a more complex
@@ -172,7 +179,7 @@ class Layout:
     # not be defined by the cell layout, but by the outer column layout.
 
     pxRender = Px('''
-     <table var="layoutCss=table.css_class;
+     <table var="layoutCss=table.css;
                  inTd=layout=='cell' and bool(column)|False"
        cellpadding=":table.cellpadding" cellspacing=":table.cellspacing"
        width=":'' if inTd else table.width"
@@ -197,7 +204,7 @@ class Layout:
       </tr>
      </table>''')
 
-    def __init__(self, layoutString=None, style=None, css_class='',
+    def __init__(self, layoutString=None, style=None, css='',
                  cellpadding=0, cellspacing=0, width='100%', align='left',
                  other=None, derivedType=None):
         if other:
@@ -216,8 +223,8 @@ class Layout:
         else:
             source = ''
             self.layoutString = layoutString
-        # Initialise simple params, either from the true params, either from
-        # the p_other Table instance.
+        # Initialise simple params, either from the true params, or from
+        # the p_other Layout instance.
         for param in Layout.baseAttributes:
             setattr(self, param, eval('%s%s' % (source, param)))
         # The following attribute will store a special Row instance used for
@@ -237,13 +244,13 @@ class Layout:
         res = res.lstrip(Layout.rowDelms); res = res.lstrip(Layout.cellDelms)
         return res
 
-    def addCssClasses(self, css_class):
-        '''Adds a single or a group of p_css_class.'''
-        if not self.css_class: self.css_class = css_class
+    def addCssClasses(self, css):
+        '''Adds a single or a group of p_css classes'''
+        if not self.css: self.css = css
         else:
-            self.css_class += ' ' + css_class
+            self.css += ' ' + css
             # Ensures that every class appears once
-            self.css_class = ' '.join(set(self.css_class.split()))
+            self.css = ' '.join(set(self.css.split()))
 
     def isHeaderRow(self, rowContent):
         '''Determines if p_rowContent specified the table header row or a
@@ -260,8 +267,8 @@ class Layout:
         # Split the p_layoutString with the row delimiters
         rowContent = ''
         for char in layoutString:
-            if char in Layout.rowDelimiters:
-                valign = Layout.rowDelimiters[char]
+            if char in self.rowDelimiters:
+                valign = self.rowDelimiters[char]
                 if self.isHeaderRow(rowContent):
                     if not self.headerRow:
                         self.headerRow = Row(rowContent, valign, isHeader=True)
@@ -310,22 +317,61 @@ class Layout:
 
     def __repr__(self): return '<Layout %s>' % self.layoutString
 
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+class LayoutF(Layout):
+    '''Alternative to parent class Layout, using a div tag with flex layout,
+       instead of a table.'''
+
+    # Flex-specific row delimiters (vertical alignment)
+    rowDelimiters =  {'-':'center', '=':'start', '_':'end'}
+    rowDelms = ''.join(rowDelimiters.keys())
+
+    pxRender = Px('''
+     <div style=":table.getStyle()" class=":table.getCss(_ctx_)"
+          id=":tagId" name=":tagName">
+      <x for="cell in table.row.cells">
+       <x for="c in cell.content">::cell.renderContent(c, layout, layoutTarget)
+       </x>
+      </x>
+     </div>''')
+
+    def __init__(self, layoutString=None, style=None, css=None, other=None,
+                 derivedType=None):
+        # Call the base constructor
+        super().__init__(layoutString, style=style, css=css, other=other,
+                         derivedType=derivedType)
+        # Unwrap the first row, that is the unique row that will be taken into
+        # account.
+        self.row = self.rows[0]
+
+    def getStyle(self):
+        '''Get content of the main tag's "style" attribute'''
+        r = 'display:flex;align-items:%s' % self.row.valign
+        if self.style:
+            r = '%s;%s' % (r, self.style)
+        return r
+
+    def getCss(self, c):
+        '''Get the CSS class(es) to apply to the main tag's "class" attribute'''
+        r = '%s %s' % (c.tagCss or '', self.css or '')
+        return r.strip()
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class ColumnLayout:
     '''A "column layout" dictates the way a table column must be rendered. Such
        a layout is of the form: <name>[*width][,|!|`|`][^].'''
 
-    # --------------------------------------------------------------------------
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     #   <name>  | is the name of the field whose content must be shown in
     #           | column's cells ;
-    # --------------------------------------------------------------------------
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     #   width   | is the width of the column. Any valid value for the "width"
     #           | attribute of the "td" HTML tag is accepted ;
-    # --------------------------------------------------------------------------
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     #   , | !   | column alignment: respectively, left, centered or right ;
-    # --------------------------------------------------------------------------
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     #     ^     | if present, indicates that the column header must be empty.
-    # --------------------------------------------------------------------------
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     def __init__(self, layoutString):
         self.layoutString = layoutString
@@ -365,20 +411,24 @@ class Layouts(dict):
     # concept of "layout type". Appy currently defines the following layout
     # types.
     types = (
-      'view',    # a read-only page for an object ;
-      'edit',    # the editable page for an object ;
-      'result',  # a cell within a list of objects ;
-      'query',   # the top-level zone on the page displaying results of some
+      'view'   , # a read-only page for an object ;
+      'edit'   , # the editable page for an object ;
+      'result' , # a cell within a list of objects ;
+      'query'  , # the top-level zone on the page displaying results of some
                  # search, typically used for rendering Pod fields allowing
                  # to export documents including these results ;
-      'sub',     # the series of buttons rendered besides the object title on a
+      'sub'    , # the series of buttons rendered besides the object title on a
                  # list of objects ;
       'buttons', # the series of buttons rendered on a view/edit page ;
-      'xml',     # similar to 'view', but in XML format, readable by a peer
+      'pre'    , # The zone in the page preamble, on object/view or edit ;
+      'xml'    , # similar to 'view', but in XML format, readable by a peer
                  # system, the 'view' layout being in XHTML format, readable
                  # by a human being ;
-      'sidebar'  # The zone in the UI situated, for left-to-right languages,
+      'sidebar', # The zone in the UI situated, for left-to-right languages,
                  # at the right of the screen.
+      'field'    # Represents the zone where some field is rendered. It means:
+                 # let the container field decide how to render some sub-field
+                 # within himself.
     )
 
     # Note that a layout will not be defined for every layout type. For example,
@@ -391,11 +441,25 @@ class Layouts(dict):
     # Layouts instance.
     baseTypes = ('edit', 'view', 'search')
 
+    # "Top" layouts correspond to a zone on the top-right corner of some pages
+    topLayouts = asDict(('query', 'pre'))
+
+    # Layouts corresponding to ranges of icons or buttons
+    buttonLayouts = asDict(('buttons', 'sub', 'view', 'xml'))
+
     # Some layouts must be explicitly specified in order to be taken into
     # account, while, for most of them, this is implicit. For example, when
     # determining visibility of a field, boolean "True" implicitly denotes any
     # layout type, excepted those listed in the following attribute.
-    explicitTypes = ('sub', 'buttons', 'sidebar')
+    explicitTypes = ('sub', 'buttons', 'sidebar', 'field')
+
+    # The following layouts do not accept all types of fields
+    restrictedTypes = asDict(('buttons', 'sub', 'pre', 'query'))
+    restrictedTypesB = asDict(('sub', 'pre', 'query'))
+
+    # Use this lambda as attribute field.renderable for a field that must be
+    # renderable on layout "buttons" in addition to standard layouts.
+    onButtons = lambda layout: layout not in Layouts.restrictedTypesB
 
     def __init__(self, edit=None, view=None, search=None):
         '''Initialise layouts based on the specified attributes'''
@@ -405,10 +469,12 @@ class Layouts(dict):
         if search: search = self['search'] = self.getLayoutFrom(search)
         # Derive unspecified layouts from the existing ones, when possible
         if edit and not view:
-            view = self['view'] = Layout(other=edit, derivedType='view')
+            class_ = edit.__class__
+            view = self['view'] = class_(other=edit, derivedType='view')
         if view and not search:
             # Do not derive a search layout from a view: because most search
             # forms are similar, use a standard layout else.
+            class_ = view.__class__
             self['search'] = Layout('l-f')
 
     def getLayoutFrom(self, this):
@@ -452,7 +518,7 @@ class Layouts(dict):
     # other Ajax-like functionalities in the future). Indeed, the machinery for
     # ajax-editing the field is inlaid in the XHTML table corresponding to the
     # layout. In such case, we will use a standard cell layout defined here.
-    cell = Layout('f|', align='center', css_class='no')
+    cell = Layout('f|', align='center', css='no')
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     #                              Class methods
@@ -549,17 +615,19 @@ Layouts.fvd = Layouts(edit=Layout(d), view=Layout('f'))
 Layouts.g   = Layouts('frvl')
 # The *d*escription is visible, even on the *v*iew layout
 Layouts.dv  = Layouts(edit=d, view='l-d-f')
+Layouts.dvt = Layouts(edit=d, view=Layout('l-d-f', css='topSpace'))
 # The *d*escription is shown on the *v*iew layout but not on *e*dit (=*s*imple)
 Layouts.dvs = Layouts(edit=b, view='l-d-f')
 # "Grid" group-related layouts
 Layouts.gn  = Layouts(Layout('f;rvl=', width=None))
 Layouts.gd  = Layouts(Layout('frvl-d', width='99%'))
-Layouts.gdn = Layouts(Layout('d2-f;rvl=', width=None))
+Layouts.dg  = Layouts(Layout('d-frvl', width='99%'))
+Layouts.gdn = Layouts(edit=Layout('d2-f;rvl=', width=None), view=Layout('f'))
 Layouts.gh  = Layouts('fhrvl')
 Layouts.gdh = Layouts('fhrvl-d')
 # Base layouts, with space above
-Layouts.t = Layouts(Layout(b, width=None, css_class='topSpace'))
-Layouts.td = Layouts(Layout(d, css_class='topSpace'))
+Layouts.t = Layouts(Layout(b, width=None, css='topSpace'))
+Layouts.td = Layouts(Layout(d, css='topSpace'))
 
 # Fields being in groups having style "grid" deserve specific layouts. This why
 # there are 2 default layouts.
@@ -583,6 +651,9 @@ class PageLayouts:
                        view=Layout('e-b|-w|', align='center'))
     # Headless (without header on "view")
     headless = Layouts(edit=Layout('b-w', width=None), view=Layout('b-w'))
+    # Complete (with preamble and header), wide or narrow
+    complete = Layouts(edit=Layout('b-w'), view=Layout('p-e-b-w'))
+    completeN = Layouts(edit=Layout('b-w', width=None), view=Layout('p-e-b-w'))
 
 Layouts.Page = PageLayouts
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
