@@ -28,6 +28,7 @@ WRONG_NAV  = 'Wrong nav key "%s".'
 class Sibling:
     '''Represents a sibling element, accessible via navigation from
        another one.'''
+
     # Existing types of siblings
     types = ('previous', 'next', 'first', 'last')
 
@@ -51,6 +52,8 @@ class Sibling:
         self.page = page
         # Are we in a popup or not ?
         self.popup = siblings.popup
+        # The CSS class to apply to the icon
+        self.iconCss = siblings.iconCss
 
     def getSiblingIndex(self):
         '''Return the index of the sibling to navigate to'''
@@ -67,7 +70,16 @@ class Sibling:
             r = 0
         return r
 
-    def get(self, ctx):
+    @classmethod
+    def getIcon(class_, c, type, css, js=None):
+        '''Return the img tag corresponding to the sibling of this p_type'''
+        # If no p_js is passed, it means that a disabled icon must be rendered
+        title = ' title="%s"' % c._('goto_%s' % type) if js else None
+        return '<img src="%s"%s class="clickable %s" style="transform: ' \
+               'rotate(%ddeg)" onclick="%s"/>' % \
+               (c.svg(Sibling.icons[type]), title, css, Sibling.rotate[type],js)
+
+    def get(self, c):
         '''Get the HTML chunk allowing to navigate to this sibling'''
         if self.o:
             # We already have the object and thus its URL
@@ -77,17 +89,12 @@ class Sibling:
             # the browser's session storage.
             container = self.container
             searchName = container.uiSearch.name
-            sessionKey = container.search.getSessionKey(searchName)
-            url = "buildSiblingUrl('%s','%s',%d)" % \
-                  (container.tool.siteUrl, sessionKey, self.getSiblingIndex())
+            key = container.search.getSessionKey(searchName)
+            url = "buildSiblingUrl('%s',%d)" % (key, self.getSiblingIndex())
         # Build the JS code allowing to navigate to the sibling element
         js = "gotoSibling(%s,'%s','%s','%s')" % \
              (url, self.nav, self.page, self.popup)
-        type = self.type
-        return '<img src="%s" title="%s" class="clickable iconXS" ' \
-               'style="transform: rotate(%ddeg)" onclick="%s"/>' % \
-               (ctx.svg(Sibling.icons[type]), ctx._('goto_%s' % type),
-                Sibling.rotate[type], js)
+        return self.getIcon(c, self.type, self.iconCss, js)
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Siblings:
@@ -95,96 +102,95 @@ class Siblings:
        its siblings.'''
 
     pxGotoNumber = Batch.pxGotoNumber
+
     # Attributes storing the siblings for going "backwards" (True) or "forwards"
     # (False)
-    byType = {True: ('firstSibling', 'previousSibling'),
-              False: ('nextSibling',  'lastSibling')}
+    byType = { True : ('firstSibling', 'previousSibling'),
+               False: ('nextSibling' , 'lastSibling'    )}
 
     # Icons for going to the current object's siblings
-    pxNavigate = Px('''<div>
+    pxNavigate = Px('''
+     <div class="snav">
       <!-- Go to the source URL (search or referred object) -->
-      <div if="not popup" class="pagePicto">::self.getGotoSource(svg, _)</div>
+      <div if="not popup and gotoSource"
+           class="pagePicto">::snav.getGotoSource(svg, _)</div>
 
       <!-- Show other navigation icons only when relevant -->
-      <div if="self.total" class="navSib">
+      <div if="snav.total" class="navSib">
        <!-- Form used to navigate to any sibling -->
-       <form name=":self.siblingsFormName" method="post" action="">
+       <form name=":snav.siblingsFormName" method="post" action="">
         <input type="hidden" name="page" value=""/>
         <input type="hidden" name="popup" value=""/>
-        <input type="hidden" name="criteria" value=""/>
        </form>
 
        <!-- Go to the first and/or previous page -->
-       <x>::self.getIcons(_ctx_, previous=True)</x>
+       <x>::snav.getIcons(_ctx_, previous=True)</x>
 
        <!-- Explain which element is currently shown -->
-       <x>:self.number</x> / <x>:self.total</x> 
+       <span class="navText">
+        <x>:snav.number</x> <span class="navSep">//</span> 
+        <span class="btot">:snav.total</span>
+       </span>
 
        <!-- Go to the next and/or last page -->
-       <x>::self.getIcons(_ctx_, previous=False)</x>
+       <x>::snav.getIcons(_ctx_, previous=False)</x>
 
        <!-- Go to the element number... -->
-       <div if="self.showGotoNumber()" class="pagePicto"
-            var2="field=self.field; sourceUrl=self.sourceObject.url;
-                  total=self.total">:self.pxGotoNumber</div>
+       <div if="snav.showGotoNumber()" class="pagePicto"
+            var2="field=snav.field; sourceUrl=snav.sourceObject.url;
+                  total=snav.total">:snav.pxGotoNumber</div>
       </div></div>''',
 
-     css='''.navSib { padding:|navsPadding|; font-size:90%; display:inline }''',
+     css='''
+      .navSib { display:inline-flex; gap:0.2em; flex-wrap:nowrap }
+      .navSib input[type=text] { text-align:center; padding:0; margin:0 }''',
 
      js='''
-       function gotoCustomSearch() {
-         // Post a form allowing to re-trigger a custom search
-         var f = document.forms['gotoSource'];
-         // Add search criteria from the browser's session storage
-         f.criteria.value = sessionStorage.getItem(f.className.value);
-         f.submit();
+       function gotoSourceSearch(key) {
+         // Post a form allowing to re-trigger a search
+         let params = getSearchInfo(key, false);
+         post(siteUrl + '/tool/Search/results', params);
        }
-       function buildSiblingUrl(siteUrl, sessionKey, index) {
-         var ids = JSON.parse(sessionStorage.getItem(sessionKey));
+
+       function askSiblings(key, index) {
+         /* Re-trigger a server search for getting siblings of the object having
+            this p_index; store and return these IDs. */
+         let url = siteUrl + '/tool/Search/jresults',
+             params = getSearchInfo(key, false);
+         params['index'] = index;
+         askAjaxChunk(url, 'POST', params, '*'+key);
+         return getSearchInfo(key, true);
+       }
+
+       function buildSiblingUrl(key, index) {
+         let ids = getSearchInfo(key, true);
          if (!(index in ids)) {
-           // Get siblings by performing a new server search
-           var parts = sessionKey.split('_'),
-               className = parts[0],
-               search = parts[1],
-               params = {'className':className, 'search':search, 'index':index},
-               url = siteUrl + '/tool/Search/jresults';
-           if (search == 'customSearch') {
-             params['criteria'] = sessionStorage.getItem(className);
-           }
-           askAjaxChunk(url, 'POST', params, '*'+sessionKey);
-           ids = JSON.parse(sessionStorage.getItem(sessionKey));
+           ids = askSiblings(key, index);
          }
-         var iid = ids[index];
+         let iid = ids[index];
          return siteUrl + '/' + iid + '/view';
        }
 
        function gotoSibling(url, nav, page, popup) {
-         var formName = (popup == 'True') ? 'siblingsPopup' : 'siblings',
+         let formName = (popup == 'True') ? 'siblingsPopup' : 'siblings',
              f = document.forms[formName];
          // Navigate to the sibling by posting a form
          f.action = url;
          f.page.value = page;
          f.popup.value = popup;
-         if (nav) {
-           // For a custom search, get criteria from the session storage
-           var elems = nav.split('.');
-           if ((elems[0] == 'search') && (elems[2] == 'customSearch')) {
-             f.criteria.value = sessionStorage.getItem(elems[1]);
-           }
-           // Avoid losing navigation
-           f.action = f.action + '?nav=' + nav;
-         }
+         // Avoid losing navigation
+         if (nav) f.action = f.action + '?nav=' + nav;
          f.submit();
         }''')
 
     @staticmethod
-    def get(nav, tool, ctx):
+    def get(nav, tool, ctx, iconCss='iconXXS'):
         '''Analyse the navigation info p_nav and returns the corresponding
           concrete Siblings instance.'''
         elems = nav.split('.')
         Siblings = RefSiblings if elems[0] == 'ref' else SearchSiblings
         try:
-            return Siblings(tool, ctx, *elems[1:])
+            return Siblings(tool, ctx, *elems[1:], iconCss=iconCss)
         except KeyError:
             # Param "nav" may be broken. Example: the silly MSN bot replays Appy
             # URLs, all lowercased.
@@ -203,10 +209,11 @@ class Siblings:
             res += batchSize
         return res
 
-    def __init__(self, tool, ctx, number, total):
+    def __init__(self, tool, ctx, number, total, iconCss):
         self.tool = tool
         self.req = tool.req
         self.ctx = ctx
+        self.iconCss = iconCss
         # Are we in a popup window or not ?
         self.popup = ctx.popup
         self.siblingsFormName = 'siblingsPopup' if self.popup else 'siblings'
@@ -227,7 +234,7 @@ class Siblings:
         if self.previousIndex > 0: self.firstNeeded = True
         self.lastNeeded = False     # Last ?
         self.lastIndex = self.total - 1
-        if (self.nextIndex < self.lastIndex): self.lastNeeded = True
+        if self.nextIndex < self.lastIndex: self.lastNeeded = True
         # Get the current object's siblings
         self.siblings = siblings = self.getSiblings()
         # Compute the URL allowing to go back to the "source" = a given page of
@@ -260,7 +267,7 @@ class Siblings:
            or search.'''
         return _('goto_source'), self.getBackText() or ''
 
-    def getIcons(self, ctx, previous=True):
+    def getIcons(self, c, previous=True):
         '''Produce icons for going to
            - the first or the previous page if p_previous is True;
            - the next or last page else.
@@ -268,28 +275,36 @@ class Siblings:
         r = ''
         for name in Siblings.byType[previous]:
             sibling = getattr(self, name)
-            if sibling: r += sibling.get(ctx)
+            if sibling:
+                r += sibling.get(c)
+            else:
+                # Get a disabled icon
+                r += Sibling.getIcon(c, name[:-7], 'disabled %s' % self.iconCss)
         return r
 
-    def getGotoSource(self, svg, _):
+    def getGotoSource(self, svg, _, imgCss=None):
         '''Get the link allowing to return to the source URL'''
         text, message = self.getBackTexts(_)
-        return '<a href="%s" title="%s"><img src="%s" class="back"/>%s</a>'% \
-               (self.sourceUrl, message, svg('arrowsA'), text)
+        # Additional CSS class for the icon
+        css = ' %s' % imgCss if imgCss else ''
+        return '<a href="%s" title="%s"><img src="%s" class="back%s"/>%s</a>'% \
+               (self.sourceUrl, message, svg('arrowsA'), css, text)
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class RefSiblings(Siblings):
     '''Class containing information for navigating from one object to another
        within tied objects from a Ref field.'''
+
     prefix = 'ref'
 
-    def __init__(self, tool, popup, sourceId, fieldName, number, total):
+    def __init__(self, tool, popup, sourceId, fieldName, number, total,
+                 iconCss='iconXXS'):
         # The source object of the Ref field
         self.sourceObject = tool.getObject(sourceId)
         # The Ref field in itself
         self.field = self.sourceObject.getField(fieldName)
         # Call the base constructor
-        Siblings.__init__(self, tool, popup, number, total)
+        super().__init__(tool, popup, number, total, iconCss)
 
     def getNavKey(self):
         '''Returns the general navigation key for navigating to another
@@ -340,9 +355,11 @@ class RefSiblings(Siblings):
 class SearchSiblings(Siblings):
     '''Class containing information for navigating from one object to another
        within results of a search.'''
+
     prefix = 'search'
 
-    def __init__(self, tool, ctx, className, searchName, number, total):
+    def __init__(self, tool, ctx, className, searchName, number, total,
+                 iconCss='iconXXS'):
         # The class determining the type of searched objects
         self.className = className
         self.class_ = tool.model.classes[className]
@@ -351,7 +368,7 @@ class SearchSiblings(Siblings):
         self.uiSearch = tool.Search.get(searchName, tool, self.class_, ctx,
                                         ui=True)
         self.search = self.uiSearch.search
-        Siblings.__init__(self, tool, ctx, number, total)
+        super().__init__(tool, ctx, number, total, iconCss)
 
     def getNavKey(self):
         '''Returns the general navigation key for navigating to another
@@ -395,23 +412,18 @@ class SearchSiblings(Siblings):
             if ref: params += '&ref=%s' % ref
             return '%s?%s' % (url, params)
 
-    def getGotoSource(self, svg, _):
-        '''Get the link or form allowing to return to the source URL'''
-        if self.searchName != 'customSearch':
-            return Siblings.getGotoSource(self, svg, _)
-        # For a custom search, post a form with the search criteria retrieved
-        # from the browser's session.
+    def getGotoSource(self, svg, _, imgCss=None):
+        '''Get the code allowing to re-trigger the source search'''
+        # Post a form with the search parameters (and criteria for a custom
+        # search) retrieved from the browser's session storage.
         text, message = self.getBackTexts(_)
-        return '<form method="post" action="%s" name="gotoSource">' \
-          '<input type="hidden" name="className" value="%s"/>' \
-          '<input type="hidden" name="search" value="customSearch"/>' \
-          '<input type="hidden" name="start" value="%d"/>' \
-          '<input type="hidden" name="ref" value="%s"/>' \
-          '<input type="hidden" name="criteria" value=""/></form>' \
-          '<a class="clickable" title="%s" onclick="gotoCustomSearch()">' \
-          '<img src="%s" class="back"/>%s</a>' % \
-          (self.sourceUrl, self.className, self.computeStartNumber(),
-           self.req.ref or '', message, svg('arrowsA'), text)
+        # The key storing search parameters in the session storage
+        key = '%s_%s' % (self.className, self.searchName)
+        # Additional CSS class for the icon
+        css = ' %s' % imgCss if imgCss else ''
+        return '<a class="clickable" title="%s" onclick="gotoSourceSearch' \
+               '(\'%s\')"><img src="%s" class="clickable back%s"/>%s</a>' % \
+                (message, key, svg('arrowsA'), css, text)
 
     def showGotoNumber(self): return
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
