@@ -19,6 +19,7 @@
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 from appy.px import Px
+from appy.utils import asDict
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Template:
@@ -35,13 +36,13 @@ class Template:
             r = tool.translate('app_name_raw')
         else:
             # The page title is based on p_o's title
-            r = o.getShownValue()
+            r = o.getShownValue() or ''
         # Ensure this is a string
         r = r if isinstance(r, str) else str(r)
         # If a XHTML paragraph is found, keep the part before it
         if '</p>' in r: r = r[3:r.index('</p>')]
         # Add p_o's page title if found
-        if o and ('page' in tool.req):
+        if o and 'page' in tool.req:
             label = '%s_page_%s' % (o.class_.name, tool.req.page)
             text = tool.translate(label, blankOnError=True)
             if text:
@@ -56,95 +57,169 @@ class Template:
     zoneCss = {'payload': {False: 'payload', True: 'payload payloadP'},
                'content': {False: 'content', True: 'contentP'}}
 
+    # Values for "show header" that do not require a global top space
+    showheaderNoTop = asDict((None, 'sub', 'portlet'))
+
     @classmethod
-    def getCssFor(class_, zone, ctx):
+    def getCssFor(class_, zone, c):
         '''Return the CSS class(es) to apply to this p_zone'''
         # In some situations (ie, in the popup), a more compact design must be
         # applied.
-        compact = ctx.popup or (not ctx.showPortlet and \
-                                (ctx._px_.name in (class_.compactPx[zone])))
+        compact = c.popup or (not c.showPortlet and \
+                              (c._px_.name in (class_.compactPx[zone])))
         return class_.zoneCss[zone][compact]
 
     @classmethod
-    def includeLoginBox(class_, ctx):
-        '''Must the login box be integrated in the current page ?'''
+    def includeLoginBox(class_, c):
+        '''Must the login box be included in the current page ?'''
         # The login box has no sense if the user is already authenticated or in
-        # the process of self-registering himself, or if browser incompatibility
-        # has be detected.
-        if not ctx.isAnon or ctx.o.isTemp() or ctx.bi: return
+        # the process of self-registering, or if browser incompatibility has be
+        # detected.
+        if not c.isAnon or c.o.isTemp() or c.bi: return
         # Moreover, if we are on the public page and the authentication will be
         # done by redirecting the user to tool/home, it is useless to include
         # the login box in this case, too.
-        if (ctx._px_.name == 'public') and (ctx.cfg.discreetLogin == 'home'):
+        if c._px_.name == 'public' and c.cfg.discreetLogin == 'home':
             return
         # Include it in any other case
         return True
 
     @classmethod
-    def getMainStyle(class_, ctx):
+    def getMainStyle(class_, c):
         '''Return the CSS properties for the main, global zone'''
         # Get the main background image
-        r = ctx.cfg.getBackground(ctx._px_, ctx, type='home')
-        # Get the global height. Deduce the footer height if the footer is shown
-        if ctx.showFooter:
-            height = 'calc(100%% - %s)' % ctx.cfg.footerHeight
+        r = c.cfg.getBackground(c._px_, c, type='home')
+        # Get the global height. Substract the footer height if the footer is
+        # shown, excepted if the portlet footer is there. In this latter case,
+        # set the main height to 100%: the portlet itself will get a reduced
+        # height taking into account its specific footer.
+        if c.showFooter and not c.showFooterP:
+            height = 'calc(100%% - %s)' % c.cfg.footerHeight
         else:
             height = '100%'
         height = 'height:%s' % height
         return '%s;%s' % (r, height) if r else height
 
     @classmethod
-    def getPayloadStyle(class_, ctx):
+    def getPayloadStyle(class_, c):
         '''Gets the CSS styles to apply to the "payload" zone'''
         # The payload zone is the block portlet / content / sidebar
         # ~~~ Determine the background image
-        r = ctx.cfg.getBackground(ctx._px_, ctx,
-                                  type='popup' if ctx.popup else 'base')
+        r = c.cfg.getBackground(c._px_, c, type='popup' if c.popup else 'base')
         # ~~~ Determine the "top" property
-        showHeader = ctx.showHeader
-        if ctx.showHeader is None or ctx.showHeader == 'sub' or \
-           ctx.payloadCss.endswith('P'):
+        showHeader = c.showHeader
+        if c.showHeader in class_.showheaderNoTop or c.payloadCss.endswith('P'):
             top = '0'
         else:
-            top = ctx.headerHeight
+            top = c.headerHeight
         top = 'top:%s' % top
         r = '%s;%s' % (r, top) if r else top
         return r
 
     @classmethod
-    def getIconsStyle(class_, ctx):
+    def getIconsStyle(class_, c):
         '''Get CSS styles to apply to the block of icons in the page header'''
-        margin = ctx.cfg.cget('headerMargin', ctx)
+        margin = c.cfg.cget('headerMargin', c)
         return 'margin-%s:auto' % margin
 
     @classmethod
-    def getContent(class_, ctx):
+    def getContentStyle(class_, c):
+        '''Get CSS dynamic CSS properties for the "content" zone'''
+        # Enable vertical scroll in such a way that the portlet and sidebar stay
+        # visible.
+        return '' if c.popup else ' style="overflow-y:auto"'
+
+    @classmethod
+    def getContent(class_, c):
         '''Returns the page content, with the sub-header when relevant'''
-        r = '<div class="%s">%s</div>' % (class_.getCssFor('content', ctx),
-                                          ctx.content(ctx, applyTemplate=False))
+        r = '<div class="%s"%s>%s</div>' % (class_.getCssFor('content', c),
+                                            class_.getContentStyle(c),
+                                            c.content(c, applyTemplate=False))
+        # Add the burger button to expand the portlet. Indeed, when the header
+        # is inlaid into the portlet, the burger button being within ths header
+        # is unavailable.
+        if c.inlaidHeader:
+            c.cookie = 'appyPortlet'
+            c.bcss = 'eburger'
+            r = '%s%s' % (class_.pxBurger(c), r)
         # Add the sub-header when appropriate
-        if ctx.showHeader == 'sub':
-            r = '<div class="subbed">%s%s</div>' % (class_.pxHeader(ctx), r)
+        if c.showHeader == 'sub':
+            r = '<div class="subbed">%s%s</div>' % (class_.pxHeader(c), r)
         return r
 
     @classmethod
-    def showConnect(class_, ctx):
-        '''Show the "connect" link for anons only, if discreet login is enabled,
-           and if we are not already on a page allowing to log in.'''
-        return ctx.isAnon and ctx.cfg.discreetLogin and \
-               not ctx._px_.isHome(ctx, orPublic=False)
+    def showConnect(class_, c):
+        '''Show the "connect" link for anons only, if discreet login is enabled
+           or if the login box is hidden, and if we are not already on a page
+           allowing to log in.'''
+        if not c.isAnon: return
+        if c.cfg.discreetLogin:
+            # Show it if we are not already on a page allowing to log in
+            r = not c._px_.isHome(c, orPublic=False)
+        else:
+            # Show it if we are on the public site, where there is no login box
+            r = c._px_.name == 'public'
+        return r
 
     # Hooks for defining PXs proposing additional links or icons, before and
     # after the links / icons corresponding to top-level pages and icons.
     pxLinksBefore = pxLinks = pxLinksAfter = None
 
+    @classmethod
+    def getBurgerStyle(class_, c):
+        '''Get dynamic CSS properties to apply to a burger button'''
+        # A portlet-related "e-burger" (=*e*xternal burger, being outside the
+        # main header), must only be shown if the portlet is collapsed.
+        if c.bcss == 'eburger' and c.req.appyPortlet != 'collapsed':
+            r = 'none'
+        else:
+            r = 'block'
+        return 'display:%s' % r
+
+    # PX for rendering a burger button
+    pxBurger = Px('''
+     <a class=":bcss|''" style=":Template.getBurgerStyle(_ctx_)"
+        onclick=":'toggleCookie(%s,%s,%s,%s,%s)' %
+                  (q(cookie), q('block'), q('expanded'), q('show'), q('hide'))">
+      <img src=":svg('burger')" class="clickable icon"/>
+     </a>''',
+
+     css='''
+      .burger { cursor:pointer; margin:|burgerMargin| }
+      .eburger { position:absolute; top:2.7em; margin:0 7px }
+      .eburger img { width:16px; transform:rotate(180deg) }''')
+
+    # Link to go to the tool
+    pxTool = Px('''
+     <a href=":'%s/view' % tool.url" title=":_('Tool')">
+      <img src=":svg('config')" class="icon"/></a>''')
+
+    # The link to open the login box, if we are in "discreet login" mode
+    pxLogin = Px('''
+     <div class="textIcon" id="loginIcon" name="loginIcon"
+          var2="loginUrl=guard.getLoginUrl(_ctx_)">
+      <a href=":loginUrl">
+       <img if="cfg.showLoginIcon" src=":svg('login')" class="icon textIcon"/>
+      </a>
+      <a href=":loginUrl"><span>:_('app_connect')</span></a>
+     </div>''')
+
+    # The link to log out
+    pxLogout = Px('''
+     <div if="not isAnon" class=":'textIcon' if cfg.logoutText else ''"
+        var2="logoutText=_('app_logout');
+              logoutUrl=guard.getLogoutUrl(tool, user)">
+      <a href=":logoutUrl" title=":logoutText">
+       <img src=":svg('logout')" class="icon"/></a>
+      <a if="cfg.logoutText" href=":logoutUrl"><span>:logoutText</span></a>
+     </div>''')
+
     # Global links, that can be shown within the template header but also from
     # other pages, like "home[s]", without the base header container.
     pxTemplateLinks = Px('''
      <!-- The burger button for collapsing the portlet -->
-     <a if="showPortlet" class="burger"
-        onclick="toggleCookie('appyPortlet','block','expanded',\
-              'show','hide')"><img src=":svg('burger')" class="icon"/></a>
+     <x if="showPortlet"
+        var2="cookie='appyPortlet'; bcss='burger'">:Template.pxBurger</x>
 
      <!-- Custom links (I) -->
      <x>:Template.pxLinksBefore</x>
@@ -156,8 +231,8 @@ class Template:
       <x>:Template.pxLinks</x>
 
       <!-- Header messages -->
-      <span class="topText" var="text=cfg.getHeaderText(tool)"
-            if="not popup and text">::text</span>
+      <span var="text=cfg.getHeaderText(tool)"
+         if="not popup and text">::text</span>
 
       <!-- The home icon -->
       <a if="not isAnon and cfg.tget('showHomeIcon', tool)"
@@ -165,57 +240,47 @@ class Template:
         <img src=":svg('home')" class="icon"/></a>
 
       <!-- Connect link if discreet login -->
-      <a if="Template.showConnect(_ctx_)" id="loginIcon" name="loginIcon"
-         href=":guard.getLoginUrl(_ctx_)" class="clickable">
-       <img if="cfg.showLoginIcon" src=":svg('login')" class="icon"/>
-       <span class="topText">:_('app_connect')</span>
-      </a>
+      <x if="Template.showConnect(_ctx_)">:Template.pxLogin</x>
 
       <!-- Root pages -->
       <x if="cfg.tget('showRootPages', tool)"
-         var2="pages=tool.OPage.getRoot(tool)">
-       <x if="pages">:tool.OPage.pxSelector</x></x>
+         var2="pagesL, pagesO=tool.OPage.getRoot(tool)">
+       <x if="pagesL or pagesO">:tool.OPage.pxSelector</x></x>
 
       <!-- Language selector -->
       <x if="ui.Language.showSelector(cfg,layout)">:ui.Language.pxSelector</x>
 
       <!-- User info and controls for authenticated users -->
       <x if="not isAnon">
+
        <!-- Config -->
-       <a if="cfg.showTool(tool)" href=":'%s/view' % tool.url"
-              title=":_('Tool')">
-        <img src=":svg('config')" class="icon"/></a>
-       <x if="cfg.tget('showUserLink', tool)">:user.pxUserLink</x>
+       <x if="cfg.showTool(tool)">:Template.pxTool</x>
+
+       <!-- User link -->
+       <x if="cfg.tget('showUserLink', tool)">:user.pxLink</x>
+
        <!-- Authentication context selector -->
        <x var="ctx=config.security.authContext" if="ctx">:ctx.pxLogged</x>
       </x>
 
       <!-- Custom links (III) -->
       <x>:Template.pxLinksAfter</x>
-     </div>
 
-     <!-- The burger button for collapsing the sidebar -->
-     <a if="showSidebar" class="burger"
-        onclick="toggleCookie('appySidebar','block','expanded',\
-           'show','hide')"><img src=":svg('burger')" class="icon"/></a>
+      <!-- The burger button for collapsing the sidebar -->
+      <x if="showSidebar" var2="cookie='appySidebar'">:Template.pxBurger</x>
 
-     <!-- Logout -->
-     <a if="not isAnon" var2="logoutText=_('app_logout')"
-        href=":guard.getLogoutUrl(tool, user)"
-        title=":logoutText">
-      <img src=":svg('logout')" class="icon"/>
-      <span>:logoutText if cfg.logoutText else ''</span>
-     </a>''',
+      <!-- Logout -->
+      <x if="not isAnon">:Template.pxLogout</x>
+     </div>''',
 
      css='''
-      .topIcons > a,
-      .topIcons > select { margin:|topIconsMargin|; color:|headerColor| }
-      .top span { text-transform:uppercase; font-weight:|headerSpanWeight|;
-                  color:|headerSpanColor|; padding:0 5px 0 10px;
-                  vertical-align:-2px }
-      .topText { letter-spacing:1px; padding: 0 1em; font-weight:bold;
-                 text-transform:uppercase }
-      .burger { cursor:pointer; margin:|burgerMargin| }''')
+      .topIcons { display:flex; align-items:center; margin:|topIconsMargin|;
+                  gap:|topIconsGap| }
+      .topIcons > select { margin:0; color:|headerColor| }
+      .topIcons > a { color:|headerColor|; font-weight:|topSpanWeight| }
+      .topIcons span { text-transform:uppercase; font-weight:|topSpanWeight|;
+                  letter-spacing:|topTextSpacing|; color:|headerSpanColor| }
+      .textIcon { display:flex; align-items:center; gap:0.5em }''')
 
     # The global page header
     pxHeader = Px('''
@@ -251,7 +316,9 @@ class Template:
                  showSidebar=ui.Sidebar.show(tool, o, layout, popup);
                  showHeader=cfg.showHeader(_px_, _ctx_, popup);
                  headerHeight=cfg.cget('headerHeight', _ctx_);
+                 inlaidHeader=showHeader == 'portlet';
                  showFooter=cfg.showFooter(_px_, _ctx_, popup);
+                 showFooterP=cfg.portletShowFooter;
                  bi=ui.Browser.getIncompatibilityMessage(tool, handler)">
 
        <!-- Browser incompatibility message -->
@@ -276,7 +343,7 @@ class Template:
         <x if="Template.includeLoginBox(_ctx_)">:guard.pxLogin</x>
 
         <!-- Payload: portlet / content / sidebar -->
-        <div var="payloadCss=Template.getCssFor('payload', _ctx_)"
+        <div id="payload" var="payloadCss=Template.getCssFor('payload', _ctx_)"
              class=":payloadCss" style=":Template.getPayloadStyle(_ctx_)">
 
          <!-- Portlet -->
