@@ -41,7 +41,7 @@ class Title:
       setSubTitles = function(value, tag) {
         createCookie('showSubTitles', value);
         // Get the sub-titles
-        var subTitles = getElementsHavingName(tag, 'subTitle');
+        var subTitles = document.getElementsByClassName('subTitle');
         if (subTitles.length == 0) return;
         // Define the display style depending on p_tag
         var displayStyle = 'inline';
@@ -62,6 +62,18 @@ class Title:
         setSubTitles(newValue, tag);
       }''')
 
+    @classmethod
+    def getClassAttribute(class_, base, more, isSelect):
+        '''Create the "class" attribute for the title link, based on CSS classes
+           potentially defined in p_base and p_more.'''
+        # p_isSelect is True if we are in "select" mode (see p_get)
+        r = base or ''
+        if more:
+            r = '%s%s%s' % (r, (' ' if r else ''), more)
+        if isSelect:
+            r = '%s%sclickable' % (r, (' ' if r else ''))
+        return ' class="%s"' % r if r else ''
+
     @classmethod
     def get(class_, o, mode='link', nav='no', target=None, page='main',
             popup=False, baseUrl=None, title=None, linkTitle=None, css=None,
@@ -72,7 +84,7 @@ class Title:
 
         # In most cases, a title must appear as a link that leads to the object
         # view layout. In this case (p_mode == "link"):
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # p_nav        | is the navigation parameter allowing navigation between
         #              | this object and others;
         # p_target     | specifies if the link must be opened in the popup or
@@ -91,7 +103,7 @@ class Title:
         #              | title chars.
         # p_pageLayout | allows to override the standard "view" page layout as
         #              | defined on p_o's class.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
         # Another p_mode is "select". In this case, we are in a popup for
         # selecting objects: every title must not be a link, but clicking on it
@@ -110,26 +122,32 @@ class Title:
         # ensure that the link will be different every time it is generated.
         # This is useful for short-circuiting the a:visited CSS style when an
         # app wants to manage link's style in some specific way.
-
-        # Compute CSS class
-        class_ = o.class_
         handler = o.H()
-        cssClass = class_.getCssFor(o, 'title')
-        if css: cssClass += ' %s' % css
+        oclass = o.class_
+
+        # Compute the "class" attribute
+        isSelect = mode == 'select'
+        tcss = oclass.getCssFor(o, 'title')
+        classAttr = class_.getClassAttribute(tcss, css, isSelect)
+        
         # Get the title, with highlighted parts when relevant
         titleIsPx = False
         if not title:
-            title = class_.getListTitle(o)
+            title = oclass.getListTitle(o, nav)
         elif isinstance(title, Px):
             title = title(handler.traversal.context)
             titleIsPx = True
         if highlight and not titleIsPx:
             title = Criteria.highlight(handler, title)
         if maxChars: title = Px.truncateText(title, width=maxChars)
-        if mode.endswith('link'): # "link" or "dlink"
-            popup = popup or (target.target != '_self')
+        if mode == 'plink':
+            # A link to stay in the public part of the site
+            url = '%s/public?rp=%d' % (o.tool.url, o.iid)
+            r = '<a href="%s">%s</a>' % (url, title)
+        elif mode.endswith('link'): # "link" or "dlink"
+            popup = popup or target.target != '_self'
             params = {'page': page, 'nav': nav, 'popup': popup,
-                      'unique': class_.produceUniqueLinks()}
+                      'unique': oclass.produceUniqueLinks()}
             if pageLayout: params['pageLayout'] = pageLayout
             # Build the link URL, or patch the given p_baseUrl
             if baseUrl is None:
@@ -145,19 +163,19 @@ class Title:
             # Set a "title" parameter when relevant
             lt = (' title="%s"' % linkTitle) if linkTitle else ''
             if mode[0] == 'd':
-                # Wrap the link into a "div". In that case, the CSS class
-                # applies to the div.
-                start = '<div class="%s"><a' % cssClass
+                # Wrap the link into a "div". In that case, the CSS class(es)
+                # apply to the div.
+                start = '<div%s><a' % classAttr
                 end = '</a></div>'
             else:
-                start = '<a class="%s"' % cssClass
+                start = '<a%s' % classAttr
                 end = '</a>'
             r = '%s name="title" href="%s" target="%s"%s%s>%s%s' %\
                 (start, url, target.target, onClick, lt, title, end)
-        elif mode == 'select':
-            r = '<span class="%s clickable" onclick="%s">%s</span>' % \
-                (cssClass, selectJs, title)
+        elif isSelect:
+            r = '<span onclick="%s"%s>%s</span>' % \
+                (selectJs, classAttr, title)
         elif mode == 'text':
-            r = '<span class="%s">%s</span>' % (cssClass, title)
+            r = '<span%s>%s</span>' % (classAttr, title)
         return r
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
