@@ -18,9 +18,8 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-import traceback, mimetypes, subprocess
-
 from persistent.list import PersistentList
+import traceback, mimetypes, subprocess, types
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 def asDict(seq):
@@ -28,7 +27,8 @@ def asDict(seq):
     return {elem: None for elem in seq}
 
 # Global variables - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-sequenceTypes = (list, tuple, PersistentList)
+listTypes = (list, PersistentList)
+sequenceTypes = listTypes + (tuple,)
 commercial = False
 
 # On these layouts, using a gobal selector and switching from one option to the
@@ -41,32 +41,37 @@ od = 'application/vnd.oasis.opendocument'
 ms = 'application/vnd.openxmlformats-officedocument'
 ms2 = 'application/vnd.ms'
 
-mimeTypes = {'odt':  '%s.text' % od,
-             'ods':  '%s.spreadsheet' % od,
-             'doc':  'application/msword',
-             'docx': '%s.wordprocessingml.document' % ms,
-             'rtf':  'text/rtf',
-             'pdf':  'application/pdf'
-             }
+mimeTypes = {
+  'odt':  '%s.text' % od,
+  'ods':  '%s.spreadsheet' % od,
+  'doc':  'application/msword',
+  'docx': '%s.wordprocessingml.document' % ms,
+  'rtf':  'text/rtf',
+  'pdf':  'application/pdf',
+  'xml':  'text/xml'
+}
 
 mimeTypesExts = {
-    '%s.text' % od:        'odt',
-    '%s.spreadsheet' % od: 'ods',
-    'application/msword':  'doc',
-    'text/rtf':            'rtf',
-    'application/pdf':     'pdf',
-    'image/png':           'png',
-    'image/jpeg':          'jpg',
-    'image/pjpeg':         'jpg',
-    'image/gif':           'gif',
-    '%s.wordprocessingml.document' % ms: 'docx',
-    '%s.spreadsheetml.sheet' % ms: 'xlsx',
-    '%s.presentationml.presentation' % ms: 'pptx',
-    '%s-excel' % ms2:      'xls',
-    '%s-powerpoint' % ms2: 'ppt',
-    '%s-word.document.macroEnabled.12' % ms2: 'docm',
-    '%s-excel.sheet.macroEnabled.12' % ms2: 'xlsm',
-    '%s-powerpoint.presentation.macroEnabled.12' % ms2: 'pptm'}
+  '%s.text'        % od:  'odt',
+  '%s.spreadsheet' % od:  'ods',
+  'application/msword':   'doc',
+  'text/rtf':             'rtf',
+  'application/pdf':      'pdf',
+  'image/png':            'png',
+  'image/jpeg':           'jpg',
+  'image/pjpeg':          'jpg',
+  'image/gif':            'gif',
+  'text/xml':             'xml',
+  'application/xml':      'xml',
+  '%s.wordprocessingml.document'               % ms:  'docx',
+  '%s.spreadsheetml.sheet'                     % ms:  'xlsx',
+  '%s.presentationml.presentation'             % ms:  'pptx',
+  '%s-excel'                                   % ms2: 'xls',
+  '%s-powerpoint'                              % ms2: 'ppt',
+  '%s-word.document.macroEnabled.12'           % ms2: 'docm',
+  '%s-excel.sheet.macroEnabled.12'             % ms2: 'xlsm',
+  '%s-powerpoint.presentation.macroEnabled.12' % ms2: 'pptm'
+}
 
 def getMimeType(fileName, default='application/octet-stream'):
     '''Tries to guess mime type from p_fileName'''
@@ -218,9 +223,19 @@ def addPair(name, value, d=None):
     else: d = {name: value}
     return d
 
+def addListValue(d, key, value):
+    '''p_d is supposed to be a dict whose values are lists. If a list already
+       exists at this p_key, p_value is appended to it. Else, [value] is added
+       at this p_key in p_d.'''
+    if key in d:
+        d[key].append(value)
+    else:
+        d[key] = [value]
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Traceback:
     '''Dumps the last traceback into a string'''
+
     @staticmethod
     def get(last=None, html=False):
         '''Gets the traceback as a string. If p_last is given (must be an
@@ -236,7 +251,7 @@ class Traceback:
         return r
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-def executeCommand(cmd, stdin=None, env=None, shell=False):
+def executeCommand(cmd, stdin=None, env=None, shell=False, stringOutput=True):
     '''Executes command p_cmd and returns a tuple (s_stdout, s_stderr)
        containing the data output by the subprocesss on stdout and stderr. p_cmd
        should be a list of args (the 1st arg being the command in itself, the
@@ -244,7 +259,14 @@ def executeCommand(cmd, stdin=None, env=None, shell=False):
        (see subprocess.Popen doc).'''
     proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stdin=stdin,
                             stderr=subprocess.PIPE, env=env, shell=shell)
-    return proc.communicate()
+    out, err = proc.communicate()
+    # The output may be a binary file as well: p_stringOutput may need to be
+    # False in that case.
+    if stringOutput and isinstance(out, bytes):
+        out = out.decode()
+    if stringOutput and isinstance(err, bytes):
+        err = err.decode()
+    return out, err
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class ImageMagick:
@@ -342,7 +364,7 @@ def multicall(o, name, block, *args):
         if hasattr(bases[i], name):
             res = getattr(bases[i], name)(o, *args)
             if block:
-                if (i==1) and not res: return
+                if i == 1 and not res: return
                 r = res
             else:
                 # If not in "block" mode, don't lose string results
@@ -363,4 +385,46 @@ def iconParts(icon):
     else:
         base = None
     return icon, base, icon.endswith('.svg')
+
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+def termColorize(s, color='2'):
+    '''Returns the colorized version of p_s, formatted to be rendered in a Unix
+       terminal.'''
+    return '\x1b[3%sm%s\x1b[0m' % (color, s)
+
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+METH_KO = 'Method named "%s" does not exist on this %s instance.'
+
+class Function:
+    '''Utility methods for functions or methods'''
+
+    @classmethod
+    def getQualifiedName(class_, fun):
+        '''Returns the name of p_fun (which is function or a method), prefixed
+           with the name of its class.'''
+        fun = fun if isinstance(fun, types.FunctionType) else fun.__func__
+        return fun.__qualname__
+
+    @classmethod
+    def scall(class_, o, spec, raiseOnError=False):
+        '''Calls a method on this p_o(bject)'''
+        # p_spec is a method "specifier", of the form:
+        #
+        #            <method_name>:<arg1>:<arg2>:...
+        #
+        # Args are optional. Char ":" is interpreted as an arg specifier: it
+        # cannot appear *within* an arg. All such args will be passed as strings
+        # to the method.
+        parts = spec.split(':')
+        name = parts[0]
+        args = parts[1:]
+        # Execute the method, if it exists on p_o
+        if hasattr(o, name):
+            r = getattr(o, name)(*args)
+        else:
+            message = METH_KO % (name, o.class_.name)
+            if raiseOnError: raise Exception(message)
+            o.log(message, type='warning')
+            r = None
+        return r
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
