@@ -17,9 +17,8 @@
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 from base64 import encodebytes
-from mimetypes import guess_type
-import xml.sax, http.client, urllib, urllib.parse, ssl
-import os, sys, re, stat, time, socket, random, hashlib, gzip
+import xml.sax, http.client, urllib.parse, ssl
+import re, time, socket, random, hashlib, gzip
 
 from appy.utils import json
 from appy.utils import copyData
@@ -27,9 +26,22 @@ from appy.model.utils import Object
 from appy.xml.marshaller import Marshaller
 from appy.xml.unmarshaller import Unmarshaller
 
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+DUR_SECS  = ', got in %.4f seconds'
+RESP_T    = '\n*Headers* %s\n*Body* %s\n'
+RESP_R    = '<HttpResponse %s (%s)%s%s>'
+D_ERR     = 'Distant server exception: %s'
+XML_ERR   = 'Invalid XML response (%s)'
+T_OUT_ERR = 'Timed out after %d second(s).'
+RESS_R    = '<Resource at %s>'
+CIC_ERR   = 'Check your Internet connection (%s)'
+CONN_ERR  = 'Connection error (%s)'
+URL_ERR   = 'Wrong URL: %s'
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class FormDataEncoder:
     '''Allows to encode form data for sending it through a HTTP request'''
+
     def __init__(self, data):
         self.data = data # The data to encode, as a dict
 
@@ -49,20 +61,22 @@ class FormDataEncoder:
             raise Exception('Cannot encode value %s' % str(value))
 
     def encode(self):
-        res = []
+        r = []
         for name, value in self.data.items():
-            res.append(self.marshalValue(name, value).encode())
-        return b'&'.join(res)
+            r.append(self.marshalValue(name, value).encode())
+        return b'&'.join(r)
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class SoapDataEncoder:
     '''Allows to encode SOAP data for sending it through a HTTP request'''
+
     namespaces = {'s'  : 'http://schemas.xmlsoap.org/soap/envelope/',
                   'xsd': 'http://www.w3.org/2001/XMLSchema',
                   'xsi': 'http://www.w3.org/2001/XMLSchema-instance'}
+
     namespacedTags = {'Envelope': 's', 'Body': 's', '*': 'py'}
 
-    def __init__(self, data, namespace='http://appyframework.org'):
+    def __init__(self, data, namespace='https://appyframe.work'):
         self.data = data
         # p_data can be:
         # - a string already containing a complete SOAP message
@@ -73,19 +87,24 @@ class SoapDataEncoder:
 
     def encode(self):
         # Do nothing if we have a SOAP message already
-        if isinstance(self.data, str): return self.data
-        # self.data is here a Python object. Wrap it in a SOAP Body.
-        soap = Object(Body=self.data)
-        # Marshall it
-        marshaller = Marshaller(rootTag='Envelope', namespaces=self.ns,
-                                namespacedTags=self.namespacedTags)
-        return marshaller.marshall(soap)
+        if isinstance(self.data, str):
+            r = self.data
+        else:
+            # self.data is here a Python object. Wrap it in a SOAP Body.
+            soap = Object(Body=self.data)
+            # Marshall it
+            marshaller = Marshaller(rootTag='Envelope', namespaces=self.ns,
+                                    namespacedTags=self.namespacedTags)
+            r = marshaller.marshall(soap)
+        return r.encode()
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class DigestRealm:
     '''Represents information delivered by a server requiring Digest-based
        HTTP authentication.'''
+
     rex = re.compile('(\w+)="(.*?)"')
+
     # List of attributes whose values must not be quoted
     unquoted = ('algorithm', 'qop', 'nc')
 
@@ -169,6 +188,9 @@ class HttpResponse:
     # Useful empty dict
     emptyDict = {}
 
+    # Redirect HTTP codes
+    redirectCodes = (301, 302, 303)
+
     def __init__(self, resource, response, body, duration=None, utf8=True,
                  responseType=None, unmarshallParams=None):
         self.resource = resource
@@ -191,12 +213,11 @@ class HttpResponse:
 
     def __repr__(self, complete=False):
         duration = suffix = ''
-        if self.duration: duration = ', got in %.4f seconds' % self.duration
+        if self.duration: duration = DUR_SECS % self.duration
         if complete:
-            suffix = '\n*Headers* %s\n*Body* %s\n' %  \
-                     (str(self.headers), str(self.body))
-        return '<HttpResponse %s (%s)%s%s>' % \
-               (self.code, self.text, duration, suffix)
+            suffix = RESP_T %  (str(self.headers), str(self.body))
+        return RESP_R % (self.code, self.text, duration, suffix)
+
     def get(self): return self.__repr__(complete=True)
 
     def getResponseUrl(self, url):
@@ -233,11 +254,12 @@ class HttpResponse:
                 self.resource.cookies[name] = value
 
     xmlHeaders = ('text/xml', 'application/xml', 'application/soap+xml')
+
     def extractData(self, responseType=None):
         '''This method extracts, from the various parts of the HTTP response,
            some useful information:
            * it will find the URI where to redirect the user to if self.code
-             is 302;
+             is 302 or 303;
            * it will return authentication-related data, if present, if
              self.code is 401;
            * it will unmarshall XML or JSON data into Python objects;
@@ -245,7 +267,7 @@ class HttpResponse:
         # Extract information from HTTP headers when relevant
         headers = self.headers
         self.extractCookies(headers)
-        if self.code == 302:
+        if self.code in self.redirectCodes:
             # The URL to redirect to is in header key 'location'
             return self.getResponseUrl(headers['location'])
         elif self.code == 401:
@@ -268,13 +290,13 @@ class HttpResponse:
             try:
                 params = self.unmarshallParams or HttpResponse.emptyDict
                 parser = Unmarshaller(**params)
-                res = parser.parse(self.body)
+                r = parser.parse(self.body)
                 if parser.rootTag == 'exception':
-                    # This is an exception: "res" contains the traceback
-                    raise Resource.Error('Distant server exception: %s' % res)
-                return res
+                    # This is an exception: v_r contains the traceback
+                    raise Resource.Error(D_ERR % r)
+                return r
             except xml.sax.SAXParseException as se:
-                raise Resource.Error('Invalid XML response (%s)' % str(se))
+                raise Resource.Error(XML_ERR % str(se))
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 urlRex = re.compile(r'(http[s]?)://([^:/]+)(:[0-9]+)?(/.+)?', re.I)
@@ -284,8 +306,10 @@ binaryRex = re.compile(r'[\000-\006\177-\277]')
 class Resource:
     '''Every instance of this class represents some web resource accessible
        through HTTP.'''
+
     class Error(Exception): pass
-    TIMEOUT_ERROR = 'Timed out after %d second(s).'
+
+    # Standard ports, by protocol
     standardPorts = {'http': 80, 'https': 443}
 
     def __init__(self, url, username=None, password=None, measure=False,
@@ -306,35 +330,43 @@ class Resource:
         # this resource object.
         self.serverTime = 0
         # Split the URL into its components
-        res = urlRex.match(url)
-        if res:
-            protocol, host, port, uri = res.groups()
-            self.protocol = protocol
-            self.host = host
-            if port:
-                self.port = int(port[1:])
-            else:
-                self.port = Resource.standardPorts[self.protocol]
-            self.uri = uri or '/'
-        else: raise self.Error('Wrong URL: %s' % str(url))
+        self.protocol, self.host, self.port, self.path = self.getUrlParts(url)
         # If some headers must be sent with any request sent through this
         # resource, you can store them in the following dict.
-        self.headers = {'Host': self.getHeaderHost(), 'User-Agent': 'Appy',
-                        'Connection': 'close', 'Accept': '*/*',
-                        'Accept-Encoding': 'gzip, identity'}
+        self.headers = {'User-Agent': 'Appy', 'Connection': 'close',
+                        'Accept': '*/*', 'Accept-Encoding': 'gzip, identity'}
         # Cookies defined hereafter will be included in self.headers at every
         # request.
         self.cookies = {}
 
-    def getHeaderHost(self):
+    def getUrlParts(self, url, raiseOnError=True):
+        '''Return p_url parts as a tuple (protocol, host, port, path). If the
+           URL is wrong (or is not a complete URL), the method raises an error
+           if p_raiseOnError is True or returns None if p_raiseOnError is
+           False.'''
+        r = urlRex.match(url)
+        if not r:
+            if raiseOnError:
+                raise Resource.Error(URL_ERR % str(url))
+            else:
+                return r
+        protocol, host, port, path = r.groups()
+        if port:
+            port = int(port[1:])
+        else:
+            port = Resource.standardPorts[protocol]
+        path = path or '/'
+        return protocol, host, port, path
+
+    def getHeaderHost(self, protocol, host, port):
         '''Gets the content of header key "Host"'''
         # Insert the port number if not standard
-        suffix = '' if self.port == Resource.standardPorts[self.protocol] \
-                    else ':%d' % self.port
-        return '%s%s' % (self.host, suffix)
+        suffix = '' if port == Resource.standardPorts[protocol] else ':%d'% port
+        return '%s%s' % (host, suffix)
 
     def __repr__(self):
-        return '<Resource at %s>' % self.url
+        '''p_self's short string representation'''
+        return RESS_R % self.url
 
     def completeHeaders(self, headers):
         # Get standard header values from self.headers if not defined in
@@ -377,42 +409,54 @@ class Resource:
                 pass
         return r
 
-    def send(self, method, uri, body=None, headers=None, bodyType=None,
-             responseType=None, unmarshallParams=None):
-        '''Sends a HTTP request with p_method, for p_uri'''
-        hc = http.client
-        if self.protocol == 'https':
-            context = ssl.SSLContext()
-            context.verify_mode = ssl.CERT_NONE
-            conn = hc.HTTPSConnection(self.host, self.port,
-                                      timeout=self.timeout, context=context)
+    def send(self, method, path, body=None, headers=None, bodyType=None,
+             responseType=None, unmarshallParams=None, timeout=None):
+        '''Sends a HTTP request with p_method, @ this p_path'''
+        # p_path can be a complete URL (http://a.b.be/c) or the "path "part (/c)
+        parts = self.getUrlParts(path, raiseOnError=False)
+        if parts is None:
+            # p_path is a real path
+            protocol, host, port = self.protocol, self.host, self.port
+            path = path or '/'
         else:
-            conn = hc.HTTPConnection(self.host, self.port, timeout=self.timeout)
+            # p_path is a complete URL
+            protocol, host, port, path = parts
+        # Initialise a HTTP or HTTPS connection
+        hc = http.client
+        timeout = self.timeout if timeout is None else timeout
+        if protocol == 'https':
+            context = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
+            context.check_hostname = False
+            context.verify_mode = ssl.CERT_NONE
+            conn = hc.HTTPSConnection(host, port, timeout=timeout,
+                                      context=context)
+        else:
+            conn = hc.HTTPConnection(host, port, timeout=timeout)
         try:
             conn.connect()
         except socket.gaierror as sge:
-            raise self.Error('Check your Internet connection (%s)' % str(sge))
+            raise self.Error(CIC_ERR % str(sge))
         except socket.timeout as se:
-            raise self.Error(self.TIMEOUT_ERROR % conn.timeout)
+            raise self.Error(T_OUT_ERR % conn.timeout)
         except socket.error as se:
-            raise self.Error('Connection error (%s)' % str(se))
+            raise self.Error(CONN_ERR % str(se))
         # Tell what kind of HTTP request it will be
-        conn.putrequest(method, uri, skip_host=True)
+        conn.putrequest(method, path, skip_host=True)
         # Add HTTP headers
         if headers is None: headers = {}
+        headers['Host'] = self.getHeaderHost(protocol, host, port)
         self.completeHeaders(headers)
         for k, v in headers.items(): conn.putheader(k, v)
         conn.endheaders()
         # Add HTTP body
         if body:
-            if not bodyType: bodyType = 'string'
-            copyData(body, conn, 'send', type=bodyType)
+            copyData(body, conn, 'send', type=bodyType or 'string')
         # Send the request, get the reply
         if self.measure: startTime = time.time()
         try:
             response = conn.getresponse()
         except socket.timeout as te:
-            raise self.Error(self.TIMEOUT_ERROR % conn.timeout)
+            raise self.Error(T_OUT_ERR % conn.timeout)
         if self.measure: endTime = time.time()
         body = self.readResponse(response)
         conn.close()
@@ -425,72 +469,45 @@ class Resource:
                             utf8=self.utf8, responseType=responseType,
                             unmarshallParams=unmarshallParams)
 
-    def mkdir(self, name):
-        '''Creates a folder named p_name in this resource.'''
-        folderUri = self.uri + '/' + name
-        #body = '<d:propertyupdate xmlns:d="DAV:"><d:set><d:prop>' \
-        #       '<d:displayname>%s</d:displayname></d:prop></d:set>' \
-        #       '</d:propertyupdate>' % name
-        return self.send('MKCOL', folderUri)
-
-    def delete(self, name):
-        '''Deletes a file or a folder (and all contained files if any) named
-           p_name within this resource.'''
-        toDeleteUri = self.uri + '/' + name
-        return self.send('DELETE', toDeleteUri)
-
-    def add(self, path, name=''):
-        '''Adds a file in this resource, whose p_path is passed'''
-        size = os.stat(path)[stat.ST_SIZE]
-        body = open(path, 'rb')
-        name = os.path.basename(path)
-        fileType, encoding = guess_type(path)
-        fileUri = self.uri + '/' + name
-        headers = {'Content-Length': str(size)}
-        if fileType: headers['Content-Type'] = fileType
-        if encoding: headers['Content-Encoding'] = encoding
-        r = self.send('PUT', fileUri, body, headers, bodyType='file')
-        body.close()
-        return r
-
-    def get(self, uri=None, headers=None, params=None, followRedirect=True,
-            responseType=None, unmarshallParams=None):
+    def get(self, path=None, headers=None, params=None, followRedirect=True,
+            responseType=None, unmarshallParams=None, timeout=None):
         '''Perform a HTTP GET on the server. Parameters can be given as a dict
            in p_params. p_responseType will be used if no "Content-Type" key is
            found on the HTTP response. In the processs of unmarshalling received
            data, specific parameters can be passed in dict
            p_unmarshallParams. '''
-        if not uri: uri = self.uri
+        path = path or self.path
         # Encode and append params if given
         if params:
-            sep = ('?' in uri) and '&' or '?'
-            uri = '%s%s%s' % (uri, sep, urllib.urlencode(params))
-        r = self.send('GET', uri, headers=headers, responseType=responseType,
-                      unmarshallParams=unmarshallParams)
+            sep = '&' if '?' in path else '?'
+            path = '%s%s%s' % (path, sep, urllib.parse.urlencode(params))
+        r = self.send('GET', path, headers=headers, responseType=responseType,
+                      unmarshallParams=unmarshallParams, timeout=timeout)
         # Follow redirect when relevant
-        if r.code == 302 and followRedirect:
+        if r.code in r.redirectCodes and followRedirect:
             # Addition durations when "measure" is True
             duration = r.duration
-            r = self.get(r.data, headers=headers)
+            r = self.get(r.data, headers=headers, timeout=timeout)
             if self.measure: r.duration += duration
             return r
         # Perform Digest-based authentication when relevant
         if r.code == 401 and self.authMethod == 'Digest' and r.data:
             # Re-trigger the request with the correct authentication headers
             headers = headers or {}
-            headers['Authorization'] = r.data.buildCredentials(self, uri)
-            return self.get(uri=uri, headers=headers, params=params,
-                       followRedirect=followRedirect, responseType=responseType)
+            headers['Authorization'] = r.data.buildCredentials(self, path)
+            return self.get(path=path, headers=headers, params=params,
+                            followRedirect=followRedirect,
+                            responseType=responseType, timeout=timeout)
         return r
     rss = get
 
-    def post(self, data=None, uri=None, headers=None, encode='form',
-             followRedirect=True):
+    def post(self, data=None, path=None, headers=None, encode='form',
+             followRedirect=True, timeout=None):
         '''Perform a HTTP POST on the server. If p_encode is "form", p_data is
            considered to be a dict representing form data that will be
            form-encoded. Else, p_data will be considered as the ready-to-send
            body of the HTTP request.'''
-        if not uri: uri = self.uri
+        path = path or self.path
         if headers is None: headers = {}
         # Prepare the data to send
         if encode == 'form':
@@ -498,34 +515,34 @@ class Resource:
             body = FormDataEncoder(data).encode()
             headers['Content-Type'] = 'application/x-www-form-urlencoded'
         else:
-            body = data
+            body = data if isinstance(data, bytes) else data.encode()
         headers['Content-Length'] = str(len(body))
-        r = self.send('POST', uri, headers=headers, body=body)
-        if (r.code == 302) and followRedirect:
+        r = self.send('POST', path, headers=headers, body=body, timeout=timeout)
+        if r.code in r.redirectCodes and followRedirect:
             # Update headers
             for key in ('Content-Type', 'Content-Length'):
                 if key in headers: del headers[key]
             # Addition durations when "measure" is True
             duration = r.duration
-            r = self.get(r.data, headers=headers)
+            r = self.get(r.data, headers=headers, timeout=timeout)
             if self.measure: r.duration += duration
         return r
 
-    def soap(self, data, uri=None, headers=None, namespace=None,
+    def soap(self, data, path=None, headers=None, namespace=None,
              soapAction=None):
         '''Sends a SOAP message to this resource. p_namespace is the URL of the
            server-specific namespace. If header value "SOAPAction" is different
            from self.url, specify it in p_soapAction.'''
-        if not uri: uri = self.uri
+        path = path or self.path
         # Prepare the data to send
         data = SoapDataEncoder(data, namespace).encode()
         if headers is None: headers = {}
         headers['SOAPAction'] = soapAction or self.url
         # Content-type could be 'text/xml'
         headers['Content-Type'] = 'application/soap+xml;charset=UTF-8'
-        res = self.post(data, uri, headers=headers, encode=None)
+        r = self.post(data, path, headers=headers, encode=None)
         # Unwrap content from the SOAP envelope
-        if hasattr(res.data, 'Body'):
-            res.data = res.data.Body
-        return res
+        if hasattr(r.data, 'Body'):
+            r.data = r.data.Body
+        return r
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
