@@ -276,9 +276,10 @@ class HtmlDiff:
                  insertMsg='Inserted text', deleteMsg='Deleted text',
                  insertCss=None, deleteCss=None, insertName='insert',
                  deleteName='delete', diffRatio=0.7):
-        # p_old and p_new are strings containing chunks of HTML
-        self.old = old.strip()
-        self.new = new.strip()
+        # p_old and p_new are strings (or possibly None) containing chunks of
+        # XHTML code.
+        self.old = (old or '').strip()
+        self.new = (new or '').strip()
         # Every time an "insert" or "delete" difference will be detected from
         # p_old to p_new, the impacted chunk will be surrounded by a tag that
         # will get, respectively, a 'title' attribute filled p_insertMsg or
