@@ -19,7 +19,7 @@
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 from pathlib import Path
-import os, os.path, time, shutil, mimetypes
+import os, os.path, time, shutil, mimetypes, tempfile
 
 from appy import utils
 
@@ -31,34 +31,40 @@ def guessMimeType(fileName):
     fileName = str(fileName) if isinstance(fileName, Path) else fileName
     return mimetypes.guess_type(fileName)[0] or defaultMimeType
 
-def getShownSize(size):
+def getShownSize(size, unbreakable=False):
     '''Express p_size (a file size in bytes) in a human-readable way'''
+    b = ' ' if unbreakable else ' '
     # Display the size in bytes if smaller than 1024 bytes
-    if size < 1024: return '%d byte(s)' % size
-    size = size / 1024.0 # This is the size, in Kb
-    if size < 1024: return '%s Kb' % utils.formatNumber(size, precision=1)
-    size = size / 1024.0 # This is the size, in Mb
-    return '%s Mb' % utils.formatNumber(size, precision=1)
+    if size < 1024: return '%d%sbyte(s)' % (size, b)
+    size /= 1024 # This is the size, in Kb
+    if size < 1024: return '%s%sKb' % (utils.formatNumber(size, precision=1), b)
+    size /= 1024 # This is the size, in Mb
+    if size < 1024: return '%s%sMb' % (utils.formatNumber(size, precision=1), b)
+    size /= 1024 # This is the size, in Gb
+    return '%s%sGb' % (utils.formatNumber(size, precision=1), b)
+
+def getFolderSize(folder, nice=False, withCounts=False):
+    '''Returns the size of this p_folder (all content, recursively)'''
+    r = 0
+    counts = 0 # Count the number of files
+    for root, dirs, files in os.walk(folder):
+        for name in files:
+            r += os.stat(os.path.join(root, name)).st_size
+            counts += 1
+    # Nicely format the size if requested
+    if nice:
+        r = getShownSize(r)
+    # Return the counts of files as well if requested
+    if withCounts:
+        r = r, counts
+    return r
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 def getOsTempFolder(sub=False):
     '''Gets the absolute path to the temp folder on this machine. If p_sub is
        True, it creates a sub-folder within this temp folder and returns its
        absolute path instead of the "root" temp folder path.'''
-    tmp = '/tmp'
-    if os.path.isdir(tmp):
-        res = tmp
-    elif 'TMP' in os.environ:
-        res = os.environ['TMP']
-    elif 'TEMP' in os.environ:
-        res = os.environ['TEMP']
-    else:
-        raise Exception("Sorry, I can't find a temp folder on your machine.")
-    if sub:
-        subFolder = os.path.join(res, 'Appy%f' % time.time())
-        os.mkdir(subFolder)
-        res = subFolder
-    return res
+    return tempfile.mkdtemp(prefix='Appy') if sub else tempfile.gettempdir()
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 def getTempFileName(prefix='', extension='', timestamp=True):
