@@ -20,7 +20,7 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-import re
+import re, string, random
 
 from appy.utils import flipDict, asDict
 
@@ -77,23 +77,26 @@ class Normalize:
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     # A. The default replacement dictionary
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+
     # The default replacement dict is the one allowing to convert latin accented
     # chars to their non-accented equivalent chars. More precisely, it concerns
     # chars from the table named "Latin-1 Supplement" on page
     #
     #        https://en.wikipedia.org/wiki/List_of_Unicode_characters
     #
-    # completed with interlaced graphemes.
+    # completed with interlaced graphemes and UTF-8 space variants.
     replacements = flipDict(
       {'A':'ÀÁÂÃÄÅÆ', 'a':'àáâãäåæ', 'C':'Ç', 'c':'ç', 'D':'Ð', 'S':'ß',
        'E':'ÈÉÊË', 'e':'èéêë', 'I':'ÌÍÎÏ', 'i':'ìíîï', 'N':'Ñ', 'n':'ñ',
        'O':'ÒÓÔÕÖØ', 'o':'òóôõöðø', 'U':'ÙÚÛÜ', 'u':'ùúûü', 'Y':'Ý', 'y':'ýÿ'},
        byChar=True)
-    replacements.update({'Æ':'AE', 'æ': 'ae', 'Œ':'OE', 'œ': 'oe'})
+    replacements.update({'Æ':'AE', 'æ': 'ae', 'Œ':'OE', 'œ': 'oe',
+                         ' ':' ', '‑':'-'})
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     # B. The default regular expressions
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+
     # The following default regular expressions define, depending on the needs,
     # various sets of chars that will not be among chars kept as-is in the
     # result.
@@ -104,29 +107,36 @@ class Normalize:
     nonAlphanumBD = re.compile('[^a-zA-Z0-9 -]') # + blanks and dashes
     nonAlphanum_  = re.compile('[^a-zA-Z0-9_]')  # Alphanums + the underscore
     nonDigit      = re.compile('[^0-9]')
-    # Access some of the previousy defined regular expressions, depending on the
-    # fact that blanks (1) and/or dashes (2) must be matched or not.
+
+    # Access some of the previously defined regular expressions, depending on
+    # the fact that blanks (1) and/or dashes (2) must be matched or not.
     textRex = { False: { False: nonAlphanum,  True: nonAlphanumD },
                 True:  { False: nonAlphanumB, True: nonAlphanumBD }}
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     # C. The default lists of chars to ignore or blankify
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+
     # The following lists define, depending on the needs, various lists of chars
     # to ignore or blankify. These lists are the converted to dicts whose values
     # are None, because lookup in dicts are more efficient.
 
     # ~~~ A common set of chars, to ignore or blankify in most situations
-    baseIgnorable = '.,:;*+=~?%^\'’"<>{}[]#|\t\\°-'
+    baseIgnorable = '.,:;*+=~?%^\'’"<>{}[]#|\t\\°-‑'
+
     # ~~~ The set of chars to ignore when producing a file name
-    fileNameIgnorable = asDict(baseIgnorable + ' $£€/\r\n')
+    moreIgnorableA = '  $£€/\r\n'
+    fileNameIgnorable = asDict(baseIgnorable + moreIgnorableA)
+
     # ~~~ The set of chars to blankify when extracting text for the purpose of
-    #     database indexing, with 2 variants, keeping dashes (True) or not.
-    moreIgnorable = '\n/()_'
+    #     database indexing or keywords cleaning, with 2 variants, keeping
+    #     dashes (True) or not.
+    moreIgnorableB = '\n/()_'
     textIgnorable = {
-      False: asDict(baseIgnorable + moreIgnorable),
-      True:  asDict(baseIgnorable.replace('-', '') + moreIgnorable)
+      False: asDict(baseIgnorable + moreIgnorableB),
+      True:  asDict(baseIgnorable[:-2] + moreIgnorableB)
     }
+    indexIgnorable = asDict(baseIgnorable + moreIgnorableA + moreIgnorableB)
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     # Base method normalizing chars by blankifying, ignoring, replacing or
@@ -137,11 +147,11 @@ class Normalize:
     def char(class_, c, blankify, ignore, replace, keep):
         '''Handles char p_c and returns the element corresponding to it that
            must be part of the result.'''
-        if blankify and (c in blankify):
+        if blankify and c in blankify:
             r = ' '
-        elif ignore and (c in ignore):
+        elif ignore and c in ignore:
             r = ''
-        elif replace and (c in replace):
+        elif replace and c in replace:
             r = replace[c]
         elif keep:
             r = c
@@ -171,25 +181,44 @@ class Normalize:
     @classmethod
     def text(class_, s, lower=True, keepDash=False, keepBlank=True):
         '''Normalizes string p_s, for producing a text being suitable for
-           purposes like database indexing.'''
-        # Apply the normalization.
-        # ~~~
-        # Choose the appropriate regular expression and set of chars to
-        # blankify, depending on the fact that dashes and/or blanks must be kept
-        # or not (p_keepDash and p_keepBlank).
+           database keyword indexing or search terms cleaning.'''
+        #
+        # If p_keepDash is ...
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # True  | Dashes (-) are kept.
+        #       |
+        #       | Example:
+        #       | Jean-François > jean-francois
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # False | Dashes are replaced with blanks.
+        #       |
+        #       | Example:
+        #       | Jean-François > jean francois
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # Choose the appropriate regular expression and set of chars to ignore
+        # or blankify, depending on the fact that dashes and/or blanks
+        # must be kept or not (p_keepDash and p_keepBlank).
         r = Normalize.string(s, class_.textRex[keepBlank][keepDash],
                              blankify=class_.textIgnorable[keepDash],
                              replace=class_.replacements)
         # Lowerize the result if required (p_lower)
-        if lower: r = r.lower()
-        return r
+        return r.lower() if lower else r
+
+    @classmethod
+    def sortable(class_, s):
+        '''Normalizes p_s such that it can be used as value for sorting'''
+        r = Normalize.string(s, class_.nonAlphanum,
+                             ignore=class_.indexIgnorable,
+                             replace=class_.replacements)
+        return r.lower()
 
     @classmethod
     def fileName(class_, s):
         '''Normalizes p_s, for producing a string suitable for naming a file on
            disk.'''
         return Normalize.string(s, class_.nonAlphanum,
-                   ignore=class_.fileNameIgnorable, replace=class_.replacements)
+                                ignore=class_.fileNameIgnorable,
+                                replace=class_.replacements)
 
     @classmethod
     def accents(class_, s):
@@ -217,6 +246,25 @@ class Normalize:
         '''Returns a version of p_s whose non-digit chars have been removed'''
         return Normalize.string(s, class_.nonDigit)
 
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+class Variables:
+    '''Replaces, in some text, variables defined with syntax |name| with content
+       as found on the homonym attribute of a passed object.'''
+
+    # Regular expression defining a variable (with pipes as delimiters)
+    withPipes = re.compile('\|(\w+?)\|', re.S)
+    # A variant with asteriscs as delimiters
+    withStars = re.compile('\*(\w+?)\*', re.S)
+
+    @classmethod
+    def replace(class_, s, o, stars=False):
+        '''Replaces, in string p_s, any variable matching class_.definition with
+           a value as found on the homonym attribute stored on this
+           p_o(bject).'''
+        fun = lambda match: getattr(o, match.group(1))
+        rex = class_.withStars if stars else class_.withPipes
+        return rex.sub(fun, s)
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 def getStringFrom(o, stringify=True, c="'"):
     '''Returns a string representation for p_o that can be transported over
@@ -406,4 +454,12 @@ class WhitespaceCruncher:
         # "r" can be a single whitespace. It is up to the caller method to
         # identify when this single whitespace must be kept or crunched.
         return r
+
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+def randomName(length=3):
+    '''Returns a randomly name made of p_length chars'''
+    r = []
+    for i in range(length):
+        r.append(random.choice(string.ascii_letters))
+    return ''.join(r)
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
