@@ -1,7 +1,7 @@
 '''Utility module related to URL manipulation'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -26,7 +26,7 @@ def split(url):
        been removed, and "params" is a dict containing these params, or None if
        there is no param in p_url.'''
     # Extract the various p_url parts
-    parsed = urllib.parse.parse(url)
+    parsed = urllib.parse.urlparse(url)
     # Return the URL as-is if it contains no parameter
     if not parsed.query: return url, None
     # Compute the dict of parameters
