@@ -126,7 +126,7 @@ class Marshaller:
 
     def dumpBytes(self, r, value):
         '''Dumps p_value, which is a bytes value'''
-        self.dumpString(value.decode())
+        self.dumpString(r, value.decode())
 
     def dumpFile(self, r, v):
         '''Dumps a file into the result'''
@@ -136,11 +136,8 @@ class Marshaller:
         # Encode it in Base64, in one or several parts.
         partTag = self.getTagName('part')
         r.write('<%s type="base64" number="1">' % partTag)
-        if v.inDb():
-            path = v.getFilePath(self.o)
-        else:
-            path = v.fsPath
-        if not os.path.isfile(path):
+        path = v.getFilePath(self.o) if v.inDb() else v.fsPath
+        if not os.path.isfile(path) or v.size == 0:
             w('</%s>' % partTag) # Close the (empty) tag
             return
         f = open(path, 'rb')
@@ -206,14 +203,17 @@ class Marshaller:
         r.write('<%s' % tagName)
         # Dump value's type as an XML attribute
         className = value.__class__.__name__
+        cname = None
         if className in self.typesMap:
             type = self.typesMap[className]
         elif self.isObject(value):
             type = 'object'
+            cname = value.__class__.__name__
         else:
             # For any other type, attribute "type" will not be dumped
             type = None
-        if type: r.write(' type="%s"' % type)
+        if type:  r.write(' type="%s"' % type)
+        if cname: r.write(' className="%s"' % cname)
         # Dump the value's length if multi-valued
         if type in ('list', 'tuple'):
             length = len(value) if value else 0
@@ -226,8 +226,14 @@ class Marshaller:
             r.write(' name=%s' % quoteattr(value.uploadName))
             # Dump the file location when appropriate
             if not self.marshallBinaries:
-                r.write(' location="%s/%s/%s"' % (self.databaseFolder,
-                                                  value.fsPath, value.fsName))
+                if value.fsName:
+                    # This is a DB-controlled file
+                    location = '%s/%s/%s' % (self.databaseFolder, value.fsPath,
+                                             Escape.xml(value.fsName))
+                else:
+                    # A not-in-db file
+                    location = value.fsPath
+                r.write(' location="%s"' % location)
         r.write('>')
         # Dump the field value
         self.dumpValue(r, value, type, className)
@@ -243,14 +249,14 @@ class Marshaller:
     def dumpObject(self, r, o, complete=False, fieldNames=None):
         '''Dumps this p_o(bject) in p_r'''
         # Is p_o an Appy object ?
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  yes | if p_complete is True, the entire object is dumped, with all
         #      | its fields being visible on the "xml" layout (p_fieldNames is
         #      | ignored). If p_complete is False, only its URL is dumped.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  no  | p_complete is ignored and the entire object is dumped, based on
         #      | its __dict__ or using names from p_fieldNames.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         if self.isAppy(o):
             # It is a Appy object
             if complete:
